-- Authors

CREATE TABLE "Authors" (
    id integer NOT NULL,
    "First Name" text,
    "Last Name" text,
    "Website" mathesar_types.uri
);

CREATE SEQUENCE "Authors_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE "Authors_id_seq" OWNED BY "Authors".id;


-- Books

CREATE TABLE "Books" (
    id integer NOT NULL,
    "Title" text,
    "Publication Year" integer,
    "Media" text,
    "Page Count" integer,
    "LC Classification" text,
    "ISBN" text,
    "Dewey Decimal" text,
    "Dewey Wording" text,
    "Author" integer,
    "Publisher" integer
);

CREATE SEQUENCE "Books_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE "Books_id_seq" OWNED BY "Books".id;


-- Checkouts

CREATE TABLE "Checkouts" (
    id integer NOT NULL,
    "Item" integer,
    "Patron" integer,
    "Checkout Time" timestamp without time zone NOT NULL,
    "Due Date" date NOT NULL,
    "Check In Time" timestamp without time zone
);

CREATE SEQUENCE "Checkouts_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE "Checkouts_id_seq" OWNED BY "Checkouts".id;


-- Items

CREATE TABLE "Items" (
    id integer NOT NULL,
    "Barcode" text,
    "Acquisition Date" date,
    "Acquisition Price" mathesar_types.mathesar_money,
    "Book" integer
);


CREATE SEQUENCE "Items_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE "Items_id_seq" OWNED BY "Items".id;


-- Media

CREATE TABLE "Media" (
    id integer NOT NULL,
    "Type" text
);

CREATE SEQUENCE "Media_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE "Media_id_seq" OWNED BY "Media".id;


-- Patrons

CREATE TABLE "Patrons" (
    id integer NOT NULL,
    "First Name" text,
    "Last Name" text,
    "Email" mathesar_types.email
);

CREATE SEQUENCE "Patrons_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE "Patrons_id_seq" OWNED BY "Patrons".id;


-- Publishers

CREATE TABLE "Publishers" (
    id integer NOT NULL,
    "Name" text
);

CREATE SEQUENCE "Publishers_id_seq"
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

ALTER SEQUENCE "Publishers_id_seq" OWNED BY "Publishers".id;


-- Set up defaults
ALTER TABLE ONLY "Authors"
  ALTER COLUMN id SET DEFAULT nextval('"Authors_id_seq"'::regclass);
ALTER TABLE ONLY "Books"
  ALTER COLUMN id SET DEFAULT nextval('"Books_id_seq"'::regclass);
ALTER TABLE ONLY "Checkouts"
  ALTER COLUMN id SET DEFAULT nextval('"Checkouts_id_seq"'::regclass);
ALTER TABLE ONLY "Items"
  ALTER COLUMN id SET DEFAULT nextval('"Items_id_seq"'::regclass);
ALTER TABLE ONLY "Media"
  ALTER COLUMN id SET DEFAULT nextval('"Media_id_seq"'::regclass);
ALTER TABLE ONLY "Patrons"
  ALTER COLUMN id SET DEFAULT nextval('"Patrons_id_seq"'::regclass);
ALTER TABLE ONLY "Publishers" 
  ALTER COLUMN id SET DEFAULT nextval('"Publishers_id_seq"'::regclass);

--
-- Data for Name: Authors; Type: TABLE DATA; Schema: Library Management; Owner: -
--

INSERT INTO "Authors" (id, "First Name", "Last Name", "Website") VALUES 
  (352, 'Lemony', 'Snicket', 'http://danielhandler.com/'),
  (300, 'Garth', 'Nix', 'https://garthnix.com/'),
  (197, 'Yuval Noah', 'Harari', 'https://www.ynharari.com/'),
  (336, 'Richard', 'Rumelt', NULL),
  (370, 'Ashlee', 'Vance', NULL),
  (284, 'Bill', 'Moggridge', NULL),
  (373, 'Oscar', 'Wilde', NULL),
  (339, 'Mary Doria', 'Russell', NULL),
  (271, 'Ian', 'McDonald', NULL),
  (218, 'Robert', 'Iger', NULL),
  (177, 'William', 'Gibson', 'https://williamgibsonbooks.com/'),
  (210, 'Robin', 'Hobb', 'http://www.robinhobb.com/'),
  (167, 'F. Scott', 'Fitzgerald', NULL),
  (269, 'Steve', 'McConnell', NULL),
  (260, 'Steven', 'Levy', 'https://stevenlevy.com/'),
  (219, 'Apple Computer', 'Inc.', NULL),
  (117, 'Adam', 'Cohen', NULL),
  (9, 'John Joseph', 'Adams', 'https://www.johnjosephadams.com/'),
  (183, 'David', 'Goodstein', NULL),
  (143, 'Philip K', 'Dick', 'https://philipdick.com/'),
  (214, 'Karen', 'Holtzblatt', NULL),
  (39, 'Fredrik', 'Backman', NULL),
  (171, 'Sigmund', 'Freud', NULL),
  (2, 'M.A.S.', 'Abdel Haleem', NULL),
  (42, 'Ronald J.', 'Baker', NULL),
  (346, 'Jason', 'Santa Maria', NULL),
  (316, 'Christopher', 'Priest', NULL),
  (264, NULL, 'Livy', NULL),
  (244, 'Jon', 'Kleinberg', NULL),
  (241, 'Erin', 'Kissane', NULL),
  (325, 'Janice (Ginny)', 'Redish', NULL),
  (140, 'Deborah A.', 'Dell', NULL),
  (326, 'Fred', 'Rogers', 'https://pbskids.org/rogers/'),
  (233, 'Jeremy', 'Keith', NULL),
  (52, 'Rick', 'Bayless', NULL),
  (98, 'Dale', 'Carnegie', NULL),
  (345, 'George', 'Sansom', NULL),
  (330, 'Patrick', 'Rothfuss', 'https://patrickrothfuss.com/'),
  (375, 'Tad', 'Williams', 'https://www.tadwilliams.com/'),
  (43, 'Iain M.', 'Banks', NULL),
  (88, 'Octavia E.', 'Butler', NULL),
  (307, 'Dan', 'Olsen', NULL),
  (164, 'Stephen', 'Few', NULL),
  (212, 'Adam', 'Hochschild', NULL),
  (49, 'Chris', 'Bates', NULL),
  (155, 'Michael', 'Ende', NULL),
  (36, 'James Edward', 'Austen-Leigh', NULL),
  (301, 'Don', 'Norman', NULL),
  (204, 'Joseph', 'Heller', NULL),
  (8, 'Douglas', 'Adams', 'https://www.douglasadams.com/'),
  (303, 'Samin', 'Nosrat', 'http://ciaosamin.com/'),
  (18, 'Christopher', 'Alexander', NULL),
  (67, 'Hassan', 'Blasim', NULL),
  (276, 'Peter', 'Menzel', 'https://www.menzelphoto.com/'),
  (249, 'Ken', 'Kocienda', NULL),
  (252, NULL, 'Kouhala', NULL),
  (245, 'Martin', 'Kleppmann', NULL),
  (152, 'Carol S.', 'Dweck', NULL),
  (55, 'George', 'Beahm', NULL),
  (61, 'Alfred', 'Bester', NULL),
  (53, 'Peter S.', 'Beagle', NULL),
  (263, 'Megan', 'Lindholm', 'https://www.meganlindholm.com/'),
  (71, 'Nate', 'Bolt', NULL),
  (202, 'Chip', 'Heath', NULL),
  (201, 'Stephen', 'Hawking', 'https://www.hawking.org.uk/'),
  (133, 'Faith', 'D''Aluisio', NULL),
  (305, 'Tsugumi', 'Ohba', NULL),
  (322, 'Melanie', 'Rawn', NULL),
  (299, 'Larry', 'Niven', NULL),
  (180, 'William', 'Goldman', NULL),
  (158, 'Anders', 'Ericsson', NULL),
  (178, 'James', 'Gleick', 'http://around.com/'),
  (226, 'James', 'Joyce', NULL),
  (11, 'Katherine', 'Addison', 'https://www.katherineaddison.com/'),
  (63, NULL, 'Biharilal', NULL),
  (192, 'Charles', 'Hallisey', NULL),
  (12, 'Mortimer Jerome', 'Adler', NULL),
  (234, 'Thomas', 'Keneally', NULL),
  (274, 'Robin', 'McKinley', 'https://robinmckinley.com/'),
  (206, 'Frank', 'Herbert', NULL),
  (62, NULL, 'Bharavi', NULL),
  (258, 'Gail Carson', 'Levine', 'https://gailcarsonlevine.com/'),
  (248, 'Jake', 'Knapp', NULL),
  (323, 'Bharatchandra', 'Ray', NULL),
  (344, 'Brandon', 'Sanderson', 'https://www.brandonsanderson.com/'),
  (162, 'Tony', 'Fadell', NULL),
  (297, 'Rachel', 'Neumeier', 'https://www.rachelneumeier.com/'),
  (315, 'Terry', 'Pratchett', 'http://terrypratchettbooks.com/'),
  (294, 'R. K.', 'Narayan', NULL),
  (220, 'Kazuo', 'Ishiguro', NULL),
  (242, 'Ellen', 'Klages', NULL),
  (29, NULL, 'Arrian', NULL),
  (348, 'Mark', 'Seemann', NULL),
  (26, 'Alan', 'Armstrong', NULL),
  (287, 'Richard', 'Moss', NULL),
  (6, 'Chinua', 'Achebe', NULL),
  (187, 'David', 'Grann', NULL),
  (58, 'Scott', 'Belsky', NULL),
  (317, 'Philip', 'Pullman', 'http://philip-pullman.com/'),
  (308, 'Ada', 'Palmer', 'https://www.adapalmer.com/'),
  (70, 'T. Richard', 'Blurton', NULL),
  (30, 'Tim', 'Ash', NULL),
  (151, 'April', 'Dunford', NULL),
  (45, 'Mark', 'Barden', NULL),
  (175, 'Neil', 'Gaiman', 'https://neilgaiman.com/'),
  (33, 'Margaret', 'Atwood', NULL),
  (76, 'Carol Ryrie', 'Brink', NULL),
  (193, 'David', 'Halpern', NULL),
  (359, NULL, 'Surdas', NULL),
  (196, 'Barbara', 'Hambly', 'http://www.barbarahambly.com/'),
  (78, 'Alton', 'Brown', 'https://altonbrown.com/'),
  (280, 'David', 'Mitchell', NULL),
  (243, 'Laura', 'Klein', NULL),
  (114, 'Arthur C.', 'Clarke', 'http://clarkefoundation.org/'),
  (134, 'Charles', 'Darwin', NULL),
  (320, 'Hannu', 'Rajaniemi', NULL),
  (364, 'Edward R.', 'Tufte', 'https://www.edwardtufte.com/'),
  (351, 'Shel', 'Silverstein', 'http://shelsilverstein.com/'),
  (125, 'Ben', 'Coppin', NULL),
  (135, 'Aliette', 'de Bodard', NULL),
  (47, 'Byron', 'Barton', NULL),
  (282, 'Masao', 'Miyoshi', NULL),
  (77, 'Charlotte', 'Bronte', NULL),
  (123, 'Alan', 'Cooper', NULL),
  (281, 'Margaret', 'Mitchell', NULL),
  (343, 'Carl', 'Sagan', 'https://carlsagan.com/'),
  (161, 'David C.', 'Evans', NULL),
  (314, 'Jeff', 'Potter', NULL),
  (130, 'Walter', 'Cronkite', NULL),
  (86, 'Frances Hodgson', 'Burnett', NULL),
  (84, 'Lois McMaster', 'Bujold', 'http://www.dendarii.com/'),
  (337, 'Salman', 'Rushdie', NULL),
  (176, 'Simon', 'Garfield', NULL),
  (148, 'Apostolos', 'Doxiadis', NULL),
  (302, 'Donald A.', 'Norman', NULL),
  (27, 'Joe', 'Armstrong', NULL),
  (342, 'Oliver', 'Sacks', 'https://oliversacks.com/'),
  (358, 'Elizabeth', 'Strout', NULL),
  (255, 'Gayle', 'Laakmann', NULL),
  (13, 'Saladin', 'Ahmed', NULL),
  (368, 'Catherynne M.', 'Valente', 'https://www.catherynnemvalente.com/'),
  (321, 'Mark', 'Ravina', NULL),
  (147, 'Korry', 'Douglas', NULL),
  (332, 'J. K.', 'Rowling', 'https://jkrowling.com/'),
  (289, 'Brandon', 'Mull', 'http://brandonmull.com/'),
  (166, 'Richard', 'Feynman', 'http://richardfeynman.com/'),
  (41, 'Rebekah', 'Baggs', NULL),
  (16, 'Julie M.', 'Albright', NULL),
  (262, 'Mel', 'Lindauer', NULL),
  (372, 'Bill', 'Watterson', NULL),
  (156, 'Jessica', 'Enders', NULL),
  (327, 'Marshall B.', 'Rosenberg', NULL),
  (104, 'Geoffrey', 'Chaucer', NULL),
  (141, 'Jared', 'Diamond', NULL),
  (120, 'Jim', 'Collins', 'https://www.jimcollins.com/'),
  (341, 'Louis', 'Sachar', NULL),
  (115, 'Susanna', 'Clarke', NULL),
  (95, 'Truman', 'Capote', NULL),
  (136, 'Daniel', 'Defoe', NULL),
  (278, 'A. A.', 'Milne', NULL),
  (208, 'Georgette', 'Heyer', 'http://www.georgette-heyer.com/'),
  (59, 'Alan', 'Bennett', NULL),
  (324, 'Satyajit', 'Ray', NULL),
  (290, 'Kevin', 'Mullet', NULL),
  (313, 'Beatrix', 'Potter', NULL),
  (309, 'Ellis', 'Peters', NULL),
  (90, 'Gaius Julius', 'Caesar', NULL),
  (37, NULL, 'Avi', NULL),
  (5, NULL, 'Abu''l-Fazl', NULL),
  (124, 'Rick', 'Copeland', NULL),
  (35, 'Jane', 'Austen', NULL),
  (69, 'Deborah', 'Blum', NULL),
  (110, 'Julia', 'Child', NULL),
  (374, 'Laura Ingalls', 'Wilder', NULL),
  (4, 'Daniel', 'Abraham', 'https://www.danielabraham.com/'),
  (85, 'Leah', 'Buley', NULL),
  (238, 'Maxwell', 'King', NULL),
  (118, 'Eliot', 'Coleman', NULL),
  (328, 'Scott', 'Rosenberg', NULL),
  (213, 'Reid', 'Hoffman', 'https://www.reidhoffman.org/'),
  (163, 'Raymond E', 'Feist', 'https://raymondefeist.com/'),
  (182, 'Emmanuel', 'Goldstein', NULL),
  (181, 'Eliyahu M.', 'Goldratt', NULL),
  (50, 'L. Frank', 'Baum', NULL),
  (80, 'Steven', 'Brust', 'http://dreamcafe.com/'),
  (209, 'Patricia', 'Highsmith', NULL),
  (312, 'Steve', 'Portigal', NULL),
  (237, 'Tracy', 'Kidder', NULL),
  (215, NULL, 'Homer', NULL),
  (254, 'Mike', 'Kuniavsky', NULL),
  (191, 'Erika', 'Hall', NULL),
  (311, NULL, 'Plato', NULL),
  (267, 'Scott', 'Lynch', NULL),
  (17, 'Louisa May', 'Alcott', NULL),
  (369, NULL, 'Valmiki', NULL),
  (367, 'Mark', 'Twain', NULL),
  (92, 'Susan', 'Cain', NULL),
  (23, 'Stephen P.', 'Anderson', NULL),
  (376, 'Gene', 'Wolfe', NULL),
  (198, 'Frances', 'Hardinge', 'http://www.franceshardinge.com/'),
  (222, 'Marius B.', 'Jansen', NULL),
  (40, 'Jono', 'Bacon', NULL),
  (203, 'William Least', 'Heat-Moon', NULL),
  (354, 'Alexander', 'Solzhenitsyn', NULL),
  (335, 'Michael', 'Ruhlman', 'https://ruhlman.com/'),
  (24, 'Katherine', 'Arden', 'http://katherinearden.com/'),
  (221, 'Makiko', 'Itoh', NULL),
  (150, 'Annie', 'Duke', 'https://www.annieduke.com/'),
  (74, 'John', 'Briggs', NULL),
  (83, 'Christopher', 'Buehlman', NULL),
  (57, 'Kent', 'Beck', NULL),
  (224, 'Diana Wynne', 'Jones', 'http://www.dianawynnejones.com/'),
  (93, 'Joseph', 'Campbell', NULL),
  (306, 'Zoe', 'Oldenbourg', NULL),
  (230, 'Sam', 'Kaner', NULL),
  (73, 'Ray', 'Bradbury', NULL),
  (132, 'William', 'Dalrymple', 'https://williamdalrymple.com/'),
  (87, 'Jim', 'Butcher', 'https://www.jim-butcher.com/'),
  (256, 'Ursula K.', 'Le Guin', 'https://www.ursulakleguin.com/'),
  (51, 'Kathy', 'Baxter', NULL),
  (81, 'Bill', 'Bryson', NULL),
  (28, 'Tavish', 'Armstrong', NULL),
  (25, 'Dan', 'Ariely', NULL),
  (7, 'Diane', 'Ackerman', 'http://www.dianeackerman.com/'),
  (174, 'Jostein', 'Gaarder', NULL),
  (68, 'Jeb', 'Blount', NULL),
  (122, 'Andrew Scott', 'Conning', NULL),
  (272, 'Harold', 'McGee', NULL),
  (228, 'Daniel', 'Kahneman', NULL),
  (116, 'Ernest', 'Cline', 'http://www.ernestcline.com/'),
  (365, NULL, 'Tulsidas', NULL),
  (318, 'Leela', 'Punyaratabandhu', 'https://leelapunyaratabandhu.com/'),
  (251, 'E. L.', 'Konigsburg', NULL),
  (240, 'Rosemary', 'Kirstein', NULL),
  (14, 'Joan', 'Aiken', 'https://www.joanaiken.com/'),
  (129, 'Douglas', 'Crockford', NULL),
  (160, 'Bert', 'Esselink', NULL),
  (72, 'Anthony', 'Bourdain', NULL),
  (350, 'Anna', 'Sewell', NULL),
  (20, 'David', 'Allen', NULL),
  (225, 'Robert', 'Jordan', NULL),
  (270, 'David', 'McCullough', NULL),
  (231, 'Guy', 'Kawasaki', NULL),
  (138, 'Samuel R.', 'Delany', NULL),
  (236, 'Daniel', 'Keyes', NULL),
  (173, 'Cornelia', 'Funke', 'https://corneliafunke.com/'),
  (273, 'Patricia A.', 'McKillip', NULL),
  (169, 'Louise', 'Fitzhugh', NULL),
  (48, 'A.L.', 'Basham', NULL),
  (142, 'Junot', 'Diaz', NULL),
  (216, 'Aldous', 'Huxley', NULL),
  (333, 'J.K.', 'Rowling', 'https://jkrowling.com/'),
  (97, 'Lillian Stewart', 'Carl', NULL),
  (126, 'James S. A.', 'Corey', 'https://www.jamessacorey.com/'),
  (103, 'Ann Mei', 'Chang', NULL),
  (331, 'J K', 'Rowling', 'https://jkrowling.com/'),
  (338, 'Bertrand', 'Russell', NULL),
  (105, 'Anton', 'Chekhov', NULL),
  (128, 'Robert X', 'Cringely', 'https://www.cringely.com/'),
  (139, 'Charles', 'De Lint', NULL),
  (112, 'Clayton M.', 'Christensen', NULL),
  (259, 'Steven D.', 'Levitt', NULL),
  (247, 'Cole Nussbaumer', 'Knaflic', NULL),
  (65, 'David J.', 'Bland', NULL),
  (91, 'Marty', 'Cagan', NULL),
  (38, 'Mahmoud M.', 'Ayoub', NULL),
  (60, 'Robert Jackson', 'Bennett', 'https://www.robertjacksonbennett.com/'),
  (144, 'Philip K.', 'Dick', 'https://philipdick.com/'),
  (121, 'Suzanne', 'Collins', 'http://www.suzannecollinsbooks.com/'),
  (235, 'Ken', 'Kesey', NULL),
  (111, 'Frank', 'Chimero', NULL),
  (149, 'Charles', 'Duhigg', NULL),
  (190, 'Andrew S.', 'Grove', NULL),
  (288, 'Steve', 'Mulder', NULL),
  (275, 'Thomas Lewis', 'M.D.', NULL),
  (349, 'Brian', 'Selznick', NULL),
  (239, 'Stephen', 'King', NULL),
  (211, 'Thomas', 'Hobbes', NULL),
  (99, 'Max', 'Caspar', NULL),
  (94, 'Laine', 'Campbell', NULL),
  (250, 'Jon', 'Kolko', NULL),
  (159, 'Steven', 'Erikson', NULL),
  (366, 'Megan Whalen', 'Turner', 'http://meganwhalenturner.org/'),
  (96, 'Philip', 'Caputo', NULL),
  (32, 'Jeff', 'Atwood', NULL),
  (362, 'J. R. R.', 'Tolkien', 'https://www.tolkienestate.com/'),
  (170, 'Karl', 'Fogel', NULL),
  (113, 'Agatha', 'Christie', 'https://www.agathachristie.com/'),
  (101, 'Michael', 'Chabon', 'http://michaelchabon.com/'),
  (227, 'Eiko', 'Kadono', NULL),
  (293, 'R K', 'Narayan', NULL),
  (172, 'Jason', 'Fried', 'https://world.hey.com/jason'),
  (189, 'Roger Lancelyn', 'Green', NULL),
  (296, 'Edith', 'Nesbit', NULL),
  (119, 'Lizzie', 'Collingham', NULL),
  (157, 'David', 'Epstein', NULL),
  (107, 'Ron', 'Chernow', NULL),
  (75, 'David', 'Brin', NULL),
  (285, 'Alan', 'Moore', 'https://www.dodgemlogic.com/'),
  (146, 'Fyodor', 'Dostoevsky', NULL),
  (127, 'James S.A.', 'Corey', 'https://www.jamessacorey.com/'),
  (179, 'William', 'Golding', NULL),
  (153, 'Umberto', 'Eco', 'http://umbertoeco.it/'),
  (21, 'Cindy', 'Alvarez', NULL),
  (31, 'Isaac', 'Asimov', NULL),
  (268, 'Niccolo', 'Machiavelli', NULL),
  (361, 'Nandi', 'Timmana', NULL),
  (356, 'Neal', 'Stephenson', 'https://nealstephenson.com/'),
  (310, 'James', 'Peterson', 'https://jimcooks.com/'),
  (186, 'Steven', 'Gould', 'https://eatourbrains.com/steve'),
  (108, 'C. J.', 'Cherryh', 'https://cherryh.com/'),
  (46, 'J. M.', 'Barrie', NULL),
  (265, 'Jack', 'London', NULL),
  (54, 'Ishmael', 'Beah', NULL),
  (106, 'Kevin', 'Cheng', NULL),
  (109, 'Ted', 'Chiang', NULL),
  (22, 'Hans Christian', 'Anderson', NULL),
  (223, 'Caroline', 'Jarrett', NULL),
  (89, 'Bill', 'Buxton', NULL),
  (371, 'Tom', 'Vanderbilt', NULL),
  (291, 'Randall', 'Munroe', 'https://xkcd.com/'),
  (229, NULL, 'Kamandaki', NULL),
  (154, 'Nadia', 'Eghbal', NULL),
  (279, 'Barbara', 'Mirel', NULL),
  (347, 'Bill', 'Scott', NULL),
  (195, 'Kristina', 'Halvorson', NULL),
  (168, 'John Dennis', 'Fitzgerald', NULL),
  (3, 'Joe', 'Abercrombie', 'https://joeabercrombie.com/'),
  (200, 'Sam', 'Harris', NULL),
  (253, 'Steve', 'Krug', 'https://sensible.com/'),
  (277, 'China', 'Mieville', 'http://www.chinamieville.net/'),
  (56, 'Elizabeth', 'Bear', NULL),
  (360, NULL, 'Svayambhudeva', NULL),
  (353, 'Edward', 'Snowden', NULL),
  (207, 'Michael J', 'Hernandez', NULL),
  (266, 'J. Kenji', 'López-Alt', NULL),
  (82, 'Amy', 'Bucher', NULL),
  (205, 'Cal', 'Henderson', NULL),
  (295, 'Jawaharlal', 'Nehru', NULL),
  (292, 'Haruki', 'Murakami', NULL),
  (165, 'Tina', 'Fey', NULL),
  (184, 'Kim', 'Goodwin', NULL),
  (64, 'Mark', 'Bittman', NULL),
  (102, 'Andrew', 'Chaikin', NULL),
  (217, 'Eva', 'Ibbotson', NULL),
  (145, 'Charles', 'Dickens', NULL),
  (137, 'Hideo', 'Dekura', NULL),
  (283, 'Shigeru', 'Mizuki', NULL),
  (100, 'Ed', 'Catmull', NULL),
  (79, 'Amy', 'Brown', NULL),
  (10, 'Richard', 'Adams', NULL),
  (194, 'Jack', 'Halpern', NULL),
  (131, 'Roald', 'Dahl', 'https://www.roalddahl.com/'),
  (188, 'Theodore', 'Gray', NULL),
  (304, 'Naomi', 'Novik', 'http://naominovik.com/'),
  (340, 'Tony', 'Russell-Rose', NULL),
  (298, 'Jakob', 'Nielsen', NULL),
  (261, 'C. S.', 'Lewis', 'https://www.cslewis.com/'),
  (257, 'Madeleine', 'L''Engle', NULL),
  (319, 'Whitney', 'Quesenbery', NULL),
  (246, 'Chuck', 'Klosterman', NULL),
  (66, 'Steve', 'Blank', NULL),
  (377, NULL, 'Xenophon', NULL),
  (363, 'Leo', 'Tolstoy', NULL),
  (334, 'Jeffrey', 'Rubin', NULL),
  (286, 'Peter', 'Morville', NULL),
  (357, 'Jonathan', 'Stroud', 'http://jonathanstroud.com/'),
  (199, 'Cyd', 'Harrell', NULL),
  (329, 'Louis', 'Rosenfeld', NULL),
  (34, 'Jean', 'Audouze', NULL),
  (19, 'Lloyd', 'Alexander', NULL),
  (355, 'Andrew', 'Stellman', NULL),
  (15, 'Josef', 'Albers', 'https://www.albersfoundation.org/'),
  (185, 'Jeff', 'Gothelf', NULL),
  (1, 'Edwin A.', 'Abbott', NULL),
  (232, 'Guy Gavriel', 'Kay', 'https://brightweavings.com/'),
  (44, 'James', 'Barclay', NULL)
;

--
-- Data for Name: Books; Type: TABLE DATA; Schema: Library Management; Owner: -
--

INSERT INTO "Books" (id, "Title", "Publication Year", "Media", "Page Count", "LC Classification", "ISBN", "Dewey Decimal", "Dewey Wording", "Author", "Publisher") VALUES 
  (813, 'Skyward Flight: The Collection', 2022, '1', 640, NULL, '05-9-356785-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 59),
  (525, 'Knight of the Demon Queen', 2000, '1', 272, 'PS3558.A4215 K57', '03-45-42189-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 196, 61),
  (247, 'Death Note, Vol. 4', 2012, '2', 208, 'PN6790.J33 O358', '978-109-05-1210-2', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (946, 'The Dawn of the Roman Empire Books 31-40', 2009, '2', 656, 'DG247 .L5513', '01995-5-568-0', '937.02', 'Ancient World > History and Geography > Italian Peninsula to 476 and adjacent territories to 476 > Italian Peninsula to 476 and adjacent territories to 476 > Republic 509-31 B.C.', 264, 170),
  (571, 'Lying', 2013, '1', NULL, 'BF637.D42', '978-19-4005-100-0', '177.3', 'Ethics > Philosophy and Psychology > Social Ethics  > Truth - Slander - Flattery', 200, 88),
  (330, 'Fairest', 2006, '1', 336, 'PZ8.L4793 F', '00607-340-8-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 258, 106),
  (283, 'Dragonshadow', 1999, '1', 297, 'PS3558.A4215 D73', '03-454-2187-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 196, 61),
  (1025, 'The Graveyard Book', 2015, '2', NULL, 'PZ7.G1273', '978-00-6-238065-4', '813', 'American fiction > English (North America) > Literature', 175, 104),
  (882, 'The Anarchy: The East India Company, Corporate Violence, and the Pillage of an Empire', 2019, '1', 576, 'DS465 .D35', '1-6355-7395-5', '954.031', '1785-1858 (East India Company) > 1785вЂ“1947 British rule > Asia > History and Geography > India and South Asia', 132, 37),
  (1195, 'The Return of the Great Brain', 1986, '2', 150, 'PZ7.F57535Re', '8-0-7128487-6', '[Fic]', NULL, 168, 64),
  (1259, 'The Summer Tree', 2001, '2', 400, 'PR9199.K39 S9', '04-5145-822-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 202),
  (64, 'A Reporter''s Life', 1996, '1', 400, 'PN4874.C84', '0394578-7-9-1', '70.92', 'Biographies > Biography And History > Information > Journalism And Publishing > Journalism And Publishing', 130, 134),
  (328, 'Fablehaven', 2013, '2', 384, 'PZ7.M9112 F', '978-583-2-07558-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 8),
  (742, 'Return of the Thief', 2020, '1', NULL, 'PZ7.T85565 R', '00628-7-447-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 366, 107),
  (456, 'Holes', 2000, '1', 240, 'PZ7.S1185', '9-7011780-4-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 341, 81),
  (1273, 'The Toll-Gate', 2011, '2', 320, 'PZ3.H514 T', '14022-3-881-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (1209, 'The Sandman Vol. 2: The Doll''s House', 1991, '2', 256, 'PN6728.S26 G35', '0-9-3028959-5', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (1193, 'The Republic of Thieves', 2013, '1', 672, 'PS3612.Y5427', '05-53-80469-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 267, 61),
  (337, 'Fantastic Beasts and Where to Find Them', 2019, '1', 160, 'PR6068.O93 F36', '978-9-0599-5955-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 331, 17),
  (214, 'Curtain', 1975, '2', 238, 'PZ3.C4637PR6005.H66Cu3', '039-60-7191-0', '823/.9/12', 'English fiction > English {except North American} > Literature > Modern Period', 113, 67),
  (197, 'Cotillion', 2007, '2', 368, 'PR6015.E795 C68', '978-14-0-221008-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (231, 'Dead Beat', 2006, '2', 528, 'PS3602.U85 D43', '0451-4609-1-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (1121, 'The Lost Conspiracy', 2009, '1', 576, 'PZ7.H21834 L', '006-08-8041-4', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 106),
  (1039, 'The Gypsy', 1996, '1', 272, 'PS3552.R84 G96', '1-92889-191-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 80, 243),
  (707, 'Princeps'' Fury', 2008, '1', 400, 'PS3602.U85 P75', '0441-0-1638-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (1352, 'Under the Dome', 2013, '2', 1088, 'PS3561.I483', '1-4-7673547-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 239, 90),
  (339, 'Farmer Boy', 1953, '2', 384, 'PZ7.W6461 F', '0-0-6440003-4', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (167, 'Children of God', 1998, '1', 436, 'PS3568.U76678 C48', '0-6794-5635-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 339, 262),
  (209, 'Cryptonomicon', 2000, '2', 918, 'PS3569.T3868 C79', '0-09-941067-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 356, 198),
  (154, 'Catch-22', 1994, '2', 569, 'PS3558.E476', '0-0-9953601-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 204, 263),
  (606, 'Mister Monday', 2005, '1', 368, 'PZ7.N647 M', '0439-703-69-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 300, 210),
  (1358, 'Usability Engineering', 1993, '1', NULL, 'QA76.U83 N54', '0-1-2518405-0', '6', 'Computing and Information > Information > Special Topics', 298, 3),
  (1032, 'The Great Hunt', 1990, '1', 600, 'PS3560.O7617 G74', '031-2-85140-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (687, 'Persuasion', 1980, '2', 288, 'PR4034 .P4', '05-5-321137-4', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 28),
  (312, 'Engaged: Designing for Behavior Change', 2020, '2', 320, NULL, '193-382-042-X', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 82, 204),
  (641, 'Now We Are Sick', 2008, '2', 108, 'PR1195.C47', '978-9-3-6837844-9', '808', 'By Topic > Literature > Rhetoric and anthologies', 175, 74),
  (363, 'Fool''s Fate', 2015, '1', 640, 'PS3558.O33636 F67', '978-951-7-84065-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (364, 'Fool''s Quest', 2015, '1', 768, 'PS3558.O33636 F675', '0-5533-9292-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 61),
  (1164, 'The Peter Rabbit Pop-up Book', 1983, '1', 12, 'PZ7 .P44154', '0-7-2322950-3', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 313, 266),
  (671, 'Past Eight O''Clock', 1986, '1', 128, 'PZ7.A2695 P', '0224-02-856-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 14, 129),
  (144, 'Caddie Woodlawn', 2006, '2', 288, 'PZ7.B78 C', '14-1-694028-6', '813', 'American fiction > English (North America) > Literature', 76, 8),
  (359, 'Fool Moon', 2011, '2', 352, 'PS3602.U85 F66', '978-147-4456-69-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (316, 'Eric', 2002, '2', 224, 'PZ7.P8865 E', '03-80-82121-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 111),
  (472, 'Information Dashboard Design: Displaying Data for At-a-Glance Monitoring', 2013, '1', 246, 'HD30.213 .F49', '193-83-7700-1', '658.4038011', 'Decision-making And Knowledge Management > Executive > Knowledge Management > Management > Management and auxiliary services > Systems > Technology and Application of Knowledge', 164, 13),
  (769, 'Sauces: Classical and Contemporary Sauce Making', 2003, '1', 624, 'TX819.A1 P47', '68-17-16728-4', '641.814', 'Appetizers > Cooking Specific Dishes > Food And Drink > Home and family management > Sauces, Relishes, Dressings > Technology and Application of Knowledge', 310, 269),
  (623, 'Myths of the Norsemen: Retold from the Old Norse Poems and Tales', 1994, '2', 274, 'BL860 .G68', '0-1-4036738-1', '293.13', 'Germanic religion > Mythologies > Mythology--Germanic religion > Other Religions > Religions', 189, 192),
  (516, 'Just My Type: A Book About Fonts', 2011, '1', 384, 'Z250 .G228', '978-1592-4-0652-4', '686.224', 'Books: Binding, Design, Printing > Manufacture of products for specific uses, handicrafts > Printing > Technology and Application of Knowledge > Typefaces > Typography', 176, 93),
  (602, 'Mistborn', 2006, '1', NULL, 'PS3619.A533', '978-1-9-3857013-1', '813', 'American fiction > English (North America) > Literature', 344, 72),
  (862, 'Taran Wanderer', 1969, '2', 272, 'PZ7.A3774 T', '0-4-4048483-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 275),
  (701, 'Powder and Patch', 2017, '2', 208, 'PZ3.H514 P', '978-3-6-4336509-5', '813', 'American fiction > English (North America) > Literature', 208, 224),
  (1385, 'Where the Sidewalk Ends: Poems and Drawings', 1974, '1', 176, 'PZ8.S5844 W', '00602-5-667-2', '811.54', '1945-1999 > 20th Century > American poetry > English (North America) > Literature', 351, 106),
  (1300, 'The White Road of the Moon', 2017, '1', 384, 'PZ7.N4448 W', '05-5350-932-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 135),
  (483, 'InterWorld', 2007, '1', 256, 'PZ7.G1273 I', '0-061-23896-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 175, 106),
  (1138, 'The Minpins', 1996, '2', 47, 'PZ7.D1515 M', '0-5-9016318-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 210),
  (1405, 'Worlds of Exile and Illusion: Three Complete Novels of the Hainish Series in One Volume--Rocannon''s World; Planet of Exile; City of Illusions', 2007, '2', 370, 'PS3562.E42 A6', '031-28621-1-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 231),
  (417, 'Gypsy Rizka', 1999, '1', 144, 'PZ7.A3774 G', '05254-612-1-3', '808.8', 'Anthologies & Collections > By Topic > Literature > Rhetoric and anthologies', 19, 75),
  (1112, 'The Lie Tree', 2016, '1', 384, 'PZ7.H21834 L', '1-41-971895-9', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 114),
  (342, 'Figure It Out: Getting from Information to Understanding', 2020, '2', 432, NULL, '1-9-3382096-9', '153', 'Cognition And Memory > Philosophy and Psychology > Psychology', 23, 251),
  (632, 'Nicholas Nickleby', 1994, '2', 158, 'PR4565.A2 F67', '185-4-71233-0', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 145, 276),
  (66, 'Armada', 2015, '1', 368, 'PS3603.L548 A89', '08041-372-5-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 116, 52),
  (321, 'Ever', 2008, '1', 256, 'PZ7.L578345 E', '00-6-122962-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 258, 106),
  (737, 'Rendezvous with Rama', 1977, '2', 303, 'PZ3.C551205PR6005.L36Ren3', '014545-79-1-7', '823/.9/14', 'English fiction > English {except North American} > Literature > Modern Period', 114, 103),
  (1011, 'The Girl Who Circumnavigated Fairyland in a Ship of Her Own Making', 2011, '1', 256, 'PZ7.V232 G', '0-3-1264961-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 368, 83),
  (697, 'Poirot Investigates', 1975, '2', NULL, 'PR6005.H66', '0-5531-0026-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 26),
  (1370, 'Warbreaker', 2009, '1', 592, 'PS3619.A533', '07-6532-030-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (574, 'Made to Stick: Why Some Ideas Survive and Others Die', 2007, '1', 291, 'HM1033 .H43', '1-400-06428-7', '302.13', 'General topics of social interaction > Social Interaction > Social Sciences > Social choice > Social sciences', 202, 198),
  (65, 'Arjuna and the Hunter', 2016, '1', 480, 'PK3791.B252 K5813', '0-6-7450496-8', '891.21', 'Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Sanskrit > Sanskrit poetry', 62, 116),
  (725, 'Rapid Contextual Design: A How-to Guide to Key Techniques for User-Centered Design', 2004, '2', 320, 'QA76.S88 H65', '978-012-354-051-5', '4.2', 'Computer Architectures > Computer science > Computing and Information > Information', 214, 156),
  (220, 'Dark Lord of Derkholm', 2001, '2', 528, 'PZ7.J684 D', '0064-47-336-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (621, 'My Lord John', 2009, '2', 464, 'PZ3.H514 PR6015 .E795', '14-02-21353-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 225),
  (96, 'Baudolino', 2002, '1', 522, 'PQ4865.C6 B3813', '01-5-100690-3', '853.914', '1900- > 1945-1999 > 20th Century > Italian > Italian fiction > Literature', 153, 102),
  (135, 'Brothers of the Wind', 2021, '1', 272, 'PS3573.I45563 B76', '0-7564-1268-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 375, 57),
  (1059, 'The History of Akbar, Volume 7', 2021, '1', NULL, NULL, '06-742-4416-8', NULL, NULL, 5, 116),
  (586, 'Master of the House of Darts', 2017, '2', 328, NULL, '1625672-5-0-0', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 135, 126),
  (163, 'Chanur''s Venture', 1985, '2', NULL, 'PS3553.H358 C45', '088-67-7183-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 108, 57),
  (1357, 'Uprooted', 2016, '2', 464, 'PS3614.O93', '08041-79-05-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (1091, 'The Kodansha Kanji Dictionary', 2022, '1', 2112, 'PL677.5 .N73', '978-00-3-775178-4', '495.6', 'Japanese > Language > Languages of East & Southeast Asia > Other Languages', 194, 136),
  (968, 'The End', 2006, '1', 368, 'PZ7.S6795 E', '00-6-441016-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 106),
  (918, 'The Cambridge Atlas of Astronomy', 1994, '1', 471, 'QB65 .G6813', '0-52-143438-6', '523', 'Astronomical objects and astrophysics > Astronomy > Natural sciences and mathematics', 34, 41),
  (590, 'Matilda', 2007, '2', 256, 'PZ7.D1515 M', '0-1-4132266-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (1081, 'The Just Bento Cookbook: Everyday Lunches To Go', 2018, '2', 127, 'TX724.J3 I85', '978-86-2-369403-7', '641.5952', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Japan > Middle East, Asia  > Technology and Application of Knowledge', 221, 137),
  (353, 'Flight of the Nighthawks', 2009, '1', 368, 'PS3556.E446 F58', '978-0-0988666-2-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 163, 113),
  (1215, 'The Sandman Vol. 6: Fables and Reflections', 2002, '2', 264, 'PN6728.S26 G29', '80-33-06185-5', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (766, 'Salt, Fat, Acid, Heat: Mastering the Elements of Good Cooking', 2017, '1', 480, 'TX651', '14767-5-383-0', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 303, 220),
  (289, 'Dreamweaver''s Dilemma', 1997, '2', 252, 'PS3552.U397 D74', '0-9-1536853-6', '813', 'American fiction > English (North America) > Literature', 84, 159),
  (989, 'The Farthest Shore', 1984, '2', 208, 'PZ7.L5215 F', '0553-2684-7-3', '813', 'American fiction > English (North America) > Literature', 256, 26),
  (1041, 'The Hallowed Hunt', 2005, '1', 480, 'PS3552.U397 H35', '0-0-6057462-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 113),
  (625, 'Naked Sun', 1993, '2', 208, 'PZ3.A8316 N', '0-5860-1016-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 95),
  (338, 'Fantastic Mr. Fox', 1988, '2', 96, 'PZ7.D1515 F', '0-14-032872-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (153, 'Cat Among The Pigeons', 1961, '2', 210, 'PR6005.H66 C3', '0-67-177704-1', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (179, 'Code That Fits in Your Head : Heuristics for Software Engineering', 2021, '2', 406, 'QA76.D47', '0-1374-6440-1', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 348, 6),
  (52, 'An Infamous Army', 2007, '2', 512, 'PR6015.E795 I64', '978-1-402210-07-5', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 225),
  (454, 'Hogfather', 1999, '1', 384, 'PR6066.R34 H64', '06-135722-2-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 104),
  (1173, 'The Prague Cemetery', 2011, '1', 464, 'PQ4865.C6', '05475-7-753-2', '853.914', '1900- > 1945-1999 > 20th Century > Italian > Italian fiction > Literature', 153, 121),
  (527, 'Lady of Quality', 2008, '2', 304, 'PR6015.E795 L34', '140-22-1077-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (369, 'Fortunately, the Milk', 2019, '2', NULL, 'PZ7.G1273 F', '978-7-8-3591566-6', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 175, 104),
  (287, 'Dreaming in Code: Two Dozen Programmers, Three Years, 4,732 Bugs, and One Quest for Transcendent Software', 2008, '2', 416, 'QA76.D47 R668', '1400-0-8247-1', '650', 'Business > Management and auxiliary services > Technology and Application of Knowledge', 328, 240),
  (892, 'The Ballad of Songbirds and Snakes', 2020, '1', NULL, 'PZ7.C6837', '1-3386351-7-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (1347, 'Ubik', 2006, '2', 224, 'PS3554.I3 U24', '0-57-507921-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 143, 92),
  (234, 'Death Comes As the End', 1972, '2', 179, 'PR6005.H66', '06-7-177445-X', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (1286, 'The Virgin in the Ice', 1995, '2', 208, 'PR6031.A49 V5', '0-4-4640428-4', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 309, 239),
  (117, 'Blue Highways: A Journey into America', 1999, '2', 448, 'E169.Z8', '0-3-1635329-9', '917.304927', 'Geography and Travel > History and Geography > North America > Travel > United States', 203, 23),
  (1384, 'When We Were Very Young', 1992, '2', 112, 'PZ8.M6354 W', '014036-12-3-5', '821.912', '1900-1944 > 20th Century > English poetry > English {except North American} > Literature > Modern period 1900-', 278, 60),
  (515, 'Just Enough Research', 2013, '2', 154, 'Q180.A1 H36', '193-75-5710-3', '1.4', 'Computing and Information > Information > Knowledge > Research', 191, 2),
  (1329, 'Thinking in Bets: Making Smarter Decisions When You Don''t Have All the Facts', 2018, '1', 288, 'HD30 .D85', '978-07352-1-635-8', '658.4', 'Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 150, 187),
  (1324, 'Thick as Thieves', 2017, '1', 400, 'PZ7.T85565 T', '0-062568-24-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 366, 98),
  (1130, 'The Marvelous Misadventures of Sebastian', 1970, '1', NULL, 'PZ7.A3774 M', '05253-473-9-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 75),
  (1218, 'The Sandman Vol. 8: Worlds'' End', 2004, '2', 168, 'PN6728.S26 G35', '99-75-83774-8', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (1340, 'Tongues of Serpents of Temeraire', 2010, '1', 288, 'PS3614.O93 T68', '0-3-4549689-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (427, 'Harry Potter and the Chamber of Secrets', 1998, '2', 256, 'PR6068.O93', '074-75-3848-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 333, 37),
  (1074, 'The Importance of Being Earnest', 1997, '2', 80, 'PS3573.I265 I47', '4-1188565-4-4', '822', 'English drama > English {except North American} > Literature', 373, 175),
  (919, 'The Candy Shop War', 2014, '2', 400, 'PZ7.M9112 C', '1-481-41119-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 8),
  (993, 'The Fires of Heaven', 1993, '1', 704, 'PS3560.O7617 F57', '0-31-285427-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (638, 'Notes From Underground', 1983, '2', 176, 'PG3326 .Z4', '0-55-321144-7', '891.733', '1800вЂ“1917 > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Russian and East Slavic languages > Russian fiction', 146, 27),
  (296, 'Effective Programming: More Than Writing Code', 2016, '2', 278, 'QA73', '978-0-44-839071-X', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 32, 50),
  (223, 'Database Design for Mere Mortals: 25th Anniversary Edition', 2020, '2', 640, 'QA76.D26 H477', '013678-80-4-1', '5.756', 'Computer programming, programs, data, security > Computing and Information > Data > Information > Relational Databases > Specific Databases And Data Files', 207, 6),
  (349, 'Fitz and The Fool: Coloring Book', 2020, '2', 52, NULL, '978-5-2-9494860-4', NULL, NULL, 210, 54),
  (1178, 'The Princess Bride: S Morgenstern''s Classic Tale of True Love and High Adventure', 2000, '2', 416, 'PS3557.O384', '034-5348-03-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 180, 61),
  (1310, 'The Wish', 2000, '1', 208, 'PZ7.L578345 W', '0-06-027901-X', '479', 'Language > Latin > Minor Italic; Medieval Latin', 258, 107),
  (977, 'The Epic of Ram, Volume 5', 2022, '1', 400, NULL, '978-08-2-380695-X', NULL, NULL, 365, 116),
  (1299, 'The Whispering Skull', 2014, '1', 448, 'PZ7.S92475 W', '14231-6-492-X', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (429, 'Harry Potter and the Cursed Child - Parts One & Two', 2016, '1', NULL, 'PR6120.H67 H37', '13-3809-913-2', '822.92', '1900- > 2000- > English drama > English {except North American} > Literature', 332, 17),
  (1166, 'The Philadelphia Adventure', 1999, '1', 150, 'PZ7.A3774 P', '4-2772101-0-9', NULL, NULL, 19, 75),
  (645, 'Observing the User Experience: A Practitioner''s Guide to User Research', 2012, '2', 664, 'QA76.H85 K86', '01238-4-869-5', '4.019', 'Computer Interaction With Humans > Computer science > Computer science -- subdivisions > Computing and Information > Information > Theory And Instruction', 254, 156),
  (518, 'Keys to the Demon Prison', 2010, '1', 544, 'PZ7.M9112 K', '1-6-0641238-8', '823', 'English fiction > English {except North American} > Literature', 289, 213),
  (5, 'A Borrowed Man', 2015, '1', 304, 'PS3573.O52 B67', '076-538-114-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 376, 243),
  (1098, 'The Landmark Xenophon''s Anabasis', 2021, '1', 672, 'PA4495 .A5313', '03-07-90685-X', '935.05092', 'Ancient World > History and Geography > Mesopotamia and Iranian Plateau to 637', 377, 173),
  (147, 'Captain''s Fury', 2007, '1', 464, 'PS3602.U85 C37', '0-4-4101527-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (1123, 'The Lost Metal', 2022, '1', 528, NULL, '0-765391-19-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 145),
  (130, 'Brave New World', 2006, '2', 288, 'PR6015.U9', '006-085-052-3', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 216, 108),
  (524, 'Knife Children', 2020, '1', NULL, NULL, '15-96069-54-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 84, 234),
  (749, 'Ringworld', 1985, '2', 352, 'PZ4.N734 R', '0-345-33392-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 299, 61),
  (494, 'Jane Austen: Seven Novels', 2011, '1', 1232, 'PR4030', '978-128-032-825-2', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 30),
  (1144, 'The Murder of Roger Ackroyd', 1980, '2', 197, 'PR6005.H66 M85', '119-66051-9-X', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (185, 'Contact', 1997, '2', 448, 'PS3569.A287 C6', '0-6710-0410-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 343, 184),
  (172, 'City of Stairs', 2014, '2', 452, 'PS3602.E66455 C58', '0-8-0413717-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 60, 40),
  (374, 'Fractals: The Patterns of Chaos: Discovering a New Aesthetic of Art, Science, and Nature', 1992, '2', 192, 'QA614 .B75', '06-71-74217-5', '514.74', 'Analytic Topology > Fractals > Mathematics > Natural sciences and mathematics > Topology', 74, 220),
  (1211, 'The Sandman Vol. 3: Dream Country', 2002, '2', 160, 'PN6728.S26 G35', '76-79-75731-0', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (899, 'The Best American Short Stories', 2005, '2', 432, 'PS648.S5', '061-8427-05-8', '813.0108', 'American fiction > By type > English (North America) > Literature > Short fiction', 101, 121),
  (188, 'Conversational Design', 2018, '2', NULL, 'QA76.S88 H35', '1937-5-5767-7', NULL, NULL, 191, 2),
  (1198, 'The Rithmatist', 2013, '1', 384, 'PZ7.S19797 R', '076-532-032-0', '813.7', 'American fiction > English (North America) > Literature', 344, 247),
  (203, 'Crispin: At the Edge of the World', 2008, '2', 256, 'PZ7.A953 C', '1-4231030-5-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 37, 122),
  (201, 'Creativity, Inc.: Overcoming the Unseen Forces That Stand in the Way of True Inspiration', 2014, '1', 368, 'HD53 .C394', '08-12-99301-2', '658.4', 'Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 100, 198),
  (865, 'Temeraire: In the Service of the King', 2006, '1', 832, 'PS3614.O93 I5', '07-3946-871-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 32),
  (31, 'A Hole in Space', 1986, '2', NULL, 'PZ4.N734 H', '034-53377-7-8', '813', 'American fiction > English (North America) > Literature', 299, 61),
  (482, 'Interviewing Users: How to Uncover Compelling Insights', 2013, '2', 176, 'H61 .P66', '1-9338-2011-X', '1.4', 'Computing and Information > Information > Knowledge > Research', 312, 204),
  (895, 'The Bear and the Nightingale', 2017, '2', 368, 'PS3601.R42 B43', '11018-85-95-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 61),
  (1351, 'Under the Banyan Tree & Other Stories', 1994, '2', NULL, 'PR9499.N3 U5', '81-85986-14-2', '823', 'English fiction > English {except North American} > Literature', 293, 125),
  (965, 'The Elements of Content Strategy', 2010, '2', NULL, 'TK5105.888 .K57', '09844-42-55-3', '5.72', 'Computer programming, programs, data, security > Computing and Information > Data > Information > Preparation and Representation', 241, 2),
  (293, 'Dune', 1999, '2', 896, 'PS3558.E63', '2-39-315816-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 206, 4),
  (775, 'Seeing Further: The Story of Science, Discovery, and the Genius of the Royal Society', 2011, '2', 512, 'Q41 .S44', '0-0-6199977-6', '506.041', 'General Science > Natural sciences and mathematics > Societies', 81, 270),
  (1271, 'The Tiger in the Well', 2004, '2', 448, 'PZ7.P968 T', '04-399-7780-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 209),
  (184, 'Consider Phlebas', 2003, '2', 480, 'PR6052.A485', '18572-31-38-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 43, 167),
  (390, 'Ghosts of Tsavo: Tracking the Mythic Lions of East Africa', 2002, '1', 300, 'QL737.C23 C345', '0792-26-362-6', '599.757', 'Carnivora > Felines > Lions > Mammals > Natural sciences and mathematics > Zoology', 96, 158),
  (941, 'The Creeping Shadow', 2016, '1', 464, 'PZ7.S92475 C', '14-847-0967-5', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (455, 'Holes', 1998, '1', 240, 'PZ7.S1185', '03-7433-265-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 341, 81),
  (491, 'I Was a Rat!', 2002, '2', 176, 'PZ7.P968 I', '04-404-1661-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 275),
  (249, 'Death Note, Vol. 6', 2006, '2', 208, 'PN6790.J34 D426', '14215-0-627-0', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (1393, 'Whittington', 2011, '2', 208, 'PZ10.A8625 W', '978-31-048-1368-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 26, 275),
  (1055, 'The History of Akbar, Volume 3', 2017, '1', 704, NULL, '0-6-7465982-1', NULL, NULL, 5, 116),
  (1392, 'Whittington', 2006, '2', 208, 'PZ10.A8625 W', '0-3-7582865-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 26, 275),
  (1322, 'The Xanadu Adventure', 2010, '1', 160, 'PZ7.A3774 X', '978-5-24-355505-7', '791.43', 'Film > Film, Radio, and Television > Public performances > Recreational and performing arts > The arts', 19, 75),
  (1104, 'The Last Samurai: The Life and Battles of Saigo Takamori', 2005, '2', 288, 'DS881.S2 R35', '0-4-7170537-3', '952.031092', '1868-1945 > Asia > History and Geography > Japan', 321, 269),
  (939, 'The Conquest of Happiness', 1971, '2', 249, 'BJ1481 .R75', '0871-4-0051-0', '615', 'Medicine and health > Pharmacology and therapeutics > Technology and Application of Knowledge', 338, 142),
  (821, 'Snow Crash', 2000, '2', 440, 'PS3569.T3868', '0553-380-95-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 356, 227),
  (787, 'Shadows for Silence in the Forests of Hell / Perfect State', 2015, '1', 176, NULL, '19-385700-6-5', NULL, NULL, 344, 72),
  (803, 'Signal to Noise', 1999, '2', 80, 'PN6727.G35 S54', '15-697-1144-5', '741', 'Drawing &  drawings > Graphic arts and decorative arts > The arts', 175, 56),
  (713, 'Ptolemy''s Gate', 2005, '2', 528, 'PZ7.S92475 P', '038560-8-68-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 68),
  (12, 'A Civic Technologist''s Practice Guide', 2020, '2', 168, NULL, '173-52865-0-8', NULL, NULL, 199, 86),
  (317, 'Esio Trot', 1992, '2', 64, 'PZ7.D1515 E', '014-0360-99-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (975, 'The Epic of Ram, Volume 4', 2018, '1', 368, NULL, '067-49750-2-2', NULL, NULL, 365, 116),
  (771, 'Schindler''s List', 1993, '2', 400, 'PR9619.K46 S3', '0-671-88031-4', '823', 'English fiction > English {except North American} > Literature', 234, 21),
  (639, 'Not Just a Witch', 2003, '1', 176, 'PZ7.I11555 N', '0525-47-101-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 75),
  (692, 'Planesrunner', 2011, '1', 269, 'PZ7.M47841776 P', '1-61-614541-2', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 271, 195),
  (580, 'Malafrena', 1980, '2', NULL, 'PZ4.L518 PS3562 .E42', '0-42504-647-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 34),
  (415, 'Grip of the Shadow Plague', 2008, '1', 487, 'PZ7.M9112 G', '978-15-9038-898-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 47),
  (1375, 'Watership Down', 1996, '1', 448, 'PZ10.A197', '0-6-8483605-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 10, 211),
  (1137, 'The Minority Report and Other Classic Stories', 2002, '2', 380, 'PS3554.I3 M46', '0806-523-79-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 144, 46),
  (459, 'House of Many Ways', 2009, '2', 432, 'PZ7.J684 H', '00-61-47797-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (461, 'Howl''s Moving Castle', 2001, '2', 336, 'PZ7.J684', '0-064-41034-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (175, 'Clockwork : Or All Wound Up', 1998, '2', 128, 'PZ7.P968 C', '059-0-12998-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 210),
  (951, 'The Dispossessed', 1999, '2', 336, 'PS3562.E42', '978-18-57988-82-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 169),
  (29, 'A History of Japan, 1615-1867', 1963, '2', 272, 'DS835.S27', '0-8-0470527-5', '952', 'Asia > History and Geography > Japan', 345, 228),
  (820, 'Snapshot / Dreamer', 2017, '1', NULL, 'PS648.F3', '978-19-38-57014-8', NULL, NULL, 344, 72),
  (745, 'Rhythm of War', 2020, '1', NULL, 'PS3619.A533', '0-765326-38-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (264, 'Designing Web Interfaces: Principles and Patterns for Rich Interactions', 2009, '2', 334, 'QA76.U83 S39', '05-9651-625-8', '5.437', 'Computer programming, programs, data, security > Computing and Information > Information > Systems programming and programs > Systems programs', 347, 168),
  (33, 'Alcatraz Versus the Evil Librarians', 2007, '1', 320, 'PZ7.S19797 A', '0-439925-50-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 210),
  (71, 'A Short History of Nearly Everything', 2004, '2', 686, 'Q162.B88', '05-529-9704-8', '600', '-- > General Technology > Technology and Application of Knowledge', 81, 36),
  (679, 'Penric''s Mission', 2017, '1', 264, 'PS3552.U397 P47', '159606-84-4-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 84, 234),
  (507, 'Jim Butcher''s The Dresden Files: Storm Front Volume 2', 2011, '1', 128, 'PN6728.D74 P69', '16-069016-0-5', '741', 'Drawing &  drawings > Graphic arts and decorative arts > The arts', 87, 76),
  (1402, 'Words of Radiance', 2015, '2', 1328, 'PS3619.A533', '07653-65-28-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 245),
  (36, 'Algorithm Design', 2005, '1', 864, 'QA76.A43 K54', '0-3-2129535-8', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 244, 6),
  (849, 'Summer Knight', 2002, '2', 371, 'PS3552.U8242 S95', '0451-458-92-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (217, 'Cytonic', 2021, '1', 432, 'PZ7.S19797 C', '03-9-955585-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 59),
  (1094, 'The Ladies of Grace Adieu: and Other Stories', 2007, '2', 256, 'PR6103.L375 L33', '0-7475-8936-4', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 115, 37),
  (1062, 'The Hollow Boy', 2015, '1', 400, 'PZ7.S92475 H', '1484709-6-8-3', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (38, 'A Little Princess', 1987, '2', 336, 'PZ7.B934 L', '00644-0-187-1', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 86, 106),
  (534, 'Law of the Broken Earth', 2010, '2', 464, 'PS3614.E553', '03-160-7993-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 167),
  (686, 'Persepolis Rising', 2017, '1', 608, 'PS3601.B677 P47', '031-63-3283-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 126, 167),
  (640, 'Now We Are Sick', 2005, '2', 108, 'PR1195.C47', '1-8-9205810-3', '808', 'By Topic > Literature > Rhetoric and anthologies', 175, 74),
  (1399, 'Wolf Moon', 2004, '2', 256, 'PS3554.E11415 W64', '0-1-4240077-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 139, 85),
  (897, 'The Beggar Queen', 2002, '2', 256, 'PZ7.A3774 B', '014-131-070-7', '839.78', 'German and Germanic > Literature > Literature in other Germanic languages  > Swedish literature > Swedish miscellany', 19, 85),
  (674, 'Pegasus', 2010, '1', 400, 'PZ7.M1988 P', '03-99246-77-0', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 274, 94),
  (84, 'At Bertram''s Hotel', 1974, '2', 180, 'PR6005.H66 A89', '0-67-177702-5', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (382, 'From The Two Rivers: The Eye of the World, Book 1', 2012, '2', 416, 'PS3560.O7617 E94', '978-5623-820-73-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 229),
  (145, 'Calamity', 2016, '1', 432, 'PZ7.S19797 C', '0-385-74360-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 59),
  (703, 'Prelude to Foundation', 1991, '2', 528, 'PS3551.S5 P7', '05-53278-39-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 227),
  (125, 'Boy: Tales of Childhood', 1999, '2', 176, 'PR6054.A35 Z', '0-1-4130305-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (963, 'The El Dorado Adventure', 1987, '1', 176, 'PZ7.A3774 E', '0-52-544313-4', '303', 'Social Processes > Social Sciences > Social sciences', 19, 75),
  (208, 'Crypto: How the Code Rebels Beat the Government--Saving Privacy in the Digital Age', 2001, '1', 352, 'QA76.A25 L49', '0670-85-950-8', '5.8', 'Computer Security > Computer programming, programs, data, security > Computing and Information > Information', 260, 260),
  (772, 'Search Patterns: Design for Discovery', 2010, '2', 192, 'QA76.D26', '05-96-80227-7', '6.7784', 'Computing and Information > Information > Multimedia Programming for Specific Systems > Multimedia systems > Special Topics', 286, 168),
  (292, 'Dune', 1990, '2', 896, 'PS3558.E63', '0-4-4117271-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 206, 4),
  (966, 'The Emperor''s Soul', 2012, '2', 176, 'PS3619.A533 E47', '16-16960-92-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 236),
  (1088, 'The Knights of Crystallia', 2009, '1', 336, 'PZ7.S19797 A', '0-43992-555-X', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 344, 210),
  (832, 'Spook Country', 2008, '2', 384, 'PS3557.I2264 S66', '0-425-22141-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 177, 34),
  (98, 'Beauvallet', 2010, '2', 304, 'PR6015.E795 B39', '14-0221-951-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (391, 'Ghost Story', 2011, '1', 496, 'PS3602.U85 G48', '04-51-46379-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (960, 'The Dragon''s Path', 2014, '2', 592, 'PS3601.B677 D73', '978-68173-11-59-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 4, 167),
  (213, 'Cursor''s Fury', 2006, '1', 448, 'PS3602.U85 C87', '0-441-01434-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (1298, 'The Wheel of Time Companion: The People, Places and History of the Bestselling Series', 2015, '1', 816, 'PS3560.O7617 W47', '07-653-1461-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (879, 'The Amber Spyglass', 2003, '2', 480, 'PZ7.P968 A', '0-4-4023815-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 139),
  (579, 'Magic: The Final Fantasy Collection', 1997, '2', 400, 'PS3551.S5 M27', '000-64-8203-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 113),
  (1191, 'The Remarkable Journey of Prince Jen', 1991, '1', 273, 'PZ7.A3774 R', '0-525-44826-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 75),
  (505, 'Jim Butcher''s The Dresden Files: Fool Moon Volume 2', 2013, '1', 120, 'PN6728.D74 B8842', '1-6-0690377-2', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 87, 76),
  (714, 'Pure Magic', 2015, '2', 350, NULL, '1-5141184-4-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 50),
  (285, 'Dragonstar', 2002, '1', 304, 'PS3558.A4215 D75', '0345441-2-1-4', '813', 'American fiction > English (North America) > Literature', 196, 61),
  (1266, 'The Theory Of Everything', 2009, '2', 132, 'QB985 .H39', '978-61324-4-142-7', '520', 'Astronomy > Astronomy > Natural sciences and mathematics', 201, 127),
  (453, 'His Majesty''s Dragon', 2006, '2', 384, 'PS3614.O93', '0-3454812-8-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (394, 'Golden Age and Other Stories', 2017, '1', 184, 'PS3614.O93 A6', '1-5-9606829-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 234),
  (694, 'Pocket Full of Rye', 2002, '2', 224, 'PR6005.H66 P6', '000-71-2097-4', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 106),
  (334, 'Fanatical Prospecting: The Ultimate Guide to Opening Sales Conversations and Filling the Pipeline by Leveraging Social Selling, Telephone, Email, Text, and Cold Calling', 2015, '1', 304, 'HF5438.25 .B568', '1-11914-475-2', '658.8', 'Management > Management and auxiliary services > Of Marketing > Technology and Application of Knowledge', 68, 269),
  (551, 'Lilavai', 2021, '1', NULL, 'PK5013.K8 L513', '067-4247-59-0', '891.3', 'Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Middle Indic languages', 252, 116),
  (514, 'Jumper', 2014, '2', 368, 'PS3557.O8947 J8', '0765-3-7816-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 186, 243),
  (893, 'The Bands of Mourning', 2016, '1', 448, 'PS3619.A533 B36', '076-5-37857-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (991, 'The Feynman Lectures on Physics, Vol. 2', 1971, '2', 592, 'QC23.F47', '02-0-102117-X', '530', 'Natural sciences and mathematics > Physics > Physics', 166, 6),
  (1373, 'Warrior of the Altaii', 2019, '1', 352, 'PS3560.O7617 W37', '1-2-5024765-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (880, 'The Amulet of Samarkand', 2003, '1', 464, 'PZ7.S92475 A', '0-7-8681859-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (853, 'Sunshine', 2006, '1', 389, 'PS3563.C3816 S86', '8365-5806-2-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 274, 34),
  (664, 'Orca', 1996, '2', 290, 'PS3552.R84', '0441-0-0196-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 80, 4),
  (1236, 'The Sharing Knife', 2008, '1', 437, 'PS3552.U397 S56', '00-61-37533-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 113),
  (846, 'Storytelling for User Experience', 2014, '2', 298, 'PE1425', '978-9-75251-394-0', '5.276', 'Computer programming, programs, data, security > Computing and Information > For Networked Multimedia Systems > Information > Programming for Specific Environments > Web Programming', 319, 204),
  (422, 'Hans Christian Anderson: The Complete Fairy Tales and Stories', 2008, '2', 972, 'PT8116 .E5', '07-60776-07-5', '839.8136', '1800вЂ“1900 > Danish > Danish and Norwegian literature  > Danish fiction > German and Germanic > Literature > Literature in other Germanic languages', 22, 30),
  (1146, 'The Name of the Rose', 1983, '1', 512, 'PQ4865.C6', '015-1-44647-4', '853.914', '1900- > 1945-1999 > 20th Century > Italian > Italian fiction > Literature', 153, 121),
  (1034, 'The Green Glass Sea', 2008, '2', 352, 'PZ7.K6768 G', '014-2411-49-3', '813', 'American fiction > English (North America) > Literature', 242, 192),
  (1308, 'The Winter of the Witch', 2019, '1', 384, 'PS3601.R42 W56', '1-1-0188599-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 61),
  (696, 'Poems from the Satsai', 2021, '1', NULL, 'PK1967.V47 S2713', '0-674-98707-1', NULL, NULL, 63, 116),
  (650, 'Oliver Twist', 1998, '2', 496, 'PR4567', '081258-00-3-6', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 145, 244),
  (529, 'Land Of The Burning Sands', 2010, '2', 464, 'PS3614.E553 L36', '184-1498-74-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 167),
  (1238, 'The Shattered Lens', 2011, '1', 304, 'PZ7.S19797 A', '978-83-243-5001-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 210),
  (17, 'A Conspiracy of Kings', 2010, '1', 336, 'PZ7.T85565 C', '0-0618709-3-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 366, 98),
  (1388, 'White Fang and the Call of the Wild', 1994, '2', 278, 'PS3523.O46 C3', '0-14062-114-8', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 265, 174),
  (225, 'Daughter of the Empire: An Epic Saga of the World on the Other Side of the Riftwar', 1988, '2', 421, 'PS3556.E446 D38', '0-5-5327211-X', '813', 'American fiction > English (North America) > Literature', 163, 26),
  (928, 'The Chronicles of Chrestomanci: The Magicians of Caprona / Witch Week', 2010, '2', 548, 'PZ7.J684 C', '978-73-25911-88-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 248),
  (675, 'Penric and the Shaman', 2017, '1', NULL, 'PS3552.U397', '1-5-9606815-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (53, 'Anna Karenina', 1954, '2', 864, 'PG3366 .A6', '01-4-044041-0', '891.733', '1800вЂ“1917 > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Russian and East Slavic languages > Russian fiction', 363, 174),
  (878, 'The Amazing Adventures of Kavalier & Clay', 2001, '2', 656, 'PS3553.H15 A82', '0312-2829-9-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 101, 183),
  (246, 'Death Note, Vol. 4', 2006, '2', 208, 'PN6790.J33 O358', '1-4-2150331-X', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (62, 'April Lady', 2012, '2', 288, 'PR6015.E795 A85', '140-2-23878-9', '813', 'American fiction > English (North America) > Literature', 208, 224),
  (407, 'Great Brain at the Academy', 1986, '1', 164, 'PZ7.F57535 G', '060603-4-58-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 168, 63),
  (1013, 'The Girl Who Raced Fairyland All the Way Home', 2016, '1', 320, 'PZ7.V232 G', '12-5-002351-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 368, 83),
  (901, 'The BFG', 2007, '2', 224, 'PZ7.D1515 B', '0141-32-262-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (983, 'The Eye of the World: The Graphic Novel, Volume Four', 2013, '1', 176, 'PN6728.W57 D585', '076-53-3631-6', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 225, 243),
  (1306, 'The Wind''s Twelve Quarters', 1976, '2', 277, 'PS3562.E42 W56', '055-30290-7-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 29),
  (403, 'Good Strategy Bad Strategy: The Difference and Why It Matters', 2011, '1', NULL, 'HD30 .R854', '978-0-30788-623-1', '658.4012', 'Executive > Management > Management and auxiliary services > Planning, control, strategy > Strategy > Technology and Application of Knowledge', 336, 53),
  (1265, 'The Theory Of Everything', 2007, '2', 132, 'QB985 .H39', '8-1-7992591-9', '520', 'Astronomy > Astronomy > Natural sciences and mathematics', 201, 127),
  (193, 'Cordelia''s Honor', 1999, '2', 596, 'PS3552.U397 S53', '067-1-57828-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (1194, 'The Return of the Great Brain', 1974, '2', 150, 'PZ7.F57535Re', '08-03-77413-3', '[Fic]', NULL, 168, 64),
  (844, 'Storm Front', 2009, '2', 384, 'PS3552.U8242', '978-3-7568498-7-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (496, 'Jane Eyre', 2001, '2', 514, 'PR4167', '817-1675-02-6', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 77, 205),
  (1169, 'The Picture of Dorian Gray', 1998, '2', 304, 'PR5819.A2', '0-375751-51-3', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 373, 155),
  (1096, 'The Landmark Arrian: The Campaigns of Alexander', 2012, '2', 560, 'DF234 .A77313', '978-14-00-07967-4', '938.07', 'Ancient World > Greece to 323 > Greece to 323 > History and Geography > Macedonian Supremacy (362-323 BC)', 29, 14),
  (659, 'Only You Can Save Mankind', 2004, '2', 179, 'PZ7.P8865 O', '0-5-5255111-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 49),
  (1129, 'The Martian Chronicles', 2001, '2', NULL, 'PS3503.R167', '0965-0-1747-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 73, 196),
  (670, 'Paladin of Souls', 2003, '1', 464, 'PS3552.U397 P355', '03-80979-02-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 113),
  (110, 'Black Dog Short Stories I & II', 2016, '2', 337, NULL, '1-5396328-7-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 50),
  (498, 'JavaScript: The Good Parts', 2016, '2', 176, 'QA76.J39 C763', '978-393-82278-8-3', '5.133', 'Computer programming, programs, data, security > Computing and Information > General Programming Languages > Information > Languages > Programming', 129, 214),
  (990, 'The Feynman Lectures on Physics, Vol. 1', 1963, '1', 535, 'QC23.F47', '02-0102-010-6', '500', 'General Science > General Science > Natural sciences and mathematics', 166, 6),
  (631, 'New Spring', 2003, '2', 336, 'PS3560.O7617 N49', '07-6-534545-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 276),
  (673, 'Peak: Secrets from the New Science of Expertise', 2016, '1', 336, 'BF378.E94 E75', '05-4-445623-8', '153.9', 'Assessment And Intelligence > Cognition And Memory > Philosophy and Psychology > Psychology', 158, 146),
  (14, 'A Civil Contract', 2011, '2', 432, 'PR6015.E795 C58', '1-4-0223877-0', '813', 'American fiction > English (North America) > Literature', 208, 224),
  (384, 'Gardens of the Moon', 2005, '2', 666, 'PR9199.E745 G37', '07-65-34878-0', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 159, 243),
  (1320, 'The Wouldbegoods', 1972, '2', 288, 'PZ7.B61 W', '0-14-030122-4', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 296, 192),
  (969, 'The English Teacher', 2007, '2', 184, 'PR9499.N3 E5', '8185-98-603-7', '823', 'English fiction > English {except North American} > Literature', 294, 125),
  (1315, 'The Wizard of Oz', 1994, '2', 188, 'PZ7.B327', '0140-366-93-8', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 50, 192),
  (788, 'Shadows of Self', 2015, '1', 384, 'PS3619.A533 S53', '0-765378-55-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (1124, 'The Magic Finger', 1993, '2', 64, 'PZ7.D1515 M', '0-14-036303-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (1038, 'The Gypsy', 1992, '1', 272, 'PS3552.R84 G96', '0312-8-5274-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 80, 243),
  (39, 'A Long Way Gone: Memoirs of a Boy Soldier', 2007, '1', 240, 'DT516.B43 A3', '0-37-410523-5', '966.404', 'Africa > History and Geography > Sierra Leone > West Africa', 54, 81),
  (487, 'I, Robot', 1999, '2', 304, 'PS3551.S5 I13', '3-887-31613-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 227),
  (573, 'Macintosh Human Interface Guidelines', 1993, '2', NULL, 'QA76.M3 M328', '978-02-0-162216-4', '5.265', 'Computer programming, programs, data, security > Computing and Information > For Specific Computers > Information > Personal Computers > Programming for Specific Environments', 219, 6),
  (646, 'Obviously Awesome: How to Nail Product Positioning so Customers Get It, Buy It, Love It', 2019, '2', NULL, NULL, '199-90230-0-5', '658.82', 'Management > Management and auxiliary services > Of Marketing > Technology and Application of Knowledge', 151, 10),
  (630, 'Neverwhere Illustrated Edition', 2017, '1', 448, 'PR6057.A319 N48', '0-0-6282133-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 270),
  (469, 'Impulse', 2014, '2', 384, 'PS3557.O8947 I47', '0-7-6536602-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 186, 246),
  (91, 'Babylon''s Ashes', 2016, '1', 544, 'PS3601.B677 B33', '03163-34-74-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 126, 167),
  (1047, 'The Heroes', 2011, '1', 560, 'PR6101.B49 H47', '0-3-1604498-9', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 167),
  (931, 'The Compass Rose', 1983, '2', 288, 'PS3562.E42 C6', '0-5-5323512-5', '813', 'American fiction > English (North America) > Literature', 256, 227),
  (1293, 'The Way of Kings', 2011, '2', 1280, 'PS3619.A533', '07-653-6527-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 245),
  (1242, 'The Slippery Slope', 2003, '1', 352, 'PZ7.S6795 S', '0-0-6029641-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 106),
  (778, 'Sense and Sensibility', 1992, '2', 320, 'PR4034 .S4', '1-8-5326016-9', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 272),
  (460, 'House of Shadows', 2012, '2', 352, 'PS3614.E553 H68', '03-160-7277-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 167),
  (348, 'Fitz and The Fool: Coloring Book', 2018, '2', 52, NULL, '0-99-961636-6', NULL, NULL, 210, 54),
  (1153, 'The Ogre Of Oglefort', 2012, '1', NULL, 'PZ7.I11555 O', '060-626-088-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 249),
  (1147, 'The Name of the Wind', 2008, '2', 736, 'PS3618.O8685 N36', '075-64047-4-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 330, 57),
  (306, 'Empire of Grass', 2019, '1', 688, 'PS3573.I45563 E57', '0-7-5641062-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 375, 57),
  (323, 'Evil Under the Sun', 1973, '2', NULL, 'PR6005.H66 E9', '06-717766-7-3', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (414, 'Grim Tuesday', 2005, '1', 336, 'PZ7.N647 G', '043-97-0370-0', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 300, 210),
  (271, 'Discourses on Livy', 2009, '2', 448, 'JC143 .M16313', '978-01-99-55555-0', '937', 'Ancient World > History and Geography > Italian Peninsula to 476 and adjacent territories to 476', 268, 171),
  (1307, 'The Wind-Up Bird Chronicle', 1998, '2', 607, 'PL856.U673 N4513', '067-9-77543-9', '895.635', '1945вЂ“2000 > Asian (east and south east) languages > Japanese > Japanese fiction > Literature > Literature of other languages', 292, 263),
  (347, 'Fish & Shellfish: The Cook''s Indispensable Companion', 1996, '1', 418, 'TX747 .P464', '0-68-812737-1', '641.692', 'Cooking game, seafood, miscellaneous foods > Cooking specific materials > Fish > Food And Drink > Home and family management > Technology and Application of Knowledge', 310, 271),
  (314, 'Epic: Legends of Fantasy', 2012, '2', 608, 'PR830.F3 E653', '1-61696-084-1', '813.0876608', 'Adventure fiction > American fiction > By type > Collections > English (North America) > Fantasy > Genre fiction > Literature > Speculative fiction', 9, 236),
  (1110, 'The Left Hand of Darkness', 1980, '1', 213, 'PS3562.E42', '0-0-6012574-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 106),
  (1072, 'The Illyrian Adventure', 2000, '2', 144, 'PZ7.A3774 I', '0-1-4130313-1', '219', '"Natural Theology and Secularism > Religions > [Formerly ""Analogies; Correspondences""; No longer used]"', 19, 192),
  (419, 'Half a King', 2014, '1', 384, 'PR6101.B49 H34', '0007-55-020-0', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 113),
  (252, 'Death Note, Vol. 9', 2007, '2', 208, 'PN6790.J33', '14-21506-30-0', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (894, 'The Bear and the Nightingale', 2017, '1', 336, 'PS3601.R42 B43', '1-10-188593-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 61),
  (884, 'The Architecture Of Open Source Applications', 2012, '2', 432, 'QA76.S46 A73', '12-57-63801-7', '5.12', 'Computer programming, programs, data, security > Computing and Information > Information > Programming > Systems Analysis And Design', 79, 143),
  (1028, 'The Great Brain Does It Again', 1975, '2', 129, 'PZ7.F57535Gu', '0-8-0375065-X', '[Fic]', NULL, 168, 64),
  (974, 'The Epic of Ram, Volume 3', 2018, '1', 336, NULL, '067-497-501-4', NULL, NULL, 365, 116),
  (924, 'The Causal Angel', 2014, '1', 304, 'PR9170.F563 R33', '07-653-2951-4', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 320, 243),
  (1409, 'Ysabel', 2007, '1', 432, 'PR9199.K39 Y83', '045146-1-29-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 202),
  (1151, 'The Ocean at the End of the Lane', 2013, '1', 181, 'PR6057.A319', '006-2-25565-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 270),
  (261, 'Designing the Search Experience: The Information Architecture of Discovery', 2013, '2', 320, 'QA76.U83 R87', '012396-9-81-6', '5.4', 'Computer programming, programs, data, security > Computing and Information > Information > Systems programming and programs', 340, 156),
  (1157, 'The Outlaws Scarlett and Browne', 2021, '1', 432, 'PZ7.S92475 O', '059-34-3036-0', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 357, 135),
  (425, 'Harpy''s Flight', 2001, '2', 320, 'PS3562.I505 H3', '000-7112-52-1', '813', 'American fiction > English (North America) > Literature', 263, 106),
  (940, 'The Convenient Marriage', 2009, '2', 320, 'PR6015.E795 C665', '1-4022-1772-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (1149, 'The Nineties: A Book', 2022, '1', 384, 'E169 .K556', '0-73-521795-5', '306.0973', 'Biography And History > Culture and Institutions > North America > Social Sciences > Social sciences > United States', 246, 189),
  (192, 'Coraline', 2017, '2', 192, 'PZ7.G1273 C', '978-5-591-33867-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 108),
  (592, 'Meat: A Kitchen Education', 2010, '1', 336, 'TX749 .P485', '15800-8-992-5', '641.66', 'Cooking specific materials > Food And Drink > Home and family management > Meat from domesticated animals > Technology and Application of Knowledge', 310, 237),
  (572, 'Lyra''s Oxford', 2007, '2', 64, 'PZ7.P968 L', '0-375843-69-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 135),
  (141, 'But What If We''re Wrong?: Thinking About the Present As If It Were the Past', 2016, '1', 288, 'E169.K5548', '039918-41-2-0', '909.83', '1800- > 2000-2099, 21st Century > History > History and Geography > World history', 246, 178),
  (1, '500 Lines or Less', 2016, '2', 478, 'QA76.6', '132-9-87127-8', NULL, NULL, 79, 143),
  (350, 'Five Children and It', 1999, '2', 208, 'PZ7.N43777 F', '0-14-062193-8', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 296, 272),
  (1083, 'The Kestrel', 2002, '2', 256, 'PZ7.A3774 K', '01-4-131069-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 85),
  (1051, 'The Hero with a Thousand Faces', 2008, '1', 432, 'BL313 .C28', '157-73-1593-6', '291.13', 'Archetypal Mythology > Comparative Religion; Mythology (No Longer Used) > Mythologies > Other Religions > Religions', 93, 162),
  (994, 'The Firework-Maker''s Daughter', 2000, '2', 97, 'PZ7.P968 F', '0590-1294-3-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 17),
  (1040, 'The Hallowed Hunt', 2005, '1', 470, 'PS3552.U397 H35', '0931-7-7105-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 120),
  (1148, 'The Neverending Story', 1997, '1', 384, 'PZ7.E6964 N', '052-5457-58-5', '833.914', '1900-1990 > 1945-1990 > German and Germanic > German fiction > Literature > Modern period (1900-)', 155, 75),
  (581, 'Malgudi Days', 2000, '2', 246, 'PR9499.N3 M34', '81-8-598617-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 294, 125),
  (276, 'Door Into Light', 2019, '2', 417, NULL, '17-29-12801-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 124),
  (1117, 'The Long Winter', 1953, '2', 352, 'PZ7.W6461 L', '06-44-00069-4', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (512, 'Journey to the River Sea', 2003, '2', 304, 'PZ7.I11555 J', '01425-018-4-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (1007, 'The Gawgon and The Boy', 2001, '1', 256, 'PZ7.A3774 G', '978-0-525-46677-2', '813', 'American fiction > English (North America) > Literature', 19, 75),
  (480, 'Interaction Design for Complex Problem Solving: Developing Useful and Usable Software', 2003, '2', 444, 'QA76.H85 M57', '978-15-5-860831-3', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 279, 156),
  (1356, 'Un Lun Dun', 2007, '1', 448, 'PR6063.I265 U5', '03454-951-6-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 277, 61),
  (1341, 'Tongues of Serpents of Temeraire', 2019, '1', 288, 'PS3614.O93 T68', '978-9-893-48500-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (30, 'A History of Japan to 1334', 1958, '2', 528, 'DS835.S27', '0804-705-23-2', '952', 'Asia > History and Geography > Japan', 345, 228),
  (1188, 'The Reindeer People', 2001, '2', 336, 'PS3562.I47 R45', '000711-42-2-2', '813', 'American fiction > English (North America) > Literature', 263, 113),
  (1154, 'The Orphans of Raspay', 2020, '1', NULL, 'PS3552.U397', '1-59606-972-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (1131, 'The Marvelous Misadventures of Sebastian', 1978, '1', NULL, 'PZ7.A3774 M', '0526-13-297-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 75),
  (1179, 'The Princess Bride: S. Morgenstern''s Classic Tale of True Love and High Adventure', 2007, '2', 512, 'PS3557.O384', '0-1-5603521-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 180, 102),
  (433, 'Harry Potter and the Half-blood Prince: Children''s Edition', 2005, '1', 608, 'PR6068.O93', '07475-8-108-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 37),
  (1002, 'The Foundling', 2009, '2', 439, 'PR6015.E795', '978-1-40-221948-1', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (399, 'Good Eats 2: The Middle Years', 2010, '1', 432, 'TX651 .B7272', '158-479-857-2', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 78, 230),
  (1289, 'The Vorkosigan Companion', 2008, '1', 480, 'PS3552.U397 Z95', '1-4-1655603-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 97, 24),
  (998, 'The Flowers of Vashnoi', 2019, '1', 96, NULL, '1596-0-6892-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 84, 234),
  (576, 'Mad Ship', 1999, '1', 656, 'PS3558.O33636 M33', '0553-10-333-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (568, 'Luna: Moon Rising', 2019, '1', NULL, 'PR6063.C38 L85', '0765391-4-7-3', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 271, 243),
  (182, 'Competing Against Luck: The Story of Innovation and Customer Choice', 2016, '1', 288, 'HD53 .C487', '006-2-43561-2', '658.4', 'Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 112, 105),
  (1297, 'The Well of Ascension', 2008, '2', 816, 'PS3619.A533 W45', '0-7-6535613-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 245),
  (823, 'So Many Steps to Death', 1956, '2', NULL, 'PZ3.C4637 D', '0-67177-708-4', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (792, 'Shards of Honor', 2000, '1', 239, 'PS3552.U385', '18-86-77820-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 159),
  (637, 'Notes from a Big Country', 1999, '2', 448, 'E169.Z82 B778', '05529-9-786-2', '910', 'Geography and Travel > Geography and Travel > History and Geography', 81, 36),
  (227, 'Dave at Night', 2006, '2', 288, 'PZ7.L578345 D', '00-64-40747-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 258, 106),
  (598, 'Mindset: The New Psychology of Success', 2006, '1', 320, 'BF773 .D85', '1-4000627-5-6', '153.8', 'Cognition And Memory > Decision Making And Persuasion > Philosophy and Psychology > Psychology', 152, 198),
  (216, 'Cyberabad Days', 2009, '2', 330, 'PR6063.C38 C93', '159-1026-99-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 271, 195),
  (1182, 'The Quantum Thief', 2011, '1', 336, 'PR9170.F563 R35', '0-7653294-9-2', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 320, 243),
  (16, 'A Connecticut Yankee in King Arthur''s Court', 2011, '2', 334, 'PS1308 .A1', '1-6138-2047-X', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 367, 218),
  (723, 'Range of Ghosts', 2012, '1', 336, 'PS3602.E2475 R36', '07-653-2754-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 56, 243),
  (666, 'Ordeal by Innocence', 2002, '2', 288, 'PR6005.H66 O74', '03-1298-162-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 152),
  (1177, 'The Prince', 2008, '2', 186, 'JC143 .M38', '978-01-995-3569-9', '320.1', 'Political Science  > Political Science  > Social sciences > The State', 268, 171),
  (81, 'A Tale of Two Castles', 2011, '1', 336, 'PZ7.L578345 T', '0061-2-2965-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 258, 106),
  (112, 'Blitzscaling: The Lightning-Fast Path to Building Massively Valuable Companies', 2018, '1', 336, 'HD62 .H624', '1524-7-6141-9', '658.11', 'Management > Management and auxiliary services > Of Corporate Finance > Startup Management > Technology and Application of Knowledge', 213, 53),
  (159, 'Cetaganda', 1996, '1', NULL, 'PS3552.U397 C48', '0606-1-7118-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 63),
  (995, 'The First Four Years', 1953, '2', 160, 'PZ7.W6461 F', '0-0-6440031-X', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (275, '"Don''t Panic: Douglas Adams and the ""Hitchhiker''s Guide to the Galaxy"""', 2003, '1', 256, 'PR6051.D3352 Z66', '18-402374-2-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 242),
  (708, 'Producing Open Source Software: How to Run a Successful Free Software Project', 2005, '2', 304, 'QA76.D47 F634', '0596007-5-9-0', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 170, 168),
  (985, 'The Eye of the World: The Graphic Novel, Volume Six', 2015, '1', 176, 'PN6728.W57 D587', '0765-374-27-7', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 225, 243),
  (567, 'Luck of the Wheels', 2009, '2', 416, 'PS3562.I479 L83', '0007348-4-3-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 263, 113),
  (791, 'Shaman''s Crossing', 2005, '1', 592, 'PS3558.O33636 S525', '00-607-5762-0', '813', 'American fiction > English (North America) > Literature', 210, 113),
  (1361, 'User Is Always Right, The: A Practical Guide to Creating and Using Personas for the Web', 2011, '2', 312, 'HF5415 .M85', '978-2-0-9467024-4', '5.7', 'Computer programming, programs, data, security > Computing and Information > Data > Information', 288, 161),
  (808, 'Six Not-so-easy Pieces: Lectures On Symmetry, Relativity, And Space-time', 1997, '1', 192, 'QC793.S9 F49', '02-01150-25-5', '530.11', 'Natural sciences and mathematics > Physics > Physics > Relativity > Theoretical Physics', 166, 31),
  (748, 'Rides a Dread Legion: Book One of the Demonwar Saga', 2009, '1', 384, 'PS3556.E446 R45', '00-6-146836-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 163, 113),
  (113, 'Blood of Dragons', 2013, '1', 448, 'PS3558.O33636 B56', '00621-168-5-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (1410, 'Zero History', 2010, '1', 416, 'PS3557.I2264 Z47', '0-399156-82-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 177, 193),
  (279, 'Dragonhaven', 2007, '1', 272, 'PZ7.M1988 D', '039-9246-75-4', '813', 'American fiction > English (North America) > Literature', 274, 194),
  (1108, 'The Lean Product Playbook: How to Innovate with Minimum Viable Products and Rapid Customer Feedback', 2015, '1', 336, 'HF5415.335', '1-118-96087-4', '658.4', 'Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 307, 269),
  (1197, 'Therigatha: Poems of the First Buddhist Women', 2015, '1', 336, 'BQ1452.E5 H35', '067442-7-73-4', '294.3', 'Buddhism > Other Religions > Religions > Religions of Indic origin', 192, 116),
  (709, 'Programming Erlang: Software for a Concurrent World', 2007, '2', 536, 'QA76.E75 A765', '193-43-5600-X', '5.133', 'Computer programming, programs, data, security > Computing and Information > General Programming Languages > Information > Languages > Programming', 27, 188),
  (180, 'Cold Days of the Dresden Files', 2012, '1', 528, 'PS3602.U85 C65', '0-4-5146440-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (1257, 'The Story of the Amulet', 2010, '2', 168, 'PZ8.B612 S5', '1-60-459694-5', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 296, 223),
  (584, 'Masquerade in Lodi', 2021, '1', NULL, 'PS3552.U397', '1-645240-47-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (886, 'The Arkadians', 1998, '2', 288, 'PZ7.A3774 A', '0-140380-73-6', '818', 'Authors, American and American miscellany > English (North America) > Literature', 19, 192),
  (1132, 'The Masqueraders', 2009, '2', 336, 'PR6015.E795 M37', '1-40-221950-4', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (1017, 'The Golden Compass', 2003, '2', 368, 'PZ7.P968', '044-02-3813-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 139),
  (1049, 'The Hero of Ages', 2008, '1', NULL, 'PS3619.A533 H47', '978-1-93-857020-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 72),
  (629, 'Never Let Me Go', 2006, '2', 288, 'PR6059.S5', '1-40-007877-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 220, 263),
  (528, 'Landing Page Optimization: The Definitive Guide to Testing and Tuning for Conversions', 2012, '2', 480, 'TK5105.888 .A7836', '978-047-0610-12-1', '6.7', 'Computing and Information > Information > Multimedia systems > Special Topics', 30, 235),
  (431, 'Harry Potter and the Goblet of Fire', 2000, '1', 636, 'PR6068.O93', '0-7-4754624-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 37),
  (1186, 'The Railway Children', 1994, '2', 192, 'PZ7.N43777 R', '185-4-71202-0', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 296, 174),
  (1152, 'The Odyssey', 2011, '2', 352, 'PA4025.A5', '1-6-1382116-6', '883.01', 'Greek and other Classical languages > Literature > Prose and Fiction, Classical Greek > Pseudo-Callisthenes', 215, 218),
  (575, 'Made to Stick: Why Some Ideas Survive and Others Die', 2016, '1', 291, 'HM1033 .H43', '978-2-9-8157172-1', '302.13', 'General topics of social interaction > Social Interaction > Social Sciences > Social choice > Social sciences', 202, 198),
  (1248, 'The Spellcoats', 2001, '2', 304, 'PZ7.J684 S', '0-06-447315-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 112),
  (644, 'Oathbringer: Book Three of the Stormlight Archive', 2017, '1', 1248, 'PS3619.A533 O38', '0-76-532637-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (1187, 'The Ramayana: A Shortened Modern Prose Version of the Indian Epic', 2006, '2', 192, 'PR9499.N3 R36', '01430-3-967-9', '297.5922', 'Islam, Babism, Bahai Faith > Moral Theology, Pillars of the Faith > Other Religions > Religions', 294, 175),
  (952, 'The Door in the Hedge', 2003, '1', NULL, 'PZ7.M1988 D', '0-7-5695676-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 274, 181),
  (1204, 'The Salmon of Doubt', 2003, '2', NULL, 'PR6051.D3352 S25', '03-454-5935-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 8, 198),
  (380, 'From the Mixed-Up Files of Mrs. Basil E. Frankweiler', 1977, '2', 176, 'PZ7.K8352', '0-440-43180-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 251, 275),
  (92, 'Barrayar', 2012, '1', 324, 'PS3552.U38 B37', '188-67-7893-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 159),
  (1381, 'What I Eat: Around the World in 80 Diets', 2010, '1', 335, 'TX353 .M437', '0984-07-440-6', '394.1209', 'Customs, Etiquette, Folklore > Eating and drinking > Eating, drinking, using drugs > General Customs > History, Geography, Peoples > Social sciences', 276, 147),
  (173, 'Civilization and Its Discontents', 1989, '2', 127, 'BF173 .F682', '0-3933-0158-3', '150.1952', 'Freudian system > Philosophy and Psychology > Psychoanalytic systems > Psychology > Psychology > Systems, schools, viewpoints > Theory And Instruction', 171, 273),
  (1046, 'The Hero and the Crown', 2007, '2', 304, 'PZ7.M1988 H', '04-410-1305-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 274, 4),
  (196, 'Cosmos', 1985, '2', 324, 'QB44.S235', '034-5-33135-4', '520', 'Astronomy > Astronomy > Natural sciences and mathematics', 343, 25),
  (373, 'Four-Season Harvest: Organic Vegetables from Your Home Garden All Year Long, 2nd Edition', 1999, '2', 236, 'SB324 .C64', '1890132-2-7-6', '635.0484', 'Agriculture & related technologies > Cultivation, harvesting > Domestic Gardening > Gardening > Organic > Special methods of cultivation > Technology and Application of Knowledge', 118, 42),
  (103, 'Best Served Cold', 2010, '2', 899, 'PR6101.B49 B47', '031604-49-5-4', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 167),
  (826, 'Sparrows in the Wind', 2022, '1', 352, NULL, '0-0-6303907-9', NULL, NULL, 258, 197),
  (1404, 'Working in Public: The Making and Maintenance of Open Source Software', 2020, '1', NULL, 'HF5548.O64', '05786-7-586-2', '5.1068', 'Business & Organizations > Computer programming, programs, data, security > Computing and Information > Information > Management > Programming > Programming -- Subdivisions', 154, 233),
  (522, 'King Leopold''s Ghost: A Story of Greed, Terror, and Heroism in Colonial Africa', 1999, '2', 384, 'DT655 .H63', '061-8-00190-5', '967.51022', 'Africa > Central Africa > Democratic Republic of the Congo (Congo-Kinshasa -- former Zaire) > Democratic Republic of the Congo (Congo-Kinshasa), Rwanda & Burundi > History and Geography', 212, 146),
  (752, 'River of Stars', 2014, '2', 656, 'PR9199.K39 R58', '04-5-141609-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 160),
  (1120, 'The Lord of the Rings', 2001, '2', 1168, 'PR6039.O32', '0-007-12381-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 362, 48),
  (1006, 'The Gathering Storm', 2010, '2', 1120, 'PS3560.O7617 G38', '0-76-534153-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (536, 'Lean Customer Development: Building Products Your Customers Will Buy', 2017, '2', 240, 'HF5415.32 .A48', '14-920-2374-4', '658.834', 'Management > Management and auxiliary services > Of Marketing > Technology and Application of Knowledge', 21, 168),
  (1004, 'Theft of a Tree', 2022, '1', 544, 'PL4780.N294 P313', '06-7424-589-X', '894.8271', 'Altaic, Finno-Ugric, Uralic and Dravidian languages > Central Dravidian languages > Dravidian literatures > Literature > Literature of other languages > Telugu > Telugu poetry', 361, 116),
  (87, 'A Wizard of Earthsea', 2004, '2', 192, 'PZ7.L5215', '0-55-338304-3', '813', 'American fiction > English (North America) > Literature', 256, 227),
  (1223, 'The Scrivener''s Bones', 2016, '1', 368, 'PZ7.S19797 A', '0-7-6537896-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 229),
  (920, 'The Canterbury Tales (original-spelling Middle English edition)', 2005, '2', 1328, 'PR1870.A1 C6', '0140-422-34-X', '821.1', 'Early English 1066-1400 > English poetry > English {except North American} > Literature', 104, 175),
  (1219, 'The Sandman Vol. 9: The Kindly Ones', 1996, '2', 352, 'PN6728.S26 G35', '15-638-9205-7', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (7, 'A Brief History of Time: From the Big Bang to Black Holes', 1990, '2', 208, 'QB981 .H377', '0-55-334614-8', '523.1', 'Astronomical objects and astrophysics > Astronomy > Natural sciences and mathematics > Universe', 201, 26),
  (1387, 'Which Witch?', 2011, '2', 249, 'PZ7.I11555 W', '978-1-8-6866029-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (262, 'Designing UX: Forms: Create Forms That Don''t Drive Your Users Crazy', 2016, '2', 276, 'QA76.H85 .E534', '099-4347-05-7', '4.019', 'Computer Interaction With Humans > Computer science > Computer science -- subdivisions > Computing and Information > Information > Theory And Instruction', 156, 222),
  (1073, 'The Importance of Being Earnest', 1995, '2', 80, 'PS3573.I265 I47', '0-1406-2172-5', '822', 'English drama > English {except North American} > Literature', 373, 175),
  (464, 'How to Read a Book', 1997, '1', NULL, 'PN83 .A43', '1-567-31010-9', '28', 'Books and Reading > Information > Library and Information Sciences', 12, 84),
  (727, 'Ready Player One', 2012, '2', 384, 'PS3603.L548', '0-307-88744-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 116, 40),
  (1326, 'Things Fall Apart', 1985, '2', 190, 'PR9387.A3', '0-449-20810-9', '823', 'English fiction > English {except North American} > Literature', 6, 82),
  (1277, 'The Two Princesses of Bamarre', 2001, '1', 256, 'PZ7.L578345 T', '0-0-6029315-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 258, 106),
  (523, 'Kitchen Confidential: Adventures in the Culinary Underbelly', 2001, '2', 320, 'TX649.B58 A3', '00-60-93491-3', '641.5092', '> > Biography > Biography And History > Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 72, 108),
  (812, 'Skyward', 2018, '1', 528, 'PS3619.A53', '039955-57-7-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 59),
  (138, 'Building Scalable Web Sites: Building, Scaling, and Optimizing the Next Generation of Web Applications', 2006, '2', 352, 'TK5105 .H46', '059-610-235-6', '5.72', 'Computer programming, programs, data, security > Computing and Information > Data > Information > Preparation and Representation', 205, 168),
  (1272, 'The Tin Princess', 2004, '2', 304, 'PZ7.P968 T', '04-3-997779-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 209),
  (212, 'Curry: A Tale of Cooks and Conquerors', 2007, '2', 352, 'GT2853.I5 C65', '0-1953-2001-8', '394.1', 'Customs, Etiquette, Folklore > Eating, drinking, using drugs > General Customs > Social sciences', 119, 171),
  (1374, 'Watchmen', 1995, '2', 416, 'PN6737.M66 W38', '0-9302-8923-4', '741.5941', 'British Isles > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > European > Graphic arts and decorative arts > The arts', 285, 58),
  (883, 'The Angel of the Crows', 2020, '1', 448, 'PS3601.D4655', '0765-3-8739-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 11, 243),
  (558, 'Little Town on the Prairie', 1953, '2', 320, 'PZ7.W6461 L', '0-064-40007-7', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (232, 'Dead Voices', 2019, '1', 256, 'PZ7.A737', '0525-51-505-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 94),
  (274, 'Don''t Make Me Think, Revisited: A Common Sense Approach to Web Usability', 2014, '2', 216, 'TK5105 .K78', '978-0-3-2196551-6', '6.7', 'Computing and Information > Information > Multimedia systems > Special Topics', 253, 161),
  (58, 'A Pattern Language: Towns, Buildings, Construction', 1977, '1', 1216, 'NA2500.A445', '01950-191-9-9', '720.1', 'Architecture > Architecture - modified standard subdivisions > The arts > Theory And Instruction', 18, 171),
  (400, 'Good Eats 3: The Later Years', 2011, '1', 432, 'TX651 .B7273', '1-5847-9903-X', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 78, 114),
  (662, 'On Web Typography', 2014, '2', 142, 'TK5105.888', '1-93755-707-3', '686.22', 'Books: Binding, Design, Printing > Manufacture of products for specific uses, handicrafts > Printing > Technology and Application of Knowledge > Typography', 346, 5),
  (361, 'Fool''s Errand', 2002, '1', 486, 'PS3558.O33636 F66', '05-5-380148-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (706, 'Prince Caspian', 1998, '2', 190, 'PZ7.L58474 P', '026-1670-49-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 261, 65),
  (219, 'Danny, the Champion of the World', 1998, '2', 224, 'PZ7.D1515 D', '014-130-114-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (375, 'Fragile Things', 2007, '2', 448, 'PR6057.A319 F73', '07-5-533415-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 118),
  (1048, 'The Hero of Ages', 2008, '1', 576, 'PS3619.A533 H47', '07-653168-9-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (889, 'The Assassins of Thasalon', 2022, '1', NULL, NULL, '1-6-4524089-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (854, 'Sur''s Ocean: Poems from the Early Tradition', 2015, '1', 1072, 'PK1967.S9 A2', '06-744277-7-7', '891.4', 'Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Modern Indic languages', 359, 116),
  (834, 'Sprint: How to Solve Big Problems and Test New Ideas in Just Five Days', 2016, '1', 288, 'HD30 .K63', '0-59307-611-7', '658.403', 'Decision-making And Knowledge Management > Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 248, 220),
  (647, 'Odd and the Frost Giants', 2009, '1', 128, 'PZ7.G1273 O', '006-167-173-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 106),
  (565, 'Lord of the Flies', 1958, '2', 176, 'PR6013.O35', '0-571084-83-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 179, 80),
  (824, 'Sophie''s World: About the History of Philosophy', 2007, '2', 544, 'PT8951.A17', '0-3-7453071-8', '839.82374', '1900вЂ“2000 > Danish and Norwegian literature  > German and Germanic > Late 20th century 1945вЂ“2000 > Literature > Literature in other Germanic languages  > Norwegian BokmГҐl fiction > Norwegian BokmГҐl, RiksmГҐl', 174, 81),
  (615, 'Murder in Mesopotamia', 1976, '2', NULL, 'PR6005.H66 M8', '27-0240-378-6', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 60),
  (922, 'The Castle of Llyr', 1969, '2', 208, 'PZ7.A3774 C', '0-44-041125-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 275),
  (719, 'Quidditch Through the Ages: The Illustrated Edition', 2020, '1', 160, 'PR6068.O93', '1-33-834056-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 210),
  (1288, 'The Vor Game', 2010, '1', 381, 'PS3552.U397', '18-867788-5-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 159),
  (850, 'Summerland', 2018, '1', 304, 'PR9170.F563 R36', '1-2-5017892-4', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 320, 243),
  (1126, 'The Making of Modern Japan', 2000, '1', 936, 'DS871 .J35', '978-06-7-400334-7', '952.025', '1185-1868 > 1603-1868 (Edo) > Asia > History and Geography > Japan', 222, 33),
  (1136, 'The Military Science of Star Wars', 2018, '1', 304, NULL, '125-0-12474-3', '791.43', 'Film > Film, Radio, and Television > Public performances > Recreational and performing arts > The arts', 55, 243),
  (1014, 'The Girl Who Soared Over Fairyland and Cut the Moon in Two', 2013, '1', 256, 'PZ7.V232 G', '1-25-002350-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 368, 83),
  (444, 'Hawk', 2014, '1', 368, 'PS3552.R84 H39', '07-65-32444-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 80, 243),
  (21, 'A Darkness at Sethanon', 1987, '2', 464, 'PS3556.E446 D3', '05-5326-328-5', '813', 'American fiction > English (North America) > Literature', 163, 227),
  (610, 'Moll Flanders', 1999, '2', 352, 'PR3404', '081-2567-01-3', '823.5', 'English fiction > English {except North American} > Literature > Queen Anne 1702-45', 136, 231),
  (722, 'Ramayana', 2000, '2', 696, 'PK3653.A2 G3', '0-14-029866-5', '813', 'American fiction > English (North America) > Literature', 369, 176),
  (59, 'A Portrait of the Artist as a Young Man', 1964, '2', 256, 'PR6019.O9 P646', '0-1-4004221-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 226, 174),
  (1031, 'The Great Ghost Rescue', 2002, '1', 144, 'PZ7.I11555 G', '0-52546-769-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 75),
  (396, 'Goldenhand', 2016, '1', 368, 'PZ7.N647 G', '0-0615615-8-4', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 300, 106),
  (634, 'Nonviolent Communication: A Language of Life: Life-Changing Tools for Healthy Relationships', 2015, '2', 280, 'BF637.C45 R645', '1892-0-0528-X', '153.6', 'Cognition And Memory > Communication > Philosophy and Psychology > Psychology', 327, 191),
  (845, 'Storytelling for User Experience', 2010, '2', 298, 'PE1425', '1933-8204-7-0', '5.276', 'Computer programming, programs, data, security > Computing and Information > For Networked Multimedia Systems > Information > Programming for Specific Environments > Web Programming', 319, 204),
  (121, 'Borders of Infinity', 2007, '1', 287, 'PS3552.U397 B67', '1-8867-7859-0', '813', 'American fiction > English (North America) > Literature', 84, 159),
  (44, 'A Memory of Light', 2013, '1', 912, 'PS3560.O7617 M46', '07-653-2595-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (253, 'Death: The High Cost of Living', 1994, '2', 104, 'PN6727.G35 D43', '156-38-9133-6', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (37, 'A Light in the Attic', 1981, '1', 176, 'PS3569.I47224 L5', '0-0-6025673-7', '811.54', '1945-1999 > 20th Century > American poetry > English (North America) > Literature', 351, 106),
  (311, 'Endless Night', 1974, '2', NULL, 'PR6005.H66 E5', '067-17-8478-1', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (945, 'The Dark Talent', 2016, '1', 304, 'PZ7.S19797 D', '0-7-6538140-0', NULL, NULL, 344, 229),
  (298, 'Elantris', 2015, '1', NULL, 'PS3619.A533', '978-07-653-8807-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 72),
  (715, 'Pyramids', 1992, '2', 10, 'PR6066.R34 P97', '0-4-5145044-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 315, 179),
  (1142, 'The Mousetrap', 1977, '2', 224, 'PR6005.H66 T43', '04-40-15860-5', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 60),
  (688, 'Peter Pan', 1991, '1', 192, 'PZ7.B27539 P', '0-6708-4180-3', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 46, 261),
  (360, 'Fool''s Assassin', 2014, '1', 688, 'PS3558.O33636 F65', '05-533924-2-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 61),
  (607, 'Mitosis', 2014, '1', 96, 'PZ7.S19797', '1473-209-35-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 92),
  (1201, 'The Rope Trick', 2002, '1', 192, 'PZ7.A3774 R', '05-2-547020-4', '303', 'Social Processes > Social Sciences > Social sciences', 19, 75),
  (356, 'Fly by Night', 2008, '2', 512, 'PZ7.H21834 F', '0060-8-7630-1', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 106),
  (588, 'Material World: A Global Family Portrait', 1994, '1', 256, 'GN406 .M45', '087-1-56437-8', '306.85', 'Culture and Institutions > Family > Marriage and Parenting > Social Sciences > Social sciences', 276, 215),
  (1030, 'The Great Gatsby', 1999, '2', 180, 'PS3511.I9', '0-7432735-6-7', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 167, 211),
  (85, 'A Walk In The Woods', 1998, '2', 350, 'F106 .B92', '055-2-99702-1', '917.40443', 'Geography and Travel > History and Geography > North America > Northeastern U.S. > Travel', 81, 36),
  (1318, 'The World According to Mister Rogers: Important Things to Remember', 2019, '1', 208, 'BJ1581 .R635', '0-3164-9271-X', '170.44', 'Essays; Special Topics > Ethics > Ethics -- Subdivisions > Normativity > Philosophy and Psychology', 326, 100),
  (299, 'Elephants Can Remember: A Hercule Poirot Mystery', 2011, '2', 240, 'PZ3.C4637 PR6005 .H66', '006-20-7403-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 270),
  (976, 'The Epic of Ram, Volume 5', 2020, '1', 400, NULL, '06749-861-4-8', NULL, NULL, 365, 116),
  (426, 'Harriet the Spy', 2001, '2', 320, 'PZ7.F5768 H', '0-4-4041679-5', '813', 'American fiction > English (North America) > Literature', 169, 275),
  (416, 'Guns, Germs, and Steel: The Fates of Human Societies', 1997, '1', 480, 'HM206 .D48', '0-393-03891-2', '303.4', 'Social Processes > Social Sciences > Social change > Social sciences', 141, 273),
  (295, 'Effective Programming: More Than Writing Code', 2012, '2', 278, 'QA73', '1-4-7830054-X', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 32, 50),
  (876, 'The Alloy of Law', 2011, '1', 336, 'PS3619.A533 A79', '0-7-6533042-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (1364, 'Vegetables, Revised: The Most Authoritative Guide to Buying, Preparing, and Cooking, with More than 300 Recipes', 2012, '1', 400, 'TX801 .P49', '16-07-74026-5', '641.65', 'Cooking specific materials > Food And Drink > Home and family management > Technology and Application of Knowledge > Vegetables', 310, 237),
  (333, 'False Colours', 2008, '2', 352, 'PR6015.E795 F35', '14-02210-75-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (166, 'Childhood''s End', 1969, '1', 216, 'PZ3.C551205 C', '01-511-7205-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 114, 102),
  (1115, 'The Long Dark Tea-Time Of The Soul', 1988, '2', 256, 'PR6051.D3352', '0-3303-0955-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 8, 172),
  (537, 'Lean Impact: How to Innovate for Radically Greater Social Good', 2018, '1', 304, 'HD60.C436', '1119-5-0660-3', '658.408', 'Executive > Management > Management and auxiliary services > Social responsability of executive management > Technology and Application of Knowledge', 103, 269),
  (755, 'Rome''s Mediterranean Empire: Books 41-45 and the Periochae', 2010, '2', 432, 'PA6452.A9 C48', '01-99-55602-4', '937.02', 'Ancient World > History and Geography > Italian Peninsula to 476 and adjacent territories to 476 > Italian Peninsula to 476 and adjacent territories to 476 > Republic 509-31 B.C.', 264, 171),
  (1105, 'The Last Siege', 2004, '2', 261, 'PR6069.T744 S7768', '05-5255-146-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 49),
  (716, 'Pyramids', 1999, '2', 10, 'PR6066.R34 P97', '2-88-359184-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 315, 179),
  (950, 'The Dispossessed', 1996, '2', 400, 'PS3562.E42', '07-842-5307-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 113),
  (1227, 'The Secret Garden', 2011, '2', 296, 'PZ7.B934 S', '161-38-2192-1', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 86, 218),
  (411, 'Gregor And The Marks Of Secret', 2007, '2', 343, 'PZ7.C6837 G', '043-97-9146-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (484, 'Into the Narrowdark', 2022, '1', 608, NULL, '0756-410-64-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 375, 57),
  (783, 'Seven Surrenders', 2017, '1', 400, 'PS3616.A33879 S48', '0765378-0-2-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 308, 243),
  (336, 'Fantastic Beasts and Where to Find Them', 2017, '1', 160, 'PR6068.O93 F36', '133-821-679-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 331, 17),
  (101, 'Beguilement', 2006, '1', 368, 'PS3552.U397 S54', '006113-75-8-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 113),
  (32, 'A Knot in the Grain and Other Stories', 1994, '1', 195, 'PZ7.M1988 K', '068-8-09201-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 274, 98),
  (816, 'Small Gods', 1994, '2', 384, 'PR6066.R34 S63', '00-6-109217-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 111),
  (770, 'Scanner Darkly', 1999, '2', 217, 'PS3554.I3 S28', '1-8-5798847-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 143, 150),
  (1400, 'Women in the Material World', 1996, '1', 256, 'HQ1381 .D35', '087-1563-98-3', '305.4', 'Groups of people > Social Sciences > Social sciences > Women', 133, 215),
  (726, 'Ratio: The Simple Codes Behind the Craft of Everyday Cooking', 2009, '1', 272, 'TX652 .R845', '141656-61-1-2', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 335, 211),
  (1366, 'Verdigris Deep', 2008, '2', 327, 'PZ7.H21834 W', '0-3304-4190-6', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 144),
  (504, 'Jim Butcher''s The Dresden Files: Dog Men', 2018, '1', 128, NULL, '1-5241054-4-9', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 87, 76),
  (269, 'Diplomatic Immunity', 2002, '1', 311, 'PS3552.U397 D57', '07-43435-33-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (160, 'Chalice', 2008, '1', 272, 'PZ7.M1988 C', '0399-246-76-2', '479', 'Language > Latin > Minor Italic; Medieval Latin', 274, 194),
  (1389, 'White Mughals: Love and Betrayal in Eighteenth-Century India', 2004, '2', 624, 'DS428 .D33', '014-30-3046-9', '813', 'American fiction > English (North America) > Literature', 132, 181),
  (1311, 'The Witches', 2007, '2', 208, 'PZ7.D1515 W', '0590-03-249-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (1280, 'The Universe in a Nutshell', 2001, '1', 224, 'QC174 .H39', '978-0-5538020-2-3', '530.12', 'Natural sciences and mathematics > Physics > Physics > Quantum Mechanics > Theoretical Physics', 201, 26),
  (801, 'Show Me the Numbers: Designing Tables and Graphs to Enlighten', 2012, '1', 371, 'HF5718 .F49', '097-0-60197-2', '153.43', 'Cognition And Memory > Philosophy and Psychology > Problem Solving > Psychology > Thought, thinking, reasoning, intuition, value, judgment', 164, 13),
  (635, 'Norse Mythology', 2017, '1', 304, 'BL860.G35', '03-9-360909-X', '293.13', 'Germanic religion > Mythologies > Mythology--Germanic religion > Other Religions > Religions', 175, 273),
  (477, 'Inside the Nudge Unit: How small changes can make a big difference', 2016, '2', 432, 'BF448.H34', '0753-5-5655-3', '153.83', 'Cognition And Memory > Decision Making > Decision Making And Persuasion > Philosophy and Psychology > Psychology', 193, 268),
  (191, 'Coraline', 2006, '2', 192, 'PZ7.G1273 C', '0-0611-3937-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 108),
  (1069, 'The Iliad', 1998, '2', 704, 'PZ8.H75', '0-1-4027536-3', '883.01', 'Greek and other Classical languages > Literature > Prose and Fiction, Classical Greek > Pseudo-Callisthenes', 215, 175),
  (1139, 'The Miserable Mill', 2000, '1', 208, 'PZ7.S6795 M', '0-064407-69-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 106),
  (471, 'Information Architecture: For the Web and Beyond', 2015, '2', 486, 'TK5105 .R67', '1-491911-68-9', '5.7', 'Computer programming, programs, data, security > Computing and Information > Data > Information', 329, 168),
  (1082, 'The Keeper of the Mist', 2016, '1', 400, 'PZ7.N4448 K', '05535-0-928-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 135),
  (1302, 'The Widow''s House', 2014, '2', 544, 'PS3601.B677 W53', '0-31-620398-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 4, 167),
  (608, 'Mockingjay', 2010, '1', 400, 'PZ7.C6837', '04-3-902351-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (134, 'Brothers in Arms', 2008, '1', 318, 'PS3552.U385 B76', '1886778-7-4-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 159),
  (1145, 'The Mysterious Affair at Styles', 1978, '2', NULL, 'PR6005.H66', '05-5-311915-X', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 26),
  (929, 'The City in the Lake', 2008, '1', 304, 'PZ7.N4448 C', '978-0375-847-04-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 135),
  (23, 'Adventures of Huckleberry Finn', 2021, '2', 358, NULL, '19536-498-0-7', NULL, NULL, 367, 212),
  (1241, 'The Silver Chair', 1998, '2', NULL, 'PZ7.L58474 S', '0-2-6167051-4', '813', 'American fiction > English (North America) > Literature', 261, 65),
  (1095, 'The Ladies of Grace Adieu: and Other Stories', 2008, '2', 256, 'PR6103.L375 L33', '978-5-90622-583-8', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 115, 37),
  (1165, 'The Philadelphia Adventure', 1990, '1', 150, 'PZ7.A3774 P', '05-2-544564-1', NULL, NULL, 19, 75),
  (618, 'Murder With Mirrors', 1973, '2', NULL, 'PR6005.H66 M884', '0-6-7177688-6', '813', 'American fiction > English (North America) > Literature', 113, 184),
  (169, 'City of Blades', 2016, '2', 496, 'PS3602.E66455 C57', '05534-1-971-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 60, 40),
  (1372, 'Warbreaker', 2021, '1', 592, 'PS3619.A533', '978-7424-7-0687-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (934, 'The Complete Adventures of Feluda, Vol. 2', 2005, '2', 755, 'PK1718.R3119 A2', '01-4-303278-X', '891.443', 'Bengali > Fiction > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Modern Indic languages', 324, 177),
  (544, 'Legion: The Many Lives of Stephen Leeds', 2018, '1', 352, 'PS3619.A533 A6', '1-25029-779-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (371, 'Foundation''s Triumph', 2000, '2', 400, 'PS3552.R4825 F68', '0-0610-5639-1', '813', 'American fiction > English (North America) > Literature', 75, 111),
  (47, 'A Morbid Taste for Bones: The First Chronicle of Brother Cadfael', 1994, '2', 208, 'PR6031.A49', '0446-4-0015-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 309, 96),
  (761, 'Ruhlman''s Twenty: 20 Techniques 100 Recipes A Cook''s Manifesto', 2011, '1', 368, 'TX651 .R738', '08-1-187643-8', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 335, 45),
  (1103, 'The Last Graduate', 2021, '1', 336, 'PS3614.O93 L37', '059-312-886-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (106, 'Black Beauty', 1993, '2', 96, 'PZ10 .B5624', '0-486-27570-1', '303', 'Social Processes > Social Sciences > Social sciences', 350, 71),
  (78, 'A Swiftly Tilting Planet', 1996, '2', 272, 'PZ7.L5385 S', '044-0-90158-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 257, 29),
  (747, 'Riddle-Master', 1999, '2', 592, 'PZ7.M19864 R', '0441-0-0596-9', '813', 'American fiction > English (North America) > Literature', 273, 4),
  (1079, 'The Just Bento Cookbook 2: Make-Ahead, Easy, Healthy Lunches To Go', 2018, '2', 128, NULL, '15-68365-79-9', '641.5952', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Japan > Middle East, Asia  > Technology and Application of Knowledge', 221, 137),
  (1003, 'The Four Steps to the Epiphany: Successful Strategies for Products that Win', 2020, '1', NULL, 'HB615 .B6225', '111-9-69035-8', '658.8', 'Management > Management and auxiliary services > Of Marketing > Technology and Application of Knowledge', 66, 269),
  (77, 'As We Saw Them: The First Japanese Embassy to the United States', 1994, '2', NULL, 'E183.J3 M57', '1568-36-028-2', '303.48', 'Causes of change > Social Processes > Social Sciences > Social change > Social sciences', 282, 137),
  (508, 'Jim Butcher''s The Dresden Files: Welcome to the Jungle', 2008, '1', 160, 'PN6727.B93 W45', '0-3-4550859-9', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 87, 62),
  (436, 'Harry Potter and the Prisoner of Azkaban', 2000, '2', 320, 'PR6068.O93', '0-7-4754629-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 333, 37),
  (1246, 'The Spanish Bride', 2008, '2', 480, 'PZ3.H514 S', '1-4022111-3-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 225),
  (332, 'Falling Free', 1998, '2', 320, 'PS3552.U397 F35', '112-78-0851-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (902, 'The BFG', 2019, '2', 224, 'PZ7.D1515 B', '978-6-1863-7832-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (520, 'Killers of the Flower Moon: The Osage Murders and the Birth of the FBI', 2018, '2', 400, 'E99.O8', '030-774-248-2', '976.6004', 'History and Geography > North America > Oklahoma > South Central U.S.', 187, 263),
  (999, 'The Food Lab: Better Home Cooking Through Science', 2015, '1', 938, 'TX651 .L526', '0393-0810-8-7', '664.07', 'Chemical Technology > Foods: Sugar, Starch, etc. > Technology and Application of Knowledge', 266, 273),
  (1084, 'The King of Attolia', 2006, '1', 400, 'PZ7.T85565 K', '0-06-083578-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 366, 98),
  (672, 'Peace Talks', 2020, '1', NULL, 'PS3602.U85 P43', '04-51-46441-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (577, 'Magician: Apprentice', 1994, '2', 512, 'PS3556.E446 M3', '05-5-356494-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 163, 227),
  (230, 'Dawnthief', 2009, '2', 403, 'PR6102.A76 D39', '1591-0-2779-9', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 44, 195),
  (948, 'The Design Of Everyday Things: Revised And Expanded Edition', 2013, '2', 368, 'TS171.4 .N67', '978-0-46505-065-9', '620.82', 'Engineering > Engineering and allied operations > Human Factors > Technology and Application of Knowledge', 301, 31),
  (1270, 'The Throne of Jade', 1998, '2', 400, 'PS3614.O93 T47', '00-0725-872-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 113),
  (967, 'The Empty Grave', 2017, '1', 437, 'PZ7.S92475 E', '14-8-477872-3', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (729, 'Red Country', 2013, '2', 480, 'PR6101.B49 R43', '03161-87-20-8', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 167),
  (1009, 'The Girl in the Tower', 2017, '1', 384, 'PS3601.R42 G57', '110-18-8596-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 61),
  (786, 'Shadows', 2013, '1', NULL, 'PZ7.M1988 S', '0-399-16579-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 274, 157),
  (711, 'Ptolemy''s Gate', 2005, '1', 501, 'PZ7.S92475 P', '14-23-10116-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 153),
  (1027, 'The Great Brain', 1987, '1', NULL, 'PZ7.F57535 G', '08-037-3076-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 168, 64),
  (305, 'Emotional Design', 2003, '1', 272, 'BF531 .N67', '0465-0513-5-9', '745.2019', 'Decorative Arts > Graphic arts and decorative arts > Industrial > Philosophy And Psychology > The arts', 301, 31),
  (263, 'Designing Visual Interfaces: Communication Oriented Techniques', 1995, '2', 304, 'QA76.U83 M85', '0-13-303389-9', '5.7', 'Computer programming, programs, data, security > Computing and Information > Data > Information', 290, 189),
  (467, 'Hungry Planet: What the World Eats', 2005, '1', 288, 'TX353 .M43', '1-5800868-1-0', '641.3', 'Food > Food And Drink > Home and family management > Technology and Application of Knowledge', 276, 147),
  (1365, 'Venetia', 2011, '2', 384, 'PR6015.E795 V44', '140-22388-4-3', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (896, 'The Beasts of Clawstone Castle', 2007, '2', 256, 'PZ7.I117 B', '01-42409-31-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (830, 'Spinning Silver', 2018, '1', 480, 'PS3614.O93', '03-991-8098-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (205, 'Crossroads of Twilight', 2003, '2', 846, 'PS3560.O7617 C75', '0-81-257133-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (268, 'Dial - A - Ghost', 1996, '2', 195, 'PZ7.I11555 D', '043-94-4165-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 210),
  (628, 'Neuromancer', 1984, '2', 288, 'PS3557.I2264', '044-1-56959-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 177, 4),
  (681, 'Penric''s Travels', 2020, '1', NULL, 'PS3552.U397 P48', '1982-1-2457-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (434, 'Harry Potter and the Order of the Phoenix', 2003, '1', 766, 'PR6068.O93', '074-75-5100-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 37),
  (56, 'Anna Karenina', 2012, '2', 1182, 'PG3366 .A6', '978-86-47-82357-7', '891.733', '1800вЂ“1917 > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Russian and East Slavic languages > Russian fiction', 363, 218),
  (1403, 'Words of Radiance', 2020, '2', 1328, 'PS3619.A533', '978-2-377-64771-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 245),
  (1292, 'The Warrior''s Apprentice', 2012, '1', 309, 'PS3552.U397 W3', '978-4-8-8224859-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 159),
  (466, 'How To Win Friends and Influence People', 2009, '1', 320, 'BF637.S8 C37', '1439-1-6734-6', '158.1', 'Applied Psychology > Personal improvement and analysis > Philosophy and Psychology > Psychology', 98, 220),
  (291, 'Dubliners', 1976, '2', 224, 'PR6019.O9 D8', '01-400-4222-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 226, 174),
  (980, 'The Eye of the World', 1990, '1', 688, 'PS3560.O7617', '0-3-1285009-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (835, 'Stardust', 2007, '2', 352, 'PS3557.A3519 S73', '03-80-80455-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 175, 104),
  (1042, 'The Handmaid''s Tale', 2017, '1', 320, 'PR9199.A8', '132-88-7994-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 33, 121),
  (1078, 'The Iron Ring', 1997, '1', 256, 'PZ7.A3774 I', '05254-5-597-3', '813', 'American fiction > English (North America) > Literature', 19, 75),
  (609, 'Moira''s Pen: A Queen''s Thief Collection', 2022, '1', 224, NULL, '00628-85-60-X', NULL, NULL, 366, 98),
  (987, 'The Eye of the World: the Graphic Novel, Volume Two', 2014, '2', 176, 'PN6728.W57 D583', '07-6-533787-8', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 225, 243),
  (1024, 'The Graveyard Book', 2008, '1', 320, 'PZ7.G1273', '00605-3-092-8', '813', 'American fiction > English (North America) > Literature', 175, 106),
  (578, 'Magician: Master', 1994, '2', 499, 'PS3556.E446 M3', '0553564-9-3-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 163, 106),
  (1050, 'The Hero of Ages', 2009, '2', 784, 'PS3619.A533 H47', '0-76535-614-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 245),
  (655, 'One Flew Over the Cuckoo''s Nest', 1966, '2', 272, 'PS3561.E667 O5', '07281-026-2-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 235, 216),
  (1119, 'The Lord of the Rings', 1991, '1', 1216, 'PR6039.O32', '03955-9-511-8', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 362, 121),
  (248, 'Death Note, Vol. 5', 2006, '2', 208, 'PN6790.J34', '14-21506-26-2', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (1102, 'The Last Battle', 2008, '2', NULL, 'PZ7.L58474 L', '978-0-9-5117480-6', '813', 'American fiction > English (North America) > Literature', 261, 65),
  (99, 'Before They Are Hanged', 2008, '2', 543, 'PR6101.B49 B45', '1-5910-2641-5', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 195),
  (627, 'Nemesis Games', 2015, '1', 544, 'PS3601.B677 N46', '031-62175-8-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 127, 167),
  (221, 'Dark One Volume 1', 2021, '1', NULL, NULL, '1-9-3942445-3', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 344, 257),
  (1313, 'The Witness for the Dead', 2021, '1', NULL, 'PS3601.D4655 W58', '0-7653-8742-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 11, 243),
  (8, 'A Brief History of Time: From the Big Bang to Black Holes', 1996, '2', 208, 'QB981 .H377', '66526-7-121-6', '523.1', 'Astronomical objects and astrophysics > Astronomy > Natural sciences and mathematics > Universe', 201, 26),
  (958, 'The Dragons of Eden: Speculations on the Evolution of Human Intelligence', 1986, '2', 271, 'BF431.S2', '0345-3-4629-7', '153', 'Cognition And Memory > Philosophy and Psychology > Psychology', 343, 25),
  (170, 'City of Dragons', 2012, '2', 352, 'PS3558.O33636 C58', '00615-6-163-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (1395, 'Wildside', 1996, '1', 316, 'PS3557.O8947 W55', '03128-547-3-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 186, 243),
  (226, 'Dave at Night', 1999, '2', NULL, 'PZ7.L578345 D', '978-0-4-3920209-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 258, 210),
  (107, 'Black Beauty', 1998, '1', 240, 'PZ10.S38 B', '05-171-8958-5', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 350, 43),
  (412, 'Gregor and the Prophecy of Bane', 2005, '2', 304, 'PZ7.C6837 G', '04-3965-076-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (176, 'Cloud Atlas', 2004, '1', 544, 'PR6063.I785', '034-082-277-5', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 280, 208),
  (535, 'League of Dragons', 2016, '1', 384, 'PS3614.O93 L43', '978-03-4552-292-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (1127, 'The Man in the Brown Suit', 1974, '2', 224, 'PR6005.H66 M35', '04-40-15230-5', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 60),
  (420, 'Handbook of Usability Testing: How to Plan, Design, and Conduct Effective Tests', 2008, '2', 384, 'QA76.U83 R82', '047018-5-48-1', '5.14', 'Computer programming, programs, data, security > Computing and Information > Information > Programming > Software Testing', 334, 269),
  (370, 'Foundation', 1991, '2', 320, 'PZ3.A8316 F PS3551', '05-53-29335-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 26),
  (900, 'The Best of 2600: A Hacker Odyssey', 2008, '1', 888, 'QA76.A25 G643', '047029-4-19-1', '5.8', 'Computer Security > Computer programming, programs, data, security > Computing and Information > Information', 182, 269),
  (20, 'A Crown of Swords', 1997, '2', 896, 'PS3560.O7617 C76', '0-8-1255028-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (776, 'Seeing with Fresh Eyes: Meaning, Space, Data, Truth', 2020, '2', 176, 'QA76.I52 T84', '19-308-2400-9', '302.226', 'Communication > Kinds of communication > Social Interaction > Social Sciences > Social sciences', 364, 97),
  (6, 'About Face: The Essentials of Interaction Design', 2014, '2', 720, 'QA76.U83 C6596', '11187-665-7-1', '5.438', 'Computer programming, programs, data, security > Computing and Information > Information > Systems programming and programs > Systems programs', 123, 269),
  (1111, 'The Lies of Locke Lamora', 2006, '1', 512, 'PS3612.Y5427', '0-575-07694-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 267, 92),
  (556, 'Little House on the Prairie', 1960, '2', 352, 'PZ7.W6461 L', '551-59639-4-8', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (1101, 'The Last Battle', 1998, '2', NULL, 'PZ7.L58474 L', '02-6-167052-2', '813', 'American fiction > English (North America) > Literature', 261, 65),
  (11, 'A Ceiling Made of Eggshells', 2020, '1', 400, 'PZ7.L578345 C', '006-28781-9-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 258, 197),
  (1035, 'The Grief of Stones', 2022, '1', 245, 'PS3601.D4655 G75', '1-2-5081389-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 11, 243),
  (35, 'Alexander Hamilton', 2005, '2', 832, 'E302.H2 C48', '014-3034-75-8', '973.4092', 'Constitutional period (1789-1809) > Constitutional period (1789-1809) > History and Geography > North America > United States', 107, 174),
  (288, 'Dream-of-Jade: The Emperor''s Cat', 2005, '1', 48, 'PZ7.A3774 D', '0-8-1262736-9', '519', 'Applied Mathematics, Probabilities > Mathematics > Natural sciences and mathematics', 19, 51),
  (303, 'Emma', 2009, '2', NULL, 'PR4034 .E5', '8-17-167409-7', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 205),
  (290, 'Drowned Ammet', 2001, '2', 336, 'PZ7.J684 D', '0-064-47314-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (102, 'Be My Enemy', 2012, '1', 268, 'PR6063.C38 B4', '978-161-61467-8-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 271, 195),
  (1172, 'The Power of Habit: Why We Do What We Do in Life and Business', 2012, '1', 400, 'BF335 .D76', '1400-0692-8-9', '158.1', 'Applied Psychology > Personal improvement and analysis > Philosophy and Psychology > Psychology', 149, 198),
  (297, 'Elantris', 2005, '1', 492, 'PS3619.A533', '0-76-531177-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (1290, 'The Voyage of the Dawn Treader', 1998, '2', 189, 'PZ7.L58474 V', '026-16705-0-6', '813', 'American fiction > English (North America) > Literature', 261, 65),
  (698, 'Portable Childhoods', 2007, '2', 210, 'PS3611.L35', '1-8-9239145-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 242, 236),
  (378, 'Frederica', 2009, '2', 448, 'PR6015.E795 F695', '1-4-0221476-6', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (736, 'Rendezvous with Rama', 1973, '2', 303, 'PZ3.C551205PR6005.L36Ren3', '0-1-5176835-8', '823/.9/14', 'English fiction > English {except North American} > Literature > Modern Period', 114, 103),
  (1401, 'Words of Radiance', 2014, '1', 1088, 'PS3619.A533', '0-765-32636-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (757, 'Royal Assassin', 1997, '2', 688, 'PS3558.O33636 R68', '0-5535-7341-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (1243, 'The Slow Regard of Silent Things', 2014, '1', 159, 'PS3618.O8685 S56', '0-75641-043-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 330, 57),
  (728, 'Reaper Man', 2005, '2', 289, 'PR6066.R34 R4', '0552-152-95-1', '813', 'American fiction > English (North America) > Literature', 315, 49),
  (921, 'The Carnivorous Carnival', 2002, '1', 286, 'PZ7.S6795 C', '00-6441-012-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 352, 106),
  (89, 'A World Without Heroes', 2011, '1', 464, 'PZ7.M9112 W', '1-41-699792-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 8),
  (218, 'Danny the Champion of the World', 1988, '2', 208, 'PZ7.D1515 D', '0-1-4032873-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (95, 'Battle Ground', 2020, '1', NULL, 'PS3602.U85 B88', '0-593-19930-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (756, 'Rose Daughter', 1997, '1', 320, 'PZ8.M1793 R', '068-81543-9-5', '398.2094401', 'Customs, Etiquette, Folklore > European folktales > Folk literature > Folklore > Folklore of France > History, geographic treatment, biography > Social sciences', 274, 98),
  (720, 'Quiet: The Power of Introverts in a World That Can''t Stop Talking', 2012, '1', 352, 'BF698.I59 C35', '03-0735-214-5', '155.2', 'Developmental And Differential Psychology > Individual Psychology > Philosophy and Psychology > Psychology', 92, 52),
  (806, 'Simple Thai Food', 2014, '1', 236, 'TX724.T5 P86', '160-77452-3-2', '641.59593', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Middle East, Asia  > Southeast Asia > Technology and Application of Knowledge > Thailand', 318, 237),
  (1224, 'The Sea-Wolf', 1993, '2', 320, 'PS3523.O46 S5', '0-8125-2276-1', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 265, 244),
  (530, 'Last Argument of Kings', 2008, '2', 639, 'PR6101.B49 L37', '1591-02-690-3', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 195),
  (1394, 'Wild Seed', 1999, '2', 288, 'PS3552.U827 W5', '0-446-60672-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 88, 19),
  (591, 'Me and My Little Brain', 1985, '2', 144, 'PZ7.F57535 M', '0-440-45533-2', '823.9', 'English fiction > English {except North American} > Literature > Modern Period', 168, 64),
  (198, 'Count Karlstein', 2000, '2', 256, 'PZ7.P968 C', '0-3758-0348-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 275),
  (1363, 'VALIS.', 1981, '2', NULL, 'PS3554.I3 V35', '0-5-5314156-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 144, 26),
  (1301, 'The Wide Window', 2000, '1', 224, 'PZ7.S6795 W', '006-4-40768-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 352, 106),
  (413, 'Gregor The Overlander', 2004, '2', 320, 'PZ7.C6837 G', '0439-6-7813-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (146, 'Caliban''s War', 2012, '2', 624, 'PS3601.B677 C35', '978-03-16-12906-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 127, 167),
  (251, 'Death Note, Vol. 8', 2006, '2', 208, 'PN6790.J33 O22', '1-4-2150629-7', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (254, 'Death: The Time of Your Life', 1997, '2', 96, 'PN6727.G35 D44', '1-56-389333-9', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (932, 'The Complete Adventures of Charlie and Mr Willy Wonka', 2001, '2', 384, 'PZ7.D1515', '01413-11-31-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (1058, 'The History of Akbar, Volume 6', 2020, '1', 688, 'DS461.3 .A2313', '067-4-98613-X', NULL, NULL, 5, 116),
  (76, 'Assassin''s Quest: The Illustrated Edition', 2021, '1', NULL, 'PS3558.O33636 A94', '059315-7-93-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 61),
  (885, 'The Architecture Of Open Source Applications, Volume Ii', 2008, '2', 390, 'QA76.S46 A73', '11-055-7181-5', '5.12', 'Computer programming, programs, data, security > Computing and Information > Information > Programming > Systems Analysis And Design', 79, 143),
  (1016, 'The Goblin Emperor', 2014, '1', 448, 'PS3601.D4655', '0-7653-2699-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 11, 243),
  (150, 'Cards on the Table', 1994, '2', 224, 'PR6005.H66 C28', '8-4-3214755-4', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 34),
  (278, 'Dragon', 1999, '2', 288, 'PS3552.R84 D73', '0-81-258916-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 80, 245),
  (677, 'Penric''s Fox', 2018, '1', 200, 'PS3552.U397 P47', '15-96-06863-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (266, 'Desolation Road', 2009, '2', 365, 'PR6063.C38 D47', '1-591027-44-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 271, 195),
  (881, 'The Amulet of Samarkand', 2003, '2', 464, 'PZ7.S92475 A', '0786-8514-3-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (228, 'David Copperfield', 2011, '2', 950, 'PR4558', '16138-2-112-3', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 145, 218),
  (1383, 'What''s a Cook to Do?: An Illustrated Guide to 484 Essential Tips, Techniques, and Tricks', 2017, '2', 422, 'TX651 .P484', '978-87800-1-545-1', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 310, 18),
  (667, 'Orsinian tales', 1977, '2', NULL, 'PS3562.E42 O75', '05-5-310705-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 26),
  (55, 'Anna Karenina', 2011, '2', 1182, 'PG3366 .A6', '1-61-382153-0', '891.733', '1800вЂ“1917 > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Russian and East Slavic languages > Russian fiction', 363, 218),
  (452, 'Hindu Art', 1997, '2', 240, 'N8195.A4 B58', '2205-7-2357-4', '704.948945', 'Hinduism in art > Hinduism, Buddhism in art > Iconography > Modified subdivisions of the arts > Non-Christian religious art > Religious art > Special topics in fine and decorative arts > The arts', 70, 116),
  (90, 'A Wrinkle in Time', 1976, '2', 190, 'PZ7.L5385', '0-440-99805-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 257, 139),
  (446, 'Heroic Hearts', 2022, '2', 368, 'PS648.F3 H47', '0-593-09918-4', '813.0876608', 'Adventure fiction > American fiction > By type > Collections > English (North America) > Fantasy > Genre fiction > Literature > Speculative fiction', 87, 4),
  (34, 'Alcatraz vs. the Evil Librarians', 2016, '1', 320, 'PZ7.S19797 A', '07653-78-94-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 229),
  (539, 'Left to Their Own Devices: How Digital Natives Are Reshaping the American Dream', 2019, '1', 304, 'HQ799.M352 .A437', '16338-8-444-9', '302.23', 'Communication > Media (Means of communication) > Social Interaction > Social Sciences > Social sciences', 16, 190),
  (171, 'City of Miracles', 2017, '2', 464, 'PS3602.E66455 C572', '0553-4197-3-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 60, 40),
  (532, 'Lathe of Heaven', 1997, '2', 176, 'PS3562.E42 L3', '0-3807918-5-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 79),
  (1407, 'xkcd: volume 0', 2010, '2', 120, 'NC1429.M86', '0-61-531446-5', '741', 'Drawing &  drawings > Graphic arts and decorative arts > The arts', 291, 39),
  (1237, 'The Shattered Lens', 2010, '1', 304, 'PZ7.S19797 A', '043-99-2557-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 210),
  (1066, 'The House at Pooh Corner', 1974, '2', 180, 'PZ7.M64 H', '01-4-036122-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 278, 60),
  (997, 'The Floating Islands', 2011, '1', 400, 'PZ7.N4448 F', '0-3-7584705-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 135),
  (1344, 'Trains', 1986, '1', 32, 'TF148 .B27', '06-94-00061-2', '385.37', 'Commerce, Communications, Transportation > Facilities and vehicles > Social sciences > Trains and Railroads', 47, 252),
  (1230, 'These Happy Golden Years', 1953, '2', 304, 'PZ7.W6461 T', '00-64400-08-5', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (463, 'How to Decide: Simple Tools for Making Better Choices', 2020, '2', 288, 'HD30.23 .D85', '0593-4184-8-4', '153.83', 'Cognition And Memory > Decision Making > Decision Making And Persuasion > Philosophy and Psychology > Psychology', 150, 186),
  (178, 'Code Complete', 1993, '2', 857, 'QA76.D47 M39', '15-56-15484-4', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 269, 149),
  (691, 'Piranesi', 2020, '1', 272, 'PR6103.L375', '1-6-3557563-X', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 115, 37),
  (517, 'Kepler', 1994, '2', 448, 'QB36.K4 C33', '048667-6-05-6', '520.92', 'Astronomer Biographies > Astronomy > Astronomy > Biography And History > Natural sciences and mathematics', 99, 71),
  (1196, 'The Ride of a Lifetime: Lessons Learned from 15 Years as CEO of the Walt Disney Company', 2019, '1', 272, 'PN1998.I28 A3', '0-399-59209-1', '658.4092', 'Executive > Leadership > Management > Management and auxiliary services > Personal Aspects > Technology and Application of Knowledge', 218, 198),
  (493, 'Jane Austen: Seven Novels', 2007, '1', 1232, 'PR4030', '1-4-3510319-X', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 30),
  (24, 'A Face Like Glass', 2017, '1', 496, 'PZ7.H21834 F', '141972-48-4-3', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 114),
  (1063, 'The Horse and His Boy', 1998, '2', NULL, 'PZ7.L58474 H', '0-2-6167048-4', '813', 'American fiction > English (North America) > Literature', 261, 65),
  (383, 'Furies of Calderon', 2005, '2', 512, 'PS3602.U85', '04410-126-8-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (440, 'Harry Potter - Fantastic Beasts & Where To Find Them', 2001, '1', 42, 'PR6068.O93 F36', '04-39-32160-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 333, 210),
  (935, 'The Complete Book of Sushi', 2004, '1', 240, 'TX747.D4384', '0-7-9460316-5', '641.5952', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Japan > Middle East, Asia  > Technology and Application of Knowledge', 137, 182),
  (239, 'Death Note, Vol. 10', 2007, '2', 208, 'PN6790.J34 D4813', '142151-15-5-X', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (142, 'By the Pricking of My Thumbs', 1985, '2', NULL, 'PR6005.H66 B9', '06-7-154208-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (570, 'Luna: Wolf Moon', 2017, '1', 400, 'PR6063.C38 L864', '0-76-537553-2', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 271, 243),
  (1207, 'The Sandman Vol. 10: The Wake', 1997, '2', 192, 'PN6728.S26 G35', '1-56389-279-0', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (1122, 'The Lost Kingdom of Bamarre', 2017, '1', 400, 'PZ7.L578345 L', '00-620-7466-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 258, 106),
  (1071, 'The Illyrian Adventure', 1999, '1', 144, 'PZ7.A3774 I', '0-808595-86-5', '219', '"Natural Theology and Secularism > Religions > [Formerly ""Analogies; Correspondences""; No longer used]"', 19, 250),
  (109, 'Black Dog', 2014, '2', 448, 'PZ7.N4448 B', '1-9-0884483-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 232),
  (869, 'The Abhorsen Trilogy Box Set', 2005, '2', NULL, 'PZ7.N647 A', '00-6073-419-1', '823.4', 'English fiction > English {except North American} > Literature > Post-Elizabethan 1625-1702', 300, 110),
  (40, 'A Man on the Moon, Vol. 1: One Giant Leap', 1999, '1', NULL, 'TL789.U6 A5244', '0-78-355675-6', '629.45', 'Astronauts and Space Travel  > Engineering and allied operations > Manned space flight > Other Branches > Technology and Application of Knowledge', 102, 241),
  (942, 'The Crown of Dalemark', 2001, '2', 496, 'PZ7.J684 C', '0064-473-16-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (187, 'Content Strategy for the Web, 2nd Edition', 2012, '2', 224, 'TK5105 .H356', '0-3218-0830-4', '6.7', 'Computing and Information > Information > Multimedia systems > Special Topics', 195, 161),
  (1210, 'The Sandman Vol. 3: Dream Country', 1991, '2', 160, 'PN6728.S26 G35', '1-56-389016-X', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (789, 'Shadows of Self', 2018, '1', 384, 'PS3619.A533 S53', '978-34-65742-25-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (445, 'Heroes of the Valley', 2009, '1', 496, 'PZ7.S92475 H', '14-23109-66-X', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 357, 122),
  (1262, 'The Tales of Beedle the Bard, Standard Edition', 2008, '1', 128, 'PZ7.R79835', '05-4512-828-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 44),
  (587, 'Master of the Phantom Isle', 2019, '1', 384, 'PZ7.M9112 M', '16-29-72604-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (604, 'Mistborn : The Final Empire', 2007, '2', 672, 'PS3619.A533', '07-65-35038-6', '813', 'American fiction > English (North America) > Literature', 344, 245),
  (653, 'One Dog and His Boy', 2012, '1', 288, 'PZ7.I11555 O', '05-4-535196-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 210),
  (449, 'Hickory Dickory Death', 1985, '2', NULL, 'PR6005.H66', '06-71-61434-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (104, 'Best Served Cold', 2013, '2', 899, 'PR6101.B49 B47', '978-9552-8856-8-1', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 167),
  (582, 'Mansfield Park', 2002, '2', 384, 'PR4034 .M3', '8-1716743-5-6', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 205),
  (511, 'Jonathan Strange & Mr. Norrell', 2004, '1', 800, 'PR6103.L375 J65', '1-5-8234416-7', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 115, 38),
  (839, 'Steles of the Sky', 2014, '1', 432, 'PS3602.E2475 S74', '07653-275-6-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 56, 243),
  (1343, 'Towers of Midnight', 2011, '2', 1248, 'PS3560.O7617 T69', '0765-364-87-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 245),
  (689, 'Peter Pan', 2008, '2', 240, 'PZ7.B27539', '160-3-40065-6', '745', 'Decorative Arts > Graphic arts and decorative arts > The arts', 46, 265),
  (538, 'Lean UX: Designing Great Products with Agile Teams', 2021, '2', 254, NULL, '1-098116-30-5', '5.437', 'Computer programming, programs, data, security > Computing and Information > Information > Systems programming and programs > Systems programs', 185, 168),
  (660, 'On the Banks of Plum Creek', 1953, '2', 352, 'PZ7.W6461 O', '006-4-40004-2', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (408, 'Great Expectations', 2002, '2', 560, 'PR4560 .A1', '014-14-3956-4', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 145, 175),
  (48, 'A Murder is Announced', 1985, '2', 221, 'PR6005.H66 M847', '00-061-7137-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (1192, 'The Reptile Room', 1999, '1', NULL, 'PZ7.S6795 R', '0064-4-0767-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 352, 106),
  (908, 'The Blue Sword', 1982, '1', 320, 'PZ7.M1988 B', '0-6880-0938-7', '813', 'American fiction > English (North America) > Literature', 274, 98),
  (663, 'On Web Typography', 2020, '2', 142, 'TK5105.888', '978-9-5-2827532-3', '686.22', 'Books: Binding, Design, Printing > Manufacture of products for specific uses, handicrafts > Printing > Technology and Application of Knowledge > Typography', 346, 5),
  (1184, 'The Queen of Attolia', 2002, '1', 288, 'PZ7.T85565 Q', '814-30-1792-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 366, 98),
  (1036, 'The Grim Grotto', 2004, '1', 323, 'PZ7.S6795 G', '006-4410-14-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 106),
  (731, 'Reflex', 2005, '2', 384, 'PS3557.O8947 R45', '08-12-57854-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 186, 246),
  (365, 'Foreigner', 2004, '2', 432, 'PS3553.H358 F67', '075-6-40251-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 108, 57),
  (562, 'Looking for Jake: And Other Stories', 2006, '2', 352, 'PR6063.I265 L66', '03-304341-8-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 277, 243),
  (211, 'Cuckoo Song', 2015, '1', 416, 'PZ7.H21834 C', '141-971-480-5', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 12),
  (80, 'A Tale of Time City', 2002, '1', 327, 'PZ7.J684 T', '00602-9-884-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (1255, 'The Steerswoman', 1989, '2', 279, 'PS3561.I776', '03-453576-2-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 240, 61),
  (1287, 'The Visual Display of Quantitative Information', 2001, '1', 200, 'QA276 .T83', '09-61-39214-2', '1.4226', 'Computing and Information > Information > Knowledge > Research > Research methods > Statistical methods', 364, 97),
  (597, 'Miles, Mystery & Mayhem', 2001, '1', 505, 'PS3552.U397 A6', '0-67-131858-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (1080, 'The Just Bento Cookbook: Everyday Lunches To Go', 2011, '2', 127, 'TX724.J3 I85', '47700-312-4-6', '641.5952', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Japan > Middle East, Asia  > Technology and Application of Knowledge', 221, 137),
  (1312, 'The Witchwood Crown', 2017, '1', 736, 'PS3573.I45563 W58', '0-7-5641060-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 375, 57),
  (802, 'Side Jobs: Stories From the Dresden Files', 2010, '1', 432, 'PS3602.U85 S53', '0-4-5146365-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (404, 'Good to Great and the Social Sectors: Why Business Thinking is Not the Answer', 2005, '2', 35, 'HD57 .C6452', '0-9773-2640-3', '658.048', 'Management > Management and auxiliary services > Technology and Application of Knowledge', 120, 106),
  (393, 'Going Solo', 1999, '2', 224, 'PR6054.A35 Z464', '01-413-0310-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (867, 'Test Driven Development: By Example', 2002, '2', 240, 'QA76.T48 B43', '0321-146-53-0', '5.14', 'Computer programming, programs, data, security > Computing and Information > Information > Programming > Software Testing', 57, 6),
  (1125, 'The Magicians Nephew', 2007, '2', NULL, 'PZ7.L58474 M', '026-16-7046-8', '813', 'American fiction > English (North America) > Literature', 261, 95),
  (273, 'Don''t Make Me Think: A Common Sense Approach to Web Usability, 2nd Edition', 2005, '2', 216, 'TK5105 .K78', '0-32134-475-8', '6.7', 'Computing and Information > Information > Multimedia systems > Special Topics', 253, 161),
  (1332, 'Throne of the Crescent Moon', 2017, '1', 288, 'PS3601.H582 T48', '978-0-4-5531756-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 13, 57),
  (546, 'Letters from a Father to His Daughter', 2004, '1', 168, 'D62 .N44', '0670-058-16-5', '930', 'Ancient History > Ancient World > History and Geography', 295, 259),
  (492, 'James and the Giant Peach', 2007, '2', 160, 'PZ8.D137 J', '0-14-132263-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (1261, 'The Talented Mr. Ripley', 2000, '1', 287, 'PS3558.I366', '07-6-218856-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 209, 123),
  (1247, 'The Sparrow', 1996, '1', 408, 'PS3568.U76678', '0679-4-5150-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 339, 262),
  (978, 'The Epic of Ram, Volume 6', 2022, '1', 384, NULL, '0-674-25821-5', '891.4312', '1345вЂ“1645 > Hindi poetry > Hindi, Urdu > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Modern Indic languages', 365, 116),
  (1128, 'The Martian Chronicles', 1982, '2', NULL, 'PS3503.R167', '0-5-5324691-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 73, 26),
  (1161, 'The Perfect Store: Inside eBay', 2003, '2', 336, 'HF5478 .C64', '031-616-493-3', '381.1702854678', 'Auctions > Commerce > Commerce, Communications, Transportation > Marketing channels > Social sciences', 117, 23),
  (1067, 'The Hunger Games', 2008, '1', 384, 'PZ7.C6837', '04390-23-48-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (569, 'Luna: New Moon', 2015, '1', 416, 'PR6063.C38 L86', '0-7-6537551-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 271, 243),
  (116, 'Blood Rites', 2004, '2', 372, 'PS3602.U85 B56', '04-51-45987-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (1075, 'The Inheritance: And Other Stories', 2011, '2', 400, 'PS3558.O33636 I54', '006-156-164-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (1043, 'The Handmaid''s Tale', 2021, '1', 320, 'PR9199.A8', '978-8-97-962329-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 33, 121),
  (545, 'Legion: The Many Lives of Stephen Leeds', 2023, '1', 352, 'PS3619.A533 A6', '978-95-3-936679-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (526, 'Komarr', 1998, '1', 320, 'PS3552.U397 K66', '0671-87-877-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 189),
  (128, 'Brandon Sanderson''s White Sand Volume 3', 2019, '1', 160, 'PN6728.W584 S262', '15-2411-006-X', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 344, 76),
  (238, 'Death Note, Vol. 1', 2005, '2', 200, 'PN6790.J33', '1421-5-0168-6', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (158, 'Cetaganda', 1996, '1', 302, 'PS3552.U397 C48', '0-671-87701-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 221),
  (1208, 'The Sandman Vol. 1: Preludes and Nocturnes', 1993, '2', 240, 'PN6728.S26 G35', '1-56-389011-9', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (833, 'Spring-Heeled Jack', 2004, '2', 112, 'PZ7.P968 S', '04-40-41881-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 275),
  (784, 'Shadow & Claw', 2021, '2', 528, 'PS3573.O52 S53', '125-0-78125-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 376, 243),
  (938, 'The Conqueror', 2008, '2', 480, 'PR6015.E795 C66', '978-14-0221-355-7', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 225),
  (229, 'David Copperfield', 2017, '2', 950, 'PR4558', '978-5-1-5846087-6', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 145, 218),
  (1331, 'Throne of the Crescent Moon', 2012, '1', 288, 'PS3601.H582 T48', '0-75-640711-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 13, 57),
  (1185, 'The Qur''an', 2010, '1', 672, 'BP109', '0-19-957071-X', '297.122521', 'Islam, Babism, Bahai Faith > Koran > Other Religions > Religions > Sacred Books and Scriptures > Textual Sources > Translations', 2, 171),
  (428, 'Harry Potter and the Chamber of Secrets: The Illustrated Edition', 2016, '1', 272, 'PR6068.O93', '0-545-79132-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 17),
  (277, 'Downbelow Station', 1981, '2', 432, 'PS3553.H358 D68', '08-799-7828-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 108, 57),
  (437, 'Harry Potter and the Prisoner of Azkaban: The Illustrated Edition', 2017, '1', 336, 'PR6068.O93', '05-457913-4-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 17),
  (250, 'Death Note, Vol. 7', 2006, '2', 208, 'PN6790.J33 O22', '1-421-50628-9', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (1303, 'The Willful Princess and the Piebald Prince', 2013, '1', 184, 'PS3558.O33636 W55', '1-59-606544-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 210, 234),
  (911, 'The Book of Three', 2006, '2', 190, 'PZ7.A3774 B', '08050-804-8-1', '813.5', '20th Century > American fiction > English (North America) > Literature', 19, 119),
  (1378, 'Well-Designed: How to Use Empathy to Create Products People Love', 2014, '1', 224, 'TS171 .K653', '162-5274-79-3', '658.5', 'Management > Management and auxiliary services > Of Production > Technology and Application of Knowledge', 250, 115),
  (282, 'Dragon Prince', 1990, '2', 576, 'PS3568.A87', '03-30312-74-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 322, 243),
  (1371, 'Warbreaker', 2019, '1', NULL, 'PS3619.A533', '978-19-3857-021-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 72),
  (501, 'Jim Butcher''s Dresden Files: Ghoul Goblin', 2013, '1', 152, 'PN6728.D74', '16-0-690438-8', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 87, 76),
  (43, 'A Memoir of Jane Austen: and Other Family Recollections', 2008, '2', 276, 'PR4036 .A8', '978-019-95407-7-8', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 36, 171),
  (1008, 'The Giraffe and the Pelly and Me', 1994, '2', 80, 'PZ7.D1515 G', '01-4037-009-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (499, 'Jim Butcher''s Dresden Files: Down Town', 2015, '1', 144, NULL, '1-606-90700-X', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 87, 76),
  (767, 'Sapiens: A Brief History of Humankind', 2015, '2', 512, 'CB113.H4 H3713', '0-0-9959008-5', '909', 'History > History and Geography > World history', 197, 198),
  (557, 'Little Lord Fauntleroy', 1998, '2', 256, 'PZ7.B934 L', '18532-6-130-0', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 86, 164),
  (542, 'Legion: Lies of the Beholder', 2018, '1', 152, 'PS3619.A533 L443', '15-960688-5-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 234),
  (438, 'Harry Potter and the Sorcerer''s Stone: MinaLima Edition', 2020, '1', 368, 'PR6068.O93', '13-385-9670-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 210),
  (1355, 'Unfinished Tales: The Lost Lore of Middle-earth', 1988, '2', 512, 'PR6039.O32 U5', '0-3-4535711-6', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 362, 61),
  (1099, 'The Landmark Xenophon''s Hellenika', 2010, '2', 672, 'DF214 .X4613', '14-00034-76-0', '938.06', 'Ancient World > Greece to 323 > Greece to 323 > History and Geography > Spartan and Theban Supremacy (404-362 BC)', 377, 14),
  (851, 'Sundiver: The Ultimate Voyage', 1980, '2', 340, 'PS3552.R4825', '055-31-3312-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 75, 26),
  (915, 'The Brothers Karamazov', 1992, '2', 816, 'PG3326 .B7', '00-9-992280-0', '813', 'American fiction > English (North America) > Literature', 146, 263),
  (315, 'Equal Rites', 2000, '2', 213, 'PR6066.R34 E69', '0-0-6102069-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 111),
  (1022, 'The Good Neighbor: The Life and Work of Fred Rogers', 2018, '1', 416, 'PN1992.R56 K56', '14-1-972772-9', '791.4502', 'Film, Radio, and Television > Public performances > Recreational and performing arts > Television > The arts', 238, 114),
  (982, 'The Eye of the World: The Graphic Novel, Volume Five', 2014, '1', 176, 'PN6728.W57 D586', '076-53-7425-0', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 225, 243),
  (510, 'John Adams', 2001, '1', 752, 'E322 .M38', '068481-36-3-7', '973.44092', 'Constitutional period (1789-1809) > History and Geography > John Adams (4 Mar. 1797-4 Mar. 1801) > North America > United States', 270, 220),
  (224, 'Database Reliability Engineering: Designing and Operating Resilient Database Systems', 2017, '2', 294, 'QA76.D3 C36365', '14-9192-594-9', '5.7565', 'Computer programming, programs, data, security > Computing and Information > Data > Information > Relational Databases > Specific Databases And Data Files', 94, 168),
  (741, 'Return of the Dragon Slayers', 2021, '1', 608, 'PZ7.M9112', '16297-2-930-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (259, 'Designing for the Digital Age: How to Create Human-Centered Products and Services', 2009, '2', 768, 'TS171 .G664', '978-0-470229-10-1', '745.2', 'Decorative Arts > Graphic arts and decorative arts > Industrial > The arts', 184, 269),
  (1267, 'The Theory Of Everything', 2011, '2', 132, 'QB985 .H39', '978-9432-2683-0-5', '520', 'Astronomy > Astronomy > Natural sciences and mathematics', 201, 127),
  (1334, 'Time Cat: The Remarkable Journeys of Jason and Gareth', 2003, '1', 224, 'PZ7.A3774 T', '080-50727-0-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 119),
  (785, 'Shadowheart: Volume Four of Shadowmarch', 2010, '1', 768, 'PS3573.I45563 S527', '0756-406-40-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 375, 57),
  (601, 'M Is for Magic', 2008, '2', 272, 'PZ7.G1273 M', '0-0-6118647-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 106),
  (1350, 'Understanding Your Users: A Practical Guide to User Research Methods', 2015, '2', 568, 'QA76.H85 C69', '0-1-2800232-8', '4.019', 'Computer Interaction With Humans > Computer science > Computer science -- subdivisions > Computing and Information > Information > Theory And Instruction', 51, 156),
  (1163, 'The Performance of Open Source Applications', 2013, '2', 182, 'QA76.S46 P47', '1304-4-8878-0', NULL, NULL, 28, 143),
  (267, 'Devil''s Cub', 2009, '2', 320, 'PR6015.E795 D48', '978-1-402-21953-5', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (42, 'A Man on the Moon, Vol. 3: Lunar Explorers', 1999, '1', NULL, 'TL789.U6 A5244', '0-783-55677-2', '629.4540973', 'Astronauts and Space Travel  > Engineering and allied operations > Manned space flight > Other Branches > Technology and Application of Knowledge', 102, 241),
  (1206, 'The Sandman: Endless Nights', 2004, '2', 160, 'PN6728.S26 G37', '14-012011-3-X', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 175, 258),
  (310, 'Empty Smiles', 2022, '1', 288, 'PZ7.A737', '059310-91-8-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 94),
  (140, 'Burning Chrome', 1986, '2', 208, 'PS3557.I2264 B875', '0441-089-34-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 177, 4),
  (1386, 'Which Witch?', 2000, '2', 249, 'PZ7.I11555 W', '0141304-2-7-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (611, 'More Mexican Everyday: Simple, Seasonal, Celebratory', 2015, '1', NULL, 'TX716.M4 B2952', '039-3081-14-1', '641.5972', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Mexico, Central America, and the Caribbean > North America > Technology and Application of Knowledge', 52, 273),
  (589, 'Matilda', 1990, '2', 240, 'PZ7.D1515 M', '01-403-4294-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (1342, 'Too Like the Lightning', 2016, '1', 432, 'PS3616.A33879 T66', '0765-378-00-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 308, 243),
  (1053, 'The History of Akbar, Volume 1', 2015, '1', 656, 'DS461 .A2313', '0-674-42775-0', '954.02', '647-1875 > Asia > History and Geography > India and South Asia', 5, 116),
  (875, 'The Alliance: Managing Talent in the Networked Age', 2014, '1', 224, 'HD58 .H6444', '978-1-6-2527577-6', '658.3', 'Management > Management and auxiliary services > Of Personnel > Technology and Application of Knowledge', 213, 115),
  (814, 'Sleeping Murder', 1977, '2', NULL, 'PZ3.C4637 S', '055310-7-06-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 29),
  (366, 'Forest Mage', 2006, '1', 736, 'PS3558.O33636 F68', '0-0-6075763-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (743, 'Revolting Rhymes', 1984, '2', 48, 'PR6054.A35 R4', '0-1-4050423-0', '821.914', '1945-1999 > 20th Century > English poetry > English {except North American} > Literature > Modern period 1900-', 131, 192),
  (439, 'Harry Potter and the Sorcerer''s Stone: The Illustrated Edition', 2015, '1', 256, 'PR6068.O93', '054-57-9035-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 17),
  (358, 'Fool Moon', 2001, '2', 352, 'PS3602.U85 F66', '0-451458-12-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (489, 'Island of the Aunts', 2001, '2', 281, 'PZ7.I11555 I', '0142-3004-9-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (406, 'Grave Peril', 2001, '2', 378, 'PS3602.U88', '04-5145-844-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 87, 203),
  (593, 'Memory', 1996, '1', 462, 'PS3552.U397 M46', '0-67187-743-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (120, 'Borders of Infinity', 1993, '2', 320, 'PS3552.U397 B67', '48114-574-5-2', '813', 'American fiction > English (North America) > Literature', 84, 24),
  (68, 'Art of War', 2005, '2', 312, 'U101 .M1613', '978-0-226-50046-1', '355.02', 'Military Science > Public Administration, Military Science > Social sciences > War', 268, 255),
  (1376, 'Web Programming: Building Internet Applications', 2006, '2', 678, 'QA76.625 .B38', '04-700-1775-9', '5.276', 'Computer programming, programs, data, security > Computing and Information > For Networked Multimedia Systems > Information > Programming for Specific Environments > Web Programming', 49, 269),
  (1330, 'Thinkpad: A Different Shade of Blue', 1999, '1', 502, 'HD9696.C64 D45', '0-6-7231756-7', '4', 'Computer science > Computing and Information > Information', 140, 207),
  (190, 'Cooking for Geeks: Real Science, Great Hacks, and Good Food', 2010, '2', 432, 'TX715 .P729', '05-968-0588-8', '641.5024', '> > Cooking, cookbooks > Food And Drink > Home and family management > Miscellany > Technology and Application of Knowledge', 314, 168),
  (583, 'Many Ways to Say I Love You: Wisdom for Parents and Children from Mister Rogers', 2019, '1', 192, 'HQ755 .R64', '03164-925-6-6', '649.1', 'Home and family management > Parenting > Parenting, Caregiving > Technology and Application of Knowledge', 326, 100),
  (506, 'Jim Butcher''s The Dresden Files: Storm Front Volume 1', 2009, '1', 128, 'PN6727.P736 J56', '0-3-4550639-1', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 87, 62),
  (559, 'Little Women', 2004, '2', 464, 'PS1017 .L5', '04-51529-30-8', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 17, 217),
  (124, 'Bottlenecks: Aligning UX Design with User Psychology', 2017, '2', 281, 'QA76.H85', '14-842-2579-1', '5.437', 'Computer programming, programs, data, security > Computing and Information > Information > Systems programming and programs > Systems programs', 161, 16),
  (959, 'The Dragon''s Path', 2011, '2', 592, 'PS3601.B677 D73', '978-0-316080-68-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 4, 167),
  (730, 'Red Seas Under Red Skies', 2007, '2', 488, 'PS3612.Y5427', '0-5-7507925-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 267, 92),
  (479, 'Instead of Three Wishes: Magical Short Stories', 2006, '2', 160, 'PZ7.T85565 I', '00608-42-31-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 366, 98),
  (164, 'Chaos: Making a New Science', 1996, '2', 380, 'Q172.C45 G54', '07-49386-06-1', '501', 'General Science > Natural sciences and mathematics > Philosophy and theory', 178, 151),
  (1056, 'The History of Akbar, Volume 4', 2018, '1', 656, NULL, '0-67497-503-0', NULL, NULL, 5, 116),
  (956, 'The Dragon Reborn', 2002, '2', 624, 'PS3560.O7617 D7', '0-7653-0511-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 245),
  (912, 'The Books of Earthsea: The Complete Illustrated Edition', 2018, '1', 1008, 'PS3562.E42 B66', '1-48146-558-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 206),
  (842, 'Storm Front', 2000, '2', 384, 'PS3552.U8242', '0-45-145781-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (1239, 'The Shattered Lens', 2016, '1', 336, 'PZ7.S19797 A', '076-5-37900-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 229),
  (984, 'The Eye of the World: The Graphic Novel, Volume One', 2013, '2', 240, 'PN6728.W57 D58', '07-6533-541-7', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 225, 243),
  (1339, 'Tom Sawyer, Detective', 2020, '2', 100, 'PZ3.C59 PS1322', '195-243-325-8', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 367, 212),
  (1229, 'The Secret of Platform 13', 1999, '2', 231, 'PZ8.I25 S', '014-1302-86-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (354, 'Flowers for Algernon', 1984, '2', 224, 'PZ4.K4453', '05-53-27450-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 236, 26),
  (69, 'A Shadow in Summer', 2006, '1', 336, 'PS3601.B677 S53', '07-6-531340-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 4, 243),
  (256, 'Deerskin', 1993, '1', 309, 'PS3563.C3816 D4', '044-1-14226-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 274, 4),
  (654, 'One Flew Over the Cuckoo''s Nest', 1963, '2', 272, 'PS3561.E667 O5', '04-511-6396-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 235, 216),
  (451, 'Hindu Art', 1994, '2', 240, 'N8195.A4 B58', '067439-1-89-6', '704.948945', 'Hinduism in art > Hinduism, Buddhism in art > Iconography > Modified subdivisions of the arts > Non-Christian religious art > Religious art > Special topics in fine and decorative arts > The arts', 70, 116),
  (819, 'Snapshot', 2017, '1', NULL, 'PS3619.A533', '978-099-855-990-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 256),
  (486, 'I, Robot', 1991, '2', 304, 'PS3551.S5 I13', '0-5532-9438-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 227),
  (368, 'Fortunately, the Milk', 2015, '2', NULL, 'PZ7.G1273 F', '978-00-623-8064-7', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 175, 104),
  (207, 'Cryoburn', 2010, '1', 352, 'PS3552.U397 C79', '1-439-13394-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (1141, 'The Mountain of Kept Memory', 2016, '1', 448, 'PS3614.E553 M68', '148-14-4894-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 206),
  (1116, 'The Long Secret', 1990, '2', 275, 'PZ7.F5768 L', '00-644033-2-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 169, 112),
  (181, 'Comet', 1986, '2', NULL, 'QB721 .S34', '0671619-1-7-9', '523.6', 'Astronomical objects and astrophysics > Astronomy > Comets > Natural sciences and mathematics', 343, 184),
  (127, 'Brandon Sanderson''s White Sand Volume 2', 2018, '1', 160, 'PN6728.W584', '1524103-4-2-X', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 344, 76),
  (300, 'Ella Enchanted', 2017, '2', 240, 'PZ7.L578345', '0-0-6440705-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 258, 106),
  (758, 'Royal Assassin: The Illustrated Edition', 2020, '1', NULL, 'PS3558.O33636 R68', '05931-57-92-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 180),
  (1015, 'The Goal: A Process of Ongoing Improvement - 30th Anniversary Edition', 2012, '2', 362, 'PR9510.G64 G6', '978-0-884-27195-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 181, 163),
  (910, 'The Book of Jhereg', 1999, '1', 471, 'PS3552.R84 A6', '0-73940-667-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 80, 4),
  (165, 'Charity Girl', 2008, '2', 288, 'PZ3.H514 PR6015 .E795', '140221-3-50-6', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (1281, 'The Unknown Ajax', 2011, '2', 384, 'PZ3.H514 PR6015 .E795', '14-022388-2-7', '813', 'American fiction > English (North America) > Literature', 208, 224),
  (1240, 'The Silmarillion', 2004, '1', 416, 'PR6039.O32 S5', '0618-39-111-8', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 362, 121),
  (41, 'A Man on the Moon, Vol. 2 : The Odyssey Continues', 1999, '1', NULL, 'TL789.U6 A5244', '0783-55-676-4', '629.4540973', 'Astronauts and Space Travel  > Engineering and allied operations > Manned space flight > Other Branches > Technology and Application of Knowledge', 102, 241),
  (796, 'Showa 1926-1939: A History of Japan', 2013, '2', 560, 'DS888.5 M5913', '1-7-7046135-3', '952.03', '1868-1945 > Asia > History and Geography > Japan', 283, 73),
  (319, 'Essential SQLAlchemy', 2008, '2', 240, 'QA76.W43', '0596-5-1614-2', '5.758', 'Computer programming, programs, data, security > Computing and Information > Data > Distributed > Information > Specific Databases And Data Files', 124, 168),
  (1220, 'The Scar', 2004, '2', 608, 'PR6063.I265 S28', '0-345460-01-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 277, 61),
  (257, 'Designing Data-Intensive Applications: The Big Ideas Behind Reliable, Scalable, and Maintainable Systems', 2017, '2', 611, 'QA76.A65', '1-4493-7332-1', '5.276', 'Computer programming, programs, data, security > Computing and Information > For Networked Multimedia Systems > Information > Programming for Specific Environments > Web Programming', 245, 168),
  (186, 'Contact', 2001, '2', 448, 'PS3569.A287 C6', '355-1-72019-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 343, 184),
  (926, 'The Chronicles of Chrestomanci, Charmed Life / The Lives of Christopher Chant', 2001, '2', 608, 'PZ7.J684 C', '0064-4726-8-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (1338, 'Tom Sawyer Abroad', 2019, '2', 112, 'PS1320 .A1', '195-04-3531-8', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 367, 212),
  (1203, 'The Ruby in the Smoke', 2012, '2', 224, 'PZ7.P968 R', '978-42364-9-788-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 209),
  (1232, 'The Shadow in the North', 2004, '2', 288, 'PZ7.P968 S', '0-4-3997781-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 209),
  (54, 'Anna Karenina', 1966, '2', 864, 'PG3366 .A6', '423-9-79477-6', '891.733', '1800вЂ“1917 > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Russian and East Slavic languages > Russian fiction', 363, 174),
  (794, 'Ship of Destiny', 2000, '1', 592, 'PS3558.O33636 S53', '0-55-310323-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (1368, 'Victory of Eagles', 2008, '1', 342, 'PS3614.O93 V53', '0-3-4549688-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (1212, 'The Sandman Vol. 4: Season of Mists', 1994, '2', 217, 'PN6728.S26 G35', '15-63-89041-0', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (389, 'Getting Things Done: The Art of Stress-Free Productivity', 2002, '2', 288, 'BF637.T5 A45', '01420-0-028-0', '646.7', 'Home and family management > Management of personal and family life > Sewing, clothing, management of personal and family life > Technology and Application of Knowledge', 20, 174),
  (1323, 'The Zookeeper''s Wife: A War Story', 2007, '1', 368, 'D804.Z33 A25', '978-03-93061-72-7', '940.5318350943840', '1918- > Europe > Europe > History and Geography > Holocaust > Social, political, economic history; Holocaust > World War II', 7, 273),
  (1225, 'The Secret Garden', 1998, '1', 241, 'PZ7.B934 S', '08-8-363202-0', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 86, 254),
  (86, 'A Wind in the Door', 1976, '2', 224, 'PZ7.L5385 W', '04409-8-761-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 257, 139),
  (970, 'The Enormous Crocodile', 1993, '2', 42, 'PZ7.D1515 E', '014-0-36556-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (327, 'Fablehaven', 2007, '2', 384, 'PZ7.M9112 F', '14-169-4720-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 8),
  (979, 'The Essence of Politics', 2021, '1', NULL, 'JC50.K2513', '067-4-97576-6', '320.01', 'Philosophy and Theory > Political Science  > Political Science  > Political Science  > Social sciences', 229, 116),
  (114, 'Blood of Tyrants', 2013, '1', 448, 'PS3614.O93 B56', '034-5-52289-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (189, 'Cooking: 600 Recipes, 1500 Photographs, One Kitchen Education', 2007, '1', 560, 'TX652 .P455', '15-8-008789-2', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 310, 237),
  (318, 'Essentials of Cooking', 1999, '1', 299, 'TX651 .P48', '157-9-65120-8', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 310, 18),
  (344, 'FirstBorn & Defending Elysium', 2013, '1', 160, NULL, '1-9-3857000-6', NULL, NULL, 344, 72),
  (540, 'Legacy', 2007, '1', 384, 'PS3552.U397 S55', '0061-1-3905-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 113),
  (174, 'Clariel: The Lost Abhorsen', 2014, '1', 400, 'PZ7.N647 C', '978-0-0-6156155-9', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 300, 106),
  (132, 'Brief Cases', 2023, '1', 448, 'PS3602.U85 A6', '978-8-72619-746-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (1349, 'Under Heaven', 2010, '1', 592, 'PR9199.K39 U53', '0-45-146330-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 202),
  (543, 'Legion: Skin Deep', 2014, '1', 208, 'PS3619', '15960-66-90-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 234),
  (1396, 'Winnie-the-Pooh', 1988, '1', 176, 'PZ7.M64 W', '0-5-2544443-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 278, 75),
  (678, 'Penric''s Labors', 2022, '1', 336, NULL, '1-9821922-4-0', '813.5400', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (45, 'American Gods', 2002, '2', 624, 'PR6057.A319', '03-80-78903-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 175, 111),
  (962, 'The Dream Hunters', 2007, '2', 128, 'PR6057.A319 S36', '978-000-0-08272-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 258),
  (255, 'Deeplight', 2020, '1', NULL, 'PZ7.H21834 D', '141-9743-20-1', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 12),
  (1037, 'The Gunslinger', 2017, '1', 288, 'PS3561.I483', '1501-182-10-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 239, 211),
  (907, 'The Blade Itself', 2013, '2', 531, 'PR6101.B49 B57', '978-1984-1-1636-1', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 195),
  (1168, 'The Physicians of Vilnoc', 2021, '1', NULL, NULL, '1-64524-021-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (123, 'Bossypants', 2014, '2', 275, 'PN2287.F4255 A3', '978-2722-92-585-8', '792.7028092', 'Recreational and performing arts > Stage presentations, Theatre > Techniques, procedures, apparatus, materials > The arts > Variety shows and theatrical dancing', 165, 23),
  (1076, 'The Interpretation of Dreams', 1980, '2', 736, 'BF1078 .F72', '038001-0-00-3', '150', 'Philosophy and Psychology > Psychology > Psychology', 171, 22),
  (1026, 'The Great Automatic Grammatizator: And Other Stories', 2001, '2', 272, 'PZ7.D1515 U', '014-1-31150-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (799, 'Showa 1944-1953: A History of Japan', 2014, '2', 536, 'DS888 .M5913', '177-046-162-0', '952.03', '1868-1945 > Asia > History and Geography > Japan', 283, 73),
  (702, 'Predictably Irrational, Revised and Expanded Edition: The Hidden Forces That Shape Our Decisions', 2009, '1', 400, 'BF448 .A75', '0-0-6185454-9', '153.83', 'Cognition And Memory > Decision Making > Decision Making And Persuasion > Philosophy and Psychology > Psychology', 25, 104),
  (385, 'Genius: The Life and Science of Richard Feynman', 1993, '2', 531, 'QC16.F49 G54', '0-6-7974704-4', '530.092', 'Biography > Biography And History > Natural sciences and mathematics > Physics > Physics > Physics', 178, 263),
  (168, 'Cibola Burn', 2014, '1', 592, 'PS3601.B677 C53', '03162-1-762-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 127, 167),
  (448, 'Hickory Dickory Death', 1984, '1', 198, 'PR6005.H66', '05533-5-012-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 26),
  (1254, 'The Startup Owner''s Manual: The Step-By-Step Guide for Building a Great Company', 2012, '2', 608, 'HD62.5 .B53', '0-984999-30-2', '658.11', 'Management > Management and auxiliary services > Of Corporate Finance > Startup Management > Technology and Application of Knowledge', 66, 138),
  (838, 'Steelheart', 2013, '1', 386, 'PZ7.S19797', '0-3-8574356-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 59),
  (566, 'Lords and Ladies', 1996, '2', 400, 'PR6066.R34 L67', '006-1056-92-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 111),
  (326, 'Fablehaven', 2006, '1', 368, 'PZ7.M9112 F', '15-90-38581-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (612, 'Mort', 2001, '2', 243, 'PR6066.R34 M67', '0-06-102068-0', '813', 'American fiction > English (North America) > Literature', 315, 111),
  (1029, 'The Great Brain Does It Again', 1981, '2', 129, 'PZ7.F57535Gu', '7-5-2225408-4', '[Fic]', NULL, 168, 64),
  (1269, 'The Thief Lord', 2003, '2', 376, 'PZ7.F96624 T', '0439-4-2089-X', '833.914', '1900-1990 > 1945-1990 > German and Germanic > German fiction > Literature > Modern period (1900-)', 173, 238),
  (1260, 'The Summer Tree', 2004, '2', 400, 'PR9199.K39 S9', '3842-05-522-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 202),
  (1200, 'The Robots of Dawn', 1983, '1', 419, 'PS3551.S5 R6', '0-385184-00-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 68),
  (57, 'Anthony Bourdain''s Les Halles Cookbook: Strategies, Recipes, and Techniques of Classic Bistro Cooking', 2004, '1', 304, 'TX719 .B736', '158-23-4180-X', '641.5944', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Europe > Food And Drink > France > Home and family management > Technology and Application of Knowledge', 72, 38),
  (648, 'Ogre Enchanted', 2018, '1', 352, 'PZ7.L578345 O', '006-2-56121-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 258, 106),
  (93, 'Bartimaeus: The Ring of Solomon', 2010, '1', 416, 'PZ7.S92475 R', '14231-237-2-7', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 357, 122),
  (874, 'The Aeronaut''s Windlass', 2015, '1', 640, 'PS3602.U85', '0451-4668-0-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (1279, 'The Uncommon Reader', 2007, '1', 120, 'PR6052.E5 U53', '03-742-8096-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 59, 81),
  (1183, 'The Queen of Attolia', 2000, '1', 288, 'PZ7.T85565 Q', '068817-4-23-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 366, 98),
  (1174, 'The Prestige', 2006, '2', 368, 'PR6066.R55 P74', '07653-5-617-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 316, 243),
  (793, 'Shattered Pillars', 2013, '1', 336, 'PS3602.E2475 S53', '07-653-2755-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 56, 243),
  (26, 'Agatha Christie''s Ten Little Indians', 1975, '2', 173, 'PR6005.H66 A84', '06-71801-51-1', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (1408, 'Young Miles', 1997, '2', 586, 'PS3552.U397 Y68', '06-7-187782-8', '813', 'American fiction > English (North America) > Literature', 84, 24),
  (815, 'Small Favor', 2008, '1', 432, 'PS3602.U85 S63', '045-146-189-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (828, 'Spindle''s End', 2002, '2', 432, 'PZ8.M1793 S', '06-9-811950-9', '813', 'American fiction > English (North America) > Literature', 274, 85),
  (521, 'King Arthur and his Knights of the Round Table', 2008, '2', 416, 'PZ8.G77 K', '0141-32-101-6', '398.2094102', 'Customs, Etiquette, Folklore > European folktales > Folk literature > Folklore > Folklore of the British Isles (other than England) > History, geographic treatment, biography > Social sciences', 189, 192),
  (906, 'The Blade Itself', 2007, '2', 531, 'PR6101.B49 B57', '1-591-02594-X', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 3, 195),
  (474, 'In Praise of Annada, Volume 1', 2017, '1', 560, 'PK1718.B463 A813', '0-6-7466042-0', '891.4413', '1645вЂ“1845 > Bengali > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Modern Indic languages > Poetry', 323, 116),
  (690, 'Phoenix and the Carpet', 1999, '2', 224, 'PZ7.N43777 P', '1-85-326155-6', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 296, 272),
  (25, 'After Dark', 2007, '1', 208, 'PL856.U673 A6613', '1-8465-5047-5', '895.635', '1945вЂ“2000 > Asian (east and south east) languages > Japanese > Japanese fiction > Literature > Literature of other languages', 292, 117),
  (930, 'The City & The City', 2010, '2', 352, 'PR6113.C28', '0-3-4549752-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 277, 61),
  (903, 'The Black Cauldron', 1999, '2', 229, 'PZ7.A3774 B', '04-40406-49-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 60),
  (840, 'Steles of the Sky', 2022, '1', 432, 'PS3602.E2475 S74', '978-8-9-8211844-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 56, 243),
  (1160, 'The Penultimate Peril', 2005, '1', 353, 'PZ7.S6795 P', '0064-41-015-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 106),
  (981, 'The Eye of the World', 1990, '2', 832, 'PS3560.O7617', '08125-118-1-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 245),
  (753, 'Rocket Surgery Made Easy: The Do-It-Yourself Guide to Finding and Fixing Usability Problems', 2009, '2', 168, 'TK5105 .K785', '03216-5-729-2', '6.7019', 'Computing and Information > Information > Multimedia systems > Special Topics', 253, 161),
  (502, 'Jim Butcher''s Dresden Files: War Cry', 2014, '1', 136, 'PN6728.D74 B95', '1-60-690574-0', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 87, 76),
  (1214, 'The Sandman Vol. 6: Fables and Reflections', 1994, '2', 264, 'PN6728.S26 G29', '1-5-6389105-0', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (108, 'Black Coffee', 1998, '1', 240, 'PR6065.S15 B58', '2-744-12143-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 152),
  (1010, 'The Girl in the Tower', 2019, '1', 384, 'PS3601.R42 G57', '978-17978-699-2-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 61),
  (870, 'The Abominables', 2013, '1', 272, 'PZ7.I117 A', '141-97078-9-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 114),
  (700, 'Powder and Patch', 2010, '2', 208, 'PZ3.H514 P', '978-1402-219-49-8', '813', 'American fiction > English (North America) > Literature', 208, 224),
  (957, 'The Dragon Reborn', 2013, '2', 624, 'PS3560.O7617 D7', '978-252-31-3366-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 245),
  (724, 'Range: Why Generalists Triumph in a Specialized World', 2019, '1', 352, 'BF378.E94 E67', '0-735-21448-4', '153.9', 'Assessment And Intelligence > Cognition And Memory > Philosophy and Psychology > Psychology', 157, 200),
  (302, 'Embassytown', 2011, '1', 368, 'PR6063.I265 E46', '03455-2-449-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 277, 61),
  (161, 'Champion of the Titan Games', 2020, '1', 544, 'PZ7.M9112 C', '162-97-2788-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (1140, 'The Morning Gift', 2007, '2', 416, 'PR6059.B3 M67', '014-240-911-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 226),
  (233, 'Dear Mister Rogers, Does It Ever Rain in Your Neighborhood?: Letters to Mister Rogers', 1996, '2', 208, 'PN1992.R56 A4', '014-0-23515-9', '791.45', 'Film, Radio, and Television > Public performances > Recreational and performing arts > Television > The arts', 326, 174),
  (111, 'Black Powder War', 2007, '2', 368, 'PS3614.O93 B53', '978-000-7-21917-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 113),
  (1250, 'The Star of Kazan', 2006, '2', 416, 'PZ7.I117 S', '014-24058-2-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (651, 'Once Upon a Time in the North: His Dark Materials', 2008, '1', 112, 'PZ7.P968 O', '0375-8451-0-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 135),
  (148, 'Captain Vorpatril''s Alliance', 2012, '1', 432, 'PS3552.U397 C37', '1-45-163845-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (372, 'Foundryside', 2018, '1', 512, 'PS3602.E66455', '15-247603-6-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 60, 52),
  (18, 'A Conspiracy of Kings', 2011, '2', 352, 'PZ7.T85565 C', '0-0-6187095-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 366, 98),
  (4, 'Abhorsen', 2003, '1', 368, 'PZ7.N647 A', '0-060278-25-0', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 300, 110),
  (873, 'The Adventures of Tom Sawyer', 2021, '2', 270, NULL, '1-95364-990-4', NULL, NULL, 367, 212),
  (626, 'Neil Gaiman and Charles Vess''s Stardust', 2019, '2', 160, 'PS3557.A3519 S73', '1-4-0128784-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 175, 258),
  (750, 'Rise of the Evening Star', 2007, '1', 456, 'PZ7.M9112 R', '1590-38-742-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (470, 'In Cold Blood', 1994, '2', 343, 'HV6533.K3', '0-67-974558-0', '364.15230978144', 'Crimes and Offenses > Criminology > History, geographic treatment, biography > Homicide > Murder > North America > Offenses against persons > Social problems and services; associations > Social sciences', 95, 263),
  (1283, 'The Very Best of Charles de Lint', 2010, '2', 425, 'PR9199.D357 A6', '18923-919-6-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 139, 236),
  (1231, 'The Seven Dials Mystery', 1976, '2', 217, 'PR6005.H66 S46', '0-553-12192-8', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 26),
  (272, 'Do Androids Dream of Electric Sheep?', 1987, '2', 272, 'PS3554.I3', '03-45-35047-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 144, 61),
  (1244, 'The Soul of a New Machine', 1981, '1', 293, 'TK7885.K53', '03164-91-70-5', '600', '-- > General Technology > Technology and Application of Knowledge', 237, 140),
  (88, 'A Wizard of Earthsea', 2009, '2', 192, 'PZ7.L5215', '978-56-3663-747-8', '813', 'American fiction > English (North America) > Literature', 256, 227),
  (809, 'Sketching User Experiences: Getting the Design Right and the Right Design', 2007, '2', 448, 'TS171 .B89', '0123740-3-7-1', '658.5752', 'Design > Management > Management and auxiliary services > Of Production > Product Development > Research And Development > Technology and Application of Knowledge', 89, 156),
  (744, 'Rework', 2010, '1', 288, 'HD31 .F755', '0307-46-374-5', '658', 'Management > Management and auxiliary services > Technology and Application of Knowledge', 172, 53),
  (936, 'The Computer Connection', 1976, '2', 217, 'PS3552.E796 C66', '04-2-503983-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 61, 34),
  (1309, 'The Wise Man''s Fear: The Kingkiller Chronicle: Day Two', 2012, '2', 1008, 'PS3618.O8685 W57', '07-564-0712-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 330, 57),
  (669, 'Oryx and Crake', 2005, '2', 376, 'PR9199.A8', '23-5-602228-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 33, 14),
  (1162, 'The Perfect Thing: How the iPod Shuffles Commerce, Culture, and Coolness', 2006, '1', 304, 'ML74.I48 L48', '0-743-28522-0', '6.5', 'Computing and Information > Digital audio > Information > Special Topics', 260, 220),
  (855, 'Surveys That Work: A Practical Guide for Designing and Running Better Surveys', 2021, '2', 368, 'HM538.J37', '19-338-2053-5', '300.723', 'Education And Research > Social Sciences > Social Sciences > Social sciences > Social sciences--descriptive research > Social sciences--research', 223, 204),
  (129, 'Brasyl', 2007, '1', 357, 'PR6063.C38 B73', '1591-025-43-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 271, 195),
  (155, 'Catching Fire', 2009, '1', 391, 'PZ7.C6837', '0-43902-349-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (961, 'The Dream Hunters', 2000, '2', 128, 'PR6057.A319 S36', '156389-62-9-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 258),
  (1359, 'Use of Weapons', 1992, '2', 432, 'PR6052.A485 U84', '185-7231-35-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 43, 167),
  (119, 'Borders of Infinity', 1989, '2', 320, 'PS3552.U397 B67', '067-1-69841-9', '813', 'American fiction > English (North America) > Literature', 84, 24),
  (872, 'The Adventures of Robinson Crusoe', 1994, '2', 157, 'PR3403 .A1', '18547-12-80-2', '823.5', 'English fiction > English {except North American} > Literature > Queen Anne 1702-45', 136, 37),
  (1268, 'The Thief', 1996, '1', 224, 'PZ7.T85565', '068814-62-7-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 366, 98),
  (684, 'Perhaps the Stars', 2021, '1', 608, 'PS3616.A33879 P47', '07-6-537806-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 308, 243),
  (863, 'Taran Wanderer', 1974, '2', 272, 'PZ7.A3774 T', '26-5254-400-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 275),
  (235, 'Death Comes As The End', 1982, '2', 222, 'PR6005.H66', '06-714-6539-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (614, 'Mrs. McGinty''s Dead', 1972, '2', NULL, 'PZ3.C4637 M', '0-671-77614-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (284, 'Dragonshadow', 2009, '1', 297, 'PS3558.A4215 D73', '978-25232-16-56-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 196, 61),
  (620, 'My Grandmother Asked Me to Tell You She''s Sorry', 2016, '2', 400, 'PT9877.A32 M5613', '1501-11-507-3', '839.73', 'German and Germanic > Literature > Literature in other Germanic languages  > Swedish fiction > Swedish literature', 39, 267),
  (355, 'Flow My Tears, the Policeman Said', 2001, '2', 208, 'PS3554.I3 F56', '1-857-98341-6', '813', 'American fiction > English (North America) > Literature', 144, 92),
  (1327, 'Thinking, Fast and Slow', 2011, '1', NULL, 'BF441 .K238', '0-374-27563-7', '153.4', 'Cognition And Memory > Philosophy and Psychology > Psychology > Thought, thinking, reasoning, intuition, value, judgment', 228, 81),
  (705, 'Pride and Prejudice', 1983, '2', 352, 'PR4034', '05-5321-310-5', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 27),
  (877, 'The Alloy of Law', 2020, '1', 336, 'PS3619.A533 A79', '978-993475-37-0-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (553, 'Lirael: Daughter of the Clayr', 2001, '1', 496, 'PZ7.N647', '006-027-823-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 300, 110),
  (596, 'Miles in Love', 2008, '1', 880, 'PS3552.U397 M54', '1416-5552-2-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (680, 'Penric''s Progress', 2020, '1', 320, 'PS3552.U397 P46', '19-82-12429-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (887, 'The Art of Community: Building the New Age of Participation', 2012, '2', 576, 'TK5105.83 .B33', '05-961-5671-5', '303', 'Social Processes > Social Sciences > Social sciences', 40, 168),
  (837, 'Starsight', 2019, '1', 480, 'PZ7.S19797 S', '0-3-9955581-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 59),
  (222, 'Dark Waters', 2021, '1', NULL, 'PZ7.A737 D', '0-5-9310915-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 94),
  (97, 'Beautiful Teams: Inspiring and Cautionary Tales from Veteran Team Leaders', 2009, '2', 510, 'QA76.D47 B4325', '0596-51-802-1', '5', 'Computer programming, programs, data, security > Computing and Information > Information', 355, 168),
  (1235, 'The Shape of Design', 2012, '2', 142, 'NK1510', '0-985472-20-0', NULL, NULL, 111, 89),
  (83, 'A Tale of Two Cities', 1998, '2', 304, 'PR4571 .A1', '04-8640-651-2', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 145, 71),
  (1176, 'The Prince', 2003, '2', 144, 'JC143 .M38', '0-1-4044915-9', '320.01', 'Philosophy and Theory > Political Science  > Political Science  > Political Science  > Social sciences', 268, 175),
  (807, 'Six Easy Pieces: Essentials of Physics By Its Most Brilliant Teacher', 2005, '2', 176, 'QC21 .F52', '046-5-02392-4', '530', 'Natural sciences and mathematics > Physics > Physics', 166, 31),
  (67, 'Artificial Intelligence Illuminated', 2004, '2', 739, 'Q335.C586', '07637-3-230-3', '6.3', 'Artificial Intelligence > Computing and Information > Information > Special Topics', 125, 130),
  (762, 'Rusalka', 1989, '1', 374, 'PS3553.H358 R8', '0-34535-953-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 108, 61),
  (1353, 'Unfettered II', 2016, '1', 567, 'PS648.F3 U5342', '194-41450-5-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 99),
  (386, 'Gentleman Jole and the Red Queen', 2016, '1', 352, 'PS3552.U397 G46', '1476-781-22-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (1314, 'The Wizard in the Tree', 1974, '1', 137, 'PZ8.A37 W', '052-543-128-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 75),
  (183, 'Complete Works of Oscar Wilde', 2001, '2', 384, 'PR5810 .F66', '1855-3-4908-6', '828.809', '1837-1899 > English miscellaneous writings > English {except North American} > Individual authors > Literature', 373, 91),
  (301, 'Elon Musk: Tesla, SpaceX, and the Quest for a Fantastic Future', 2015, '1', NULL, 'HC102.M88 V36', '978-0-0-6230123-9', '338.7', 'Business Enterprises > Economics > Production > Social sciences', 370, 77),
  (944, 'The Curse of Chalion', 2002, '2', 512, 'PS3552.U397 C87', '03-8-081860-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 111),
  (1093, 'The Kodansha Kanji Learner''s Dictionary: Revised and Expanded', 2013, '2', 1248, 'PL677.5', '1-5-6836407-5', '495', 'Language > Languages of East & Southeast Asia > Other Languages', 194, 136),
  (63, 'Arcanum Unbounded: The Cosmere Collection', 2016, '1', 672, 'PS3619.A533 A6', '076-5-39116-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (825, 'Soul Music', 1995, '1', 373, 'PR6066.R34 S66', '0-6-1357277-7', '813', 'American fiction > English (North America) > Literature', 315, 111),
  (1319, 'The World of Robert Jordan''s The Wheel of Time', 2001, '2', 304, 'PS3560.O7617 W6', '03128-6-936-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (649, 'Olive Kitteridge', 2008, '2', 336, 'PS3569.T736', '081-29-7183-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 358, 198),
  (447, 'Hexwood', 2002, '1', 464, 'PZ7.J684 H', '0-0-6029888-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (733, 'Remembered Death', 1975, '2', NULL, 'PR6005.H66', '067180-2-95-X', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (1181, 'The Psychology Of Everyday Things', 1988, '1', 272, 'TS171.4 .N67', '046-5-06709-3', '620.82', 'Engineering > Engineering and allied operations > Human Factors > Technology and Application of Knowledge', 302, 31),
  (335, 'Fantastic Beasts and Where to Find Them', 2001, '2', 42, 'PR6068.O93 F36', '074755-4-66-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 333, 37),
  (1109, 'The Leap', 2001, '2', 192, NULL, '0-099-40285-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 199),
  (424, 'Haroun and the Sea of Stories', 1991, '2', 224, 'PR6068.U757 H37', '014014-04-3-3', '823', 'English fiction > English {except North American} > Literature', 337, 174),
  (1284, 'The Vicar of Nibbleswicke', 1994, '2', 48, 'PZ7.D1515 V', '01-403-6837-X', '828', 'English miscellaneous writings > English {except North American} > Literature', 131, 192),
  (1019, 'The Golden Enclaves', 2022, '1', 416, 'PS3614.O93 G65', '0593-15-835-0', '813.00', 'American fiction > By type > English (North America) > Literature', 304, 61),
  (19, 'Across the Wall: A Tale of the Abhorsen and Other Stories', 2005, '1', 320, 'PZ7.N647 A', '0-06074-713-7', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 300, 110),
  (1202, 'The Ruby in the Smoke', 2004, '2', 224, 'PZ7.P968 R', '0-439-97778-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 209),
  (831, 'Splendid Soups: Recipes and Master Techniques for Making the World''s Best Soups', 2000, '1', 640, 'TX757 .P4823', '0-471-39136-0', '641.813', 'Appetizers > Cooking Specific Dishes > Food And Drink > Home and family management > Soups > Technology and Application of Knowledge', 310, 121),
  (286, 'Dragonwatch', 2017, '1', 384, 'PZ7.M9112 D', '16297-225-6-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (260, 'Designing Interactions', 2007, '1', 766, 'QA76.H85 M64', '0262-1347-4-8', '4.019', 'Computer Interaction With Humans > Computer science > Computer science -- subdivisions > Computing and Information > Information > Theory And Instruction', 284, 154),
  (1150, 'The Nonesuch', 2009, '2', 337, 'PR6015.E795 N66', '1-4-0221770-6', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (367, 'Forms that Work: Designing Web Forms for Usability', 2008, '2', 199, 'TK5105.888 .J37', '008094-8-48-0', '5', 'Computer programming, programs, data, security > Computing and Information > Information', 223, 156),
  (859, 'Tales from Watership Down', 1996, '1', 192, 'PR6051.D345 T3', '009180-1-66-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 10, 9),
  (162, 'Changes  of the Dresden files', 2010, '2', 441, 'PS3602.U85C53 2010', '978-045-14631-7-3', '813/.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (555, 'Little House on the Prairie', 1953, '2', 352, 'PZ7.W6461 L', '006-44-0002-6', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (1012, 'The Girl Who Fell Beneath Fairyland and Led the Revels There', 2012, '1', 272, 'PZ7.V232 G', '0-3126-4962-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 368, 83),
  (717, 'Quicksilver:', 2006, '2', 480, 'PS3569.T3868 Q52', '0-0-6083316-5', '813', 'American fiction > English (North America) > Literature', 356, 111),
  (668, 'Oryx and Crake', 2004, '2', 376, 'PR9199.A8', '978-0-3-8572167-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 33, 14),
  (550, 'Life''s Journeys According to Mister Rogers', 2019, '1', 176, 'BJ1581 .R633', '0-3164-9329-5', '170.44', 'Essays; Special Topics > Ethics > Ethics -- Subdivisions > Normativity > Philosophy and Psychology', 326, 100),
  (1333, 'Thunderbolt''s Waxwork', 1996, '2', 128, 'PR6066.U44', '014-03641-0-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 192),
  (1171, 'The Pleasure of Finding Things Out: The Best Short Works of Richard Feynman', 2001, '2', 288, 'Q171 .F385', '0-1-4029034-6', '600', '-- > General Technology > Technology and Application of Knowledge', 166, 174),
  (1170, 'The Player of Games', 1989, '2', 320, 'PR6052.A485', '1-8-5723146-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 43, 167),
  (1118, 'The Long Winter', 1961, '2', 352, 'PZ7.W6461 L', '910-909-429-1', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (151, 'Cart and Cwidder', 2001, '2', 240, 'PZ7.J684 C', '0-06-447313-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (973, 'The Epic of Ram, Volume 2', 2016, '1', 416, 'PK1947.T83 R313', '0-6740-8861-1', '891.4312', '1345вЂ“1645 > Hindi poetry > Hindi, Urdu > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Modern Indic languages', 365, 116),
  (811, 'Sky Raiders', 2014, '1', 432, 'PZ7.M9112 S', '14-424970-0-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 8),
  (779, 'SEO for Everyone', 2021, '2', NULL, NULL, '978-1-9526160-7-5', NULL, NULL, 41, 2),
  (699, 'PostgreSQL', 2005, '2', 1032, 'QA76.D3 D685', '06723-27-56-2', '5.7565', 'Computer programming, programs, data, security > Computing and Information > Data > Information > Relational Databases > Specific Databases And Data Files', 147, 207),
  (843, 'Storm Front', 2007, '1', 320, 'PS3552.U8242', '0451-461-97-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (795, 'Ship of Magic', 1998, '1', 704, 'PS3558.O33636', '0-5-5310324-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (847, 'Storytelling for User Experience', 2019, '2', 298, 'PE1425', '978-742-47-1293-X', '5.276', 'Computer programming, programs, data, security > Computing and Information > For Networked Multimedia Systems > Information > Programming for Specific Environments > Web Programming', 319, 204),
  (50, 'Anathem', 2009, '2', 928, 'PS3569.T3868 A53', '18-4-354917-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 356, 20),
  (613, 'Mountain of Black Glass', 2000, '2', 784, 'PS3573.I45563 M68', '08-86-77906-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 375, 57),
  (488, 'Islam: Faith and History', 2005, '2', 256, 'BP161 .A96', '18-5-168350-X', '200', 'Religion > Religion > Religions', 38, 165),
  (1321, 'The Xanadu Adventure', 2005, '1', 160, 'PZ7.A3774 X', '0-525-47371-8', '791.43', 'Film > Film, Radio, and Television > Public performances > Recreational and performing arts > The arts', 19, 75),
  (822, 'Snuff of Discworld', 2011, '1', NULL, 'PR6066.R34 S65', '0-0620-1184-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 104),
  (547, 'Letting Go of the Words: Writing Web Content that Works', 2012, '2', 368, 'TK5105 .R427', '012-3859-30-1', '6.7', 'Computing and Information > Information > Multimedia systems > Special Topics', 325, 156),
  (1354, 'Unfettered III', 2019, '1', 768, NULL, '1-9-4414523-0', '808.83', 'Anthologies & Collections > By Topic > Fiction > Literature > Rhetoric and anthologies', 375, 99),
  (866, 'Terciel & Elinor', 2021, '1', 352, NULL, '00-6-304932-5', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 300, 132),
  (1114, 'The Limbreth Gate', 2002, '2', 368, 'PS3562.I479 L56', '00-0711-254-8', '813', 'American fiction > English (North America) > Literature', 263, 106),
  (1233, 'The Shadow Rising', 1992, '1', 704, 'PS3560.O7617 S48', '031-2-85431-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (143, 'By the Shores of Silver Lake', 1953, '2', 304, 'PZ7.W6461 B', '006-440-005-0', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (131, 'Brief Cases', 2018, '1', 448, 'PS3602.U85 A6', '04514-921-0-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (500, 'Jim Butcher''s Dresden Files: Fool Moon Part 1', 2011, '1', 128, 'PN6728.D74 B884', '16-0-690210-5', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 87, 76),
  (1379, 'Westmark', 2002, '2', 192, 'PZ7.A3774 W', '978-01-41-31068-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 85),
  (765, 'Sailing to Sarantium', 2012, '2', 546, 'PR9199.K39 S25', '978-8-818-06748-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 79),
  (352, 'Flight of the Nighthawks', 2006, '1', 368, 'PS3556.E446 F58', '978-0060792-7-8-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 163, 113),
  (495, 'Jane Eyre', 1997, '2', 480, 'PR4167', '04515-2-655-4', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 77, 217),
  (79, 'A Swiftly Tilting Planet', 2005, '2', 272, 'PZ7.L5385 S', '6871-6-4263-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 257, 29),
  (485, 'Iraq 100: The First Anthology of Science Fiction to Have Emerged from Iraq', 2017, '2', 224, 'PJ8046.E5 I73', '1-250161-32-0', '892.73010832567', 'Arabic (Egypt, Lebanon, Palestine, Saudi Arabia, Sudan) > Arabic fiction > Literature > Literature of other languages > Middle Eastern languages', 67, 243),
  (22, 'A Deadly Education', 2020, '1', NULL, 'PS3614.O93', '05-93128-48-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (656, 'On Food And Cooking: The Science And Lore Of The Kitchen', 2004, '1', 884, 'TX651 .M27', '068-4800-01-2', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 272, 211),
  (149, 'Cards on the Table', 1987, '2', 224, 'PR6005.H66 C28', '0-42-510567-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 34),
  (864, 'Tehanu', 1991, '2', 288, 'PZ7.L5215 T', '05-5-328873-3', '813', 'American fiction > English (North America) > Literature', 256, 227),
  (1060, 'The History of Akbar, Volume 8', 2022, '1', 672, NULL, '0-674-24417-6', NULL, NULL, 5, 116),
  (1133, 'The Meaning Of It All: Thoughts Of A Citizen-scientist', 1998, '1', 144, 'Q175 .F52', '0201-3-6080-2', '500', 'General Science > General Science > Natural sciences and mathematics', 166, 31),
  (585, 'Mastering the Art of French Cooking', 2009, '1', NULL, 'TX719.B388', '0307-5-9352-5', '641', 'Food And Drink > Home and family management > Technology and Application of Knowledge', 110, 134),
  (1180, 'The Prisoner of Limnos', 2019, '1', 256, 'PS3552.U397 P75', '1-5-9606884-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 84, 234),
  (1086, 'The King''s Blood', 2012, '2', 528, 'PS3601.B677 K56', '978-03-1608-077-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 4, 167),
  (943, 'The Curse of Chalion', 2001, '1', 448, 'PS3552.U397 C87', '0-38097-901-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 113),
  (1397, 'Winter of Ice and Iron', 2017, '1', NULL, 'PS3614.E553 W56', '1-4-8144897-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 206),
  (914, 'The Brief Wondrous Life of Oscar Wao', 2007, '1', 352, 'PS3554.I259', '15-94-48958-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 142, 201),
  (341, 'Feynman''s Lost Lecture: The Motion of Planets Around the Sun', 2000, '2', 192, 'QB603.M6 G66', '0-39331-995-4', '521.3', 'Astronomy > Celestial Mechanics > Natural sciences and mathematics > Orbits', 183, 273),
  (1068, 'The Hydrogen Sonata', 2013, '2', 544, 'PR6052.A485 H93', '0-316212-36-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 43, 167),
  (74, 'Assassin''s Apprentice', 2019, '1', 464, 'PS3558.O33636', '19-84-81785-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 61),
  (397, 'Golem''s Eye', 2005, '2', 576, 'PZ7.S92475 G', '0-5-5255273-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 49),
  (1256, 'The Stone of Farewell', 1990, '1', 589, 'PS3573.I45563 S76', '08-8-677435-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 375, 57),
  (554, 'Little House in the Big Woods', 1971, '2', 256, 'PZ7.W6461 L', '00644-00-01-8', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 374, 106),
  (490, 'It Doesn''t Have to Be Crazy at Work', 2018, '1', 240, 'HD58 .F78', '006-28-7478-0', '650.1', 'Business > Management and auxiliary services > Personal success in business > Technology and Application of Knowledge', 172, 105),
  (9, 'Academ''s Fury', 2006, '2', 720, 'PS3602.U85 A63', '0-44-101340-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (304, 'Emma', 2011, '2', 448, NULL, '0-4864-7804-1', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 71),
  (1367, 'V for Vendetta', 1998, '2', 288, 'PN6737.M66', '185286-29-1-2', '741.5941', 'British Isles > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > European > Graphic arts and decorative arts > The arts', 285, 242),
  (51, 'And Then There Were None', 1972, '2', 173, 'PR6005.H66 A84', '067-1-77581-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (1085, 'The King of Attolia', 2007, '2', 432, 'PZ7.T85565 K', '00608-35-79-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 366, 98),
  (1276, 'The True Confessions of Charlotte Doyle', 1997, '2', 240, 'PZ7.A953 T', '038-0-72885-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 37, 106),
  (513, 'Julia''s Kitchen Wisdom: Essential Techniques and Recipes from a Lifetime of Cooking', 2000, '1', 144, 'TX651 .C523', '037-54115-1-8', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 110, 134),
  (481, 'Interaction of Color: 50th Anniversary Edition', 2013, '2', 192, 'ND1489 .A4', '030-0179-35-9', '701.85', 'Color > Inherant features > Modified subdivisions of the arts > Philosophy and theory of fine and decorative arts > The arts', 15, 274),
  (953, 'The Drackenberg Adventure', 1988, '1', 152, 'PZ7.A3774 D', '05-254-4389-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 75),
  (157, 'Caves of Steel', 1993, '2', 208, 'PS3551.S5', '05-8-600835-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 95),
  (1205, 'The Sandman: Book of Dreams', 2002, '2', 416, 'PS648.F3 S26', '0-3-8081770-5', '813', 'American fiction > English (North America) > Literature', 175, 111),
  (810, 'Skin Game', 2014, '1', 464, 'PS3602.U85 S55', '0451-4-6439-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (340, 'Faro''s Daughter', 2008, '2', 304, 'PR6015.E795 F37', '978-1-40221-352-6', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (636, 'Northanger Abbey', 2002, '2', 256, 'PR4034 .N7', '037-575-917-4', '823.7', 'Early 19th century 1800-37 > English fiction > English {except North American} > Literature', 35, 155),
  (718, 'Quidditch Through the Ages', 2001, '1', 64, 'PR6068.O93', '04-393216-1-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 333, 210),
  (947, 'The Dervish House', 2010, '1', 410, 'PR6063.C38 D46', '16-1-614204-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 271, 195),
  (858, 'Tale of Benjamin Bunny', 1988, '2', NULL, 'PZ7.P85 T', '155-78201-6-3', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 313, 266),
  (410, 'Gregor And The Curse Of The Warmbloods', 2006, '2', 368, 'PZ7.C6837 G', '04-3965-624-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (1052, 'The High King', 1990, '2', 304, 'PZ7.A3774 H', '04-40435-74-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 29),
  (665, 'Ordeal by Innocence', 1970, '2', 192, 'PR6005.H66 O74', '000-612-450-X', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 87),
  (759, 'Royal Assassin: The Illustrated Edition', 2021, '1', NULL, 'PS3558.O33636 R68', '978-02-15-37027-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 180),
  (362, 'Fool''s Fate', 2004, '1', 640, 'PS3558.O33636 F67', '05-538015-4-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (1167, 'The Phoenix Guards', 1992, '2', 491, 'PS3552.R84 P4', '0-812506-89-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 80, 243),
  (746, 'Rick Bayless Mexico One Plate At A Time', 2000, '1', 384, 'TX716.M4 B29523', '068484-18-6-X', '641.5972', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Mexico, Central America, and the Caribbean > North America > Technology and Application of Knowledge', 52, 211),
  (954, 'The Dragonbone Chair', 1988, '1', 672, 'PS3573.I45563 D7', '0809-90-003-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 375, 57),
  (503, 'Jim Butcher''s Dresden Files: Wild Card', 2016, '1', 144, NULL, '15241-00-98-6', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 87, 76),
  (136, 'Build: An Unorthodox Guide to Making Things Worth Making', 2022, '1', 384, 'HB615 .F255', '0-063-04606-7', '658.421', 'Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 162, 105),
  (281, 'Dragon Keeper', 2010, '1', 496, 'PS3558.O33636 D73', '00-6156-162-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (280, 'Dragon Haven', 2010, '1', 528, 'PS3558.O33636 D69', '0-06-193141-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (27, 'A General Theory of Love', 2000, '2', 274, 'BF575.L8 L49', '03755-038-9-7', '152.41', 'Emotions > Emotions And Senses > Love > Philosophy and Psychology > Psychology', 275, 263),
  (1092, 'The Kodansha Kanji Learner''s Course: A Step-by-Step Guide to Mastering 2300 Characters', 2013, '2', 720, 'PL539.E5 C68', '15-68365-26-8', '495.61', 'Japanese > Language > Languages of East & Southeast Asia > Other Languages > Writing system, phonology, phonetics of standard Japanese', 122, 137),
  (409, 'Gregor and the Code of Claw', 2008, '2', 412, 'PZ7.C6837 G', '0-43-979144-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 121, 210),
  (986, 'The Eye of the World: The Graphic Novel, Volume Three', 2014, '2', 176, 'PN6728.W57 D584', '0-7-6533788-6', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 225, 243),
  (1199, 'The Rithmatist', 2022, '1', 384, 'PZ7.S19797 R', '978-58-2-292889-0', '813.7', 'American fiction > English (North America) > Literature', 344, 247),
  (243, 'Death Note, Vol. 2', 2005, '2', 200, 'PN6790.J34 D43', '1-4215-0169-4', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (70, 'A Shadow in Summer', 2016, '1', 336, 'PS3601.B677 S53', '978-9-5-7802049-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 4, 243),
  (652, 'One Day in the Life of Ivan Denisovitch', 1998, '2', 160, 'PG3488.O4 O3313', '0-45-152310-5', '891.7344', 'Late 20th century 1917вЂ“1991 > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Russian and East Slavic languages > Russian fiction > USSR 1917вЂ“1991', 354, 217),
  (1245, 'The Soul of a New Machine', 1982, '1', 294, 'TK7885.K53', '07139-14-82-3', '600', '-- > General Technology > Technology and Application of Knowledge', 237, 179),
  (1258, 'The Subtle Knife', 2003, '2', 304, 'PZ7.P968', '0-44-023814-5', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 317, 139),
  (782, 'Seveneves', 2015, '1', 880, 'PS3569.T3868 S47', '006-2190-37-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 356, 270),
  (309, 'Empress of the Sun', 2014, '1', 290, 'PZ7.M47841776 E', '161-61-4865-9', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 271, 195),
  (245, 'Death Note, Vol. 3', 2006, '2', 200, 'PN6790.J33 O358', '14-2150-170-8', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (533, 'Lavinia', 2009, '2', 288, 'PS3562.E42 L38', '0-156-03368-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 146),
  (402, 'Good Omens: The Nice and Accurate Prophecies of Agnes Nutter, Witch', 2006, '2', 432, 'PR6057.A319', '00-608539-8-0', '813', 'American fiction > English (North America) > Literature', 175, 111),
  (988, 'The Fairy''s Return and Other Princess Tales', 2006, '1', 400, 'PZ8.L4793 F', '0061-1-3061-3', '398.2', 'Customs, Etiquette, Folklore > Folk literature > Folklore > Social sciences', 258, 106),
  (595, 'Midnight''s Children', 1998, '2', 672, 'PR6068.U757 M5', '009957-8-51-4', '813', 'American fiction > English (North America) > Literature', 337, 263),
  (152, 'Castle in the Air', 2008, '2', 400, 'PZ7.J684 C', '006-147-877-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (888, 'The Art of the Start 2.0: The Time-Tested, Battle-Hardened Guide for Anyone Starting Anything', 2015, '1', 336, 'HD62.5 .K38', '1-5-9184784-2', '658.1', 'Management > Management and auxiliary services > Of Corporate Finance > Technology and Application of Knowledge', 231, 186),
  (1295, 'The Well of Ascension', 2007, '1', 592, 'PS3619.A533 W45', '0-7653168-8-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (1064, 'The Hostile Hospital', 2001, '1', 272, 'PZ7.S6795 H', '006-440-866-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 106),
  (712, 'Ptolemy''s Gate', 2005, '1', 512, 'PZ7.S92475 P', '0786-81-861-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (661, 'On the Origin of Species', 2011, '2', 320, 'QH365 .O2', '161949-1-30-3', '576.82', 'Evolution > Genetics and evolution > Life Sciences, Biology > Natural sciences and mathematics > Theories of evolution', 134, 78),
  (442, 'Harry Potter: The Illustrated Collection', 2018, '1', NULL, 'PZ7.R79835', '1-3-3831291-X', '823', 'English fiction > English {except North American} > Literature', 332, 17),
  (1061, 'The Hobbit', 1986, '2', 320, 'PZ7.T5744 H', '0-345-33968-1', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 362, 25),
  (1113, 'The Life of Padma, Volume 1', 2018, '1', 768, 'PK1428.S9 P313', '06-746-6036-6', '891.3', 'Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Middle Indic languages', 360, 116),
  (773, 'Secrets of the Dragon Sanctuary', 2009, '1', 528, 'PZ7.M9112 S', '160-64104-2-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (465, 'How to Win Friends and Influence People', 1990, '2', 276, 'BF637.S8 C37', '067-1-72365-0', '158.1', 'Applied Psychology > Personal improvement and analysis > Philosophy and Psychology > Psychology', 98, 184),
  (1325, 'Thief of Time', 2002, '2', 384, 'PR6066.R34 T47', '0-0-6103132-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 111),
  (1278, 'The Tyrant''s Law', 2013, '2', 528, 'PS3601.B677 T97', '0-316-08070-5', '813.6', '21st Century > American fiction > English (North America) > Literature', 4, 167),
  (1222, 'The Scrivener''s Bones', 2008, '1', 336, 'PZ7.S19797 A', '978-04399-2-553-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 210),
  (388, 'Getting Real: The Smarter, Faster, Easier Way to Build a Successful Web Application', 2006, '2', 194, 'TK5105 .G48', '0-5780-1281-2', '6.76', 'Computing and Information > Information > Multimedia systems > Special Topics > Web & Multimedia Programming', 172, 1),
  (194, 'Cordelia''s Honor', 2000, '2', 596, 'PS3552.U397 S53', '7-2-5721296-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (1226, 'The Secret Garden', 1998, '2', 384, 'PZ7.B934 S', '0-0644-0188-X', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 86, 106),
  (685, 'Permanent Record', 2019, '1', 352, 'JF1525.W45 S655', '1250-237-23-8', '327.12730092', 'Espionage and subversion > Foreign policy and specific topics in international relations > International Relations > North America > Political Science  > Social sciences > United States', 353, 148),
  (909, 'The Bogleheads'' Guide to Investing', 2021, '2', 336, 'HG4521 .L3194', '1-1-1984767-2', '332.6', 'Economics > Finance > Investing > Social sciences', 262, 269),
  (237, 'Death Masks: Book Five of The Dresden Files', 2003, '2', 464, 'PS3552.U8242 D42', '0-45145-940-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (294, 'Earwig and the Witch', 2012, '1', NULL, 'PZ7.J684 E', '978-0-0-6207511-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 98),
  (964, 'The Elements: A Visual Exploration of Every Known Atom in the Universe', 2009, '1', 240, 'QD466 .G73', '157912-8-14-9', '546', 'Chemistry > Inorganic > Natural sciences and mathematics', 188, 35),
  (857, 'Sylvester: or The Wicked Uncle', 2011, '2', 400, 'PZ3.H514 PR6015 .E795', '978-14-02-23880-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (122, 'Bossypants', 2012, '2', 275, 'PN2287.F4255 A3', '03-16-05687-1', '792.7028092', 'Recreational and performing arts > Stage presentations, Theatre > Techniques, procedures, apparatus, materials > The arts > Variety shows and theatrical dancing', 165, 23),
  (1285, 'The Vile Village', 2002, '2', 256, 'PZ7.S6795 V', '0439-4512-9-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 210),
  (856, 'Sword & Citadel', 1994, '2', 416, 'PS3573.O52 S88', '03-1289-018-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 376, 166),
  (739, 'Republic', 2004, '2', 392, 'JC71', '0872-2-0736-6', '321.07', 'Ideal state; Utopias > Political Science  > Political Systems > Political Systems > Social sciences', 311, 101),
  (619, 'Musicophilia: Tales of Music and the Brain, Revised and Expanded Edition', 2008, '2', 425, 'ML3830 .S13', '140-0-03353-5', '781.11', 'Basic principles of music > General principles and musical forms > Music > The arts', 342, 263),
  (1054, 'The History of Akbar, Volume 2', 2016, '1', 592, 'DS461.3 .A2313', '0-67-450494-1', NULL, NULL, 5, 116),
  (10, 'Accidental Empires: How the Boys of Silicon Valley Make Their Millions, Battle Foreign Competition, and Still Can''t Get a Date', 1992, '2', 324, 'HD9696.C63 U51586', '0-2-0157032-7', '338.470040979473', 'California > Computer Industry > Economics > Great Basin and Pacific Slope of United States    Pacific Coast states > History, geographic treatment, biography > North America > Production > Santa Clara county > Secondary industries and services > Services and specific products > Social sciences > Southern Coast Range counties', 128, 55),
  (764, 'Sailing to Sarantium', 2000, '2', 546, 'PR9199.K39 S25', '0-06-105990-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 79),
  (475, 'In Praise of Annada, Volume 2', 2020, '1', 400, 'BV4832.B43', '0674-970-98-5', NULL, NULL, 323, 116),
  (617, 'Murder in Three Acts', 1961, '2', NULL, 'PR6005.H66 M8447', '00-44500-44-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 185),
  (1294, 'The Way of Kings', 2020, '1', NULL, 'PS3619.A533', '978-1-938570-22-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 72),
  (624, 'Myths of the Norsemen: Retold from the Old Norse Poems and Tales', 1997, '2', 274, 'BL860 .G68', '8236-0290-9-8', '293.13', 'Germanic religion > Mythologies > Mythology--Germanic religion > Other Religions > Religions', 189, 192),
  (2, 'Abaddon''s Gate', 2013, '2', 576, 'PS3601.B677 A64', '03161-29-07-0', '813.6', '21st Century > American fiction > English (North America) > Literature', 127, 167),
  (798, 'Showa 1939-1944: A History of Japan', 2017, '2', 536, 'DS888 .M5913', '978-2906-366-71-6', '952.03', '1868-1945 > Asia > History and Geography > Japan', 283, 73),
  (240, 'Death Note, Vol. 11', 2007, '2', 208, 'PN6790.J33', '142-15-1178-9', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (200, 'Creative Selection: Inside Apple''s Design Process During the Golden Age of Steve Jobs', 2018, '1', 304, 'HD9696 .K63', '978-12-50-19446-6', '4.2', 'Computer Architectures > Computer science > Computing and Information > Information', 249, 231),
  (1107, 'The League of Extraordinary Gentlemen, Vol. 1', 1999, '2', 176, 'PN6737.M66 L56', '15638-98-58-6', '741.5941', 'British Isles > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > European > Graphic arts and decorative arts > The arts', 285, 11),
  (329, 'Facilitator''s Guide to Participatory Decision-Making', 2014, '2', 432, 'HD30 .K2753', '11184-049-5-5', '658.4036', 'Decision-making And Knowledge Management > Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 230, 131),
  (923, 'The Cat Who Wished to Be a Man', 2000, '2', 128, 'PZ7.A3774 C', '014-13-0704-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 192),
  (622, 'Mysteries of Pittsburgh', 2005, '2', 320, 'PS3553.H15 M97', '0-06079-059-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 101, 108),
  (1070, 'The Illyrian Adventure', 1990, '2', 144, 'PZ7.A3774 I', '044-0-40297-2', '219', '"Natural Theology and Secularism > Religions > [Formerly ""Analogies; Correspondences""; No longer used]"', 19, 275),
  (754, 'Rome''s Italian Wars: Books 6-10', 2013, '2', NULL, 'PA6452 .A5', '01-9-956485-X', '937.01', 'Ancient World > History and Geography > Italian Peninsula to 476 and adjacent territories to 476 > Italian Peninsula to 476 and adjacent territories to 476 > Kings 753-509 B.C.', 264, 171),
  (937, 'The Conan Chronicles', 1995, '1', 512, 'PS3560.O7617 A6', '0-3-1285929-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (199, 'Cracking the Coding Interview, Fourth Edition', 2008, '2', 310, 'QA76 .M34', '14-51-57827-X', '5.1028', 'Computer programming, programs, data, security > Computing and Information > Information > Miscellany > Programming > Programming -- Subdivisions', 255, 50),
  (115, 'Blood of Tyrants', 2018, '1', 448, 'PS3614.O93 B56', '978-544-7-54815-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (206, 'Crucible of Gold', 2012, '1', 323, 'PS3614.O93 C78', '03-455-2286-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 61),
  (1317, 'The Wonder That Was India', 2005, '2', 572, 'DS425 .B33', '03-30-43909-X', '954', 'Asia > History and Geography > India and South Asia', 48, 183),
  (75, 'Assassin''s Fate', 2017, '1', 640, 'PS3558.O33636 A92', '0-553-39295-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 61),
  (509, 'Jingo', 1999, '2', 448, 'PR6066.R34 J56', '0-06-105906-4', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 315, 106),
  (195, 'Cosmos', 1983, '2', 384, 'QB44.S235', '039-4-71596-9', '520', 'Astronomy > Astronomy > Natural sciences and mathematics', 343, 198),
  (848, 'Storytelling with Data: A Data Visualization Guide for Business Professionals', 2015, '2', 288, 'QA76.I52 K64', '1-11-900225-7', '1.4226', 'Computing and Information > Information > Knowledge > Research > Research methods > Statistical methods', 247, 269),
  (1057, 'The History of Akbar, Volume 5', 2019, '1', 784, NULL, '0-6-7498395-5', NULL, NULL, 5, 116),
  (683, 'Perfectly Reasonable Deviations from the Beaten Track', 2006, '2', 512, 'QC16.F49 A4', '04-6-502371-1', '530', 'Natural sciences and mathematics > Physics > Physics', 166, 31),
  (600, 'Mirror Dance', 1994, '1', 392, 'PS3552.U397 M57', '0-67-172210-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (817, 'Small Spaces', 2018, '1', 224, 'PZ7.A737 S', '978-0525-51-502-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 24, 94),
  (1348, 'Uncle Tungsten: Memories of a Chemical Boyhood', 2002, '2', 352, 'RC339.S23 A3', '0-3-7570404-3', '616.8092', 'Diseases > Diseases of nervous system and mental disorders > Medicine and health > Technology and Application of Knowledge', 342, 263),
  (1346, 'Turn Coat', 2009, '1', 432, 'PS3602.U85 T87', '045146-25-6-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (1291, 'The Warrior''s Apprentice', 2002, '1', 309, 'PS3552.U397 W3', '1-8-8677827-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 159),
  (202, 'Crime and Punishment', 1998, '2', 448, 'PG3326 .P7', '0140-6-2180-6', '813', 'American fiction > English (North America) > Literature', 146, 174),
  (1251, 'The Stars, Like Dust', 1983, '2', 231, 'PS3551.S5 S73', '0-34-531194-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 61),
  (768, 'Sauces: Classical and Contemporary Sauce Making', 1998, '1', 624, 'TX819.A1 P47', '04712-927-5-3', '641.814', 'Appetizers > Cooking Specific Dishes > Food And Drink > Home and family management > Sauces, Relishes, Dressings > Technology and Application of Knowledge', 310, 269),
  (473, 'Inkheart', 2009, '2', 576, 'PZ7.F96624 I', '0-5450462-6-2', '833.914', '1900-1990 > 1945-1990 > German and Germanic > German fiction > Literature > Modern period (1900-)', 173, 210),
  (418, 'Hackers: Heroes of the Computer Revolution', 2010, '2', 520, 'QA76 .L469', '1449-3-8839-6', '5', 'Computer programming, programs, data, security > Computing and Information > Information', 260, 168),
  (1189, 'The Reindeer People', 2008, '2', 336, 'PS3562.I47 R45', '978-9-9-0820341-4', '813', 'American fiction > English (North America) > Literature', 263, 113),
  (379, 'Friday''s Child', 2008, '2', 432, 'PR6015.E795 F7', '14-0-221079-5', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (244, 'Death Note, Vol. 2', 2013, '2', 200, 'PN6790.J34 D43', '978-9-5-1130476-6', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (351, 'Flatland: A Romance of Many Dimensions', 1992, '2', 96, 'QA699 .A13', '0-486-27263-X', '530.11', 'Natural sciences and mathematics > Physics > Physics > Relativity > Theoretical Physics', 1, 71),
  (137, 'Build Better Products: A Modern Approach to Building Successful User-Centered Products', 2016, '2', 368, 'TS171', '193-3-82058-6', '658.421', 'Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 243, 204),
  (971, 'The Enormous Crocodile', 1996, '2', 42, 'PZ7.D1515 E', '846-725-822-2', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 131, 192),
  (1253, 'The Start-up of You: Adapt to the Future, Invest in Yourself, and Transform Your Career', 2012, '1', 272, 'HF5384 .H63', '0-30788-890-8', '650.1', 'Business > Management and auxiliary services > Personal success in business > Technology and Application of Knowledge', 213, 53),
  (927, 'The Chronicles of Chrestomanci: The Magicians of Caprona / Witch Week', 2001, '2', 548, 'PZ7.J684 C', '0-0-6447269-8', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 224, 248),
  (1023, 'The Grand Sophy', 2009, '2', 384, 'PR6015.E795 G73', '1-402-21894-X', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (343, 'Firefight', 2015, '1', 432, 'PZ7.S19797 F', '0-38-574358-0', '813.7', 'American fiction > English (North America) > Literature', 344, 59),
  (308, 'Empowered: Ordinary People, Extraordinary Products', 2020, '1', 432, 'HF5415.15 .C244', '1-1-1969129-X', '658.4022', 'Executive > Internal organization > Management > Management and auxiliary services > Teams > Technology and Application of Knowledge', 91, 269),
  (777, 'See What I Mean: How To Use Comics to Communicate Ideas', 2012, '2', 216, 'NC1764 .C44', '193-382-027-6', '741.595195', 'Asian > Cartoons, Caricatures, Comics > China and adjacent areas > Collections > Drawing &  drawings > Graphic arts and decorative arts > Korea > South Korea > The arts', 106, 204),
  (549, 'Leviathan Wakes', 2011, '2', 592, 'PS3601.B677', '03-16-12908-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 127, 167),
  (1100, 'The Landmark Xenophon''s Hellenika', 2016, '2', 672, 'DF214 .X4613', '978-7663194-8-8-5', '938.06', 'Ancient World > Greece to 323 > Greece to 323 > History and Geography > Spartan and Theban Supremacy (404-362 BC)', 377, 14),
  (376, 'Fragile Things', 2019, '2', 448, 'PR6057.A319 F73', '978-32932-3-140-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 118),
  (781, 'Servant of the Underworld', 2016, '2', 338, 'PR6102.O637', '1-6-2567204-7', '813', 'American fiction > English (North America) > Literature', 135, 126),
  (1249, 'The Spirit Ring', 1992, '1', 369, 'PZ7.B91114 S', '0-6717214-2-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (1234, 'The Shadow Rising', 1993, '2', 1008, 'PS3560.O7617 S48', '08125-13-73-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 245),
  (721, 'Radiance', 2015, '1', 432, 'PS3622.A4258 R33', '0765-3-3529-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 368, 243),
  (774, 'Seductive Interaction Design: Creating Playful, Fun, and Effective User Experiences', 2011, '2', 223, 'TK5105 .A5284', '03-217-2552-2', '6.7', 'Computing and Information > Information > Multimedia systems > Special Topics', 23, 161),
  (443, 'Harry Potter: The Illustrated Collection', 2021, '1', NULL, 'PZ7.R79835', '978-4124-9-0677-1', '823', 'English fiction > English {except North American} > Literature', 332, 17),
  (156, 'Catherine the Great', 1965, '1', NULL, 'DK170.O373', '0-3-9441869-7', '947.063', 'Catharine I - Catharine II 1725-96 > Europe > History and Geography > Russia and eastern Europe [and formerly Finland] > Russian & Slavic History by Period', 306, 198),
  (704, 'Prelude to Foundation', 1994, '2', 464, 'PS3551.S5 P7', '05-860-7111-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 106),
  (458, 'Horizon', 2009, '1', 464, 'PS3552.U397 S57', '0061-37-536-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 113),
  (925, 'The Caves of Steel', 1991, '2', 288, 'PS3551.S5', '055-3-29340-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 227),
  (852, 'Sunshine', 2003, '1', 389, 'PS3563.C3816 S86', '0-4251917-8-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 274, 34),
  (100, 'Beginners: The Joy and Transformative Power of Lifelong Learning', 2021, '1', 320, 'BF637.S4 V375', '1-524-73216-8', '646.7', 'Home and family management > Management of personal and family life > Sewing, clothing, management of personal and family life > Technology and Application of Knowledge', 371, 134),
  (331, 'Falling Free', 1988, '2', 320, 'PS3552.U397 F35', '06-71-65398-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (392, 'Going Offline', 2018, '2', NULL, NULL, '1-9375-5765-0', NULL, NULL, 233, 2),
  (780, 'Servant of the Empire', 1991, '2', 704, 'PS3556.E446 S48', '05-5-329245-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 163, 227),
  (531, 'Last Chance to See', 1992, '2', 256, 'QL82 .A33', '034-53-7198-4', '591.529', 'Abode; Migration > Habits and behavior > Natural sciences and mathematics > Specific topics in natural history of animals > Zoology', 8, 25),
  (73, 'Assassin''s Apprentice', 1996, '2', 448, 'PS3558.O33636', '0553-57-339-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (760, 'Royal Escape', 2008, '2', 464, 'PZ3.H514 PR6015 .E795', '1-4022-1076-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 225),
  (320, 'Ethan of Athos', 2003, '1', 224, 'PS3552.U46 E74', '1886-778-39-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 159),
  (72, 'A Skinful of Shadows', 2017, '1', 432, 'PZ7.H21834 S', '141-97257-2-6', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 114),
  (676, 'Penric''s Demon', 2016, '1', 184, 'PS3552.U397', '15-9-606769-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (1398, 'Wizard of the Pigeons: The 35th Anniversary Illustrated Edition', 2020, '1', NULL, 'PS3562.I4786', '19-44145-59-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 263, 99),
  (800, 'Showa 1953-1989: A History of Japan', 2015, '2', 552, 'DS888 .M59413', '1-7-7046201-5', '952.03', '1868-1945 > Asia > History and Geography > Japan', 283, 73),
  (15, 'A Civil Contract', 2017, '2', 432, 'PR6015.E795 C58', '978-12-3-671481-6', '813', 'American fiction > English (North America) > Literature', 208, 224),
  (60, 'A Portrait of the Artist as a Young Man', 1976, '2', 256, 'PR6019.O9 P646', '19408-6-458-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 226, 174),
  (139, 'Buried Fire', 2003, '2', 304, 'PZ7.S92475 B', '055254-9-33-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 49),
  (1221, 'The Screaming Staircase', 2013, '1', 400, 'PZ7.S92475 S', '1423-1649-1-1', '820', 'English literature > English {except North American} > Literature', 357, 66),
  (421, 'Hannibal''s War', 2009, '2', 800, 'PA6452 .C1', '0199555-9-7-4', '355', 'Military Science > Public Administration, Military Science > Social sciences', 264, 171),
  (541, 'Legion', 2012, '1', 88, 'PS3619.A533', '159606-48-5-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 234),
  (1337, 'To Hold the Bridge', 2016, '1', 416, 'PZ7.N647 T', '978-88-23590-92-9', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 300, 106),
  (682, 'Perdido Street Station', 2003, '2', 640, 'PR6063.I265 P47', '0-34-545940-7', '813', 'American fiction > English (North America) > Literature', 277, 61),
  (423, 'Harbinger of the Storm', 2017, '2', 322, NULL, '16-25-67249-7', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 135, 126),
  (46, 'American Gods', 2006, '2', 624, 'PR6057.A319', '707-400-666-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 175, 111),
  (1274, 'The Tombs of Atuan', 2001, '2', 192, 'PZ7.L5215 T', '0-68-984536-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 219),
  (1143, 'The Murder of Roger Ackroyd', 1973, '2', 197, 'PR6005.H66 M85', '06-7-177706-8', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (1134, 'The Merchant and the Alchemist''s Gate', 2007, '1', 62, 'PS3603.H53 M47', '15-9-606100-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 109, 234),
  (861, 'Tales of the Greek Heroes', 2009, '2', 304, 'BL781.G66', '01-4-132528-3', '292.13', 'Classical religion (Greek and Roman religion) > Mythologies > Mythology--Greek religion > Other Religions > Religions', 189, 192),
  (657, 'Only the Paranoid Survive', 1996, '1', 224, 'HD58 .G765', '0385-482-58-2', '658.406', 'Executive > Management > Management and auxiliary services > Managing Change > Technology and Application of Knowledge', 190, 69),
  (1228, 'The Secret History of Mac Gaming', 2018, '1', NULL, NULL, '1-7-8352486-3', '794.8', 'Electronic and video games > Indoor games of skill; board games > Recreational and performing arts > The arts', 287, 253),
  (1213, 'The Sandman Vol. 5: A Game of You', 1993, '2', 192, 'PN6728.S26 G35', '15-638-9089-5', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (871, 'The Adventures of Huckleberry Finn', 2010, '2', 320, 'PS1305 .A1', '0-00-735103-8', '813.4', 'American fiction > English (North America) > Later 19th Century 1861-1900 > Literature', 367, 109),
  (241, 'Death Note, Vol. 12', 2007, '2', 216, 'PN6790.J33', '14-2-151327-7', '741.5952', 'Asian > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > Japanese > The arts', 305, 264),
  (1106, 'The Last Unicorn', 1991, '2', 304, 'PS3552.E13 L37', '045145-05-2-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 53, 202),
  (1089, 'The Knights of Crystallia', 2016, '1', 320, 'PZ7.S19797 A', '0765-37-898-1', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 344, 229),
  (1369, 'Visual Explanations: Images and Quantities, Evidence and Narrative', 1997, '1', 156, 'P93 .T846', '0961-392-12-6', '302.23', 'Communication > Media (Means of communication) > Social Interaction > Social Sciences > Social sciences', 364, 97),
  (1335, 'To Green Angel Tower', 1993, '1', 1083, 'PS3573.I45563 T6', '0-8-8677521-3', '813', 'American fiction > English (North America) > Literature', 375, 57),
  (643, 'Oathbringer', 2018, '2', 1264, 'PS3619.A533 O38', '1250-2971-4-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (401, 'Good Eats: The Early Years', 2009, '1', 396, 'TX651 .B727', '978-15-8-479795-1', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 78, 230),
  (82, 'A Tale of Two Cities', 1997, '2', 371, 'PR4571 .A1', '0-4515-2656-2', '823.8', 'English fiction > English {except North American} > Literature > Victorian period 1837-1900', 145, 217),
  (215, 'Curtain', 1985, '2', 238, 'PZ3.C4637PR6005.H66Cu3', '573-9-36173-3', '823/.9/12', 'English fiction > English {except North American} > Literature > Modern Period', 113, 67),
  (552, 'Lions of Al-Rassan', 2011, '2', 656, 'PR9199.K39 L56', '00-073420-6-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 232, 113),
  (435, 'Harry Potter and the Order of the Phoenix: The Illustrated Edition', 2022, '1', 576, 'PR6068.O93', '0-54-579143-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 210),
  (827, 'Spellbound: Fantasy Stories', 2007, '2', 280, 'PZ5 .F2148', '0-753-46144-7', '823.0876608', 'Adventure fiction > By Type > Collections > English fiction > English {except North American} > Fantasy fiction > Genre fiction > Literature > Speculative fiction', 224, 133),
  (270, 'Dirk Gently''s Holistic Detective Agency', 1988, '2', 256, 'PR6051.D3352', '03-3-030162-4', '813', 'American fiction > English (North America) > Literature', 8, 243),
  (118, 'Bone in the Throat', 2000, '2', 304, 'PS3552.O8328 B65', '15-82-34102-8', '813', 'American fiction > English (North America) > Literature', 72, 38),
  (1018, 'The Golden Dream of Carlo Chuchio', 2007, '1', 306, 'PZ7.A3774 G', '0-80508-333-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 19, 119),
  (1264, 'The Telling', 2000, '2', 272, 'PS3562.E42 T45', '01510-05-67-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 102),
  (105, 'Beyond the Dreams We Know: A Collection', 2018, '2', NULL, NULL, '1-9-8076043-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 124),
  (1021, 'The Golem''s Eye', 2004, '1', 576, 'PZ7.S92475 G', '07868-186-0-3', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 357, 66),
  (1005, 'The Further Chronicles of Conan', 1999, '1', 512, 'PS3560.O7617 A6', '0-3-1287195-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (1155, 'The Outlaws of Sherwood', 1989, '2', 288, 'PZ8.M198 O', '04416-445-1-1', '398.2094202', 'Customs, Etiquette, Folklore > European folktales > Folk literature > Folklore > Folklore of England > History, geographic treatment, biography > Social sciences', 274, 4),
  (955, 'The Dragonfly Pool', 2009, '2', 384, 'PZ7.I117 D', '0142-4-1486-7', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (1304, 'The Will to Battle: Book 3 of Terra Ignota', 2017, '1', 352, 'PS3616.A33879 W55', '07653-780-4-3', '813.6', '21st Century > American fiction > English (North America) > Literature', 308, 243),
  (28, 'A History of Japan, 1334-1615', 1961, '2', 478, 'DS835.S27', '08-04-70525-9', '952', 'Asia > History and Geography > Japan', 345, 228),
  (594, 'Mexican Everyday', 2005, '1', 288, 'TX716.M4 B293', '03-930-6154-X', '641.5972', 'Cooking characteristic of specific geographic environments, ethnic cooking > Cooking, cookbooks > Food And Drink > Home and family management > Mexico, Central America, and the Caribbean > North America > Technology and Application of Knowledge', 52, 264),
  (695, 'Pocket Full of Rye', 2006, '2', 224, 'PR6005.H66 P6', '98-7355-048-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 106),
  (1296, 'The Well of Ascension', 2007, '1', NULL, 'PS3619.A533 W45', '978-19-385701-8-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 72),
  (1000, 'The Forgotten Beasts of Eld', 2017, '2', 240, 'PZ7.M19864 F', '1-6-1696277-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 273, 236),
  (633, 'Noise: A Flaw in Human Judgment', 2021, '1', 464, 'BF447 .K34', '031-6-45140-1', '153.83', 'Cognition And Memory > Decision Making > Decision Making And Persuasion > Philosophy and Psychology > Psychology', 228, 141),
  (992, 'The Feynman Lectures on Physics, Vol. 3', 1971, '2', 400, 'QC23.F47', '0-20-102118-8', '530', 'Natural sciences and mathematics > Physics > Physics', 166, 6),
  (603, 'Mistborn: Secret History', 2022, '1', 240, 'PS3619.A533', '1-2-5085914-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 344, 243),
  (1275, 'The Town Cats and Other Tales', 1998, '2', 144, 'PZ8.A37 T', '01413-012-2-8', '813.5', '20th Century > American fiction > English (North America) > Literature', 19, 192),
  (805, 'Simon the Coldheart', 2008, '2', 348, 'PR6015.E795 S56', '14-02213-54-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 225),
  (1282, 'The User Experience Team of One: A Research and Design Survival Guide', 2013, '2', 246, 'QA76.U83 B86', '1-93382-018-7', '4.019', 'Computer Interaction With Humans > Computer science > Computer science -- subdivisions > Computing and Information > Information > Theory And Instruction', 85, 204),
  (325, 'Exploring Calvin and Hobbes: An Exhibition Catalogue', 2015, '2', 152, 'NC1429.W36 A4', '14-4-946036-4', '741.56973', 'Cartoons, Caricatures, Comic Strips > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 372, 15),
  (913, 'The Boy Who Lost Fairyland', 2015, '1', NULL, 'PZ7.V232 B', '12-500234-9-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 368, 83),
  (548, 'Leviathan', 1982, '2', 736, 'JC153 .H65', '01-4-043195-0', '320.1', 'Political Science  > Political Science  > Social sciences > The State', 211, 175),
  (693, 'Plays', 1999, '2', 292, 'PG3456.A19 G3', '8-17-167404-6', '891.723', '1800вЂ“1917 > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Russian and East Slavic languages > Russian drama', 105, 205),
  (735, 'Remote Research: Real Users, Real Time, Real Research', 2010, '2', 266, 'QA76.H85', '19-3-382077-2', '1.4202854678', 'Computing and Information > Information > Knowledge > Research > Research methods', 71, 204),
  (1382, 'What''s a Cook to Do?: An Illustrated Guide to 484 Essential Tips, Techniques, and Tricks', 2007, '2', 422, 'TX651 .P484', '15-796531-8-9', '641.5', 'Cooking, cookbooks > Food And Drink > Home and family management > Technology and Application of Knowledge', 310, 18),
  (322, 'Every Thing On It', 2011, '1', 208, 'PS3569.I47224 E94', '0-06-199816-8', '811.54', '1945-1999 > 20th Century > American poetry > English (North America) > Literature', 351, 106),
  (1190, 'The Reluctant Widow', 2008, '2', 320, 'PR6015.E795', '978-1-402-21351-9', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (1216, 'The Sandman Vol. 7: Brief Lives', 1995, '2', 256, 'PN6728.S26 G35', '1-56-389138-7', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (478, 'Inspired: How to Create Tech Products Customers Love', 2017, '1', 368, 'HF5415.157', '1119-3875-0-7', '658.575', 'Management > Management and auxiliary services > Of Production > Product Development > Research And Development > Technology and Application of Knowledge', 91, 269),
  (210, 'Cryptonomicon', 2004, '2', 918, 'PS3569.T3868 C79', '134-3-09286-2', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 356, 198),
  (1380, 'What Do You Care What Other People Think?: Further Adventures of a Curious Character', 2001, '2', 256, 'QC16.F49 A3', '0-393-32092-8', '530.092', 'Biography > Biography And History > Natural sciences and mathematics > Physics > Physics > Physics', 166, 273),
  (398, 'Gone With the Wind', 1991, '2', 1024, 'PS3525.I972', '03303-2-349-0', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 281, 172),
  (790, 'Shadow Twin', 2018, '2', 400, NULL, '198-0-30490-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 297, 124),
  (258, 'Designing Data-Intensive Applications: The Big Ideas Behind Reliable, Scalable, and Maintainable Systems', 2020, '2', 611, 'QA76.A65', '978-5-5-4152976-5', '5.276', 'Computer programming, programs, data, security > Computing and Information > For Networked Multimedia Systems > Information > Programming for Specific Environments > Web Programming', 245, 168),
  (1316, 'The Wonderful Story of Henry Sugar', 2000, '2', 240, 'PZ7.D1515 W', '01-413-0470-7', '813', 'American fiction > English (North America) > Literature', 131, 192),
  (387, 'Gentlemen of the Road: A Tale of Adventure', 2007, '1', 204, 'PS3553.H15 G46', '034550-17-4-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 101, 61),
  (94, 'Bastille vs. the Evil Librarians', 2022, '1', 272, 'PZ7.S19797 B', '1-2508110-6-6', NULL, NULL, 344, 229),
  (732, 'Regency Buck', 2008, '2', 416, 'PR6015.E795 R455', '140-2-21349-2', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (377, 'Freakonomics', 2006, '2', 320, 'HB74.P8 L479', '00-612427-0-5', '330', 'Economics > Economics > Social sciences', 259, 106),
  (1159, 'The Path of Daggers', 1999, '2', 704, 'PS3560.O7617 P38', '08-125-5029-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (307, 'Empire of Ivory', 2008, '2', 400, 'PS3614.O93 E57', '0007256-7-4-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 304, 113),
  (432, 'Harry Potter and the Goblet of Fire: The Illustrated Edition', 2019, '1', 464, 'PR6068.O93', '054579-14-2-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 17),
  (1362, 'UX for Lean Startups: Faster, Smarter User Experience Research and Design', 2018, '2', 236, 'QA76.H85 K54', '1492-0-4958-1', '658.11', 'Management > Management and auxiliary services > Of Corporate Finance > Startup Management > Technology and Application of Knowledge', 243, 168),
  (242, 'Death Note, Vol. 13: How to Read', 2008, '2', 192, 'PN6790.J33', '14-21-51888-0', '741.56952', 'Asian > Cartoons, Caricatures, Comic Strips > Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > The arts', 305, 264),
  (836, 'Stardust', 2009, '2', 288, 'PS3557.A3519 S73', '0-0616-8924-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 175, 106),
  (563, 'Lord of Chaos', 1994, '1', 720, 'PS3560.O7617 L67', '0-31-285428-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 243),
  (462, 'How To Cook Everything: Simple Recipes for Great Food', 2006, '2', 960, 'TX714 .B573', '0-4-7178918-6', '641', 'Food And Drink > Home and family management > Technology and Application of Knowledge', 64, 269),
  (204, 'Crispin: The Cross of Lead', 2004, '2', 262, 'PZ7.A953 C', '04-39-57468-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 37, 210),
  (13, 'A Civil Campaign: A Comedy of Biology and Manners', 1999, '1', 405, 'PS3552.U397 C58', '0-67-157827-8', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (829, 'Spindle''s End', 2003, '2', 432, 'PZ8.M1793 S', '0-6-6859991-6', '813', 'American fiction > English (North America) > Literature', 274, 85),
  (1020, 'The Golden Fool', 2008, '2', 712, 'PS3558.O33636 G65', '0-0-0648602-9', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (1045, 'The Heart of What Was Lost of Osten Ard', 2017, '1', 224, 'PS3573.I45563 H43', '0-756-41248-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 375, 57),
  (560, 'Living with Complexity', 2010, '1', 308, 'T14 .N673', '0-2-6201486-6', '601', 'General Technology > Philosophy and theory > Technology and Application of Knowledge', 302, 154),
  (1001, 'The Foundling', 1982, '2', 10, 'PZ7.A3774 F', '0440-425-36-0', '800.1988', 'By Topic > Literature > Literature: general works', 19, 275),
  (916, 'The Buried Giant', 2015, '1', 336, 'PR6059.S5', '030-7-27103-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 220, 134),
  (476, 'INSANELY GREAT: The Life and Times of Macintosh, the Computer that Changed Everything', 1994, '1', 304, 'QA76.M3 L487', '978-0-6-7085244-4', '338.761004165', 'Business Enterprises > By Industry > Economics > Production > Service > Social sciences', 260, 260),
  (890, 'The Bad Beginning', 1999, '1', 162, 'PZ7.S6795 B', '0-0-6075589-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 352, 106),
  (1345, 'Trains', 1995, '1', 32, 'TF148 .B27', '4-75-082097-2', '385.37', 'Commerce, Communications, Transportation > Facilities and vehicles > Social sciences > Trains and Railroads', 47, 252),
  (1158, 'The Pale Horse', 1972, '2', NULL, 'PR6005.H66 P28', '06717-7-615-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (1263, 'The Tales of Beedle the Bard: The Illustrated Edition', 2018, '1', 160, 'PZ7.R79835', '978-1338-2621-8-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 17),
  (177, 'Cloven Hooves', 2019, '2', 368, 'PS3558.O33636', '0-00-828739-2', '813', 'American fiction > English (North America) > Literature', 263, 113),
  (917, 'The Call of the Wild', 1990, '2', 128, 'PS3523.O46 C3', '08-1250-432-1', '813.52', '1900-1944 > 20th Century > American fiction > English (North America) > Literature', 265, 7),
  (564, 'Lord of Chaos', 1995, '2', 1011, 'PS3560.O7617 L67', '081-25137-5-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 245),
  (605, 'Mistborn: The Final Empire', 2006, '1', 544, 'PS3619.A533', '0765-31-178-X', '813', 'American fiction > English (North America) > Literature', 344, 243),
  (898, 'The Best American Science and Nature Writing 2014', 2014, '2', 352, 'PN6071.S3 B46', '0-544-00342-X', '810.8', 'American literature > Anthologies and Collections > English (North America) > Literature', 69, 146),
  (61, 'A Practical Guide to Localization', 2000, '2', 490, 'QA76.D63 E87', '1-5-8811006-0', '5.1', 'Computer programming, programs, data, security > Computing and Information > Information > Programming', 160, 128),
  (616, 'Murder in the Calais Coach', 1974, '2', NULL, 'PR6005.H66 M88', '0-6-7177448-4', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 113, 184),
  (841, 'Stolen Magic', 2015, '1', NULL, 'PZ7.L578345 S', '0-0-6170638-8', '813.6', '21st Century > American fiction > English (North America) > Literature', 258, 106),
  (891, 'The Bad Beginning', 2012, '1', 162, 'PZ7.S6795 B', '978-6-8329-2113-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 352, 106),
  (1328, 'Thinking, Fast and Slow', 2011, '1', NULL, 'BF441 .K238', '03-8-567651-4', '153.4', 'Cognition And Memory > Philosophy and Psychology > Psychology > Thought, thinking, reasoning, intuition, value, judgment', 228, 70),
  (1217, 'The Sandman Vol. 8: Worlds'' End', 1995, '2', 168, 'PN6728.S26 G35', '1-5-6389171-9', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 175, 258),
  (324, 'Exo', 2014, '1', NULL, 'PS3557.O8947 E96', '0-765336-54-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 186, 243),
  (972, 'The Epic of Ram, Volume 1', 2016, '1', 416, 'PK1947.T83 R313', '06-7442-501-4', '891.4', 'Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Modern Indic languages', 365, 116),
  (740, 'Republic', 2014, '2', 392, 'JC71', '978-17970-80-43-1', '321.07', 'Ideal state; Utopias > Political Science  > Political Systems > Political Systems > Social sciences', 311, 101),
  (450, 'High Output Management', 1995, '2', 272, 'HD31 .G764', '978-0-679762-88-1', '658.5', 'Management > Management and auxiliary services > Of Production > Technology and Application of Knowledge', 190, 263),
  (905, 'The Blacktongue Thief', 2021, '1', 416, 'PS3602.U3395 B53', '12506-2-119-4', '813.6', '21st Century > American fiction > English (North America) > Literature', 83, 243),
  (751, 'River of Gods', 2006, '1', 599, 'PR6063.C38 R58', '1-5-9102436-6', '813', 'American fiction > English (North America) > Literature', 271, 195),
  (430, 'Harry Potter and the Deathly Hallows', 2007, '2', 608, 'PR6068.O93', '0-5907-6200-1', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 332, 37),
  (1044, 'The Haunting of Granite Falls', 2005, '2', 216, 'PZ7.I117 H', '1-4176949-8-X', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 217, 192),
  (949, 'The Dispossessed', 1994, '2', 400, 'PS3562.E42', '00-6-105488-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 256, 113),
  (236, 'Deathless', 2011, '1', 352, 'PS3622.A4258', '076-53-2630-2', '813.6', '21st Century > American fiction > English (North America) > Literature', 368, 243),
  (818, 'Smoke and Mirrors: Short Fictions and Illusions', 2005, '2', 384, 'PR6057.A319 S6', '03-80-78902-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 175, 22),
  (804, 'Silverthorn', 1986, '2', 343, 'PS3556.E446 S5', '0-55327-054-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 163, 26),
  (1135, 'The Messy Middle: Finding Your Way Through the Hardest and Most Crucial Part of Any Bold Venture', 2018, '1', 416, 'HD62.5', '073-52-1807-2', '658.4', 'Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 58, 186),
  (599, 'Mira''s Last Dance', 2018, '1', 160, 'PS3552.U397 M567', '15-9606-854-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 234),
  (133, 'Brothers in Arms', 2001, '2', 338, 'PS3552.U385 B76', '06-7169-799-4', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 84, 24),
  (3, 'A Beautiful Constraint: How To Transform Your Limitations Into Advantages, and Why It''s Everyone''s Business', 2015, '1', 288, 'HF5386 .M674', '978-1-118-89901-4', '650.1', 'Business > Management and auxiliary services > Personal success in business > Technology and Application of Knowledge', 45, 269),
  (996, 'The First Two Lives of Lukas-Kasha', 1998, '2', 224, 'PZ7.A3774 F', '0-14-130057-4', '813', 'American fiction > English (North America) > Literature', 19, 192),
  (1065, 'The Hostile Hospital', 2005, '1', 272, 'PZ7.S6795 H', '76-7-317912-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 352, 106),
  (1090, 'The Kodansha Kanji Dictionary', 2013, '1', 2112, 'PL677.5 .N73', '15-6-836408-3', '495.6', 'Japanese > Language > Languages of East & Southeast Asia > Other Languages', 194, 136),
  (1406, 'Wrath of the Dragon King', 2018, '1', 416, 'PZ7.M9112 W', '1-6-2972486-6', '813.6', '21st Century > American fiction > English (North America) > Literature', 289, 213),
  (265, 'Designing Web Usability', 1999, '2', 432, 'TK5105 .N55', '1-5-6205810-X', '5.2', 'Computer programming, programs, data, security > Computing and Information > Information > Programming for Specific Environments', 298, 161),
  (1390, 'White Night', 2007, '1', 407, 'PS3602.U85 W48', '0-4-5146140-1', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (497, 'JavaScript: The Good Parts', 2008, '2', 176, 'QA76.J39 C763', '0596-51-774-2', '5.133', 'Computer programming, programs, data, security > Computing and Information > General Programming Languages > Information > Languages > Programming', 129, 214),
  (904, 'The Black Moth', 2009, '2', 368, 'PR6015.E795 B49', '140221-95-2-0', '823.912', '1901-1945 > 1901-1999 > English fiction > English {except North American} > Literature > Modern Period', 208, 224),
  (710, 'Proven Guilty', 2007, '2', 576, 'PS3602.U85 P76', '0-45-146103-7', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 202),
  (1360, 'User Is Always Right, The: A Practical Guide to Creating and Using Personas for the Web', 2006, '2', 312, 'HF5415 .M85', '032-1-43453-6', '5.7', 'Computer programming, programs, data, security > Computing and Information > Data > Information', 288, 161),
  (738, 'Renegade''s Magic', 2008, '1', 672, 'PS3558.O33636 R46', '0-06075-764-7', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 113),
  (49, 'Anansi Boys', 2006, '2', 387, 'PR6057.A319 A85', '0-0611-4060-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 175, 111),
  (313, 'Envisioning Information', 1990, '1', 126, 'P93 .T84', '0961-392-11-8', '302.23', 'Communication > Media (Means of communication) > Social Interaction > Social Sciences > Social sciences', 364, 97),
  (1377, 'We Can Remember It for You Wholesale', 2012, '2', 381, 'PS3554.I3 A15', '978-0-806-53648-4', '813.0876208', 'Adventure fiction > American fiction > By type > Collections > English (North America) > Genre fiction > Literature > Science fiction > Speculative fiction', 144, 46),
  (1087, 'The Knight', 2005, '2', 544, 'PS3573.O52 K58', '07653-4-701-6', '813', 'American fiction > English (North America) > Literature', 376, 245),
  (457, 'Homicidal Psycho Jungle Cat: A Calvin and Hobbes Collection', 1994, '2', 176, 'PN6728.C34', '0836-2176-9-1', '741.5973', 'Cartoons, Caricatures, Comics > Collections > Drawing &  drawings > Graphic arts and decorative arts > North American > The arts > United States (General)', 372, 15),
  (1336, 'To Hold the Bridge', 2015, '1', 416, 'PZ7.N647 T', '978-006-229-252-0', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 300, 106),
  (346, 'First Lord''s Fury', 2009, '1', 480, 'PS3602.U85 F57', '0-441-01769-X', '813.6', '21st Century > American fiction > English (North America) > Literature', 87, 4),
  (561, 'Logicomix: An Epic Search for Truth', 2009, '2', 352, 'PA5615.O87 L64', '159-691-452-1', '192', 'British philosophers > Modern western philosophy > Philosophy and Psychology', 148, 38),
  (1175, 'The Pride of Chanur', 1982, '2', 223, 'PS3553.H358 P75', '0886772-9-2-3', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 108, 57),
  (658, 'Only the Paranoid Survive', 2009, '1', 224, 'HD58 .G765', '978-8601-6-0687-3', '658.406', 'Executive > Management > Management and auxiliary services > Managing Change > Technology and Application of Knowledge', 190, 69),
  (1097, 'The Landmark Julius Caesar: The Complete Works', 2019, '2', 896, 'DG261 .L36', '0-3074-5544-0', '937.05', 'Ancient World > Civil strife 146-21 B.C. > History and Geography > Italian Peninsula to 476 and adjacent territories to 476 > Italian Peninsula to 476 and adjacent territories to 476', 90, 14),
  (519, 'Kiki''s Delivery Service', 2020, '1', NULL, 'PZ7.K1167 K', '198-48-9666-0', '895.635', '1945вЂ“2000 > Asian (east and south east) languages > Japanese > Japanese fiction > Literature > Literature of other languages', 227, 59),
  (1156, 'The Outlaws of Sherwood', 1991, '2', 288, 'PZ8.M198 O', '367-54-2237-6', '398.2094202', 'Customs, Etiquette, Folklore > European folktales > Folk literature > Folklore > Folklore of England > History, geographic treatment, biography > Social sciences', 274, 4),
  (357, 'Fly Trap', 2011, '1', 592, 'PZ7.H21834 F', '006-08804-4-9', '823.92', '2000- > English fiction > English {except North American} > Literature > Modern Period', 198, 106),
  (797, 'Showa 1939-1944: A History of Japan', 2014, '2', 536, 'DS888 .M5913', '17-704615-1-5', '952.03', '1868-1945 > Asia > History and Geography > Japan', 283, 73),
  (345, 'FirstBorn & Defending Elysium', 2022, '1', 160, NULL, '978-57-95438-30-8', NULL, NULL, 344, 72),
  (468, 'Implementing Value Pricing: A Radical Business Model for Professional Firms', 2010, '1', 400, 'HF5416.5 .B34', '978-0-4705846-1-3', '658.8', 'Management > Management and auxiliary services > Of Marketing > Technology and Application of Knowledge', 42, 269),
  (405, 'Good to Great: Why Some Companies Make the Leap and Others Don''t', 2001, '1', 400, 'HD57 .C645', '978-0066-6-2099-2', '658', 'Management > Management and auxiliary services > Technology and Application of Knowledge', 120, 105),
  (868, 'Testing Business Ideas: A Field Guide for Rapid Experimentation', 2019, '2', 368, 'HD30 .B5926', '1119-5-5144-7', '658.4034', 'Decision-making And Knowledge Management > Executive > Management > Management and auxiliary services > Technology and Application of Knowledge', 65, 269),
  (441, 'Harry Potter - Fantastic Beasts & Where To Find Them', 2004, '1', 42, 'PR6068.O93 F36', '2-8-0422828-9', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 333, 210),
  (642, 'Now You See It: An Introduction to Visual Data Sensemaking', 2021, '1', 301, 'TK7882.I6 F48', '193-83771-2-5', '650', 'Business > Management and auxiliary services > Technology and Application of Knowledge', 164, 13),
  (1252, 'The Stars, Like Dust', 1989, '2', 231, 'PS3551.S5 S73', '1-34529-472-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 31, 61),
  (126, 'Brandon Sanderson''s White Sand Volume 1', 2016, '1', 160, 'PN6728.W584 H67', '1606-9088-5-5', '741.5', 'Cartoons, Caricatures, Comics > Drawing &  drawings > Graphic arts and decorative arts > The arts', 344, 76),
  (734, 'Remote: Office Not Required', 2013, '1', 256, 'HD2336 .F75', '0-8-0413750-1', '658.3', 'Management > Management and auxiliary services > Of Personnel > Technology and Application of Knowledge', 172, 53),
  (1033, 'The Great Hunt', 1991, '2', 705, 'PS3560.O7617 G74', '08125-17-72-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 245),
  (1077, 'The Invention of Hugo Cabret', 2007, '1', 544, 'PZ7.S4654 I', '0-4-3981378-6', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 349, 210),
  (1305, 'The Windsingers', 2002, '2', 400, 'PS3562.I505 W5', '0-0071-1253-X', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 263, 106),
  (395, 'Golden Fool', 2003, '1', 528, 'PS3558.O33636 G65', '0-55-380151-1', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 210, 227),
  (860, 'Tales of Neveryon', 1979, '2', 264, 'PS3554.E437 T3', '0-55312-333-5', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 138, 26),
  (1391, 'White Sands, Red Menace', 2008, '1', 352, 'PZ7.K6768 W', '067-00-6235-9', '813.6', '21st Century > American fiction > English (North America) > Literature', 242, 261),
  (933, 'The Complete Adventures of Feluda, Vol. 1', 2004, '2', 785, 'PK1718.R3119', '0-14-303277-1', '891.44371', '1940- > Bengali > Fiction > Literature > Literature of east Indo-European and Celtic languages > Literature of other languages > Modern Indic languages', 324, 174),
  (763, 'Sabriel', 2021, '1', 368, 'PZ7.N647', '00-630-4050-6', '823.914', '1901-1999 > 1945-1999 > English fiction > English {except North American} > Literature > Modern Period', 300, 106),
  (381, 'From The Two Rivers: The Eye of the World, Book 1', 2002, '2', 416, 'PS3560.O7617 E94', '0-7-6534184-0', '813.54', '1945-1999 > 20th Century > American fiction > English (North America) > Literature', 225, 229)
;

--
-- Data for Name: Items; Type: TABLE DATA; Schema: Library Management; Owner: -
--

INSERT INTO "Items" (id, "Barcode", "Acquisition Date", "Acquisition Price", "Book") VALUES
  (82, '2D4C-8311-4C68-3CD3', '2019-09-08', 18.16, 1056),
  (796, '8CDC-9FDE-A1BF-747C', '2022-09-22', 12.72, 609),
  (1712, '9489-7A6D-4BB8-C1D2', '2016-10-31', 7.23, 1195),
  (885, 'B116-3C21-FF60-128F', '2021-06-26', 19.84, 1196),
  (116, '01FF-1940-F0B7-4B4B', '2020-02-11', 5.40, 1197),
  (39, '996B-5731-4635-35D7', '2022-08-20', 13.25, 1198),
  (40, 'FDEC-93AF-1ABF-AF77', '2022-10-14', 11.28, 1199),
  (107, '1F99-56DF-AC3E-2DA1', '2000-07-19', 12.56, 1200),
  (454, '3B7D-AEF3-B976-6772', '2007-11-06', 15.29, 1201),
  (1296, 'A921-2CDF-D6C3-90FB', '2010-08-14', 10.52, 1202),
  (1297, 'A1F2-01BC-B43F-6FF5', '2020-11-08', 7.08, 1203),
  (1341, 'B26C-8190-1717-C712', '2003-12-21', 10.05, 1204),
  (628, '5C27-2F14-4149-F23C', '2018-02-25', 9.61, 1205),
  (167, '0A24-2AE6-33DF-A177', '2005-12-06', 15.92, 1206),
  (168, 'A0EC-2CB7-05B9-DF65', '2016-07-11', 7.41, 1206),
  (1079, '4BBE-E3D5-6803-3D2E', '2007-05-24', 5.00, 1207),
  (1103, '192B-A4E4-AFFD-C773', '2009-05-09', 8.63, 1208),
  (526, 'CA6E-9AD9-B597-35F0', '1995-05-10', 10.92, 1209),
  (421, '436C-AAB1-5F6C-7B9F', '2016-10-16', 6.77, 1210),
  (422, '6898-1DFB-76AD-5B4E', '2003-09-22', 16.73, 1211),
  (1299, 'B3B7-4A9B-BE50-A8B0', '2004-05-15', 5.14, 1212),
  (111, 'EAEA-7E20-B6DB-6A08', '1995-01-03', 18.92, 1213),
  (693, '9B25-A8E5-3C3C-F263', '2002-06-01', 15.23, 1214),
  (694, '82AE-DECF-D0AD-AD9F', '2002-02-01', 12.01, 1215),
  (695, 'D8C5-49F5-CEDE-871A', '2019-05-12', 12.37, 1215),
  (280, 'C4A9-A382-01AD-5DC4', '2014-07-12', 6.58, 1216),
  (340, '9AC1-F044-5307-0BF0', '2006-12-27', 7.57, 1217),
  (341, 'B27E-D273-0098-F5CC', '2009-12-05', 7.69, 1218),
  (1342, 'D7D6-575D-FD9B-9ABE', '2016-10-26', 15.05, 1219),
  (1193, 'BA0E-D679-0E27-CE4A', '2018-03-19', 17.56, 1330),
  (393, 'ACF3-FB6D-EA88-0724', '2021-08-18', 11.96, 1055),
  (61, '489E-DB4F-C079-8D4A', '2014-06-10', 7.82, 811),
  (745, '65A7-C8CB-5881-E5C5', '2022-03-16', 16.57, 23),
  (823, '90F6-1FC7-7A80-6AB5', '2022-05-05', 15.71, 475),
  (499, '3506-41F0-2B7D-CEFC', '2014-06-18', 18.87, 802),
  (939, '8B5E-1F32-681C-52F7', '2013-04-27', 6.04, 1235),
  (1343, '4447-2EDB-A30B-2C35', '2022-09-27', 18.14, 392),
  (1521, 'B9CC-E85C-FF31-34E2', '2022-08-13', 10.73, 188),
  (1428, 'F596-63E1-A8D2-85AD', '2019-09-27', 12.76, 351),
  (373, '6DD2-B593-2D2C-08F8', '2019-02-02', 11.18, 419),
  (1267, 'CCFE-8E62-BB3E-DBFC', '2012-11-17', 13.11, 1185),
  (1094, '3D8B-E6A7-1826-B058', '2019-12-08', 14.69, 1078),
  (1693, 'A59A-6E85-A51E-024F', '2021-12-23', 12.75, 1163),
  (867, 'EA09-62A7-D999-0B1C', '2023-01-03', 10.78, 779),
  (248, '3673-91CB-DBE0-42F4', '2019-02-09', 19.95, 664),
  (1704, '7BCE-62A1-FD84-5658', '1991-03-27', 17.22, 1029),
  (1699, '216B-4B9E-241F-F2BB', '2022-06-03', 15.87, 1025),
  (538, 'B5C5-A6F4-BCA4-6493', '2021-11-28', 18.43, 348),
  (24, '8A39-15E3-4B50-3D0B', '2016-06-24', 9.02, 667),
  (492, '1D90-C5BB-DD2E-2288', '2015-10-15', 15.00, 1310),
  (1573, '162B-4C55-EBC5-9607', '2018-02-17', 8.30, 682),
  (507, '759C-B9DA-83B0-E401', '2021-07-10', 12.38, 22),
  (562, 'BBF3-FFB1-CE0D-4197', '2018-08-26', 5.53, 442),
  (309, 'CCC2-F896-BC36-C926', '1990-04-23', 13.15, 580),
  (1626, '8054-CA93-4431-531D', '2022-11-04', 6.59, 812),
  (1522, '7B38-4B55-7D76-8EF8', '2022-10-20', 19.26, 974),
  (968, '0640-E0E3-F79D-E206', '2020-04-09', 16.09, 722),
  (752, 'FCCE-E5AD-6B8A-0EB2', '2019-12-31', 6.31, 131),
  (753, '9227-9FE7-1C7D-936F', '2023-01-06', 13.32, 132),
  (1236, 'B0B9-C34E-68FD-0FFC', '1996-10-23', 7.93, 140),
  (1408, '2371-7AB9-4BCE-EFF6', '2021-12-31', 17.28, 147),
  (1407, '5A48-1640-C444-75D7', '2017-02-19', 8.67, 147),
  (1542, '51DB-B624-B22F-440E', '2013-08-24', 6.75, 346),
  (901, '80AA-F9CA-5BD3-D105', '2010-11-01', 10.43, 388),
  (1344, '583F-9A17-6B67-6336', '2020-12-16', 13.21, 392),
  (1381, '84B5-4539-8001-FE8A', '2014-07-02', 14.35, 515),
  (635, 'AF4C-24A1-33D8-2C10', '2017-10-29', 19.06, 628),
  (1337, '39D9-044F-AC24-848A', '2016-12-03', 5.12, 707),
  (359, '0BC4-25DF-0217-9B29', '2001-05-09', 18.66, 910),
  (360, '87CB-E4A0-74D7-ED43', '2006-12-19', 7.19, 910),
  (287, '87CA-20DC-A4FC-7586', '2020-02-06', 9.88, 965),
  (1308, 'F7B8-E2A8-7A11-DC79', '2001-01-10', 10.22, 1156),
  (532, '5571-669D-9928-B20C', '2005-04-28', 7.60, 1358),
  (592, '4E96-44A7-5E48-14A8', '2009-07-20', 15.45, 1408),
  (678, '4FB9-1E44-96FD-116C', '2015-11-16', 6.63, 1007),
  (242, '41DC-9C77-817D-5B98', '1987-07-05', 11.91, 908),
  (1692, '97C6-C0D5-75D6-65DA', '2002-08-07', 13.87, 96),
  (491, '17E1-7827-4422-C3FF', '2012-02-04', 9.89, 1310),
  (644, '70B2-36C0-DDC5-DBEC', '2007-10-11', 6.69, 488),
  (430, 'C053-0E46-9DCB-5D46', '2018-10-22', 8.71, 155),
  (743, '4D5B-84F2-F54A-7976', '2021-03-13', 18.42, 23),
  (1636, '6866-F70B-A0B2-8E03', '2021-06-03', 11.27, 14),
  (250, '5D1E-8DBD-845E-F3A6', '2022-12-22', 15.59, 945),
  (144, '7638-CE15-EE4F-4EEB', '2022-01-31', 5.54, 89),
  (1561, 'F7CD-7888-FB3C-2747', '2019-09-20', 10.27, 211),
  (1115, 'BB31-16FE-13D4-B58A', '2020-01-03', 10.41, 255),
  (801, '12F4-F504-53CF-7CA9', '2022-10-07', 7.65, 386),
  (763, 'B515-0C6A-BAE8-44F9', '2014-11-23', 9.43, 472),
  (1224, '0F27-06ED-B9FD-21E7', '2004-02-04', 14.71, 859),
  (320, 'B1EC-697B-D8FB-8AB7', '2016-06-28', 9.72, 298),
  (1166, '57AB-A898-5690-23F9', '2022-08-23', 12.83, 1024),
  (524, 'AC01-4340-1F10-2D80', '2011-12-11', 12.65, 391),
  (467, 'F8D2-B7BB-7899-49E4', '2022-07-22', 13.66, 1),
  (1140, '4886-330F-571A-CC9F', '2022-08-01', 8.25, 1034),
  (349, '757B-EB7A-0F26-324A', '2021-01-10', 8.86, 12),
  (744, '26DC-AEE6-09F7-8D55', '2021-02-27', 16.89, 23),
  (468, 'D06D-D5A0-9978-95A9', '2022-01-27', 17.06, 1),
  (1129, 'DEDE-B2C6-81CA-3761', '2016-03-31', 18.98, 2),
  (1128, '0FE7-8900-E44F-182B', '2017-02-12', 17.31, 2),
  (665, '40FB-7111-BBF4-29E2', '2020-11-20', 5.38, 3),
  (809, 'F823-986B-B978-1634', '2005-08-04', 8.39, 4),
  (339, 'A48F-7AF2-6634-13F1', '2018-01-18', 13.39, 5),
  (503, '52ED-AB40-0D72-9724', '2014-06-16', 14.46, 6),
  (282, '6C68-C48E-79C5-F240', '1996-06-05', 10.80, 7),
  (283, '4480-58AD-5F86-5A08', '1998-07-22', 10.37, 8),
  (284, 'B3AE-E09F-61B7-8374', '2013-09-01', 9.55, 8),
  (1066, '25B6-5406-CD54-2FD8', '2019-09-20', 19.00, 9),
  (1067, '315B-D151-286B-D0A4', '2020-12-05', 13.50, 9),
  (1707, '09D0-DCB3-7765-5BAA', '1998-01-15', 13.62, 10),
  (574, '97AE-F52B-5D63-A669', '2020-07-28', 19.41, 11),
  (573, '5987-54A5-5E7E-1032', '2022-05-22', 13.98, 11),
  (119, '7670-3114-59E2-0536', '2007-07-07', 12.36, 13),
  (1637, '854F-6967-4217-00E1', '2022-04-03', 5.51, 15),
  (1336, '453D-E6FB-2934-8A40', '2021-10-02', 16.50, 16),
  (549, '8F06-4AA3-9D2D-BDB6', '2010-06-21', 7.80, 17),
  (1406, '7767-6D7D-718A-AE9D', '2017-01-20', 6.65, 18),
  (1508, '98D8-C8B0-31F0-F2E6', '2016-09-06', 7.14, 19),
  (1374, 'A832-46A6-CB17-A4A7', '2007-06-10', 13.91, 20),
  (632, '6713-5D3D-3975-6E18', '1996-11-04', 13.12, 21),
  (560, 'B66B-6597-3679-C941', '2022-01-19', 11.43, 24),
  (1536, 'D0D7-C43E-A6AA-CC3A', '2018-08-30', 8.17, 25),
  (1537, '0331-35E1-EBD8-5408', '2008-11-24', 7.97, 25),
  (1430, '3DDA-999C-1926-C926', '2011-05-30', 10.41, 26),
  (1059, 'EDB0-9AB4-C064-EE3C', '2010-02-03', 10.66, 27),
  (1286, '2ADC-49EF-07DB-1B68', '2022-02-10', 19.55, 28),
  (1427, 'DA89-4B6B-05DE-AE06', '1994-04-13', 10.51, 29),
  (189, '3729-1784-2F26-EA5E', '1981-11-02', 19.33, 30),
  (190, 'D4FB-AD2D-8127-1772', '2009-03-31', 10.33, 30),
  (188, '1C91-3E1F-94D3-7106', '1966-06-23', 10.61, 30),
  (502, 'E72F-19FC-ED4C-3477', '2005-12-03', 7.86, 31),
  (1063, '8A0B-A949-DB25-643F', '2013-02-13', 7.66, 32),
  (805, 'A6B1-757F-E43C-90DB', '2018-10-14', 10.35, 33),
  (969, '7087-711C-3BD1-67FC', '2017-08-28', 14.38, 34),
  (1333, 'C753-AB81-0D9C-C30C', '2010-05-13', 18.87, 35),
  (1334, 'E731-ACCA-30BD-AA39', '2018-04-03', 5.15, 35),
  (182, '38A2-1EF6-7163-21F3', '2013-08-31', 5.90, 36),
  (857, '367E-AF36-7908-CD9F', '2020-10-31', 19.11, 37),
  (579, 'EEB3-D137-AF8B-72A2', '2016-01-14', 16.31, 38),
  (408, '92F8-5F5A-CC7E-EAC4', '2011-03-16', 13.51, 39),
  (1260, '4791-F198-A44D-A346', '2011-07-21', 5.47, 40),
  (1259, '38DF-988E-4BDA-519E', '2007-05-11', 19.95, 40),
  (1100, '64BC-39CD-0558-CD08', '2006-11-21', 8.08, 41),
  (1099, '7787-F199-F773-831E', '1999-12-13', 16.66, 41),
  (1216, '5826-522C-D203-0043', '2000-09-25', 17.26, 42),
  (923, '5753-C803-B2A0-182F', '2021-05-08', 13.06, 43),
  (1348, '35D9-2A7E-7EE0-70D5', '2019-03-05', 9.88, 44),
  (1643, 'ACAC-C42C-D483-1EC7', '2006-06-24', 10.66, 45),
  (1642, 'A4CA-04A0-3AF8-5E69', '2004-11-15', 12.10, 45),
  (1641, '9989-1376-53BE-26E8', '2015-11-06', 17.99, 45),
  (1644, '04CE-AC4B-4995-FF88', '2014-05-25', 12.24, 46),
  (618, '23DE-C827-7D7F-2252', '2000-01-13', 16.49, 47),
  (994, '4220-E0B2-B7D7-58DF', '2015-03-22', 6.58, 48),
  (1033, '748B-3827-E48C-E793', '2016-06-18', 17.00, 49),
  (607, 'CD33-467C-413C-043B', '2011-07-05', 6.83, 50),
  (617, '30C5-B47F-209C-BDD9', '2013-10-16', 11.45, 51),
  (875, '1F38-DC9B-6D06-7E62', '2015-06-06', 19.15, 52),
  (544, 'CA42-8D24-FF68-AEA7', '1984-11-13', 13.38, 53),
  (545, '6DBF-9E30-A55A-4F3B', '2010-08-29', 5.18, 54),
  (1208, '5EBE-8E17-7DF0-00B0', '2020-11-25', 13.06, 55),
  (1209, '9414-90D6-1948-E24D', '2021-06-22', 13.09, 56),
  (1470, '21D3-5E1B-BEDA-19E1', '1978-10-17', 17.83, 86),
  (516, '8E8E-D677-69DC-0A16', '2007-04-13', 12.21, 57),
  (311, '8196-E09C-4380-2582', '1985-05-13', 9.85, 58),
  (613, '3373-6505-69C1-A141', '1992-12-25', 6.29, 59),
  (614, '9708-E39B-8014-D7BA', '2007-03-30', 10.14, 60),
  (1293, '2FF0-538A-5E01-D5C5', '2021-11-03', 5.04, 61),
  (895, '56BB-52E3-E8ED-050F', '2013-12-04', 9.91, 62),
  (469, 'A3B1-063D-B008-417C', '2017-06-19', 8.45, 63),
  (973, '0A08-459C-B582-CE8C', '1998-07-22', 9.84, 64),
  (890, '22CA-A27B-D973-E0B8', '2019-08-30', 7.53, 65),
  (50, '55EE-DB79-99C6-16A8', '2017-09-23', 5.03, 66),
  (113, '92CC-5A7A-1334-396B', '2010-01-04', 14.45, 67),
  (1268, '3936-E981-1BC1-3499', '2010-07-31', 5.63, 68),
  (504, 'BE8B-3048-4D71-20D5', '2011-08-18', 18.75, 69),
  (505, '52FF-4D36-AD6B-9709', '2017-03-15', 7.71, 70),
  (819, '8979-49AC-75F0-A31E', '2018-07-11', 15.41, 71),
  (1650, '0C3C-9587-A7D0-E0ED', '2018-02-19', 13.62, 72),
  (476, '9B41-12C7-99AF-EC6A', '2004-08-09', 10.59, 73),
  (477, '8B47-AE15-C978-51F3', '2003-10-31', 18.78, 73),
  (357, '87F0-EBD0-848B-9967', '2021-03-20', 19.48, 74),
  (322, 'BF65-9EED-1B98-AE1D', '2020-10-26', 15.69, 75),
  (133, '36E9-6A13-B6E5-E74A', '2022-08-17', 16.26, 76),
  (134, 'F39C-6818-B4C1-5627', '2021-04-11', 14.40, 76),
  (35, '4A81-0EC2-5DFD-75C8', '1995-12-13', 5.80, 77),
  (163, '1685-7AFA-82A2-93DC', '2022-01-27', 16.62, 78),
  (164, '3962-F3D0-8485-053C', '2011-06-14', 18.86, 79),
  (1666, 'D85A-193E-7BAB-9A19', '2015-11-18', 5.07, 80),
  (301, '8FC7-FB16-D6B3-222C', '2015-02-09', 12.97, 81),
  (1338, '4A23-56CC-D405-82F7', '2007-06-18', 15.97, 82),
  (594, 'B96E-CA85-777E-0C7B', '2016-10-22', 6.68, 83),
  (1455, '63C1-B55A-327B-10FC', '1992-01-22', 17.60, 84),
  (1454, '0F66-EF01-2198-46D8', '1974-11-19', 10.23, 84),
  (702, '7EC8-53E7-C1B5-82B7', '1999-03-05', 7.92, 85),
  (146, '2823-7F82-DBE3-E2AC', '2015-06-09', 19.35, 87),
  (148, '51AF-CEA7-07E5-9BF9', '2016-02-08', 12.31, 88),
  (147, '894F-F143-09C2-2DB9', '2020-10-09', 10.49, 88),
  (554, '0557-B587-A832-3BB9', '2010-11-10', 12.75, 90),
  (555, 'FFC6-8D77-2561-8042', '2022-05-15', 19.28, 90),
  (302, '4D05-F342-D350-3A24', '2017-07-27', 14.80, 91),
  (137, 'A738-DE1B-69B1-F199', '2022-08-31', 16.40, 92),
  (138, '1B5B-E2D0-4C6A-45F7', '2019-01-16', 8.21, 92),
  (139, '73E2-BD55-1628-685B', '2012-03-03', 19.53, 92),
  (1221, '620B-901F-4339-F754', '2016-09-27', 10.25, 93),
  (304, '68D7-442D-3110-403E', '2022-11-30', 11.82, 94),
  (1110, '3C60-EB8E-C41E-66AF', '2020-10-23', 19.43, 95),
  (576, '4523-B790-826D-9080', '2011-03-01', 17.63, 97),
  (1677, 'D74B-0A94-EBF2-29D1', '2015-08-21', 7.17, 98),
  (568, 'BF69-FF27-5588-3301', '2021-11-08', 6.76, 99),
  (569, 'B719-A5E2-789E-EF95', '2021-12-21', 15.30, 99),
  (1691, 'BAAE-60CF-70F2-A91F', '2021-08-09', 5.70, 100),
  (200, '7194-E49E-DEBE-8BFD', '2018-08-28', 12.71, 101),
  (802, '1277-7DBC-36D7-DF37', '2014-01-31', 6.94, 102),
  (44, '16AE-C720-BC15-417C', '2010-12-28', 10.10, 103),
  (45, 'D4A6-8BC3-76D8-27E9', '2021-09-02', 8.31, 104),
  (1634, 'AFB6-7A8C-F5A9-115A', '2020-07-20', 19.77, 105),
  (1119, 'BB90-D8E3-9413-B4A4', '1995-11-26', 17.23, 106),
  (1153, 'DFDC-2752-F265-777E', '2008-06-25', 17.80, 107),
  (223, '52F4-9BDF-C65B-9C2B', '2007-07-23', 7.95, 108),
  (1320, '01C5-65AD-EFFA-E12E', '2019-12-25', 14.44, 109),
  (286, 'F4FD-4662-0665-8055', '2021-06-25', 16.73, 110),
  (1, '2989-356B-A9F0-91F6', '2015-12-25', 5.27, 111),
  (546, '346E-B71B-A973-39A3', '2020-10-10', 12.30, 112),
  (1006, 'AF27-46CB-C7AE-59D5', '2022-08-16', 15.95, 113),
  (1007, '5A37-62D5-BC8C-EC35', '2021-03-22', 16.82, 113),
  (1524, 'EBDC-C814-ABB9-C83B', '2021-01-10', 14.86, 114),
  (1525, 'B7C5-9144-8EAD-B722', '2018-10-16', 18.14, 115),
  (1526, '37DC-E9DB-8EE2-8E11', '2023-01-05', 8.95, 115),
  (898, 'A096-0777-C225-F4F4', '2014-03-08', 18.99, 116),
  (20, '817A-B93E-03E8-D9BE', '2013-03-03', 8.33, 117),
  (933, '43C0-CD40-3811-4B3C', '2006-07-29', 7.75, 118),
  (98, 'D98E-971E-1A56-5579', '2012-06-14', 11.59, 119),
  (99, 'E83D-D1B8-628F-9ADA', '1998-08-07', 19.58, 119),
  (100, '2952-EEEF-95B4-FC31', '1998-09-11', 8.92, 120),
  (825, 'BA9C-C49A-932A-DF6A', '2007-12-23', 16.64, 121),
  (1384, '3744-6F33-8CAA-C060', '2015-07-30', 6.43, 122),
  (1385, 'D216-5AE7-E582-9791', '2019-07-12', 18.89, 123),
  (391, 'F532-05D1-060D-4566', '2020-01-29', 13.80, 124),
  (48, 'C8DC-3511-4D49-47C5', '2001-07-25', 18.25, 125),
  (1230, 'C0DD-CF47-C677-EED0', '2022-06-26', 18.96, 126),
  (290, '2E9A-7A8E-769F-068D', '2022-08-19', 10.39, 127),
  (1420, 'F682-E61F-599A-9414', '2021-09-07', 17.52, 128),
  (1550, '5572-6C10-F0D3-16E1', '2014-09-28', 14.12, 129),
  (629, '9880-CFA9-4474-BBCB', '2008-10-25', 8.35, 130),
  (751, '73C8-0743-747C-AA0E', '2022-06-08', 16.25, 131),
  (672, '5796-3DB1-68A2-8543', '2002-03-02', 18.62, 133),
  (1615, 'CC39-7F37-8F5F-33CB', '2018-11-02', 12.86, 134),
  (1614, '7831-D033-84CE-DFFB', '2013-12-22', 7.55, 134),
  (1613, '25B4-33FE-62D0-662C', '2019-06-06', 10.49, 134),
  (216, 'A570-2586-523F-4253', '2021-07-24', 19.16, 135),
  (1453, 'F48A-2904-C97E-B5DB', '2022-09-14', 11.34, 136),
  (327, '0738-C0CE-D68A-7B61', '2021-04-17', 8.10, 137),
  (995, '2C99-871F-C858-E8E7', '2021-08-02', 8.71, 138),
  (1435, '63BA-8DEB-9AA5-8E27', '2018-10-11', 16.37, 139),
  (1404, 'EC52-0332-D215-3EF0', '2020-05-09', 18.06, 141),
  (1405, '3904-89BE-0AEB-798C', '2018-08-12', 9.35, 141),
  (181, '5098-527D-49A1-18E9', '1995-08-11', 12.57, 142),
  (470, 'C71B-B843-54EC-DD0C', '2022-07-28', 9.27, 143),
  (688, '4962-F9E7-C1D8-B7CF', '2021-01-01', 17.99, 144),
  (523, '150D-AF6C-EFAF-1487', '2018-07-06', 10.63, 145),
  (1417, '4AEF-3324-4701-CEED', '2015-03-16', 14.08, 146),
  (1088, 'F1B7-6B06-0920-D1FD', '2018-03-15', 15.12, 148),
  (807, 'C736-BA2C-910F-0067', '2016-03-24', 16.54, 149),
  (808, 'BD02-428E-429C-B002', '2022-11-11', 16.54, 150),
  (1018, '2D4F-D1F1-012C-FD9A', '2003-09-14', 7.37, 151),
  (1648, '59DD-ECA8-A833-8C2C', '2020-10-20', 18.47, 152),
  (1649, 'CCC1-56E0-12EE-5EBA', '2009-10-24', 14.85, 152),
  (1304, 'B0A9-ED5C-58D0-D4D4', '2021-04-07', 9.43, 153),
  (1447, '6F8B-E612-86C8-21CD', '2018-05-24', 11.32, 154),
  (429, '0ACC-70A2-04CD-3643', '2020-04-01', 8.15, 155),
  (428, 'AF31-D97B-E9D2-DC03', '2011-07-07', 9.18, 155),
  (136, '66DE-41D4-7C06-A90C', '2008-01-19', 9.77, 156),
  (932, 'B4D2-A93B-71A2-C075', '2010-01-30', 8.51, 157),
  (1507, '1EDA-5F20-A036-D8DC', '2001-09-17', 11.68, 158),
  (986, 'A101-5067-5583-13E1', '2011-10-13', 17.51, 159),
  (1114, '45AE-FC36-9C0A-D54C', '2013-07-17', 7.80, 160),
  (1323, 'FB41-2D7F-36EC-9494', '2020-04-18', 7.37, 161),
  (1324, '37BA-985E-6840-4E67', '2021-09-04', 6.52, 161),
  (1700, '8F51-8598-BC1D-2AE8', '2013-11-20', 5.11, 162),
  (1701, '48B7-8AA5-D270-4F86', '2015-08-13', 6.76, 162),
  (1411, 'F7B8-D580-1A55-B616', '2001-01-19', 7.25, 163),
  (1163, 'C8CB-EB55-A5D3-C244', '2003-07-04', 5.10, 164),
  (862, '0904-ACDE-9478-18AD', '2009-03-10', 9.10, 165),
  (522, 'E973-D02C-3DD9-D707', '2001-06-15', 15.98, 166),
  (140, '401B-1493-472C-094F', '2011-12-18', 7.79, 167),
  (1291, '3DC8-191D-E70B-A8DE', '2018-05-14', 12.29, 168),
  (1647, '95C8-64F7-B9B2-9966', '2017-05-14', 7.38, 169),
  (519, 'C317-41CC-FBD6-4197', '2013-10-18', 15.71, 170),
  (192, '07F2-38FA-8827-5828', '2017-05-11', 7.99, 171),
  (999, 'C746-4071-94A4-E5C2', '2016-06-15', 14.48, 172),
  (186, '7012-C120-9B74-8E36', '2009-09-18', 15.12, 173),
  (1232, '7D22-BFE1-165B-1162', '2015-12-08', 14.86, 174),
  (649, '072E-DA19-5637-A3EB', '2011-03-11', 12.10, 175),
  (73, 'FE5F-6B46-1065-B587', '2010-04-18', 17.69, 176),
  (514, 'DC3D-0A3A-8F56-E764', '2020-05-22', 7.10, 177),
  (914, 'F4FB-C4D5-D626-9F86', '2022-07-12', 9.82, 178),
  (755, '6EA2-D3FB-2155-EEEC', '2021-06-01', 9.41, 179),
  (361, 'BCCE-AE25-4CA8-B007', '2015-12-17', 11.11, 180),
  (865, 'F789-0DD5-3DE8-F358', '2016-07-12', 19.80, 181),
  (909, 'CBB5-D6E2-BCC9-31F0', '2020-07-10', 16.35, 182),
  (612, 'CB31-477F-C784-654A', '2021-01-29', 8.27, 183),
  (1278, '4B76-A3E9-91E3-2383', '2007-11-07', 8.96, 184),
  (960, '6446-70DD-5AAC-B4B8', '2019-01-05', 5.97, 185),
  (959, '6578-4AAC-4E6C-F025', '2009-05-28', 15.94, 185),
  (961, '46AA-66F9-9FCE-EEE4', '2020-11-29', 10.99, 186),
  (178, '37A9-0DC5-6830-84D4', '2021-10-27', 12.65, 187),
  (1535, '5E66-AAE7-116C-9108', '2008-03-12', 8.20, 189),
  (1538, 'AF5F-323D-1741-898F', '2014-11-05', 15.47, 190),
  (334, '4469-DB70-587D-4CCA', '2019-06-11', 7.71, 191),
  (335, 'C99C-CBFB-756E-4AC7', '2022-08-28', 14.29, 192),
  (883, '5B73-215C-30BC-EA87', '2012-11-09', 7.64, 193),
  (884, '634B-12D0-549F-E9E5', '2015-12-31', 8.12, 194),
  (1315, '154D-1C38-205D-726D', '2021-01-03', 7.95, 195),
  (850, '86C5-F301-62A0-19BC', '1989-02-25', 10.79, 196),
  (849, '4EB4-0463-4AE8-562D', '2017-06-28', 18.95, 196),
  (1563, '42F0-B0A5-2102-D690', '2021-08-30', 10.20, 197),
  (766, '9E96-3AA7-EF86-2E3A', '2003-09-24', 7.15, 198),
  (749, '837E-AAEE-7505-34BE', '2012-06-06', 7.76, 199),
  (253, 'B017-81B9-B976-573B', '2020-08-27', 15.66, 200),
  (1418, '8E2F-7BA8-BF2D-65F1', '2018-03-24', 11.95, 201),
  (976, '3C94-0284-5785-34D6', '2005-09-02', 19.99, 202),
  (1084, '3C32-855C-F60C-A20A', '2018-05-03', 8.78, 203),
  (936, 'EC94-6628-66D5-01B0', '2006-05-29', 9.81, 204),
  (726, 'C88B-0EAF-6A70-B426', '2015-03-26', 13.79, 205),
  (1409, 'F4DB-A260-BC37-E0D4', '2019-01-28', 6.02, 206),
  (846, '9F9A-FFE6-3755-CBAC', '2014-04-29', 10.06, 207),
  (1466, '665C-9071-3A07-D835', '2010-07-21', 8.41, 208),
  (1159, 'EE3A-D6B4-4252-E2C1', '2005-03-09', 7.46, 209),
  (1160, '1896-22B5-0EED-F131', '2005-04-18', 11.89, 210),
  (781, '69B0-ADD3-10D6-C3D1', '2010-06-09', 19.58, 212),
  (1178, '1DAC-7E6B-0C0E-A242', '2015-11-22', 5.24, 213),
  (1696, '66F6-47D0-5C80-33DE', '1997-09-10', 17.52, 214),
  (1695, '6F93-D1FC-0131-A66D', '2022-09-02', 17.75, 214),
  (1694, 'EB1F-FC3C-048E-D60F', '2005-07-24', 18.09, 214),
  (1697, 'C12A-EC73-5769-65CE', '1992-01-03', 17.20, 215),
  (1583, '153F-4C6C-511F-470B', '2014-10-12', 18.33, 216),
  (353, '8B38-4D8B-6C59-E61B', '2022-06-21', 14.39, 217),
  (1346, '986C-C66F-3DE4-22DB', '2014-07-25', 17.42, 218),
  (71, '4E95-A60C-DE16-72B3', '2021-09-29', 8.82, 219),
  (863, 'D81D-EF4C-4391-DC02', '2022-07-13', 15.85, 220),
  (438, 'A265-F3BC-65AA-2E8D', '2021-03-24', 14.21, 221),
  (1179, 'D1CC-03C1-5191-4DC7', '2022-06-14', 9.41, 222),
  (984, '845C-2F66-AC3C-C87D', '2020-02-22', 13.29, 223),
  (854, '995A-5483-96F1-E717', '2019-10-22', 13.04, 224),
  (1112, '3E74-6D70-81A1-D537', '1990-12-29', 13.51, 225),
  (518, '7A76-05F5-170C-3C1B', '2019-01-21', 16.69, 226),
  (1574, '86EE-0746-A494-852D', '2017-06-07', 11.23, 227),
  (947, 'FA20-E84E-F296-3A95', '2018-01-17', 5.71, 228),
  (948, 'A9F6-D825-7F05-138D', '2018-11-02', 11.62, 229),
  (1503, 'D921-3A6C-6546-52D4', '2014-04-11', 10.45, 230),
  (1275, '4E98-90B0-ED4A-96A0', '2009-09-26', 8.83, 231),
  (896, '9562-9E9C-A091-90B5', '2021-01-13', 19.28, 232),
  (1572, 'C689-C276-BA3A-A6FD', '2011-06-27', 16.23, 233),
  (145, 'D5B9-FE74-7769-2471', '2001-03-16', 10.52, 234),
  (1227, '7FE8-53BC-9FB4-14CC', '1995-10-12', 12.64, 235),
  (1250, 'C353-96E5-A145-3211', '2017-05-01', 9.99, 236),
  (731, 'B470-D4B5-C7BB-DF91', '2006-07-26', 10.15, 237),
  (648, 'E043-7B99-F9BC-9EC8', '2007-06-14', 12.61, 238),
  (67, 'C0E0-97CF-8023-F5A7', '2012-06-28', 9.74, 239),
  (66, '31F0-6B9A-4130-7EDC', '2020-11-20', 16.56, 239),
  (586, '7128-86D4-766F-4BC5', '2019-02-06', 5.87, 240),
  (587, 'E652-16F6-C19F-C6CE', '2018-04-06', 18.73, 240),
  (1212, 'BF56-0B89-AEC0-CCBC', '2020-10-27', 15.30, 241),
  (1624, '3290-1EC4-ECEB-C4C1', '2012-09-17', 14.11, 242),
  (565, '27DA-B771-95F4-52F7', '2021-04-16', 5.72, 243),
  (564, '9FF7-2A22-7105-52B7', '2017-05-08', 15.67, 243),
  (566, '6400-08F4-326A-8978', '2020-03-04', 19.63, 244),
  (1473, '1C19-E70E-9225-C164', '2009-05-02', 9.68, 245),
  (814, '816B-B753-8B0B-3486', '2019-11-11', 10.52, 246),
  (815, '169C-1A4F-A1CB-3BB5', '2018-11-05', 8.33, 247),
  (457, 'A791-BFCD-E176-5D1F', '2008-08-06', 12.36, 248),
  (494, 'DBC1-49C6-4D4B-F937', '2007-05-18', 15.27, 249),
  (493, '797D-B0D6-6414-BB40', '2016-09-05', 19.10, 249),
  (234, '25E9-3405-E853-BCF7', '2019-07-17', 9.25, 250),
  (235, '0C9E-209B-F1FF-BD84', '2010-01-13', 13.94, 250),
  (1682, 'B201-57D4-53EF-B465', '2019-10-16', 11.81, 251),
  (1683, '16CF-581F-DD05-3764', '2019-05-11', 14.38, 251),
  (1137, '9FEE-8A73-3C16-1E92', '2022-08-08', 11.73, 252),
  (904, 'FF23-043A-632C-D57B', '2000-07-17', 16.14, 253),
  (197, '09E5-C16C-A7F6-4B80', '2008-02-19', 19.18, 254),
  (1632, '5BEF-C7B2-F9AE-E77D', '2017-03-10', 18.36, 256),
  (142, '8A23-8354-C088-D32B', '2021-04-06', 11.57, 257),
  (1456, 'C6FA-2C7E-5DC4-9F1A', '2004-04-08', 18.81, 282),
  (141, '9EDC-883A-1FFF-D3C6', '2020-07-17', 13.84, 257),
  (143, '012F-1B8E-03FF-AC44', '2021-06-30', 16.07, 258),
  (450, 'F5AC-0D9A-A965-886F', '2016-03-13', 5.38, 259),
  (543, 'FDD1-C74B-0D3D-8415', '2016-04-14', 11.03, 260),
  (864, 'C686-EE01-2D1A-41AF', '2014-08-28', 13.05, 261),
  (1393, '0C06-EF14-6FE5-F0C5', '2019-03-15', 15.12, 262),
  (660, 'A90F-3B73-B2FF-841C', '2015-07-28', 7.09, 263),
  (659, 'AD5B-26F5-02D5-DDB9', '2007-02-16', 6.41, 263),
  (1043, 'B9F4-F96A-4CE7-5E6D', '2017-03-24', 15.12, 264),
  (897, '15BA-22BF-9C77-571A', '2015-12-08', 17.80, 265),
  (671, '6496-6985-D729-A302', '2020-01-27', 10.28, 266),
  (670, 'EEFC-F038-9565-94AA', '2019-09-18', 8.24, 266),
  (1431, 'B632-E3E4-8F3D-1988', '2021-11-04', 12.57, 267),
  (1180, '4165-798E-6B9D-A82B', '2010-11-18', 9.48, 268),
  (813, 'B5D8-CD4A-FFEC-16D8', '2008-03-10', 14.29, 269),
  (85, '7167-8C09-B621-71E1', '1994-04-18', 7.44, 270),
  (269, '0F67-43B5-B5BB-328B', '2020-05-01', 13.55, 271),
  (54, '7839-6375-8632-27E8', '2008-03-15', 10.29, 272),
  (1042, '5257-5E90-9372-3551', '2006-11-30', 10.62, 273),
  (1258, '3511-2719-A73E-E6A1', '2020-07-09', 16.79, 274),
  (291, 'AD76-6823-0D13-B55F', '2018-07-16', 7.84, 275),
  (728, 'C4B1-3D45-8E0D-F281', '2019-08-19', 10.43, 276),
  (1571, '19C2-E610-95A7-8CCA', '1994-01-19', 5.42, 277),
  (1684, '1EBC-51E0-F31A-1568', '2006-07-24', 7.75, 278),
  (1126, 'B2B5-1136-EE54-D661', '2018-08-10', 12.76, 279),
  (378, '780B-4A07-17A3-DCFA', '2015-05-06', 12.97, 280),
  (376, 'E5D0-8C79-917B-B377', '2020-10-05', 10.05, 280),
  (377, 'EF8D-F966-9C7C-FFAC', '2019-02-02', 7.99, 280),
  (1127, '54E2-AAD1-8C74-A4CB', '2017-12-11', 9.23, 281),
  (86, '77A1-34F9-4DF9-4708', '2020-09-16', 16.13, 283),
  (87, 'CFEB-4409-B46C-EA97', '2021-03-08', 6.51, 284),
  (732, 'E23B-CA9B-F16D-C00F', '2011-07-13', 7.62, 285),
  (739, '7D48-E5C4-BFA5-06FD', '2020-12-28', 7.85, 286),
  (700, '87CC-CB39-3F34-0A9E', '2012-05-23', 16.88, 287),
  (699, 'C0C6-0997-60AB-6A9A', '2009-06-16', 8.29, 287),
  (637, 'EC1D-0D39-4715-E668', '2018-02-04', 7.75, 288),
  (150, 'F621-3D6E-1687-FDC8', '1999-01-22', 5.17, 289),
  (149, '4601-CA7F-F18C-252F', '2018-10-07', 7.94, 289),
  (848, 'D76C-0C20-96C1-A922', '2007-12-23', 10.52, 290),
  (486, 'EF8F-5AAD-2D35-3D59', '1986-10-26', 14.65, 291),
  (1091, 'EA1C-09EC-4E11-81E1', '2020-02-13', 11.47, 292),
  (1092, '3D6F-F53C-DDF1-C32D', '2007-01-09', 12.37, 293),
  (300, '72C8-0C05-6C28-221B', '2018-03-27', 14.35, 294),
  (1302, '96AE-038A-BCA1-D5EF', '2018-02-06', 9.46, 295),
  (1303, 'A995-BFF4-92C5-479B', '2020-03-19', 16.73, 296),
  (158, '9E1F-5E40-7E72-449F', '2010-01-02', 17.78, 297),
  (567, '3AE6-1DBE-41D1-7ACF', '2020-07-03', 8.12, 299),
  (531, 'D281-E8AF-2A1D-20D6', '2021-08-25', 11.39, 300),
  (81, '40D2-4E4D-B3B2-47DA', '2021-02-27', 16.43, 301),
  (80, '2077-939D-5316-A6D4', '2015-05-18', 16.80, 301),
  (481, '3DF2-027B-B280-3668', '2011-12-14', 8.83, 302),
  (1062, '71C6-9F7B-320E-02BB', '2010-07-14', 13.82, 303),
  (416, '2F62-BD30-E1EE-0E9F', '2017-01-14', 18.06, 304),
  (77, '0CDA-708A-71BA-09AA', '2010-10-12', 5.45, 305),
  (1107, '5A18-291B-D735-E699', '2022-09-22', 14.74, 306),
  (1203, '5B78-7F5E-20D1-925B', '2013-03-09', 15.19, 307),
  (571, 'C79B-09B2-3DF9-B571', '2022-11-09', 8.16, 308),
  (570, '2C4A-4D0E-B311-6A8D', '2022-07-31', 5.04, 308),
  (1218, '8B0C-5034-F77A-0AB5', '2017-07-19', 9.49, 309),
  (615, 'F8CF-57B6-3A87-391F', '2022-01-06', 12.31, 310),
  (1294, '7244-E1B3-A71A-004D', '2021-07-02', 10.60, 311),
  (922, 'EDB7-A2F3-D624-A570', '2020-10-27', 10.52, 312),
  (1279, '4D9F-3B2E-A629-DD1F', '1993-06-25', 17.17, 313),
  (65, 'CA74-19F6-53F4-0FA5', '2019-01-23', 15.96, 314),
  (64, '70EB-0A0C-35B2-38A8', '2022-01-26', 8.33, 314),
  (1529, 'A8C1-41CB-930A-7101', '2018-08-14', 10.84, 315),
  (1487, '1F7B-FC81-CBF9-85B6', '2014-04-17', 13.48, 316),
  (1496, 'E46F-01E1-D74A-4FF6', '2009-05-13', 16.29, 317),
  (196, '6C81-49A7-21B7-B0C4', '1999-12-19', 19.06, 318),
  (194, '6B55-73EC-722C-42E9', '2005-01-10', 18.04, 318),
  (195, '93F5-E747-8B89-C166', '2017-09-27', 13.76, 318),
  (1223, '95D8-53B6-906A-C5C4', '2014-01-19', 5.21, 319),
  (1090, 'B51B-99FF-322D-7150', '2008-11-10', 13.58, 320),
  (622, 'DDD7-9402-C6AC-DFEA', '2012-02-15', 16.32, 321),
  (584, '3489-4853-8BC0-ABBD', '2011-06-08', 7.01, 322),
  (344, 'B3A3-89BB-3C70-C2E1', '1997-07-28', 13.31, 323),
  (346, '9DBF-FFC1-C6F9-399F', '2015-05-17', 17.89, 323),
  (345, 'E075-1C93-9C19-031D', '2021-12-10', 15.50, 323),
  (184, '37C4-398A-CFBF-2323', '2016-02-27', 14.69, 324),
  (719, '2D53-20EE-64EC-A2D3', '2022-06-04', 15.30, 325),
  (718, 'B609-DBEA-8751-4A16', '2015-12-21', 5.92, 325),
  (657, '4C1E-CD42-3CE9-B75A', '2008-10-27', 5.34, 326),
  (1474, 'D45C-E1BD-C0EA-7B38', '2012-01-10', 10.04, 327),
  (1475, 'E57C-74DC-1C81-03BA', '2013-12-28', 11.60, 328),
  (1476, 'D436-C346-E313-63A9', '2021-09-04', 17.47, 328),
  (651, '0784-51A7-7F27-FB48', '2016-09-02', 10.84, 329),
  (650, 'E9D4-B0EF-5080-ECF0', '2016-02-20', 19.00, 329),
  (255, 'A017-C393-B640-02CB', '2008-03-18', 12.69, 330),
  (997, '3D52-8F0E-FAE2-197B', '2016-10-28', 11.46, 331),
  (998, '922E-138C-3C24-444F', '2012-12-14', 11.66, 332),
  (866, 'D425-9244-2210-D0AC', '2008-07-31', 17.55, 333),
  (83, '22D0-370A-6600-F0C7', '2021-05-02', 5.79, 421),
  (1234, '381F-94BC-CD53-A6C6', '2021-03-16', 5.12, 334),
  (633, '4801-E4BF-7CD7-C414', '2003-07-29', 11.04, 335),
  (1440, 'EDFF-875A-2066-53F4', '2017-05-23', 17.26, 336),
  (1441, 'B586-F7DE-E1BC-6901', '2022-01-20', 11.89, 337),
  (447, 'A423-B2FC-1B4A-0152', '1995-12-06', 16.11, 338),
  (924, '97CC-58D7-D1C0-DF90', '1996-03-18', 7.96, 339),
  (1377, '2320-93D1-8EBE-435B', '2020-10-19', 12.87, 340),
  (1480, '8ACB-7689-5459-604E', '2006-07-27', 8.54, 341),
  (1464, '9433-F222-0E5A-9BFE', '2020-07-08', 14.99, 342),
  (415, '5B47-2619-7248-09D7', '2015-07-10', 6.05, 343),
  (552, '83A3-F692-8126-37F8', '2014-04-10', 17.92, 344),
  (553, '785D-C883-8454-7298', '2022-03-04', 18.47, 345),
  (241, '0049-AEBC-5D4C-4EA5', '2011-11-09', 17.42, 347),
  (536, '28BA-EC68-AF62-156D', '2018-05-25', 17.48, 348),
  (537, '5ADD-C0A8-73B8-DD47', '2021-12-17', 14.85, 348),
  (539, '1BC9-4018-5242-227C', '2022-11-09', 5.72, 349),
  (1485, '6B7A-CD95-A692-261D', '2011-02-18', 6.60, 350),
  (7, '10D9-FDDF-EAF0-2DD6', '2007-05-13', 11.52, 352),
  (6, 'B9C0-E39E-1BBB-1035', '2011-07-03', 18.75, 352),
  (8, '4E2F-362E-BAC9-E0D3', '2017-09-30', 15.71, 353),
  (733, '935C-9D2C-1A76-3853', '1995-02-23', 6.06, 354),
  (401, '95AB-AA44-8016-90A9', '2020-05-11', 19.22, 355),
  (400, 'FBB3-4C02-0BB0-F973', '2011-11-18', 10.38, 355),
  (1363, '01D8-C02C-823D-B4B2', '2019-09-29', 11.86, 356),
  (25, '3A24-456C-977A-C7B7', '2021-04-02', 10.75, 357),
  (511, '2A1D-ED57-2543-377E', '2005-07-14', 11.64, 358),
  (512, '0F98-74F4-855B-2620', '2016-04-05', 19.67, 359),
  (1073, '7ED4-B5C8-CC36-46D2', '2018-02-19', 9.99, 360),
  (1012, 'A831-927A-F4F1-8935', '2017-03-13', 10.93, 361),
  (619, '0068-427E-F357-CF41', '2017-12-02', 9.15, 362),
  (620, 'D284-1316-B233-4930', '2011-06-01', 12.87, 362),
  (621, '1F20-20B2-60DD-0C07', '2020-08-20', 17.67, 363),
  (337, 'E7C2-5EBE-B7B3-A216', '2015-03-18', 9.25, 364),
  (1355, '3BD1-AA41-2E06-6E29', '2014-07-14', 12.90, 365),
  (840, '92A6-1E84-699B-AE4F', '2019-08-22', 10.00, 366),
  (10, '649B-0D31-7EAD-2F86', '2011-02-22', 9.49, 367),
  (9, 'FF6C-10C7-6BE7-9DD6', '2021-05-01', 16.51, 367),
  (1708, '48DA-CE9B-E1C7-B3BF', '2015-12-03', 10.52, 368),
  (1710, 'B3EA-191A-AE80-D5E4', '2020-12-11', 13.17, 369),
  (1709, '3AEA-409E-6A84-1B91', '2020-06-18', 19.94, 369),
  (1181, 'F635-10C7-E1F9-7D2A', '2000-08-19', 5.48, 370),
  (681, 'CEC6-8043-FC18-6DEB', '2015-01-18', 12.13, 371),
  (680, 'C522-AF3C-D7B9-3C78', '2009-08-31', 6.18, 371),
  (697, 'B2F7-8ECB-B4F3-B05D', '2021-07-26', 7.01, 372),
  (102, 'FB9D-322A-495F-AE06', '1999-04-04', 15.12, 373),
  (16, 'CBEA-96A3-92A3-FFCE', '2016-05-29', 19.01, 374),
  (642, 'DCBC-F009-04CA-199B', '2012-02-01', 9.87, 375),
  (643, 'AE58-98FA-AFF2-48A7', '2022-02-01', 14.22, 376),
  (1436, '8F9F-B635-8103-92A5', '2012-04-06', 12.86, 377),
  (1102, 'A2BC-45B2-8BBD-C631', '2013-05-22', 5.18, 378),
  (1516, '40C6-4567-20AC-8F35', '2010-06-06', 5.99, 379),
  (1679, 'E974-6B6A-407F-A58F', '1979-02-21', 18.30, 380),
  (500, '1E3C-D378-2856-939E', '2015-10-20', 6.33, 381),
  (501, 'DF1B-7336-A807-084B', '2013-12-30', 18.16, 382),
  (436, '9613-86ED-64C8-8A4C', '2016-05-03', 18.77, 383),
  (853, '356F-AB26-1A62-8FA0', '2011-02-12', 5.35, 384),
  (852, '3ED2-5B03-6AF4-608E', '2005-03-02', 13.64, 384),
  (851, '1DCF-BC55-2706-05DF', '2009-07-13', 18.18, 384),
  (1174, '8892-0A55-209B-3A85', '1997-05-21', 12.02, 385),
  (800, '5F69-E997-2ADF-6934', '2022-03-06', 16.11, 386),
  (1398, 'A315-67F8-81BE-1DA1', '2011-06-30', 17.69, 387),
  (590, '47E6-9270-DAA8-8A07', '2004-04-14', 18.45, 389),
  (1544, '3CE0-4352-0373-8B3C', '2019-12-29', 10.80, 390),
  (1482, '5808-C577-7A8C-CE14', '2010-04-29', 14.62, 393),
  (1198, '9619-A196-7781-7380', '2021-08-12', 14.22, 394),
  (1109, '2246-27C2-3B90-B49B', '2013-01-14', 5.26, 395),
  (1347, 'F0B1-C9CA-9046-E2C4', '2018-04-14', 8.42, 396),
  (237, 'D5AF-31E2-F95B-1189', '2020-08-16', 6.15, 397),
  (770, '2070-4C4D-689A-03F2', '2011-11-06', 18.64, 398),
  (638, 'F8EC-FD80-6070-C185', '2013-01-24', 8.05, 399),
  (596, 'D927-B220-1EB0-DA37', '2018-05-14', 9.10, 400),
  (1570, '205C-1B9C-8206-FE0B', '2013-03-01', 14.43, 401),
  (1271, 'DEEF-4884-DEA8-CAE8', '2007-04-13', 19.81, 402),
  (1688, '715E-5BFC-3AA3-A9E1', '2015-06-18', 10.47, 403),
  (106, '3AD3-0020-84B3-66B6', '2020-11-22', 10.29, 404),
  (171, '57E1-7CD2-9ABB-3F05', '2019-01-03', 11.27, 405),
  (653, '5E07-393F-36C8-3B10', '2013-08-05', 19.89, 406),
  (652, 'BCE8-C13A-E673-1CF0', '2014-05-31', 14.33, 406),
  (654, 'AC50-DAD9-BC58-2C25', '2019-01-06', 5.62, 406),
  (1283, 'A39F-ED9B-EB2D-0010', '1986-04-20', 5.96, 407),
  (1284, '661A-8F78-140C-98EC', '2016-09-06', 15.46, 407),
  (232, '0FA9-B24F-E8D2-4A81', '2021-07-24', 7.78, 408),
  (233, 'A3DB-C5C6-F135-E1FA', '2020-03-24', 10.82, 408),
  (784, '9E14-C398-22E4-1AEE', '2013-04-01', 8.96, 409),
  (1305, '9C17-F35A-0D86-DCC6', '2017-10-25', 11.16, 410),
  (1331, 'F952-859D-EEE3-29CD', '2012-03-08', 10.72, 411),
  (135, '8F34-9F6D-787D-C255', '2014-10-16', 18.48, 412),
  (157, '5052-1C19-8EFB-BE68', '2009-01-30', 6.19, 413),
  (1446, 'B934-51C7-09ED-5438', '2017-03-13', 6.61, 414),
  (78, '73E3-7AA8-9028-7045', '2014-06-22', 17.55, 415),
  (882, 'A3FC-2390-D14B-07AB', '2017-12-03', 10.44, 416),
  (1512, 'C76E-36F6-37E6-6915', '2020-12-28', 7.91, 417),
  (1513, '9176-52AD-81D0-E434', '2006-12-18', 16.65, 417),
  (777, 'BF46-5AB5-B13C-C121', '2017-04-24', 15.86, 418),
  (375, 'F3BF-38B5-CD59-62B7', '2020-08-29', 8.23, 419),
  (374, 'F4BA-288B-2941-3E34', '2015-06-29', 18.69, 419),
  (1472, '2CB4-3542-687E-F517', '2017-05-22', 6.14, 420),
  (1698, 'FBE9-184A-8AAE-A099', '2008-09-05', 8.53, 422),
  (1564, '5256-F8D2-98E9-5467', '2019-12-28', 11.89, 423),
  (1154, '39CE-26F6-847A-47F8', '2010-07-13', 19.85, 424),
  (1228, 'EBF4-8BB4-A07E-BF31', '2008-07-08', 5.11, 425),
  (575, '6B04-7FBE-98CA-0267', '2022-11-05', 6.95, 426),
  (1186, '6EFD-BCDF-3286-16FF', '2010-01-14', 6.79, 427),
  (1187, 'EBE8-19A5-E885-DE18', '2001-03-09', 6.16, 427),
  (1356, '6AA5-2D7C-C7D8-615A', '2019-11-09', 14.16, 428),
  (1357, '9AC7-42CC-E5E0-0F61', '2019-09-08', 15.34, 428),
  (1410, '02A8-32D9-E0E5-95C7', '2022-01-26', 16.41, 429),
  (1144, 'DF81-E9D8-D826-0A9F', '2011-11-02', 5.13, 430),
  (1288, 'FDA5-D30D-639F-2A70', '2009-01-09', 10.24, 431),
  (1289, 'C52E-AD07-14FD-0C6A', '2000-05-03', 17.58, 431),
  (873, 'D28B-42D7-E990-70CC', '2021-07-13', 11.46, 432),
  (877, '9385-56C7-5652-37D9', '2021-07-01', 18.09, 433),
  (461, '3F5F-F1DE-93DB-DDCB', '2013-08-28', 10.25, 434),
  (1050, '3520-4152-3038-295E', '2022-09-08', 18.11, 435),
  (366, '0B48-F217-E98F-371A', '2015-06-26', 13.29, 436),
  (1511, '91E0-0C48-958B-2793', '2021-12-15', 8.44, 437),
  (640, '6212-27A7-1896-F202', '2020-10-10', 5.02, 438),
  (910, 'E449-31FF-EA5D-348D', '2020-08-12', 15.96, 439),
  (1651, '898A-D2F5-74F2-DED4', '2018-11-28', 10.88, 440),
  (1652, 'BC83-FD44-75ED-DE59', '2017-11-05', 8.58, 441),
  (563, '0702-A0C8-4094-8529', '2022-05-02', 7.55, 443),
  (1183, '71A2-6B2C-1B92-DB93', '2016-05-21', 14.11, 444),
  (675, '8050-CC5B-A098-F494', '2012-09-12', 15.83, 445),
  (49, 'AD06-4EA0-A48B-E857', '2022-06-08', 15.53, 446),
  (1096, '9D05-7A33-1564-C80F', '2012-09-03', 16.78, 447),
  (247, '63D5-027F-DB9E-A27A', '2016-06-20', 5.18, 448),
  (891, '4B67-7535-2CA6-90B4', '2005-04-18', 5.21, 449),
  (534, 'FE11-6A78-E4D4-420C', '1997-03-03', 13.50, 450),
  (1276, 'C18D-C1A3-183C-9D6A', '1996-07-11', 15.27, 451),
  (1277, '0E64-A34C-99FB-19AC', '2014-01-12', 18.98, 452),
  (1439, '61C2-8944-D4DC-A139', '2012-05-13', 5.39, 453),
  (1592, '0D50-B682-BBD6-F21D', '2001-06-09', 19.57, 454),
  (1499, 'A5FF-6821-A389-8E46', '2014-09-22', 15.00, 455),
  (1500, '10CF-5064-01A2-2509', '2015-12-22', 19.94, 456),
  (1617, 'B7B2-DCFA-4C95-4E28', '2019-02-28', 18.80, 457),
  (1616, 'B836-E9D8-1D0F-4C7D', '2000-09-26', 17.30, 457),
  (198, '1A69-8E73-C610-7CA3', '2012-05-01', 15.37, 458),
  (199, 'E9A1-D036-31AD-241B', '2019-01-23', 8.55, 458),
  (1562, '5FF9-1CDE-4778-2E78', '2020-08-31', 11.50, 459),
  (443, 'D57D-CB63-59A4-6AA7', '2016-07-28', 5.69, 460),
  (593, '2759-FAF0-2E9B-CBAA', '2021-03-16', 10.37, 461),
  (88, 'F55A-B96A-911B-5A33', '2022-08-05', 15.45, 462),
  (722, 'AFF5-6188-61E0-DBFE', '2020-10-11', 6.30, 463),
  (913, '1717-DCEB-2ECB-A4E9', '2007-05-06', 8.00, 464),
  (645, '59D9-5C96-905E-F7CE', '1994-06-25', 6.63, 465),
  (484, '4B14-9335-D212-018B', '2019-06-07', 10.76, 466),
  (483, 'FAD4-CAF3-B517-A2C9', '2011-05-08', 18.83, 466),
  (946, 'D247-2FB9-EC05-4F57', '2007-07-05', 19.39, 467),
  (944, '80C2-8987-D961-AF48', '2014-10-01', 18.30, 467),
  (945, '1A5F-A2CB-8F66-E613', '2011-04-25', 12.09, 467),
  (1394, '79FF-A589-D0A1-4764', '2020-01-24', 15.81, 468),
  (1527, '55E9-4625-11A3-EEAC', '2020-06-23', 18.69, 469),
  (114, 'CA30-E4CA-2B7B-A597', '1999-01-19', 10.18, 470),
  (84, '1D24-CC52-7D0A-B695', '2015-11-04', 14.93, 471),
  (921, '51F8-EC39-F95C-C622', '2010-12-17', 8.63, 473),
  (1548, '04F8-195C-6DC3-9AD6', '2020-08-22', 7.59, 474),
  (950, 'F53B-9B2A-FD13-BB9C', '1973-03-23', 14.57, 554),
  (677, '34A8-DAAF-5772-7326', '2011-04-05', 15.77, 476),
  (721, '9094-A135-A9E2-0E4D', '2021-03-15', 14.53, 477),
  (720, 'F811-3619-FF28-7E1F', '2018-06-28', 13.80, 477),
  (381, '3A17-C37F-44F8-4284', '2019-08-02', 9.74, 478),
  (1300, '01EF-233A-70E0-FFD7', '2016-11-21', 14.58, 479),
  (1322, 'B802-0701-5A95-59AD', '2009-11-12', 19.71, 480),
  (1540, '78B7-A7BF-765A-6079', '2017-01-25', 11.35, 481),
  (172, '2C9A-8A94-06CF-E168', '2017-07-09', 6.00, 482),
  (1240, 'F049-82B3-80C4-D2FE', '2009-09-21', 6.90, 483),
  (1239, '716B-5378-56A2-2E05', '2007-01-18', 14.54, 483),
  (125, 'D2EC-2270-FE42-C166', '2022-07-26', 9.99, 484),
  (1352, '12E1-BF81-B243-2918', '2021-04-24', 13.05, 485),
  (328, '6155-55BA-369C-4035', '2018-06-02', 19.45, 486),
  (329, '5A3D-2856-45F3-76DC', '2014-07-06', 9.93, 487),
  (330, '45CC-11DC-ECE5-0598', '2022-01-21', 11.88, 487),
  (816, '88A9-443C-C44E-44F1', '2017-03-15', 7.75, 489),
  (1214, '6E26-BCD3-7780-396F', '2019-09-23', 11.31, 490),
  (1567, 'BB6D-0FD3-E4C3-C06E', '2018-07-15', 7.88, 491),
  (1580, 'B3D9-D8AD-418B-217F', '2008-08-16', 13.12, 492),
  (1504, 'B88A-0D0A-7C35-2D48', '2017-03-15', 9.36, 493),
  (1506, '13D9-83E6-7DAE-CA10', '2022-01-22', 14.46, 494),
  (1505, 'ADA5-5E36-FC1C-1465', '2020-08-02', 18.70, 494),
  (187, '414B-727B-BF37-ACBD', '2012-09-02', 14.61, 495),
  (36, '7AD0-B3EF-4384-9D81', '2014-09-16', 14.40, 496),
  (624, '4046-48F7-CA86-CED2', '2010-01-02', 18.49, 497),
  (626, '7B79-68AF-98EE-7E1F', '2019-09-17', 14.07, 498),
  (625, '8E33-FF5C-1AE2-498F', '2020-11-26', 6.15, 498),
  (480, 'AD92-6E97-454F-687D', '2022-05-21', 6.98, 499),
  (1317, 'F76C-FEDF-2A19-DD10', '2019-04-17', 9.86, 500),
  (869, 'B8A5-F5DC-CB57-7788', '1953-07-21', 13.50, 555),
  (1058, '4C78-A9D9-A06B-C733', '2021-10-05', 19.30, 501),
  (696, 'F99F-5922-B0B2-B610', '2019-02-10', 17.84, 502),
  (1306, 'E917-FE24-C0F0-3D9B', '2020-11-12', 7.91, 503),
  (230, '2C29-DB38-C50F-E2D9', '2020-08-04', 9.40, 504),
  (231, '4DE6-D3C7-2A3B-E9D8', '2019-06-24', 7.12, 504),
  (841, '091C-5520-7A58-4386', '2019-10-30', 16.85, 505),
  (281, '4149-94DC-3216-D9EE', '2013-03-22', 19.22, 506),
  (1071, 'BD80-BBA0-9B93-8AA2', '2022-10-05', 17.51, 507),
  (254, '3867-2FDC-AFF1-B2EA', '2013-07-10', 15.05, 508),
  (204, '2311-2B9F-5F02-8F19', '2017-12-03', 17.07, 509),
  (1486, 'ADF0-4844-6C61-AEE3', '2009-03-18', 11.81, 510),
  (783, '3420-6978-CDF5-6603', '2022-01-23', 19.82, 511),
  (634, 'D12B-E6B9-31ED-A218', '2018-10-09', 15.47, 512),
  (1601, '7097-41B3-9AC4-B52A', '2007-05-30', 13.26, 513),
  (1339, '3CF9-EF57-8B46-E40A', '2015-02-16', 11.64, 514),
  (451, '278B-52E8-E24B-1BBA', '2014-06-03', 10.30, 516),
  (715, '1CF3-322B-3DF4-4BD7', '2009-08-24', 19.85, 517),
  (1569, 'D6B0-94F6-A4CF-C792', '2013-03-16', 9.31, 518),
  (1055, '6041-5C52-1CEB-C7DB', '2022-07-22', 12.24, 519),
  (1457, '1123-C401-2CF5-1D01', '2020-08-05', 6.43, 520),
  (517, '5574-83DA-3284-F0F6', '2022-10-15', 7.00, 521),
  (918, 'CF1E-9685-72CC-00A9', '2016-04-21', 6.37, 522),
  (1656, '89A7-75D2-6C11-119B', '2013-09-22', 15.90, 523),
  (1620, 'A35C-1AD3-DC71-98D8', '2022-11-19', 9.79, 524),
  (1659, 'F087-38D9-CE42-7FE6', '2005-06-09', 12.37, 525),
  (382, 'A92C-0ABF-C3D3-71BF', '2008-11-13', 7.81, 526),
  (1247, '326B-5019-64BE-4F30', '2014-07-10', 7.34, 527),
  (305, 'D175-1B64-BA1C-BE18', '2020-10-22', 8.35, 528),
  (306, '30FA-B260-7512-A3CB', '2018-04-17', 14.10, 528),
  (307, '213B-021D-5B95-9364', '2018-01-11', 16.65, 528),
  (1608, 'FB42-4EA3-538F-FEF7', '2014-02-13', 18.80, 529),
  (1609, '67A1-1767-529C-135C', '2018-10-04', 10.03, 529),
  (679, 'D8BE-61F5-94A6-8DCE', '2018-06-11', 9.99, 530),
  (183, '358D-B527-19F8-9196', '2007-04-12', 10.59, 531),
  (1391, '4905-C9AB-B205-6662', '2019-10-25', 17.38, 532),
  (893, 'F960-FB35-1701-57C1', '2017-11-18', 6.14, 533),
  (174, '148C-F8C5-8BDA-4E83', '2015-02-01', 8.35, 534),
  (1332, 'A88B-B7FE-1DAF-14EB', '2018-04-27', 18.75, 535),
  (1365, 'A6FF-FF03-A2B3-C1BE', '2017-05-11', 15.22, 536),
  (874, '72FB-9D9B-FDF8-43DB', '2018-07-27', 18.44, 537),
  (541, '2A3B-12E3-B092-280D', '2021-01-22', 15.69, 538),
  (578, '8D8F-206D-561E-CBA8', '2021-04-27', 19.80, 539),
  (577, 'C20C-4D87-A2F3-D95A', '2020-01-04', 14.43, 539),
  (1373, '0DDE-CB39-EB1D-EC70', '2018-06-09', 19.49, 540),
  (256, 'A75C-59C7-90B0-7A29', '2016-01-16', 12.50, 541),
  (257, '8EA0-460C-C8DA-54A8', '2015-05-18', 8.16, 541),
  (806, '8288-C5EC-C230-2E64', '2018-07-21', 16.95, 542),
  (826, '54C4-722E-02FC-6CE1', '2018-11-23', 7.60, 543),
  (1501, '003A-1E86-4CAF-2853', '2018-08-01', 11.07, 544),
  (1502, 'B4A6-5508-DCF1-BD0E', '2023-01-02', 6.94, 545),
  (1579, 'C29D-592D-43ED-A9CF', '2008-06-09', 18.81, 546),
  (1678, '1F58-DAFC-3F5E-CA28', '2014-05-27', 9.83, 547),
  (369, '14CA-8D1B-AA27-E84E', '1984-05-09', 5.86, 548),
  (76, '1F3D-3317-321C-7CC8', '2012-02-23', 16.62, 549),
  (515, '464A-8F61-3A06-CD6C', '2020-07-24', 6.26, 550),
  (1635, '0687-1A7E-C4D4-C91F', '2021-11-29', 17.31, 551),
  (559, 'EFCE-B799-2056-D64F', '2022-06-25', 13.23, 552),
  (1039, '055F-A162-6B2A-D49F', '2007-03-08', 9.15, 553),
  (949, '92E0-9A38-7205-A212', '1998-03-04', 5.24, 554),
  (870, 'E8F3-98F7-A1E2-BAAD', '2010-01-10', 8.22, 556),
  (398, 'A968-1F40-815F-0207', '2015-11-29', 13.90, 557),
  (399, '5CA6-8416-9DC8-097D', '2012-06-05', 9.21, 557),
  (397, '08E7-8DFB-90EA-E774', '2015-07-31', 16.99, 557),
  (1104, 'FEEA-8A39-8D0D-4013', '1990-10-07', 12.38, 558),
  (1584, '9B20-5E24-9267-41C7', '2017-04-22', 15.67, 559),
  (839, '77CE-84F3-E831-8414', '2010-05-04', 6.39, 560),
  (701, 'A683-D306-A89D-9061', '2010-04-11', 8.34, 561),
  (1027, '4FF7-4B68-E186-893B', '2018-01-12', 5.16, 562),
  (294, 'EE2D-22B3-1CC3-929C', '2022-04-01', 11.83, 563),
  (295, '5F86-4379-6B69-9277', '1999-01-27', 14.62, 563),
  (51, '712D-E638-866B-6963', '2002-12-18', 9.32, 564),
  (1189, '5F85-0497-3912-1750', '1973-01-18', 15.79, 565),
  (1188, '34B3-1EB4-E316-FBC7', '1968-05-09', 18.30, 565),
  (101, '2192-4DC5-E938-F3D8', '2001-08-08', 6.09, 566),
  (1557, 'CC0F-7DC9-52E4-1D29', '2009-07-04', 14.84, 567),
  (1558, '4B51-B851-8EA8-D806', '2011-11-26', 9.36, 567),
  (754, 'ABB2-EED9-5344-742B', '2020-06-14', 15.77, 568),
  (674, 'C09D-409D-E212-AB7D', '2021-08-16', 18.35, 569),
  (908, 'AD3F-2C00-9FFD-014C', '2020-01-07', 19.75, 570),
  (460, '99DC-FB92-C88C-31A5', '2017-05-30', 9.95, 571),
  (175, 'B159-B57B-0934-4DE4', '2015-10-11', 7.91, 572),
  (983, '77B6-C831-5541-66FB', '2016-01-13', 14.73, 573),
  (68, '0D3A-C73C-D31A-9A20', '2015-08-01', 6.15, 574),
  (69, 'A084-DF91-9145-4FB2', '2017-10-10', 15.90, 574),
  (70, '10DA-1EA3-36D7-304E', '2019-08-17', 9.66, 575),
  (764, '67F2-6294-2F9F-AE14', '2019-11-10', 17.51, 576),
  (252, 'BADD-086B-A0D2-FD90', '2009-06-15', 11.13, 577),
  (1675, '4EAC-E168-36B3-C240', '2013-04-05', 6.01, 578),
  (600, '2C2D-F095-7098-62B2', '2007-12-20', 19.63, 579),
  (310, '316A-F27E-9FA7-E8BC', '2022-10-16', 17.42, 580),
  (308, '3EB2-75F5-D4CC-F1B1', '2008-04-08', 18.06, 580),
  (1490, '3E33-EB73-0F21-9A1C', '2001-05-12', 5.77, 581),
  (1488, '29E0-D74C-C64E-8234', '2011-04-16', 6.90, 581),
  (1489, '11E3-91B0-96B7-E34F', '2008-03-05', 7.49, 581),
  (1661, 'AB88-84E4-0537-2F96', '2018-02-26', 7.70, 582),
  (860, '5229-0319-62A8-7C78', '2019-10-31', 19.40, 583),
  (669, 'C901-AE20-DABA-97CE', '2021-11-29', 17.11, 584),
  (1177, '5E20-8C0E-1327-7C00', '2018-05-04', 19.71, 585),
  (1387, 'CA2A-2B3D-115B-A9A4', '2022-12-10', 9.49, 586),
  (222, '8DE4-FFAE-1C9D-C746', '2019-09-11', 17.37, 587),
  (791, 'F0FE-7442-664D-A5AE', '2018-04-19', 8.88, 588),
  (1196, '1A3B-B42B-9726-F34A', '2008-02-04', 9.76, 589),
  (1254, '3D7F-EF4B-B4B0-C336', '2022-07-13', 6.77, 590),
  (1253, '8FA7-BCAE-B814-AF98', '2014-10-06', 9.05, 590),
  (859, '4A8E-EFC7-38C8-A2B8', '1985-02-09', 12.10, 591),
  (521, '8638-703B-067B-F040', '2017-04-26', 5.45, 592),
  (1123, 'D8F3-E53D-9F48-F6F5', '2006-01-14', 18.49, 593),
  (293, '46CF-198B-DF13-6CE3', '2022-10-01', 8.23, 594),
  (292, 'EA21-A30E-DC2A-9CEE', '2012-01-22', 6.34, 594),
  (1054, '3F0F-FA45-0ECC-0A79', '2013-09-03', 18.04, 595),
  (1479, '2869-48B1-0C5D-EBE2', '2013-03-07', 11.91, 596),
  (236, '8701-D5E1-F931-2263', '2010-10-05', 13.91, 597),
  (589, 'E498-3D1C-02D0-7348', '2013-11-07', 11.11, 598),
  (1658, '80D8-AF22-E530-2ACA', '2019-04-07', 6.85, 599),
  (1215, '7965-3381-17FD-C3E0', '1998-01-05', 15.44, 600),
  (704, 'F08A-E6FE-7889-1FB2', '2008-02-29', 16.90, 601),
  (938, '1009-7DD3-119E-5688', '2009-08-17', 5.31, 602),
  (937, '8DF3-3D5A-C54C-419C', '2020-02-13', 8.19, 602),
  (37, 'C5CC-A313-1889-CB0A', '2022-09-30', 6.04, 603),
  (1222, '2303-69F5-552F-5308', '2008-05-13', 6.82, 604),
  (1195, 'A48E-BF71-3299-88E9', '2010-10-21', 15.27, 605),
  (1360, 'B536-F947-A0C0-CEA2', '2014-06-18', 12.52, 606),
  (1359, '254A-6F8F-2730-E18C', '2013-02-20', 17.13, 606),
  (285, 'E16D-96C1-CB3B-F83B', '2017-06-11', 12.93, 607),
  (1152, 'C0DF-A459-AA70-2A87', '2018-11-25', 7.55, 608),
  (336, '6E37-CF1B-81EC-9CED', '2022-06-19', 16.06, 610),
  (439, '662D-2570-C4DE-6DF8', '2018-10-09', 7.84, 611),
  (1383, '4ACC-E50E-EC41-1456', '2012-01-18', 11.24, 612),
  (703, '35F4-F5D5-D978-B009', '2022-06-26', 19.65, 613),
  (1199, '8C95-671B-1897-8B86', '2013-01-30', 19.55, 614),
  (394, '5BBA-F881-0FC2-6E62', '2011-10-01', 11.71, 615),
  (466, '9185-F5D4-5C7B-9DFE', '1984-09-25', 14.70, 616),
  (508, '7258-1FC8-F542-C1CF', '2020-10-21', 19.85, 617),
  (509, '2957-2E27-E464-3041', '2002-12-01', 14.13, 617),
  (1629, 'E2B6-A5B9-55AE-B3CE', '1977-09-01', 10.53, 618),
  (1628, '63CB-48AD-A0A6-5BB5', '2016-11-04', 9.03, 618),
  (1038, '427C-ABE7-C59B-AEBB', '2018-02-12', 7.61, 619),
  (1169, '8C6C-7E41-66F0-18AB', '2019-03-29', 5.67, 620),
  (985, 'E7D5-BE87-E291-2AFA', '2017-11-11', 12.95, 621),
  (1001, '97C1-A942-EE95-6365', '2013-02-19', 13.18, 622),
  (1518, 'DE08-4A03-8DEB-93A3', '1996-10-17', 15.56, 623),
  (1519, '8FDD-C2A8-04C6-58F0', '2016-01-10', 11.89, 624),
  (109, '1EA3-B7FE-D136-9D40', '2011-11-07', 13.50, 625),
  (227, '4AE9-55FE-9523-5DE4', '2020-08-08', 19.83, 626),
  (21, '99ED-9B61-9784-77A1', '2018-12-24', 6.51, 627),
  (736, 'F6C1-2BB3-07AB-A504', '2022-04-01', 11.93, 629),
  (980, '56DE-4A6B-B315-D19A', '2022-12-02', 17.49, 630),
  (298, '84B4-FE3B-7D93-687B', '2005-04-28', 6.36, 631),
  (676, 'FD33-BF09-551E-1CBD', '2017-01-29', 5.53, 632),
  (419, 'F056-FFC7-8CBF-00CC', '2021-01-30', 8.93, 633),
  (420, '91E4-12EE-C06C-29B3', '2021-01-19', 9.23, 633),
  (1669, 'AF6F-8016-122D-D9F5', '2017-11-26', 7.34, 634),
  (609, '1B8F-5C0C-BE9D-A5DE', '2018-02-09', 12.18, 635),
  (608, 'F75E-E3A7-AD59-3AA7', '2021-06-27', 14.68, 635),
  (610, '4A70-3875-9D91-3BB7', '2022-02-20', 11.87, 635),
  (762, '2869-11A4-EC06-4416', '2002-01-26', 14.81, 636),
  (761, 'DF70-0957-D709-0218', '2009-04-10', 17.56, 636),
  (1045, '858E-7291-0543-C007', '2002-12-19', 16.42, 637),
  (224, '994D-D4E8-159B-E37E', '1987-07-22', 15.18, 638),
  (225, 'F997-C7E6-C53B-EBAD', '2009-11-30', 17.60, 638),
  (692, '5C81-4405-E190-0BA4', '2006-12-19', 11.10, 639),
  (206, '9CD1-37BF-ABE4-DDCB', '2008-10-08', 18.87, 640),
  (207, '53D6-9029-F824-7160', '2013-04-16', 6.62, 641),
  (1053, '757F-A28F-BDB0-1114', '2021-10-28', 19.03, 642),
  (1052, '20E6-3E73-8C2D-0A88', '2022-09-28', 11.76, 642),
  (1028, 'E6C3-3FD6-A0B4-1E48', '2021-05-05', 19.38, 643),
  (1026, 'A1C3-64C2-A2F1-145E', '2018-01-29', 14.74, 644),
  (708, 'EEAE-E13D-1B39-F152', '2017-04-20', 19.00, 645),
  (810, 'A3FF-3F95-3F1C-AB04', '2020-05-02', 10.55, 646),
  (1646, '11C7-882C-2A7A-FD8F', '2012-04-28', 18.64, 647),
  (1566, '45C7-09AD-4E31-28EA', '2018-08-20', 11.33, 648),
  (1367, '74F3-61B5-BDAF-089C', '2015-07-30', 6.22, 649),
  (1316, 'E355-5022-B251-68CE', '2002-08-08', 14.81, 650),
  (203, '8027-AFC4-7BDB-0585', '2018-06-09', 14.79, 651),
  (811, 'D8B3-D1FC-D1D0-C229', '2009-05-28', 12.07, 652),
  (1680, 'A52A-B181-FD2F-7F9C', '2022-07-10', 9.60, 653),
  (13, 'A3FF-2582-DB9A-0571', '1989-05-28', 5.55, 654),
  (14, '7393-76DE-7146-EF00', '2003-11-14', 11.25, 655),
  (1400, 'F9CD-59F5-EC6E-0974', '2013-01-26', 12.76, 656),
  (1399, '872B-0961-F7C4-0F2F', '2014-02-21', 17.49, 656),
  (989, 'BA88-91F7-C1CF-2C9F', '2012-06-05', 14.74, 657),
  (990, 'F288-68E4-1D39-C0EB', '2021-10-04', 6.44, 657),
  (991, '09CA-8F06-D986-0C78', '2010-07-05', 19.44, 658),
  (121, 'DD52-D2CF-53AB-028F', '2017-07-22', 7.89, 659),
  (297, '1C7E-769B-DB70-A81C', '1966-01-17', 6.47, 660),
  (1151, 'F7EE-5089-917E-A9E3', '2022-09-11', 14.75, 661),
  (529, 'D2E1-9ED6-0778-83D7', '2019-07-12', 6.17, 662),
  (530, '86DE-2329-98E2-6BA0', '2020-01-16', 12.19, 663),
  (331, 'FF89-EB88-E04A-812A', '1974-03-10', 12.53, 665),
  (1645, 'D860-D6AB-3A8A-69DC', '2009-03-03', 13.57, 666),
  (987, '084C-7833-A760-09D7', '2023-01-05', 19.62, 668),
  (988, '70E2-E74E-C6BE-7AD6', '2011-08-25', 10.17, 669),
  (1081, '9447-C07C-8E9E-F8EE', '2004-12-24', 11.77, 670),
  (767, 'CCD4-A090-852A-C863', '1986-05-07', 15.45, 671),
  (856, '64AF-D13D-8DDA-FF9C', '2022-12-22', 8.88, 672),
  (855, 'B517-A3FC-A210-7862', '2022-08-18', 12.36, 672),
  (1429, '6809-EFBF-9E0A-F9DF', '2020-05-11', 12.67, 673),
  (1368, '1A45-7B4C-2D9B-4847', '2022-01-08', 8.50, 674),
  (390, '34E7-0D08-7536-887F', '2017-08-16', 17.32, 675),
  (1130, '3B54-2072-D8A4-BBB4', '2021-04-10', 19.40, 676),
  (1403, '0562-670A-C01A-EE4A', '2020-08-24', 15.86, 677),
  (488, 'CA53-351B-D534-A584', '2022-09-26', 10.35, 678),
  (489, '3EB3-1318-0222-D44F', '2022-08-01', 11.24, 678),
  (221, '0FEC-D851-7A8D-DC93', '2020-07-22', 17.33, 679),
  (479, '41ED-D4BE-41EC-4D81', '2021-08-30', 19.36, 680),
  (787, 'F573-C47B-3B95-B15B', '2021-03-20', 7.13, 681),
  (220, '0622-C091-96EC-7904', '2008-06-24', 9.25, 683),
  (463, '7A1B-F3FC-4D6D-3373', '2021-09-13', 17.93, 684),
  (462, 'C8B8-7210-6F58-AB75', '2021-06-24', 9.40, 684),
  (15, 'AFC0-81A4-5BFA-4E90', '2019-12-13', 7.05, 685),
  (981, 'E07A-B7BD-9C7F-2D2E', '2018-11-20', 8.14, 686),
  (1264, '270F-5AAD-356E-796E', '2012-03-01', 17.39, 687),
  (1266, 'AF3C-53DC-1AA5-37B4', '1999-09-04', 6.66, 687),
  (1265, 'B1EC-96E6-51F7-2ECF', '2012-11-15', 16.40, 687),
  (1097, 'F2A0-512E-095D-9220', '1999-11-27', 6.99, 688),
  (92, '4D9A-511A-8116-0E93', '2014-02-03', 11.62, 689),
  (1290, '95E1-DDF4-3F61-043F', '2022-07-24', 16.40, 690),
  (72, '7703-82E9-DC8C-5B49', '2022-04-18', 8.29, 691),
  (1361, 'EEE1-7B28-10F4-4E7C', '2022-12-28', 13.57, 692),
  (818, 'BB5B-E19A-9C05-FBE0', '2008-08-10', 14.64, 693),
  (785, '1C8C-2591-F547-D024', '2008-01-21', 11.89, 694),
  (786, '7669-A395-1575-F9E5', '2013-08-31', 17.93, 695),
  (1273, '9F47-21FA-410F-0E60', '2022-12-20', 10.00, 696),
  (193, '5148-C082-5AAA-B1FD', '1994-04-14', 16.03, 697),
  (1372, 'CA0F-D932-9182-FC03', '2017-05-21', 12.86, 698),
  (1654, 'CE48-A8E8-4A73-3A24', '2005-10-27', 16.14, 699),
  (1655, '633D-049C-7796-3AA2', '2018-08-08', 7.78, 699),
  (1653, '54C0-FA8D-7A0C-327A', '2005-06-07', 18.58, 699),
  (258, '0AD0-4CCD-98D2-BEC6', '2012-01-02', 7.66, 700),
  (259, 'A7DA-BE12-2284-0431', '2022-10-30', 12.98, 701),
  (1192, 'ABF9-80A6-369B-628D', '2014-04-28', 8.28, 702),
  (47, 'D395-641D-F013-04E2', '2004-06-27', 16.71, 703),
  (392, '3415-A0F1-35DC-B4E1', '2003-09-04', 7.72, 704),
  (778, 'B1E8-A597-6F54-0086', '2013-12-02', 18.67, 705),
  (261, 'C6D3-BA56-299D-CDBF', '2001-05-18', 15.38, 706),
  (647, '5D46-0313-1630-FE71', '2006-10-10', 14.59, 708),
  (795, 'AF6C-71D7-58BB-2DDE', '2011-11-14', 19.45, 709),
  (384, '8A46-6DF4-8A7C-AB73', '2016-01-22', 17.19, 710),
  (385, 'B43F-0199-F60D-BB41', '2015-08-09', 16.97, 710),
  (383, '4DBE-FB6F-D97E-4991', '2011-11-11', 19.57, 710),
  (1077, '6C7C-1D84-EFF1-C226', '2005-07-10', 10.44, 711),
  (424, '0D4B-AF87-1DB3-28FB', '2015-06-15', 9.59, 712),
  (425, '4546-2CB9-80C8-A27B', '2012-09-25', 18.92, 712),
  (426, '1704-F6D1-F63D-1D80', '2005-02-25', 17.93, 712),
  (1532, '9B57-C93F-2666-CFEC', '2015-10-02', 10.48, 713),
  (132, 'F446-D08F-01E2-A478', '2019-01-12', 18.36, 714),
  (789, '71FC-20CB-4072-4CC4', '2019-06-13', 11.84, 715),
  (790, '1348-ACFB-7D83-9E06', '2000-08-05', 6.29, 716),
  (831, '246E-2563-33B1-ED71', '2014-09-24', 18.72, 717),
  (1582, '88FC-79C3-0C2C-C443', '2022-01-31', 15.41, 718),
  (580, 'D148-9620-DD5A-63B1', '2021-05-09', 15.38, 719),
  (963, '7920-79A7-B385-AA6C', '2021-08-05', 7.88, 720),
  (1419, '96E5-E81D-C77D-C479', '2018-01-28', 15.36, 721),
  (362, '368C-BE15-9BE4-2133', '2014-04-17', 12.12, 723),
  (363, '9255-7A17-6003-DF40', '2012-08-19', 14.42, 723),
  (1041, 'CA89-8AF3-3A62-B2F7', '2020-09-22', 14.49, 724),
  (1477, '20AF-0157-AA4E-9CDE', '2014-12-11', 16.60, 725),
  (1478, 'F85F-1609-A1E0-B233', '2018-02-07', 10.58, 725),
  (705, '730C-3DFA-FCF9-E295', '2012-04-09', 11.72, 726),
  (757, 'A591-DDF4-F3A6-9D86', '2018-09-10', 11.79, 727),
  (1262, '48F5-0CCC-C0D6-547C', '2015-02-05', 7.61, 728),
  (1263, '2CCC-30D0-7899-0EF2', '2019-06-30', 8.60, 728),
  (1676, 'A7D1-453C-B967-BBC5', '2013-07-09', 14.30, 729),
  (595, 'B5CD-AF60-8018-15C6', '2016-06-20', 15.68, 730),
  (903, '0420-730C-65F8-6283', '2015-04-25', 19.28, 731),
  (902, 'FE59-FB4A-8292-9098', '2011-05-17', 10.87, 731),
  (1493, '3E1A-DA6F-3A0C-664A', '2011-07-31', 14.30, 732),
  (1255, '4312-5FD4-F3E9-5025', '2019-05-24', 12.75, 733),
  (1003, '5CEE-2B7C-B52D-E187', '2022-10-06', 10.81, 734),
  (249, '77E3-3132-41F4-CA48', '2010-11-26', 19.71, 735),
  (1705, '31DD-D6AC-5730-D459', '1990-08-28', 15.23, 736),
  (1706, '7600-031C-BB55-9628', '1993-07-07', 14.68, 737),
  (730, '3BE5-F343-D0D8-8044', '2012-11-11', 12.08, 738),
  (729, '5362-88A1-CE3A-5827', '2014-08-10', 18.03, 738),
  (444, '05D7-41E8-2DFC-BDFC', '2013-03-14', 19.51, 739),
  (445, 'D468-0A8A-8AF5-43B7', '2017-09-18', 11.77, 740),
  (1035, '6E8F-33E3-E1DB-4CD7', '2022-08-05', 17.55, 741),
  (1034, 'EA6C-555F-2FF0-650C', '2021-07-18', 9.23, 741),
  (1023, '0077-E50A-D84F-B769', '2022-10-01', 10.73, 742),
  (1069, 'F6F6-B44E-90B6-2D06', '1991-05-25', 16.09, 743),
  (1301, 'B204-0485-799E-D2E8', '2017-09-30', 14.05, 744),
  (407, '648B-6EEA-DA87-2733', '2021-03-07', 17.99, 745),
  (1555, '55F4-6536-7167-5175', '2018-05-16', 18.13, 746),
  (792, '5AA3-61B2-CA94-F88F', '2017-03-25', 11.20, 747),
  (1602, 'FE36-7E04-8F91-7FE4', '2022-07-28', 5.16, 748),
  (418, '649B-CE5E-4853-38BF', '2003-09-11', 5.42, 749),
  (417, '24CC-96A4-F3D8-5988', '2008-07-09', 19.40, 749),
  (1463, '2D83-D01F-66BD-1476', '2020-12-05', 13.30, 750),
  (1462, 'ADE5-E02F-21F1-7A5A', '2011-07-08', 17.80, 750),
  (79, 'A46C-59ED-AB91-C080', '2022-11-27', 18.06, 751),
  (1217, 'BF55-E096-987F-6B45', '2016-12-28', 19.28, 752),
  (646, 'E966-FB69-1350-6F4F', '2011-11-09', 14.00, 753),
  (34, '9B1D-861F-F540-89E6', '2020-03-05', 11.37, 754),
  (1207, 'CDD4-DEA4-A66E-4B76', '2020-03-21', 18.95, 755),
  (513, '38D9-6DF1-AB11-F56C', '1997-11-16', 16.91, 756),
  (1389, 'D59F-E126-574F-6E94', '1999-11-14', 17.08, 757),
  (684, 'D338-717C-F23E-1266', '2022-09-14', 7.51, 758),
  (686, '4815-9D04-B2CA-1219', '2022-10-23', 13.14, 758),
  (685, '8CE2-CE54-CC33-D148', '2021-02-21', 14.92, 758),
  (687, '53BF-7C91-6D50-53A3', '2021-06-03', 19.48, 759),
  (1362, '2739-2430-A677-4A5B', '2010-02-18', 12.00, 760),
  (1541, 'A8A0-4609-14E5-8477', '2018-06-19', 9.92, 761),
  (55, 'FA2A-E865-77BE-1026', '1999-10-31', 16.69, 762),
  (56, 'ACE2-2798-47EF-277A', '2016-11-25', 13.59, 762),
  (1025, '59AD-41E6-7CB8-EFC2', '2021-02-26', 8.56, 763),
  (1598, '24A5-5A3A-FB5D-894D', '2001-08-20', 19.39, 764),
  (1596, '5DAD-7F44-10CA-627D', '2003-05-10', 5.75, 764),
  (1597, '5248-CAF0-41B6-B8F4', '2005-12-11', 15.57, 764),
  (1599, '8966-59C6-7C93-5991', '2015-10-19', 10.28, 765),
  (1353, '9D0D-7A4A-706E-E6FC', '2020-05-13', 19.43, 766),
  (1354, 'F0EB-11B2-0BD6-B55C', '2018-04-22', 7.68, 766),
  (611, 'A9B1-BAAB-8DBA-2B5F', '2016-04-24', 12.59, 767),
  (1311, '27A3-A9EF-05A7-DA6F', '2015-05-09', 10.62, 821),
  (771, '3288-BD9F-EBA1-A5E8', '2001-02-22', 8.01, 768),
  (773, '07D3-0A81-DD08-03AC', '2017-03-22', 8.06, 769),
  (772, '5FB7-BAAF-ADEC-748D', '2013-04-11', 9.27, 769),
  (788, 'F8E1-3517-6ED0-2909', '2000-07-17', 8.33, 770),
  (682, 'DDDD-9940-C5C5-A6D9', '1995-12-12', 17.28, 771),
  (1036, 'CC8D-FF84-1FB1-B1D1', '2022-07-12', 9.61, 772),
  (1517, '5236-F82B-D263-D467', '2014-01-28', 16.43, 773),
  (1235, '1409-ADB9-A677-41A0', '2019-10-10', 19.80, 774),
  (1376, '15C2-899B-0F07-A216', '2013-11-28', 11.11, 775),
  (1375, 'D97C-6654-63A5-1C45', '2020-11-20', 12.91, 775),
  (639, '52C9-4CA8-94D9-5580', '2022-08-19', 7.25, 776),
  (581, 'F56E-4DA4-2BE6-F502', '2020-05-17', 10.58, 777),
  (582, '3331-8344-2C1A-456C', '2016-11-25', 9.76, 777),
  (1200, 'A7E2-107B-6B2D-BCB9', '2009-11-14', 16.98, 778),
  (889, '573C-C279-CEF2-8CA8', '2017-04-06', 8.96, 780),
  (97, 'B146-2652-1B81-20EA', '2021-11-05', 13.93, 781),
  (1575, 'B566-01E2-0E2E-3355', '2022-04-29', 7.74, 782),
  (599, '7B5F-EA52-BA77-46ED', '2021-03-19', 13.26, 783),
  (598, '4F78-6698-622A-99F3', '2018-06-15', 9.04, 783),
  (666, '6B03-1631-DC1B-55F8', '2021-06-10', 10.97, 784),
  (1335, '28A7-76CB-8FE0-D71A', '2021-03-21', 5.66, 785),
  (1131, '53FF-FA69-346E-FA4E', '2019-12-10', 7.95, 786),
  (1468, '6D63-9A09-1FDF-E95C', '2015-12-02', 14.28, 787),
  (160, '6A9E-87B8-9CEF-6C36', '2022-02-11', 14.25, 788),
  (162, '9251-BC07-D3EF-9FCD', '2019-05-01', 13.25, 789),
  (161, '9312-EF1C-9DFB-C660', '2021-02-09', 17.61, 789),
  (219, 'ED90-D1C3-C912-A999', '2022-05-09', 12.73, 790),
  (1369, '1227-508D-E3FE-60AC', '2020-07-03', 14.04, 791),
  (1633, 'BC15-D116-D2B4-428F', '2016-06-27', 16.32, 792),
  (413, '4944-0392-EDC4-59A4', '2013-09-03', 19.63, 793),
  (27, '5B2B-C98A-4B69-3F4D', '2008-11-01', 5.28, 794),
  (1061, '4CA0-AD62-8C43-458C', '2018-12-04', 13.98, 795),
  (1060, 'F831-A92D-AA5B-BE80', '2005-10-02', 14.34, 795),
  (112, '1303-17EA-5130-38EA', '2021-08-04', 14.64, 796),
  (1008, '0DC7-3745-955A-0457', '2015-05-21', 12.13, 797),
  (1009, '5A7A-D5DE-1D8B-158A', '2018-06-20', 19.15, 798),
  (616, '802E-A008-2C8E-B93F', '2019-08-03', 19.41, 799),
  (1040, '96B3-FB75-3790-729F', '2022-12-19', 7.02, 800),
  (1202, '4E90-EABC-B94C-2F9F', '2019-03-27', 10.45, 801),
  (1556, '4DB6-1B6E-5C11-701C', '2004-02-12', 11.08, 803),
  (1134, '6235-5DCB-0581-1811', '2002-11-25', 17.12, 804),
  (824, 'F828-489B-0BC0-B9C5', '2010-09-09', 8.37, 805),
  (1017, '63FF-9509-EDE6-462B', '2021-03-13', 5.39, 806),
  (1016, 'F2C4-8ED6-C9C1-F9C4', '2021-04-07', 12.69, 806),
  (117, '6C8A-C3DC-F6A5-2DA5', '2013-11-08', 5.60, 807),
  (917, '8D5C-3F4D-C28F-E88D', '2017-08-12', 16.18, 808),
  (916, '2FD0-7ECE-F619-4FF3', '2007-11-28', 9.88, 808),
  (915, '0146-6B97-513D-3CF4', '2018-08-21', 8.36, 808),
  (1261, '7959-F804-B200-B830', '2016-02-09', 7.91, 809),
  (1510, 'DEAA-B1DA-3505-1378', '2019-05-24', 18.57, 810),
  (1165, '3FDC-7A47-E1DF-3981', '2022-01-13', 7.47, 813),
  (888, 'EB57-E33E-A08C-B338', '2000-10-04', 11.59, 814),
  (887, 'DD5A-4B07-A639-446C', '1987-06-23', 19.33, 814),
  (886, '6569-6AFC-EB66-A234', '1977-06-17', 5.41, 814),
  (765, '67D6-948E-3EAB-E086', '2008-08-06', 7.75, 815),
  (244, '1FFE-AE8F-181B-2DF2', '1996-01-09', 6.89, 816),
  (246, '95BD-BC05-43C7-5FAE', '2014-11-26', 10.94, 816),
  (245, 'B0D2-2E5F-3A67-DD97', '2015-03-20', 19.71, 816),
  (1201, 'DB84-4C8E-4CC1-0FF3', '2021-12-03', 9.04, 817),
  (471, 'BC69-511C-0C67-F915', '2012-05-15', 6.88, 818),
  (472, '5910-8E51-9A49-D916', '2006-09-30', 10.40, 818),
  (1515, '3C68-9D34-73C4-3F98', '2023-01-02', 10.92, 819),
  (820, '173F-8A5B-C268-6A16', '2022-05-10', 15.24, 820),
  (1703, 'E0D7-97FE-0AF6-3595', '2015-06-17', 5.55, 1029),
  (1539, 'E487-1FA5-6CD5-E787', '2016-10-20', 13.85, 822),
  (929, '6EDA-2D88-2671-10A6', '1975-01-10', 8.54, 823),
  (1364, '8656-A29E-6214-9AC5', '2012-08-26', 6.94, 824),
  (585, '7811-BB11-23C6-4301', '2015-10-06', 5.36, 825),
  (179, '52D0-2111-A447-C9C6', '2022-06-28', 10.85, 826),
  (423, '0AB2-783E-42C5-9EAB', '2009-11-23', 17.11, 827),
  (1325, 'B3FD-BE3C-85A3-3F66', '2009-12-26', 7.29, 828),
  (1326, '0A14-F20A-CE46-27F7', '2020-06-15', 18.41, 829),
  (1327, '6423-C35C-57FF-3A9C', '2003-06-24', 7.03, 829),
  (1523, 'A78A-547E-FA8C-E5CB', '2020-07-01', 17.70, 830),
  (1150, 'BC18-6C4C-E847-1ACE', '2003-03-06', 14.96, 831),
  (396, '6DE7-D7B1-F228-9CDF', '2015-08-17', 15.81, 832),
  (395, '1F87-F972-1714-CBAF', '2010-10-15', 18.31, 832),
  (550, '2346-2209-8C62-BC01', '2010-04-11', 10.41, 833),
  (118, '6267-B180-9136-B084', '2022-06-28', 8.42, 834),
  (1593, '320C-E9C2-80DE-8F4B', '2018-04-12', 8.77, 835),
  (1149, '2078-3F94-AF79-8BFB', '2011-02-17', 11.16, 836),
  (1390, '4C24-7DD7-E4C4-C886', '2020-05-25', 9.52, 837),
  (1483, '11B4-14BB-21EE-3C19', '2018-07-02', 12.10, 838),
  (354, '2D70-2239-D132-9410', '2021-05-18', 7.32, 839),
  (355, 'B7C5-AB17-1339-189F', '2022-08-31', 17.39, 840),
  (356, '5501-6CB5-E5BD-3797', '2022-03-09', 16.99, 840),
  (1269, '818D-0168-5A59-1541', '2020-07-01', 9.10, 841),
  (323, 'B72E-26B5-CC7E-2ABC', '2001-12-23', 5.06, 842),
  (1051, 'E551-25CD-BBEB-F1B1', '2015-11-21', 14.99, 843),
  (324, '017A-F6A3-CDE6-0EE3', '2013-08-05', 19.30, 844),
  (325, 'BE45-AA14-2FAF-2D2C', '2018-08-05', 10.49, 844),
  (952, '166F-15C3-BBF1-E62C', '2019-06-12', 6.23, 845),
  (955, 'ACC2-9C0A-A8B3-26A5', '2018-05-06', 6.23, 846),
  (954, '4292-0810-1839-D206', '2014-10-26', 14.81, 846),
  (953, '6549-19DC-13A4-D1C7', '2019-06-19', 18.41, 846),
  (1148, 'C04C-A001-4CF5-A206', '2021-02-21', 16.69, 1059),
  (956, 'B08E-CFE6-24CC-82E5', '2019-11-25', 10.67, 847),
  (803, 'DFEC-23AD-4398-95F1', '2021-11-01', 14.07, 848),
  (131, 'C269-8E92-4610-2442', '2021-04-20', 5.22, 849),
  (1106, 'DE6E-AB05-BB24-EED3', '2020-07-23', 7.29, 850),
  (1551, '7390-1A9D-0462-FB9D', '2015-01-12', 6.37, 851),
  (706, '0143-DD93-4A66-1789', '2016-04-16', 18.41, 852),
  (707, '30A8-CA52-3740-8B36', '2014-10-06', 7.57, 853),
  (191, '454E-D2A5-36AB-2EA6', '2021-01-03', 6.90, 854),
  (1484, '7932-5F0A-1327-8874', '2021-05-18', 15.12, 855),
  (1074, '37D9-B9B8-4CD0-9F9B', '2005-02-20', 9.81, 856),
  (1559, '9C7F-A1AD-E717-6D86', '2012-01-13', 13.17, 857),
  (673, 'D0CC-9A6E-6380-69DB', '1993-03-07', 10.60, 858),
  (53, '8B99-08EC-F4B5-98CC', '2015-09-03', 16.94, 860),
  (165, 'C8E6-C257-A7BE-8E83', '2016-01-27', 14.05, 861),
  (970, 'AD8F-6F1C-45E9-BF3B', '1975-05-20', 6.82, 862),
  (971, '5B0E-23B4-19E9-1889', '1990-11-26', 9.05, 863),
  (1388, '64CB-B0F0-1649-CDC8', '2016-10-21', 6.97, 864),
  (1274, '94AA-838D-3352-FC35', '2018-07-23', 16.33, 865),
  (1514, '74AC-785A-E83F-DD36', '2021-10-29', 19.85, 866),
  (1423, 'EEE1-3CF9-34C5-0C8A', '2014-05-05', 10.59, 867),
  (1233, 'C388-A4E7-9115-FB3D', '2019-03-16', 6.41, 868),
  (551, 'E67F-D0E3-4732-0993', '2014-02-01', 7.72, 869),
  (1117, '0266-9A7F-8677-2518', '2015-11-13', 10.73, 870),
  (1118, 'AE58-DCA1-1294-579C', '2020-07-19', 9.93, 870),
  (525, '35AF-65C1-65F5-6EFE', '2022-07-15', 18.22, 871),
  (1125, 'FE2F-A9B1-848C-1C0A', '2001-09-09', 16.10, 872),
  (1124, '33B8-1C79-20E4-20D1', '1996-02-18', 18.34, 872),
  (1229, 'ED0F-E9B3-FEE2-4BF2', '2021-03-29', 13.85, 873),
  (1619, 'A6EE-BFB5-80C2-58C5', '2018-07-11', 9.33, 874),
  (151, '1E1B-97F8-5B6A-8929', '2020-04-08', 14.80, 875),
  (1184, '5258-7BD4-C316-0183', '2013-03-18', 16.96, 876),
  (1075, '9965-8A00-103C-E97F', '2022-12-07', 19.48, 1060),
  (1185, 'AEA4-5D41-6621-63D4', '2020-12-11', 5.43, 877),
  (1664, 'F98A-7692-7A57-4AA0', '2002-10-06', 12.95, 878),
  (1665, '458C-DB9E-B7D0-2211', '2011-12-16', 6.79, 878),
  (185, '9930-C4D3-B482-8FC7', '2015-03-13', 18.52, 879),
  (1101, 'AFBE-3EE8-0598-1EBA', '2017-10-07', 8.62, 880),
  (769, '165C-E0C2-D0C3-0D55', '2003-08-26', 19.04, 881),
  (1425, 'C424-B508-0CE0-1B90', '2022-07-19', 11.63, 882),
  (238, '2E4A-E7CA-F89C-D43B', '2021-12-22', 9.31, 883),
  (1249, '5EE7-A12C-138C-9B4B', '2013-12-31', 5.22, 884),
  (1108, '117C-5299-4272-51C8', '2018-10-21', 16.49, 885),
  (326, '6B0B-8DAD-8AEC-0357', '2003-07-15', 18.66, 886),
  (347, 'B303-BC47-0DB2-C1C4', '2012-07-15', 11.74, 887),
  (951, '2491-9822-3A59-D847', '2019-11-18', 17.15, 888),
  (431, '73D8-4C17-6CEA-A37F', '2022-12-29', 13.72, 889),
  (432, 'C166-B3FF-031E-0F04', '2022-12-15', 15.14, 889),
  (497, '7D3E-CC41-5939-F457', '2000-12-17', 10.90, 890),
  (498, '00B0-6C6B-A8BB-974C', '2017-08-27', 5.03, 891),
  (201, '9BA1-9D9E-9EBC-36D2', '2021-07-31', 13.16, 892),
  (1625, 'F7A1-65BD-7A7C-E95B', '2019-06-09', 11.56, 893),
  (905, '32C1-A2D7-2D2E-E2A8', '2019-09-10', 10.50, 894),
  (1029, '9B22-684F-23CC-8B3B', '2020-11-01', 19.82, 895),
  (1690, 'C859-8A17-A015-032F', '2011-10-01', 14.72, 896),
  (661, '38FC-DCE6-AEB7-352B', '2005-11-28', 5.30, 897),
  (1056, 'B0FA-6068-C9E4-D6EA', '2020-01-20', 15.74, 898),
  (1057, '2069-742D-42BF-3441', '2020-07-31', 9.78, 898),
  (1083, '21BA-D781-3768-EBE0', '2019-07-06', 9.82, 899),
  (1168, '35E6-3C82-2EDF-BCDB', '2015-04-14', 5.46, 900),
  (1167, 'C626-CB17-2561-CF80', '2013-11-09', 16.71, 900),
  (169, 'EF1F-F895-04E9-4AAF', '2008-12-18', 8.04, 901),
  (170, 'E079-9A0B-7316-EBDE', '2021-01-31', 12.66, 902),
  (260, '99EA-C514-208A-AFB2', '2015-05-07', 7.80, 903),
  (1197, '87E3-1055-12C5-8B69', '2017-10-27', 12.82, 904),
  (828, 'E534-64E5-E7DF-817C', '2022-04-12', 9.05, 905),
  (830, 'BE1F-471C-5593-1B36', '2022-04-21', 8.84, 905),
  (829, 'B6E5-FBD8-116B-D300', '2021-01-31', 17.91, 905),
  (1401, '6CBC-3133-5C7B-8827', '2009-08-26', 15.57, 906),
  (1402, '405F-49F7-93E4-A511', '2016-12-09', 13.85, 907),
  (588, '4BC3-69A3-5923-1762', '2021-10-08', 15.46, 909),
  (698, 'E174-E7ED-493D-3A8D', '2012-05-02', 19.15, 911),
  (1366, '892A-C492-38D3-1509', '2022-02-24', 7.77, 912),
  (1552, 'A5EC-2A1D-C552-9F8A', '2017-02-24', 9.65, 913),
  (1553, '3C57-A3BB-5724-2B6C', '2019-06-01', 7.21, 913),
  (871, '5D17-17F8-CD9D-F72C', '2020-09-27', 5.93, 914),
  (1607, 'EAC9-F3E0-A2A2-1ADA', '1999-10-17', 6.28, 915),
  (1173, '0C7A-E47C-AD0B-8E8A', '2016-02-22', 15.81, 916),
  (1172, '7F06-17D9-2CC2-BCE3', '2019-03-14', 6.81, 916),
  (26, '24C7-C41D-FF03-6089', '2002-09-03', 9.55, 917),
  (1594, '9152-A811-143B-518C', '2007-01-01', 9.71, 918),
  (958, '868C-D022-78E8-1CED', '2014-12-08', 8.20, 919),
  (1147, 'AF48-C963-725E-2D48', '2008-08-28', 19.24, 920),
  (876, '8C66-BB6C-6886-88E0', '2002-04-12', 10.00, 921),
  (1349, 'BD36-88FC-3D88-C304', '2008-10-16', 8.40, 922),
  (1351, '93C1-4EED-2DB1-E8C9', '2022-08-07', 17.99, 922),
  (1350, '3ADE-0088-0899-4F5C', '1981-07-23', 7.68, 922),
  (1098, 'DD90-6A55-AF5C-03D7', '2006-10-29', 7.27, 923),
  (1170, 'CC9F-CFA5-38A1-578D', '2015-08-05', 13.21, 924),
  (1171, '6F2E-74C3-0394-D074', '2020-05-07', 5.82, 924),
  (57, 'DCC9-181C-E456-273F', '1997-04-07', 9.48, 925),
  (1282, '9DEA-6C12-FF60-8411', '2022-11-02', 19.34, 926),
  (1281, '98E1-F215-AE5D-A88E', '2022-08-04', 10.39, 926),
  (1662, '0FD2-9787-09C4-302F', '2010-04-11', 17.00, 927),
  (1663, '0663-37E1-DE96-3C91', '2012-09-18', 17.05, 928),
  (942, '3119-9FD3-E066-6FF5', '2022-01-13', 16.87, 929),
  (528, '60A9-767B-D666-8F0A', '2018-03-10', 13.66, 1166),
  (943, '0CD9-A1D5-E60B-8911', '2011-06-12', 11.72, 929),
  (371, '4D7A-DB70-CA69-9C7F', '2014-09-18', 9.83, 930),
  (115, '797F-6AF9-5478-C35D', '1986-05-18', 12.01, 931),
  (123, '7AFC-C5EA-11E9-48B0', '2002-06-23', 8.65, 932),
  (1534, '3717-AFA2-5E89-A953', '2017-07-24', 14.73, 933),
  (389, 'BC92-4889-C410-D51E', '2014-02-26', 7.48, 934),
  (1030, '6999-FED8-3094-0E46', '2004-05-24', 10.97, 935),
  (1395, 'C0F0-8C97-5D1C-C444', '2022-12-09', 10.92, 936),
  (1396, '23E0-D2D2-FCFC-D647', '1980-11-01', 19.89, 936),
  (60, '6A22-8DBA-F3B8-2EEC', '2021-11-07', 19.61, 937),
  (1621, '5A12-D8E2-AB4A-8795', '2015-02-25', 5.23, 938),
  (1220, 'BB6B-1BB8-71F7-8DAF', '2003-01-06', 6.42, 939),
  (215, '1F86-251C-5F94-596A', '2012-12-31', 6.14, 940),
  (213, 'ACCC-99FA-CE81-7079', '2017-08-14', 9.51, 940),
  (214, 'DBF6-D03C-7697-8EBD', '2022-05-09', 13.41, 940),
  (22, '54F0-DCC4-F5C6-4140', '2019-05-18', 7.77, 941),
  (606, 'A342-48A7-6D11-2E6E', '2021-01-02', 5.49, 942),
  (1687, '86B2-C5B4-00A7-CA3B', '2012-12-27', 18.98, 943),
  (1686, '4A4E-E34D-4EB7-BD36', '2016-01-30', 15.72, 943),
  (41, '639B-F93C-4B10-153F', '2014-02-02', 12.40, 944),
  (42, 'F6DD-0964-3420-D2C6', '2005-09-23', 14.05, 944),
  (1443, '9503-1460-6028-CF7E', '2016-03-06', 19.53, 946),
  (1444, '615E-8530-A976-3F8F', '2020-10-06', 17.10, 946),
  (1442, '2539-13B8-DDE8-42B2', '2020-02-06', 19.52, 946),
  (992, 'F30D-BC52-0A8C-EE45', '2011-04-26', 9.76, 947),
  (265, '44B9-A1E6-8B89-92B2', '2016-04-25', 8.76, 948),
  (264, '840E-EECB-2DAB-D33C', '2014-10-03', 14.51, 948),
  (1587, 'EC1C-4A71-BB2B-3882', '2003-05-13', 12.37, 949),
  (1588, 'F9D1-E926-CEE3-FD22', '2005-01-05', 13.20, 950),
  (1711, '5EA4-4459-6D9F-D17F', '2015-07-09', 12.32, 1194),
  (1589, '3BAF-FC6D-21FF-77C2', '2005-04-25', 8.52, 950),
  (1660, '6F74-112E-07CE-2DE3', '2001-01-19', 16.31, 951),
  (817, '43D8-FB02-3662-756B', '2020-11-17', 11.63, 952),
  (1139, '62AE-EB8F-2726-7207', '2001-05-03', 10.84, 953),
  (1138, 'FB1F-448B-E725-9975', '1996-09-07', 19.66, 953),
  (1135, '180B-0FE2-41DA-9189', '2001-03-19', 15.76, 954),
  (1422, '7ECC-9E81-56FB-2DD6', '2015-10-29', 12.67, 955),
  (1639, '137F-FE43-8582-7320', '2021-07-08', 10.63, 956),
  (1640, '782E-7D13-2A37-081A', '2016-01-08', 18.01, 957),
  (1256, '9449-6478-8A7E-D7F7', '2004-05-13', 14.02, 958),
  (1257, 'EA57-4CBA-78F9-A68E', '2007-07-11', 10.07, 958),
  (1046, 'F8E5-2F9D-F9A3-BDF0', '2014-06-02', 15.82, 959),
  (1047, 'BBEB-1224-D16E-123E', '2014-09-26', 5.75, 960),
  (1458, '0AB7-1D1B-6499-02DB', '2003-02-03', 16.24, 961),
  (1459, '5FF0-5C45-6698-FCAD', '2017-02-03', 14.56, 962),
  (1461, '47FD-AE7F-3860-4D23', '2017-11-04', 13.87, 962),
  (1460, 'BF9F-1E99-0AB2-F30C', '2012-07-20', 15.13, 962),
  (459, '7B8B-F18A-9629-AECA', '2018-07-29', 14.40, 963),
  (458, '1847-13D0-56F1-678D', '2011-08-06', 18.42, 963),
  (1345, '82E8-2430-4BBE-F431', '2020-11-25', 8.72, 964),
  (1024, '0CBB-4C4E-F89B-32C7', '2012-03-22', 13.43, 966),
  (836, '9EAA-EF3D-F97F-6207', '2021-10-21', 11.51, 967),
  (919, 'A07D-BE75-8FC3-CCB3', '2015-11-20', 11.11, 968),
  (920, '1842-B7C8-F0F4-AC88', '2014-08-10', 13.27, 968),
  (4, '962C-9F4F-E892-574A', '2012-04-02', 10.50, 969),
  (3, 'FDD6-C43D-82BC-9ABA', '2015-04-18', 5.35, 969),
  (350, '0F2F-6C8D-F5FC-D924', '1993-01-15', 6.40, 970),
  (351, '126A-D455-4763-84F9', '2016-04-04', 14.61, 971),
  (892, '60C3-8BF3-3AD8-BD42', '2019-04-05', 17.86, 972),
  (1585, 'EE52-4342-F78F-C1F2', '2022-08-19', 16.44, 973),
  (1586, '4340-752C-9743-E919', '2017-01-18', 19.01, 973),
  (1237, '41DD-6CD3-2665-E180', '2019-10-21', 19.58, 975),
  (940, '1972-E93D-5762-6A87', '2021-12-21', 14.13, 976),
  (941, '6664-BBC7-702D-2D54', '2022-05-16', 13.60, 977),
  (656, '454B-3006-59E1-5870', '2022-05-28', 5.32, 978),
  (1627, 'A564-2651-558A-0DD5', '2022-11-28', 16.74, 979),
  (911, '9AFF-9A51-98B1-14DA', '2012-05-31', 17.41, 980),
  (338, '6E0B-F4C3-8B1A-63A6', '2000-12-26', 17.24, 981),
  (1491, 'C8E7-4AF1-BD30-2BF8', '2022-04-27', 11.19, 982),
  (266, '6914-8D8B-DE4B-593E', '2013-02-19', 16.19, 983),
  (1481, 'A04C-9FAE-DE48-B32F', '2017-01-28', 9.45, 984),
  (1295, 'A68C-CA1B-6F10-3247', '2020-09-06', 19.27, 985),
  (465, 'FB1A-B6B5-534F-6641', '2020-05-17', 11.73, 986),
  (879, '27E2-3507-BC3B-0E65', '2020-04-25', 10.96, 987),
  (880, '6260-140D-C9D4-190B', '2022-10-29', 16.74, 987),
  (412, '20BF-60B2-F6F1-E3D1', '2009-06-17', 5.82, 988),
  (411, '1D23-31AD-1F16-64EC', '2021-06-11', 5.69, 988),
  (410, 'C6BF-91F3-1ACD-D322', '2019-09-21', 18.29, 988),
  (683, '02AD-BC72-4C72-AF9E', '1990-08-03', 8.49, 989),
  (273, '44A0-F210-0989-BEF9', '1997-12-10', 17.44, 990),
  (1064, 'D5B5-00E6-16DB-CBDB', '2011-03-21', 11.97, 991),
  (1065, '1DFD-9BA0-1530-062E', '1973-07-01', 10.60, 991),
  (1681, '9C9E-B909-7589-2EFB', '1994-05-23', 17.35, 992),
  (1080, 'C43F-2C8C-7EB8-71AB', '2008-04-05', 17.74, 993),
  (490, '937B-48AA-8019-0990', '2003-03-01', 7.50, 994),
  (533, '9D28-3701-C793-1BE2', '1987-07-27', 12.02, 995),
  (1412, 'B418-D14B-5443-9C95', '2005-07-08', 18.60, 996),
  (1116, '49EA-BA87-8C54-28C8', '2013-03-05', 17.71, 997),
  (1241, '5B47-46D5-F9A9-DA76', '2022-08-17', 6.80, 998),
  (475, 'D5FA-2096-27B5-BC31', '2020-10-26', 18.21, 999),
  (474, 'ED85-E7C5-5CD9-1507', '2019-04-12', 18.86, 999),
  (835, '7A18-F20F-14C5-B16F', '2019-08-03', 6.70, 1000),
  (724, '3A28-470C-D534-EF48', '1997-12-28', 17.59, 1001),
  (723, 'F9EF-9F54-093E-BB7F', '2012-11-04', 11.67, 1001),
  (725, 'D852-5B88-51A7-D133', '1989-04-06', 14.43, 1001),
  (159, '516A-EEAF-F88F-3F43', '2020-05-10', 18.79, 1002),
  (1528, '72A6-BFCB-891A-3679', '2020-07-17', 19.40, 1003),
  (663, 'CCFC-940B-3C48-4F70', '2022-06-05', 5.22, 1004),
  (662, '25AD-5C42-E859-7557', '2022-05-26', 10.54, 1004),
  (742, 'AED5-DF12-6C25-DD61', '2020-05-20', 5.01, 1005),
  (406, '9AAB-7CD1-FC63-F3F9', '2022-04-28', 16.43, 1006),
  (405, 'A67A-12EA-14B7-E36C', '2021-04-07', 10.01, 1006),
  (540, '7066-55E9-6E47-3FB8', '2010-05-24', 5.73, 1008),
  (1545, '0BD9-5F3F-BB6A-2698', '2019-12-26', 16.90, 1009),
  (1547, '440F-1145-ED06-46D2', '2020-06-18', 17.09, 1010),
  (1546, 'D597-36DD-15E7-67A7', '2022-07-18', 10.24, 1010),
  (775, '5621-C93B-362A-6633', '2013-08-24', 10.90, 1011),
  (776, '685E-070B-E4EA-2F4A', '2018-06-14', 16.37, 1011),
  (1689, 'C8B0-2E1E-C023-221A', '2020-02-01', 17.34, 1012),
  (321, '0442-96CD-79A9-B8C8', '2017-07-12', 5.01, 1013),
  (978, 'DB03-E79F-194C-D819', '2022-04-06', 15.38, 1014),
  (979, 'F376-945D-8D48-3B2A', '2021-01-02', 16.09, 1014),
  (977, '1286-9AFB-AA6B-0750', '2019-04-26', 17.62, 1014),
  (630, 'FE8E-BEF6-E733-0791', '2019-06-20', 11.20, 1015),
  (1120, 'B886-24AB-4DFD-660F', '2018-05-25', 12.14, 1016),
  (1434, '0CFB-FDED-2E39-F0C2', '2012-04-11', 17.95, 1017),
  (1014, '04DA-82E8-529A-5B27', '2022-02-09', 8.06, 1018),
  (1015, 'BF27-CD36-C75B-2770', '2013-05-10', 8.89, 1018),
  (1013, '069F-67AB-90D1-DF74', '2016-06-08', 19.68, 1018),
  (907, '9CE7-E33E-36CE-2705', '2022-07-19', 16.25, 1019),
  (623, '6484-0B15-F8C7-A836', '2016-08-03', 6.86, 1020),
  (1432, 'E586-6FF0-A600-E1D5', '2018-08-07', 15.25, 1021),
  (1242, '1199-73B8-011E-466B', '2022-05-27', 17.20, 1022),
  (664, 'EC9B-98EC-5412-29AF', '2020-02-10', 19.25, 1023),
  (89, 'C83A-9B21-A27D-C558', '2021-10-24', 8.50, 1026),
  (1048, '1CD0-002E-54FE-E7A6', '1998-12-09', 8.04, 1027),
  (1702, '82A8-AFFE-17BB-FAF0', '1983-08-22', 18.91, 1028),
  (1604, 'FC94-319D-B7BA-2CED', '2020-04-02', 8.10, 1030),
  (1605, '557F-B4D7-85AE-2D78', '2019-04-14', 13.32, 1030),
  (1606, '0B56-18FD-671F-934B', '2003-09-26', 17.37, 1030),
  (368, '706A-7349-47D5-B397', '2019-04-17', 12.94, 1031),
  (74, 'CB64-8DD4-DD4F-54D2', '2003-09-04', 18.60, 1032),
  (75, '11FC-886F-ACDD-33A4', '2017-12-11', 5.89, 1032),
  (104, '55DA-7759-9D42-F5C7', '1995-07-05', 15.92, 1033),
  (105, '966C-A12E-9D84-5EA9', '1994-12-31', 7.47, 1033),
  (202, 'D9E5-84E8-28F3-B301', '2022-10-26', 6.90, 1035),
  (1591, '6B1B-BCE2-BABF-5B3D', '2011-11-19', 13.82, 1036),
  (1078, 'B9EB-9570-9DFB-10F7', '2017-03-22', 7.49, 1037),
  (832, 'B464-8049-1D5E-1418', '1995-06-15', 11.01, 1038),
  (833, 'C1FA-4063-73DF-629C', '1992-01-06', 18.87, 1038),
  (834, '4BD0-0784-3936-7404', '2003-11-21', 13.07, 1039),
  (1549, '8F8D-FA02-241B-1623', '2018-03-31', 11.14, 1040),
  (1685, '8700-D94D-150A-5421', '2010-11-30', 6.92, 1041),
  (1085, '0438-EAC0-7C30-DDD1', '2018-06-02', 14.88, 1042),
  (1086, '444A-BE22-DE21-4F89', '2022-02-15', 10.64, 1043),
  (1087, '4F5B-2F4C-D6EF-659D', '2022-08-12', 7.50, 1043),
  (1044, '5064-4BA2-3527-5E7F', '2020-04-30', 18.77, 1044),
  (1213, '6EC6-4FC2-53D9-0B07', '2020-07-13', 8.82, 1045),
  (1246, 'D45C-59AD-B773-E9B0', '2013-07-21', 18.41, 1046),
  (303, '7D8F-93F8-F3E8-3651', '2011-04-24', 16.48, 1047),
  (713, 'EF3F-4BEF-E57B-1D87', '2010-11-25', 15.37, 1048),
  (712, 'C0A2-C33E-BD9B-414B', '2014-09-26', 13.84, 1048),
  (711, 'F7B1-7836-8FE4-72C0', '2018-04-03', 11.00, 1048),
  (821, 'F6EE-C860-B38E-B83B', '2008-11-04', 8.80, 1049),
  (1037, '1F7C-0CCB-4BBD-6BAF', '2012-06-03', 17.10, 1050),
  (1141, '7296-6374-3D8C-DF5C', '2017-08-19', 19.47, 1051),
  (912, '4F44-99F2-E1FE-887C', '1994-08-22', 10.84, 1052),
  (228, '6B26-FEAA-B18E-68FD', '2020-08-26', 13.41, 1053),
  (409, 'DA33-A9D3-A739-BD8F', '2018-07-01', 16.56, 1054),
  (1319, 'DA90-81A3-CD8D-F744', '2019-01-15', 12.56, 1057),
  (1318, '42E6-8719-4F8C-65AF', '2021-05-02', 14.18, 1057),
  (735, 'B840-0199-81B9-AF4A', '2022-11-23', 9.65, 1058),
  (734, 'C87F-A493-E211-A616', '2020-09-18', 8.07, 1058),
  (1494, '46D6-C45E-A1CF-B9F4', '2001-08-03', 9.17, 1061),
  (120, '0E4E-E097-8DBF-7953', '2018-10-22', 6.45, 1062),
  (1392, '442D-C873-79D0-F92E', '2013-05-25', 16.89, 1063),
  (402, '428B-29A7-13CB-B0CD', '2007-01-17', 16.89, 1064),
  (403, '42ED-487F-D1E2-9174', '2016-07-20', 10.59, 1065),
  (404, '58D6-23D8-8512-0BB1', '2017-08-17', 11.78, 1065),
  (641, '2AA0-8996-1D64-DEB9', '2006-01-27', 12.86, 1066),
  (11, '26EF-8645-C733-B1A2', '2012-10-01', 16.90, 1067),
  (1414, 'CE3D-ABF9-ECBD-52C7', '2016-09-26', 14.23, 1068),
  (1415, '0C6F-6885-FB71-23FA', '2019-10-14', 9.96, 1068),
  (1416, '4BFD-7B11-823B-2CE4', '2019-05-02', 15.07, 1068),
  (1082, 'FE39-4AEC-6CD5-06D8', '2000-12-16', 9.62, 1069),
  (343, '236C-84F3-3748-28D0', '1991-10-05', 10.79, 1070),
  (774, 'A81C-4752-26B4-0B2B', '2020-06-09', 6.28, 1071),
  (1445, 'D107-0929-65AD-AAC3', '2017-11-05', 5.49, 1072),
  (603, '9C2A-2DC1-5359-159F', '2008-05-23', 17.70, 1073),
  (604, 'DE33-FF2E-96B1-74DA', '2021-07-18', 15.84, 1074),
  (605, '71AE-CECF-58E8-30E6', '2014-08-01', 18.87, 1074),
  (1287, '173D-ADA5-F9CA-3AF3', '2014-06-26', 18.79, 1075),
  (38, '06EB-FCCC-5327-A670', '1983-10-06', 9.39, 1076),
  (1465, 'A5B5-0693-EC7E-8C12', '2015-07-09', 13.87, 1077),
  (1093, '87BA-7BBA-2D43-03E6', '2009-08-31', 17.84, 1078),
  (1437, '2B17-B965-4A91-1730', '2018-03-08', 11.77, 1079),
  (556, 'C989-A677-E787-453B', '2021-12-30', 6.47, 1080),
  (558, 'EA1D-D2D8-34E9-15CC', '2020-03-12', 14.28, 1081),
  (557, '0F42-F430-E2C0-FD5E', '2020-12-22', 9.80, 1081),
  (271, '713D-12CE-2678-7EFF', '2017-09-30', 19.81, 1082),
  (272, '4AC0-7518-5105-161B', '2017-05-31', 16.29, 1082),
  (270, 'F898-BA2F-D5AD-526F', '2018-03-03', 13.45, 1082),
  (1000, '7391-F9AC-7FBC-B9C2', '2012-05-10', 5.63, 1083),
  (974, '35E8-CA4C-B255-0CEE', '2008-05-04', 14.52, 1084),
  (1321, '8140-AA35-5831-C671', '2016-10-11', 13.01, 1085),
  (1226, 'B55B-9091-C36A-CD4B', '2021-04-30', 10.74, 1086),
  (1019, 'B042-8FDA-515B-7D02', '2021-01-08', 9.90, 1087),
  (96, 'B075-8ADC-B8CB-3AC8', '2017-05-12', 17.46, 1088),
  (95, '6D6E-5119-EC30-8872', '2019-10-16', 11.70, 1088),
  (1452, '1E74-65E8-D37A-2B05', '2018-06-13', 14.93, 1089),
  (440, 'C944-0B26-FA20-DEDE', '2020-11-17', 8.92, 1090),
  (441, '0236-22AA-4A94-B16B', '2022-03-07', 11.72, 1091),
  (442, '21AF-D394-C689-2CA1', '2022-11-14', 18.38, 1091),
  (211, '3593-E9EE-FBB6-8F5F', '2017-05-27', 5.04, 1092),
  (210, '5E9A-FF83-F7D8-2ADC', '2017-03-11', 7.42, 1092),
  (212, 'B6E1-7650-F724-00E1', '2020-11-21', 12.46, 1092),
  (243, '9674-B730-4683-BFAE', '2015-08-07', 12.32, 1093),
  (1312, '0957-59CF-A344-3693', '2020-10-23', 6.61, 1094),
  (1313, '50B3-F1A9-1C3E-4630', '2008-10-28', 14.12, 1095),
  (1314, '212D-1225-C904-7992', '2022-11-02', 5.29, 1095),
  (370, 'D570-1219-96A8-C56F', '2021-08-23', 15.12, 1096),
  (1595, 'EA77-6BCB-DD48-9EBA', '2022-08-28', 13.37, 1097),
  (1005, '00D0-F8C0-C83F-738B', '2022-07-12', 18.86, 1098),
  (288, '800C-A6B7-1B88-E574', '2019-10-03', 12.88, 1099),
  (289, '7D15-20A5-3DCC-688C', '2021-10-12', 9.00, 1100),
  (1611, '1B0C-148D-C130-8580', '2022-10-04', 17.46, 1101),
  (1612, 'FD33-F203-8FD1-AB43', '2016-06-18', 7.92, 1102),
  (1610, '8BE2-58D6-CD95-14F5', '2021-05-07', 6.34, 1103),
  (1136, '80F8-0184-5211-731D', '2022-05-15', 18.10, 1104),
  (1211, '3398-921C-879D-2CF6', '2013-01-02', 9.28, 1105),
  (510, '9E79-7C27-0178-6F14', '2015-04-14', 8.45, 1106),
  (780, '2F8A-858C-1FF5-1C3B', '1999-07-29', 15.53, 1107),
  (1002, '1E04-973C-535E-1BCF', '2016-11-16', 11.32, 1108),
  (1105, '0F96-EB40-DED2-2879', '2005-11-29', 15.38, 1109),
  (358, 'B153-F7A6-AF41-7538', '1987-08-18', 16.16, 1110),
  (1210, 'EC5D-29B3-AFF6-0F71', '2009-05-06', 10.97, 1111),
  (156, '0C1C-31C3-6D20-47F4', '2017-02-19', 17.81, 1112),
  (822, 'ED1C-4D77-5DCD-F99E', '2018-08-25', 14.01, 1113),
  (878, '3191-C624-BDBD-094D', '2018-01-02', 10.67, 1114),
  (387, 'DD3B-40B6-30BE-3876', '2021-10-14', 14.31, 1115),
  (208, '1050-F374-E312-28C9', '2003-05-15', 14.58, 1116),
  (758, '55AB-C878-F0A7-D30A', '1990-01-09', 13.42, 1117),
  (759, '0887-05FF-3CC7-EC3D', '1984-07-05', 19.57, 1118),
  (760, 'E5B5-3190-C861-AEA2', '2010-05-08', 13.54, 1118),
  (449, '10B2-ECF0-B990-F513', '2002-01-17', 8.24, 1119),
  (1155, '8CB9-640F-41B8-AC66', '2017-07-19', 16.53, 1120),
  (601, '1AED-10C8-787B-09E5', '2011-08-15', 16.24, 1121),
  (602, '1975-3282-7C2E-28CD', '2017-02-05', 19.34, 1121),
  (1467, 'AB64-C40C-0DA5-197F', '2019-03-05', 13.46, 1122),
  (658, 'C8B2-6B4F-ED3C-EF0B', '2022-01-18', 14.08, 1123),
  (1448, 'D3C6-42E8-ECE0-5ED3', '1996-01-12', 16.06, 1124),
  (894, 'DA9C-29C5-49AF-8256', '2013-06-10', 18.46, 1125),
  (1270, 'FE34-0ECF-44D2-4ADE', '2002-08-24', 9.19, 1126),
  (312, '5480-3A82-C690-5904', '1985-02-22', 10.49, 1127),
  (1298, '1E32-563C-CE5E-D676', '2003-04-28', 13.81, 1128),
  (352, 'FCAA-6530-2F1F-AA7A', '2017-12-31', 13.12, 1129),
  (837, '5262-6ACD-76B7-78EB', '2015-04-20', 17.11, 1130),
  (838, '7532-627E-E3D7-56BA', '2012-01-11', 15.22, 1131),
  (858, 'E25E-A52E-01CD-E840', '2019-01-27', 14.57, 1132),
  (547, 'C78A-7AE0-A4A1-AC7A', '2008-11-10', 19.12, 1133),
  (496, '56DA-7041-A8F6-C0E5', '2022-12-24', 14.49, 1134),
  (495, '5549-CB48-2A9B-C27A', '2021-04-20', 5.48, 1134),
  (274, 'C3D8-EFC7-720D-9D2A', '2018-04-14', 11.02, 1135),
  (934, '4C8B-2DF4-E4DA-AE36', '2022-11-13', 8.50, 1136),
  (279, 'F834-9EFB-115F-ED0A', '2018-05-02', 10.86, 1137),
  (278, 'CDA0-94FD-8B12-743A', '2006-04-13', 11.94, 1137),
  (453, 'F39A-3F54-1012-3E51', '2017-02-09', 8.60, 1138),
  (1072, '4F06-CEE0-BE24-2DF2', '2022-06-30', 13.13, 1139),
  (655, 'F291-687C-429C-46F9', '2012-01-20', 16.38, 1140),
  (456, '7594-2BDD-377A-2563', '2017-10-25', 6.61, 1141),
  (1340, 'BFFA-905F-DE2C-8DE1', '2010-08-09', 7.93, 1142),
  (689, '530B-1424-7A5C-20BA', '2011-05-24', 19.90, 1143),
  (690, 'EFBA-D690-691E-038A', '1992-06-22', 9.89, 1143),
  (691, '6CDC-FC19-5E90-E53A', '2003-08-20', 19.35, 1144),
  (1397, '5FDB-6CCA-5B57-DCA1', '2017-09-15', 7.36, 1145),
  (768, '1612-C883-03C6-2ED5', '2001-06-18', 14.58, 1146),
  (1578, 'DE59-9DB2-F7FF-9F6C', '2009-07-06', 5.71, 1147),
  (668, 'B135-19F9-1C1D-7DCF', '2021-10-31', 12.73, 1148),
  (667, '1959-37F1-68E8-5CFD', '2022-04-14', 15.93, 1149),
  (1413, '7CC3-014F-E325-72FF', '2017-08-23', 14.92, 1150),
  (975, 'C140-DEB1-C8A9-1E16', '2020-12-08', 15.91, 1151),
  (318, 'E9CB-A091-BA2C-F4F6', '2021-05-25', 16.60, 1152),
  (319, 'AD20-0F0B-266A-99F5', '2014-08-06', 10.25, 1152),
  (1576, 'B7EB-482A-8557-A065', '2020-06-04', 16.06, 1153),
  (1577, 'B9E7-98A6-B960-8B8C', '2018-08-14', 15.51, 1153),
  (1113, '0A0C-39B3-240F-1619', '2021-08-10', 17.01, 1154),
  (1307, 'B19E-32F6-687C-9B5A', '2022-11-30', 15.53, 1155),
  (1310, 'CF63-172C-F98A-B6BB', '1996-01-05', 14.47, 1156),
  (1309, '15F3-6E9D-CB32-F243', '2003-05-15', 9.32, 1156),
  (1672, '1A3F-EF89-3752-7FB5', '2022-06-05', 9.84, 1157),
  (446, 'F59D-3165-61F9-5C37', '2009-12-08', 9.31, 1158),
  (226, 'E163-B2CF-0CF9-44E7', '2002-02-01', 13.99, 1159),
  (572, 'C9D5-9D36-610C-CD7D', '2022-03-09', 6.92, 1160),
  (964, 'AFF0-26B7-8FD9-3F46', '2006-05-17', 13.64, 1161),
  (126, 'D4BA-10A4-0D78-57D5', '2007-12-27', 6.57, 1162),
  (1371, '1485-9D9A-6E1F-9167', '1997-12-16', 18.50, 1164),
  (1370, '5081-0824-2431-AEFF', '2003-03-29', 12.21, 1164),
  (527, '5843-10B3-7517-C6A7', '1999-07-12', 18.45, 1165),
  (583, '294E-7E78-722F-3440', '2005-04-26', 7.11, 1167),
  (1630, '4FE4-7388-279B-3D9C', '2022-01-06', 16.81, 1168),
  (782, '94FB-3E3D-6DF6-2E3C', '2018-12-27', 7.16, 1169),
  (1225, 'EC28-97D5-BB71-3705', '2008-06-14', 9.14, 1170),
  (927, '4054-5FE1-D769-1619', '2003-10-23', 5.27, 1171),
  (925, 'C317-A127-860A-6871', '2012-03-10', 8.62, 1171),
  (926, '66FF-E46D-B096-38A7', '2020-01-22', 13.66, 1171),
  (176, 'F00B-7946-E484-22A0', '2017-05-31', 7.19, 1172),
  (177, '91A6-2A6E-4E7A-1D4A', '2016-08-06', 6.73, 1172),
  (1111, '3202-7F9B-10C6-E1C9', '2019-09-21', 9.99, 1173),
  (251, 'A2E3-B268-14B9-87EA', '2020-11-12', 10.19, 1174),
  (152, 'D689-038B-DCBB-6BA5', '1991-02-25', 7.71, 1175),
  (727, '2278-95A2-D669-8ADC', '2013-05-22', 16.73, 1176),
  (847, '4A8B-E3AE-BD5F-CD93', '2016-02-26', 9.06, 1177),
  (482, '0260-49C0-9909-B552', '2017-06-27', 15.26, 1178),
  (1175, '748E-A873-45AA-A684', '2015-01-10', 13.01, 1179),
  (299, '37A4-A703-A9E8-58DF', '2021-02-01', 15.94, 1180),
  (316, 'F770-CB27-CA63-F703', '2015-10-22', 7.80, 1181),
  (317, 'E317-9E45-C4CC-C059', '2005-02-23', 8.25, 1181),
  (1280, '3A7B-D97D-B33B-EBCE', '2013-08-21', 17.40, 1182),
  (930, '7FD4-E0CE-6025-B6BE', '2008-05-24', 18.85, 1183),
  (931, 'A066-6B02-4F09-05F2', '2013-04-14', 10.77, 1184),
  (1433, '448A-1367-C513-181F', '2009-04-07', 9.26, 1186),
  (437, '2B37-0DF6-DF9B-4E5F', '2021-10-05', 16.72, 1187),
  (797, '0712-7A66-0B6A-89CF', '2001-05-17', 10.00, 1188),
  (798, '5983-3C09-43D1-9ED4', '2020-10-07', 5.91, 1189),
  (861, '10D4-601A-AF52-FF5E', '2012-04-24', 19.33, 1190),
  (31, '4F59-C4C9-D285-2077', '2020-09-19', 17.16, 1191),
  (32, '37F2-B2E8-9490-FF73', '2015-10-06', 9.98, 1191),
  (33, '7F5F-C9B6-D598-7BAC', '2012-03-20', 17.35, 1191),
  (448, '24E9-D56E-32B1-8271', '2000-01-29', 10.35, 1192),
  (1176, '7385-A048-3B38-6CBD', '2013-01-25', 8.71, 1193),
  (714, 'E6BC-1A6A-DC4F-081B', '2004-03-05', 10.66, 1220),
  (427, '898B-0872-D232-DFF8', '2017-09-10', 8.78, 1221),
  (1424, '2998-A4CD-B604-2D91', '2012-07-16', 16.38, 1222),
  (750, '234F-2DB1-65B2-8F2B', '2019-07-10', 10.47, 1223),
  (1076, 'A409-4BF8-3B22-F525', '2013-01-06', 8.76, 1224),
  (485, '0CB3-4EF9-1822-650E', '2008-04-14', 17.75, 1225),
  (1358, 'CF18-F589-254D-3475', '2000-09-17', 19.02, 1226),
  (1565, '781F-B62E-E40B-53E2', '2017-10-27', 7.24, 1227),
  (110, '953B-0FE6-2B61-8DE3', '2018-09-01', 15.02, 1228),
  (627, 'E4B5-6353-0BE3-372F', '2021-07-30', 8.71, 1229),
  (414, '3AF5-0BD5-C49B-2B85', '1991-12-12', 17.38, 1230),
  (535, 'FD0C-C273-C435-94F1', '1982-06-08', 13.13, 1231),
  (872, '87B8-78E7-BC9C-E93A', '2016-02-19', 14.43, 1232),
  (122, '48E7-5002-9BF0-8D54', '2008-08-08', 18.94, 1233),
  (794, '2490-471D-C008-A01A', '2005-06-19', 12.62, 1234),
  (793, '41DB-8015-2AAD-FC86', '1999-10-25', 13.77, 1234),
  (1095, '4BF6-A364-13A7-75BB', '2010-04-14', 12.25, 1236),
  (746, 'ABBE-668B-9F7D-EAAA', '2011-01-17', 16.93, 1237),
  (747, 'E487-E6D2-38F5-F4D3', '2022-12-06', 12.25, 1237),
  (748, '2A13-0042-B4F6-CAEB', '2019-04-14', 17.66, 1238),
  (1292, '6966-1242-64F8-E059', '2022-01-20', 11.56, 1239),
  (1623, '160A-9D9D-9986-BD00', '2016-05-14', 17.91, 1240),
  (1622, '27A5-96AC-42A2-6FF8', '2018-08-28', 18.18, 1240),
  (1161, 'B006-6E4C-5A41-F11E', '2022-03-13', 10.94, 1241),
  (1162, 'B710-3088-A5A2-A79C', '2019-10-22', 16.12, 1241),
  (561, '7F20-558F-7BDF-088B', '2019-01-24', 5.60, 1242),
  (1638, '03A1-1461-3ACE-0CEA', '2015-01-31', 10.97, 1243),
  (1590, '8293-24C0-8F82-B646', '2019-04-29', 13.75, 1244),
  (756, '7D10-1B32-F331-946F', '2001-06-21', 13.01, 1245),
  (1380, '054F-3125-CC50-03C0', '2021-03-23', 5.10, 1246),
  (1379, '829C-EC9E-A072-622D', '2017-04-13', 11.18, 1246),
  (962, 'F09D-9D4F-0B6F-0CC5', '2021-05-03', 9.77, 1247),
  (1191, '3D96-991D-49E9-FE20', '2003-01-06', 8.14, 1248),
  (996, '170C-9E62-EBDA-1EE8', '2008-11-18', 18.40, 1249),
  (1245, '1BF0-64EC-C69F-CEA1', '2009-07-15', 12.33, 1250),
  (1244, 'F87E-6DB0-6004-D427', '2008-03-17', 7.55, 1250),
  (1243, 'E396-EB82-2983-B988', '2017-06-04', 17.70, 1250),
  (716, 'E5E0-7F4B-262B-7F7E', '2002-03-23', 11.68, 1251),
  (717, 'FB90-1B6A-A73F-FD33', '2012-07-13', 17.09, 1252),
  (464, 'C5F4-4366-C16C-1517', '2019-09-05', 14.25, 1253),
  (1248, 'CA7C-F224-F0A1-C207', '2020-05-25', 8.80, 1254),
  (1670, '9A2E-AA1D-1A7A-D45D', '1994-06-01', 17.72, 1255),
  (478, '62D9-5A84-1B12-1BD9', '2016-02-05', 11.82, 1256),
  (506, '0647-54FB-510B-8E65', '2021-12-07', 17.54, 1257),
  (1495, '88CE-AA09-BCF3-8EAF', '2016-11-11', 12.08, 1258),
  (1132, 'C972-144A-FA3C-54E8', '2004-11-15', 13.31, 1259),
  (1133, '59D1-9DE6-83F9-97F9', '2006-01-17', 5.14, 1260),
  (1657, '6472-4C39-1755-8F26', '2013-11-04', 14.56, 1261),
  (710, '4E32-F9F3-8D93-EE07', '2011-05-14', 18.99, 1262),
  (709, 'D476-9599-867B-0FFB', '2020-09-04', 6.46, 1262),
  (23, 'BAB7-B3FE-6957-FE15', '2021-04-17', 17.12, 1263),
  (597, 'BBA4-C949-DF1D-93DF', '2014-01-27', 10.83, 1264),
  (127, '9617-8EDA-13BC-1DC0', '2010-04-08', 11.28, 1265),
  (128, 'A41C-BCC7-7C0C-DD77', '2016-09-20', 6.53, 1266),
  (129, '1C90-DA3F-7940-881B', '2018-12-06', 11.91, 1267),
  (906, '1A55-2732-312F-77FD', '2022-03-24', 6.79, 1268),
  (2, 'B587-3100-B3D1-0408', '2005-07-24', 18.31, 1269),
  (229, '6BBC-2C50-05DA-28DC', '2015-05-18', 15.72, 1270),
  (218, '93C8-3372-659A-784F', '2022-06-11', 19.00, 1271),
  (827, '43AC-0762-2B39-A7B6', '2022-04-22', 5.35, 1272),
  (155, 'ABE8-19AB-283A-0730', '2011-08-26', 9.70, 1273),
  (154, '16AD-67A7-9ED5-595E', '2012-08-17', 10.51, 1273),
  (153, 'FE4E-0661-3AD3-DFF4', '2014-11-14', 11.90, 1273),
  (1382, '5CC8-D188-3652-C134', '2016-02-18', 12.68, 1274),
  (1509, '9544-E23A-9062-5A12', '2016-05-18', 19.73, 1275),
  (315, '45A7-77E1-099E-0552', '2015-02-23', 7.81, 1276),
  (12, '8564-8CCE-846E-C62D', '2003-02-11', 13.45, 1277),
  (1231, '8CE3-6E4D-8010-5CD5', '2016-02-26', 5.62, 1278),
  (972, 'A579-AF9A-E118-8667', '2015-11-05', 11.72, 1279),
  (267, '37D1-839F-1AE5-D51B', '2004-10-05', 16.84, 1280),
  (1438, '5146-AC28-4B90-3F8C', '2021-11-01', 5.99, 1281),
  (30, '0FC0-D43E-73C0-CC6F', '2021-10-16', 12.87, 1282),
  (29, 'BB5D-6229-CE33-0194', '2017-09-12', 7.26, 1282),
  (28, '5AEF-6A79-D016-5E0E', '2021-02-13', 16.26, 1282),
  (1004, '92C3-059C-55D5-D665', '2019-10-06', 14.15, 1283),
  (108, '4F0A-FF6E-CB43-122F', '2017-03-31', 10.79, 1284),
  (333, '4AC8-78FA-856E-9096', '2014-12-15', 12.59, 1285),
  (332, 'B8BA-55DB-7375-3BD0', '2018-03-30', 6.93, 1285),
  (1378, '3E7D-4B17-719F-EC26', '2015-11-26', 15.20, 1286),
  (779, 'FC59-B610-678E-C582', '2011-06-27', 5.14, 1287),
  (1581, 'EB90-826A-7430-691A', '2021-11-15', 14.62, 1288),
  (314, 'E654-0D52-1979-C98B', '2018-07-10', 9.42, 1289),
  (313, '90C0-BB7D-F346-9313', '2015-12-07', 13.88, 1289),
  (205, 'D34A-C65C-9340-2A71', '2009-10-11', 11.76, 1290),
  (1142, 'E437-A0AC-417B-C740', '2020-01-24', 13.59, 1291),
  (1143, '0E03-4DA0-F6FC-BCC8', '2014-04-19', 16.50, 1292),
  (804, '98A4-6660-380D-4287', '2018-07-17', 8.90, 1293),
  (59, '68D2-7DC7-560A-1E9F', '2020-09-16', 9.99, 1294),
  (58, '903B-7277-E4D4-9B4E', '2021-12-20', 12.31, 1294),
  (1068, 'E2C3-422F-C072-50AD', '2007-06-29', 19.93, 1295),
  (1497, '72FA-B9D4-F80D-658E', '2014-11-10', 16.59, 1296),
  (1498, '0C19-685D-F25C-A497', '2013-12-07', 7.36, 1296),
  (1089, '3993-B859-0496-199D', '2017-07-06', 15.77, 1297),
  (5, '5117-5964-EFE9-534E', '2023-01-03', 18.28, 1298),
  (1471, '5279-C4CE-97B1-B648', '2017-06-19', 9.78, 1299),
  (1238, 'D6AA-911E-C343-0AC0', '2020-02-15', 18.99, 1300),
  (845, '2CCC-F49C-4C99-D063', '2011-07-16', 9.57, 1301),
  (268, '9182-7D4A-5EDC-5823', '2016-07-28', 14.44, 1302),
  (1530, '8096-3CB6-9122-B5CB', '2020-01-01', 14.91, 1303),
  (1531, '179A-2ADD-F82E-D521', '2022-10-10', 17.97, 1303),
  (166, '86A7-76BF-5817-1FEC', '2017-04-12', 11.13, 1304),
  (1190, '87B3-C1A0-3A7A-B884', '2020-11-04', 5.36, 1305),
  (1543, 'BF33-D6BD-0237-AC85', '1977-08-27', 15.05, 1306),
  (1194, 'F32A-CF0A-1E71-669A', '2019-07-10', 19.12, 1307),
  (1631, 'E342-9009-443B-35CD', '2019-12-22', 11.26, 1308),
  (542, '5D8A-3E22-E095-3BE6', '2015-02-22', 16.40, 1309),
  (380, '1D75-FFAD-DB1B-3481', '2010-12-26', 9.16, 1311),
  (379, '1E9C-5A50-EA1E-EC66', '2022-08-13', 14.33, 1311),
  (1451, '16A0-A509-B3EB-0748', '2019-09-18', 8.16, 1312),
  (1560, '6963-B0A1-DBDF-6F05', '2021-04-13', 14.50, 1313),
  (124, '4096-C16E-A9A6-53CB', '1995-12-18', 7.96, 1314),
  (631, 'A42D-9B75-6754-5C90', '2004-03-24', 5.98, 1315),
  (435, 'B1D1-F736-3C1A-A67F', '2006-09-10', 7.28, 1316),
  (434, 'FE78-9F49-FE62-7AE6', '2004-10-14', 15.39, 1316),
  (433, 'BF91-CD7C-AFCD-AC20', '2015-07-25', 8.55, 1316),
  (1421, 'D1A1-80D8-DFE4-C661', '2014-11-16', 6.24, 1317),
  (364, '993A-5FAF-82CE-9B8B', '2022-06-21', 5.97, 1318),
  (365, '9BF0-A316-5E19-6465', '2019-06-30', 14.36, 1318),
  (367, 'BF04-7820-693E-FECD', '2009-06-30', 15.67, 1319),
  (262, '5AA9-AF1D-E151-D287', '2022-02-18', 11.31, 1320),
  (263, '6FD3-7EB4-FD4B-E925', '1996-05-13', 18.22, 1320),
  (1010, '0D62-5D56-24E3-076A', '2007-09-26', 6.95, 1321),
  (1011, '69ED-E036-0454-7039', '2015-08-21', 11.23, 1322),
  (935, 'B786-3737-A1A8-49ED', '2015-11-25', 10.17, 1323),
  (473, '426E-8109-5C8D-EC77', '2020-09-15', 8.63, 1324),
  (52, 'CE2F-6479-58E2-784B', '2015-11-15', 8.89, 1325),
  (982, 'C18A-99A5-4289-71BB', '2015-02-18', 10.97, 1326),
  (342, 'B1FE-95BE-66EB-A756', '2016-10-12', 15.92, 1327),
  (348, '2004-BAAF-107F-A3D5', '2022-12-01', 8.99, 1328),
  (1272, '1033-2BE2-CE6E-A37E', '2020-02-05', 11.16, 1329),
  (842, '932F-6650-06C0-0B1D', '2012-08-11', 5.72, 1331),
  (843, '4282-ACB3-E55D-BCB2', '2019-04-15', 19.79, 1331),
  (844, '1B3E-6015-AE34-3326', '2022-12-13', 9.61, 1332),
  (63, '574A-A737-FE86-6113', '2007-09-01', 6.58, 1333),
  (62, 'BF78-2C59-C52E-5B66', '2003-05-03', 15.68, 1333),
  (1049, '0798-EEC1-4A5D-0FA9', '2005-07-26', 11.00, 1334),
  (868, 'D4CA-84CD-5B50-9DA7', '2019-08-29', 6.00, 1335),
  (1031, 'C0EE-9AA5-9C9F-DF24', '2018-01-10', 7.75, 1336),
  (1032, '5B76-B3D6-9167-CBCC', '2017-09-14', 8.89, 1337),
  (1164, 'A8DD-AE6E-722B-4C98', '2019-06-19', 17.83, 1338),
  (900, '7409-E957-7E26-86FC', '2020-05-09', 14.77, 1339),
  (899, 'E17F-F258-53FA-0848', '2022-11-16', 7.80, 1339),
  (1667, 'B105-5498-5333-17BF', '2019-06-18', 8.71, 1340),
  (1668, 'FF0A-C1BE-26BB-935F', '2022-04-05', 9.77, 1341),
  (1554, '06A0-4989-0901-C706', '2016-02-15', 17.63, 1342),
  (1469, 'DA77-EF34-D9F3-ED4A', '2020-06-24', 8.24, 1343),
  (17, '5CCE-9773-66C2-FABA', '1997-12-13', 8.97, 1344),
  (19, '9CDC-4BA3-774D-1B7E', '2015-03-03', 6.04, 1345),
  (18, 'C5A2-3B0F-8066-6C72', '1995-03-19', 19.51, 1345),
  (957, '007D-7C06-C493-6718', '2016-07-11', 13.95, 1346),
  (636, '40E3-1178-A263-7FEE', '2009-04-06', 18.59, 1347),
  (103, 'B835-3CC5-4638-3CBD', '2009-07-13', 5.59, 1348),
  (388, '3B5F-5B08-911E-1C1D', '2019-06-20', 7.17, 1349),
  (1492, 'D23F-BD1F-B1B1-758A', '2016-08-03', 11.76, 1350),
  (520, '1D7B-6407-D4CC-0CA5', '1994-10-26', 13.49, 1351),
  (1671, 'DCB7-3F5A-7E39-D753', '2016-01-23', 16.14, 1352),
  (94, '622A-5DBE-23BB-97E1', '2022-08-03', 8.37, 1353),
  (93, '0C85-15E1-D353-5184', '2016-09-07', 12.04, 1353),
  (386, '523A-7FF9-EE8A-A22A', '2022-05-10', 8.52, 1354),
  (740, '6A0B-9A50-5BC3-C8FD', '2009-09-28', 6.17, 1355),
  (812, 'C4F6-DC8C-46BA-2A04', '2017-08-10', 6.72, 1356),
  (455, '5F78-9644-15D2-5068', '2021-12-22', 15.18, 1357),
  (1158, '55F5-D114-3960-1446', '2010-09-06', 18.69, 1359),
  (1156, '717B-9615-F46C-8A7D', '1992-08-20', 18.80, 1359),
  (1157, 'E142-3ED1-19A1-4BE2', '2019-12-07', 5.89, 1359),
  (1205, '2F4D-B426-1DD3-98F4', '2013-01-12', 8.54, 1360),
  (1204, '0DC4-5363-E2B9-934C', '2021-07-12', 6.94, 1360),
  (1206, '2748-F84B-0CAC-5E6E', '2021-12-15', 13.39, 1361),
  (130, 'C064-F1C3-F645-627A', '2021-09-09', 15.28, 1362),
  (209, 'D8A7-A8C6-CDC3-5153', '2020-03-18', 6.02, 1363),
  (1021, '2459-FE77-25D2-DDC5', '2021-11-29', 10.27, 1364),
  (1020, '71E9-A695-4BAD-0826', '2020-04-30', 17.71, 1364),
  (1022, 'BEDD-7242-3E27-D4C6', '2021-12-11', 16.49, 1364),
  (1600, 'C3A8-8F92-4CC5-464F', '2013-11-11', 17.87, 1365),
  (928, 'C78F-3A4C-2C79-8B99', '2009-08-02', 12.54, 1366),
  (1568, '3B37-57A2-1C3A-2ED3', '2013-11-11', 16.25, 1367),
  (966, 'D195-5298-1D25-8595', '2021-07-14', 10.01, 1368),
  (965, 'F50A-EF67-CFC9-F913', '2014-12-12', 12.36, 1368),
  (967, '8C95-7793-AC62-C3FF', '2013-06-13', 9.81, 1368),
  (1618, '5225-23A5-FE3B-48D4', '2006-04-07', 8.07, 1369),
  (1449, 'CD50-7EBF-2543-DCFA', '2015-08-09', 11.08, 1370),
  (180, '68E7-8904-C2D0-5AD4', '2019-06-27', 5.52, 1371),
  (1450, 'B10B-C571-C4A4-5571', '2022-08-19', 7.76, 1372),
  (1426, 'E5BA-76E0-AF6D-25FD', '2021-06-05', 18.94, 1373),
  (1673, '9F4E-081A-DDC1-686C', '2000-09-06', 5.59, 1374),
  (1674, '496A-BDB9-96AF-8D5F', '2006-08-27', 12.23, 1374),
  (43, '27CC-CBE6-6A86-836F', '1999-01-10', 11.99, 1375),
  (1070, 'B76A-E74F-B1A3-5D8D', '2016-09-25', 19.46, 1376),
  (799, 'EB16-F030-D1A0-C392', '2022-12-12', 7.31, 1377),
  (487, '410A-1BEC-1518-22F6', '2021-08-01', 10.45, 1378),
  (372, '847D-88E9-B1A4-3335', '2015-04-20', 5.48, 1379),
  (1386, '5694-77F5-B5CA-E5F0', '2019-03-19', 13.19, 1409),
  (296, 'C6B3-DFF3-0B6F-F891', '2013-04-10', 16.37, 1380),
  (1182, '6EE6-C35B-F841-C726', '2010-05-21', 17.68, 1381),
  (275, '435B-A5AA-F249-3D8E', '2020-09-17', 13.48, 1382),
  (276, '7DF1-8057-36E0-A73A', '2009-04-30', 11.36, 1382),
  (277, '7BAA-FC98-7AFE-9901', '2022-10-02', 12.81, 1383),
  (91, 'C6D0-73B4-855C-2A69', '2008-07-26', 6.92, 1384),
  (217, '6725-7E91-E9E8-FA92', '1988-11-15', 10.74, 1385),
  (1121, '69F7-1DA5-2620-09AE', '2000-05-22', 5.33, 1386),
  (1122, '4CED-FCB4-3EB3-A546', '2013-07-06', 6.16, 1387),
  (881, 'F5D8-91C5-05B7-A5AF', '2004-09-24', 6.31, 1388),
  (452, '177B-5D76-CBDE-9B3A', '2021-02-24', 11.65, 1389),
  (1285, '66C5-581A-F6F6-F1B2', '2021-03-05', 7.62, 1390),
  (1145, '8999-E364-C15B-2322', '2020-10-11', 12.93, 1391),
  (737, 'FF11-F047-4A78-05BF', '2007-10-07', 15.46, 1392),
  (738, 'AA8D-A950-4C50-3792', '2013-04-01', 10.24, 1393),
  (90, '1738-8E9F-A363-AA25', '2004-07-03', 9.32, 1394),
  (1520, 'D9D5-0D11-A122-E730', '2014-11-30', 14.96, 1395),
  (173, '265C-2A09-8F87-FDC6', '2019-01-18', 11.33, 1396),
  (993, '035D-EE85-03F4-5FEB', '2022-04-01', 16.02, 1397),
  (1533, '7643-5167-8894-E7C4', '2021-01-12', 7.54, 1398),
  (741, '5633-E027-EEAE-B747', '2009-04-27', 19.81, 1399),
  (548, 'CC6B-A721-5E0E-E693', '2013-02-01', 5.97, 1400),
  (1146, 'F3C7-0754-42AA-7795', '2019-07-02', 5.27, 1401),
  (239, '58DA-5825-909D-2703', '2022-04-12', 13.89, 1402),
  (240, '9D0E-A5A3-3814-EB04', '2022-04-19', 9.18, 1403),
  (1603, 'F5F6-2311-0DB2-291B', '2022-12-11', 5.91, 1404),
  (1219, '4E37-41C2-C768-9367', '2017-09-03', 17.85, 1405),
  (1252, '8DF0-871A-2329-77C1', '2021-02-03', 5.49, 1406),
  (1251, '2AA0-1E38-A2AA-0DE4', '2022-05-03', 9.45, 1406),
  (46, '144A-898E-772D-665A', '2021-01-30', 19.35, 1407),
  (591, 'BC94-2BD8-F9E3-84CB', '2000-11-18', 12.98, 1408),
  (1329, '3796-1037-1251-E64A', '2022-10-13', 16.46, 1410),
  (1328, 'EBD6-6277-7A67-CB8F', '2010-06-08', 11.28, 1410),
  (1330, 'AA01-1990-3DF7-7031', '2015-06-22', 11.64, 1410)
;

--
-- Data for Name: Media; Type: TABLE DATA; Schema: Library Management; Owner: -
--

INSERT INTO "Media" (id, "Type") VALUES 
  (1, 'Hardcover'),
  (2, 'Paperback');

--
-- Data for Name: Patrons; Type: TABLE DATA; Schema: Library Management; Owner: -
--

INSERT INTO "Patrons" (id, "First Name", "Last Name", "Email") VALUES 
  (1, 'Tanya', 'Ramirez', 'tanyaramirez43@hernandez.com'),
  (2, 'Keith', 'Jones', 'keithj11@sanders.com'),
  (3, 'Tom', 'Matthews', 'tmatthews17@alexander-woods.com'),
  (4, 'Brooke', 'Williams', 'brooke.williams@randolph.info'),
  (5, 'Mathew', 'Burnett', 'mburnett@fletcher.com'),
  (6, 'Samantha', 'Robinson', 'srobinson20@barnes.com'),
  (7, 'Aaron', 'Johnson', 'aaronjohnson53@wise.com'),
  (8, 'Donna', 'Baker', 'd.baker@castro.com'),
  (9, 'Susan', 'Stevens', 's.stevens63@martinez.biz'),
  (10, 'Justin', 'Bowman', 'jbowman@williams.com'),
  (11, 'Deborah', 'Torres', 'deborahtorres40@reyes-christensen.com'),
  (12, 'Stacey', 'Hall', 's.hall@rodriguez.info'),
  (13, 'Nancy', 'Campbell', 'nancyc38@gray-vasquez.com'),
  (14, 'Michelle', 'Reynolds', 'michelle.r66@payne-davis.net'),
  (15, 'Leroy', 'Herring', 'lherring26@preston-gardner.org'),
  (16, 'Stephanie', 'King', 's.king75@murray.com'),
  (17, 'Rachel', 'Cruz', 'rcruz@parrish.com'),
  (18, 'Lauren', 'Richards', 'laurenr@hamilton-wilson.com'),
  (19, 'Jennifer', 'Ortiz', 'jortiz@casey-mendez.com'),
  (20, 'Samantha', 'Lawson', 'samanthalawson47@ray-taylor.biz'),
  (21, 'Hailey', 'Wright', 'hwright88@silva-manning.biz'),
  (22, 'Alice', 'Thompson', 'athompson76@stein.com'),
  (23, 'Robert', 'Medina', 'robert.medina@brooks.com'),
  (24, 'Tiffany', 'Evans', 'tiffanyevans76@sharp-reid.com'),
  (25, 'William', 'Lewis', 'williamlewis@mason.net'),
  (26, 'Katie', 'Moody', 'katiem90@mason.com'),
  (27, 'Richard', 'Ortiz', 'richardortiz63@wade-wolfe.org'),
  (28, 'Walter', 'Heath', 'wheath42@rodriguez.com'),
  (29, 'James', 'Nguyen', 'james.nguyen61@douglas.info'),
  (30, 'Samantha', 'Butler', 's.butler@curry-green.info'),
  (31, 'Aaron', 'Black', 'aaronb@bond.com'),
  (32, 'Jacob', 'Carson', 'j.carson55@cox-hernandez.com'),
  (33, 'Ashley', 'Miller', 'ashleym44@randolph-carr.biz'),
  (34, 'Ethan', 'Deleon', 'edeleon40@friedman.com'),
  (35, 'Sandra', 'Bates', 'sandrabates26@johnson.com'),
  (36, 'Wendy', 'James', 'wendy.j70@richard-white.biz'),
  (37, 'Kara', 'Dawson', 'kara.dawson@johnson-summers.net'),
  (38, 'Cynthia', 'Jacobs', 'cynthia.jacobs@greene.com'),
  (39, 'Richard', 'Martin', 'richard.m37@johnson.info'),
  (40, 'Jared', 'Underwood', 'junderwood92@duarte-smith.net'),
  (41, 'Beth', 'Payne', 'bpayne@price-gray.com'),
  (42, 'Theresa', 'Wallace', 't.wallace26@harper-werner.biz'),
  (43, 'David', 'Nguyen', 'davidn96@douglas.com'),
  (44, 'Kristin', 'Weiss', 'kristin.w@young.info'),
  (45, 'Billy', 'Robinson', 'billy.r82@evans.info'),
  (46, 'Thomas', 'Aguirre', 'thomasa6@morris-anderson.com'),
  (47, 'Tanner', 'Suarez', 'tsuarez61@lopez.com'),
  (48, 'Nicholas', 'Cruz', 'n.cruz@james.biz'),
  (49, 'Arthur', 'Whitney', 'arthurw65@valdez-mendoza.com'),
  (50, 'Pamela', 'Mendez', 'pamelam@stewart.com'),
  (51, 'Steven', 'Davis', 's.davis@diaz.com'),
  (52, 'Tyler', 'Martinez', 'tyler.martinez95@nichols.biz'),
  (53, 'Erica', 'Mendoza', 'erica.mendoza@collins-tyler.info'),
  (54, 'Jeremy', 'Arroyo', 'j.arroyo61@johnson.com'),
  (55, 'Wayne', 'Lyons', 'wlyons@faulkner.biz'),
  (56, 'Jesus', 'Cox', 'j.cox45@kennedy-jacobs.biz'),
  (57, 'Amanda', 'Moore', 'amanda.m@smith-jones.com'),
  (58, 'Nathan', 'Sutton', 'nathans33@sullivan-wallace.info'),
  (59, 'Joseph', 'Jones', 'jjones74@black.com'),
  (60, 'Ashley', 'Kennedy', 'ashleykennedy36@camacho.biz'),
  (61, 'David', 'Anderson', 'david.anderson62@petty-dalton.com'),
  (62, 'Beth', 'Owens', 'betho@lopez.com'),
  (63, 'Julie', 'Hamilton', 'jhamilton@dominguez.com'),
  (64, 'Deborah', 'Harris', 'deborahh96@williams.org'),
  (65, 'April', 'Anthony', 'a.anthony@mcgee-barnett.info'),
  (66, 'Mathew', 'Rosales', 'm.rosales@schwartz.com'),
  (67, 'Elizabeth', 'Buchanan', 'elizabeth.buchanan21@owens.com'),
  (68, 'Leslie', 'Houston', 'lesliehouston95@wilson.com'),
  (69, 'Paul', 'Marshall', 'p.marshall@hinton.info'),
  (70, 'Jennifer', 'Lee', 'jennifer.l65@carr.com'),
  (71, 'Mark', 'Harrison', 'mark.harrison@campbell.com'),
  (72, 'Joan', 'Austin', 'joan.austin68@george.net'),
  (73, 'Steven', 'Miles', 'smiles@gilmore.org'),
  (74, 'Mary', 'Aguilar', 'marya@melendez.com'),
  (75, 'Elizabeth', 'Gross', 'e.gross@cantu-hendrix.org'),
  (76, 'Tammy', 'Perry', 'tperry13@jones.com'),
  (77, 'Jose', 'Wilson', 'jose.wilson42@ruiz.net'),
  (78, 'Paula', 'Johnson', 'paula.johnson@orr.com'),
  (79, 'Jared', 'Smith', 'jaredsmith29@lopez-howard.com'),
  (80, 'Stephanie', 'Johnson', 'stephaniej@wright-kim.com'),
  (81, 'John', 'Hernandez', 'j.hernandez@cole.com'),
  (82, 'Sara', 'Nicholson', 'sara.nicholson@brewer-blackburn.com'),
  (83, 'Carrie', 'Chapman', 'carrie.chapman@rhodes-cordova.net'),
  (84, 'Larry', 'Kirk', 'larryk@wallace-wilson.com'),
  (85, 'Alicia', 'Coleman', 'a.coleman33@price.com'),
  (86, 'Sarah', 'Snyder', 'sarah.s@herrera-kennedy.com'),
  (87, 'Joseph', 'Herrera', 'joseph.herrera31@perez-murphy.com'),
  (88, 'Kristen', 'Sheppard', 'kristens71@mccoy.com'),
  (89, 'Francisco', 'Harvey', 'f.harvey89@garza-smith.com'),
  (90, 'Kevin', 'Berry', 'kberry@gregory.com'),
  (91, 'Jose', 'Ross', 'j.ross@nixon-johnson.com'),
  (92, 'Emily', 'Walker', 'e.walker72@daniels.com'),
  (93, 'Heather', 'Hunt', 'heather.h32@jackson.com'),
  (94, 'Kimberly', 'Garcia', 'kimberly.garcia@cole-greene.info'),
  (95, 'Gregory', 'Ramos', 'g.ramos@james.com'),
  (96, 'Douglas', 'Foley', 'douglasfoley@brown.com'),
  (97, 'Johnny', 'George', 'johnny.g89@fleming-andrade.biz'),
  (98, 'Dennis', 'Mendez', 'd.mendez71@conner.org'),
  (99, 'Jennifer', 'Miranda', 'jenniferm93@fisher-campbell.org'),
  (100, 'Kent', 'Hawkins', 'k.hawkins58@morales-johnson.biz'),
  (101, 'Bruce', 'Williams', 'bruce.williams@scott.com'),
  (102, 'Blake', 'Palmer', 'bpalmer@dodson-moore.biz'),
  (103, 'Ryan', 'Wright', 'rwright@oconnor-allen.com'),
  (104, 'Corey', 'Nelson', 'c.nelson86@johnson.com'),
  (105, 'Victor', 'Haney', 'victorhaney@robinson.org'),
  (106, 'Lori', 'Hunter', 'l.hunter34@little.com'),
  (107, 'Maria', 'Marks', 'maria.m83@wood-short.com'),
  (108, 'Ryan', 'Macdonald', 'rmacdonald@wood.biz'),
  (109, 'Javier', 'Hansen', 'javierh@hernandez.com'),
  (110, 'Amanda', 'Golden', 'a.golden59@martin-lara.com'),
  (111, 'Brandon', 'Miller', 'brandon.m89@hampton-long.biz'),
  (112, 'Caitlin', 'Gallagher', 'cgallagher54@bradley.com'),
  (113, 'Madeline', 'Miller', 'm.miller@patton.com'),
  (114, 'Kristie', 'Macdonald', 'k.macdonald89@gonzalez.com'),
  (115, 'Paul', 'Swanson', 'paulswanson@turner-lee.com'),
  (116, 'Thomas', 'Lloyd', 'thomas.lloyd@fox-ortiz.org'),
  (117, 'Tammy', 'Mcbride', 'tammym@middleton.org'),
  (118, 'Robert', 'Jones', 'robert.j15@petty-george.org'),
  (119, 'Louis', 'Beck', 'l.beck37@rowe.com'),
  (120, 'John', 'Barron', 'john.b84@smith-johnson.biz'),
  (121, 'William', 'Fitzpatrick', 'wfitzpatrick@morris.info'),
  (122, 'William', 'Torres', 'williamtorres@wilson.biz'),
  (123, 'Jodi', 'Austin', 'jaustin@ho.com'),
  (124, 'Cristina', 'Martinez', 'cmartinez41@ortiz-schmidt.com'),
  (125, 'Carrie', 'Rios', 'crios@miller-cruz.com'),
  (126, 'Joshua', 'Peterson', 'joshua.peterson@leonard.com'),
  (127, 'Brandon', 'White', 'bwhite38@smith.com'),
  (128, 'Jennifer', 'Miller', 'jmiller@cruz.com'),
  (129, 'Hunter', 'Guzman', 'hunter.g31@buck.com'),
  (130, 'Adam', 'Diaz', 'a.diaz15@dominguez.com'),
  (131, 'Robert', 'Gibbs', 'robertgibbs@rubio.com'),
  (132, 'Kelly', 'Hodges', 'kelly.hodges@singh.com'),
  (133, 'Thomas', 'Gillespie', 'thomas.gillespie@greer-horne.com'),
  (134, 'James', 'Gutierrez', 'james.gutierrez20@wright.com'),
  (135, 'Kevin', 'Vasquez', 'kevinvasquez@davis.com'),
  (136, 'Sara', 'Huff', 'sarahuff@fuller.com'),
  (137, 'Stephen', 'Acosta', 'stephenacosta32@long.com'),
  (138, 'David', 'Myers', 'david.myers@miller-walker.com'),
  (139, 'Stephanie', 'Miller', 'stephanie.miller79@watson.com'),
  (140, 'Sara', 'Williamson', 'swilliamson71@rodriguez.net'),
  (141, 'Patricia', 'Martinez', 'p.martinez11@mejia-tran.com'),
  (142, 'Jose', 'Rhodes', 'jose.r@jones.com'),
  (143, 'Margaret', 'Cook', 'margaret.c41@orozco-randolph.com'),
  (144, 'Kristina', 'Dickerson', 'kdickerson@murray.biz'),
  (145, 'Jay', 'Lang', 'jlang48@johnson.com'),
  (146, 'Diana', 'Roberson', 'droberson@brewer.com'),
  (147, 'Laura', 'Lara', 'llara71@gibbs-newton.com'),
  (148, 'Cynthia', 'Reese', 'cynthiar12@rosales.com'),
  (149, 'Mary', 'Hughes', 'mary.hughes@simmons.com'),
  (150, 'Lisa', 'Fowler', 'lfowler45@thomas.com'),
  (151, 'Shawna', 'Ferguson', 'shawna.ferguson@giles-hansen.info'),
  (152, 'Mathew', 'Key', 'm.key@phillips.com'),
  (153, 'Tara', 'Wright', 'twright@lee.info'),
  (154, 'Mary', 'Bennett', 'mbennett@ross-morris.com'),
  (155, 'Matthew', 'Green', 'mgreen82@jackson.info'),
  (156, 'Amber', 'Joseph', 'ajoseph79@brewer.biz'),
  (157, 'Shane', 'Norman', 'shanenorman@arias.com'),
  (158, 'Debbie', 'Wilson', 'dwilson@sanchez.com'),
  (159, 'Cynthia', 'Johnston', 'c.johnston@thompson-morales.com'),
  (160, 'Kathy', 'Thompson', 'kathythompson@ramirez-johnson.info'),
  (161, 'David', 'Anderson', 'david.anderson@rodriguez.com'),
  (162, 'Scott', 'Carey', 'scarey48@phillips.info'),
  (163, 'Holly', 'Johnson', 'hjohnson67@gonzalez.info'),
  (164, 'William', 'Garcia', 'william.garcia@jackson-nguyen.com'),
  (165, 'Bernard', 'Henry', 'bernard.h35@colon.com'),
  (166, 'Deborah', 'Mason', 'dmason@barker.com'),
  (167, 'Rebecca', 'Mitchell', 'rebeccamitchell74@miller.biz'),
  (168, 'Julie', 'Jackson', 'jjackson23@bennett.com'),
  (169, 'Jesse', 'Buck', 'j.buck90@johnston.com'),
  (170, 'Tammy', 'Powell', 'tammypowell@parsons-rowland.com'),
  (171, 'Lisa', 'Mullins', 'lisamullins@gomez-francis.com'),
  (172, 'Michelle', 'Dominguez', 'michelledominguez44@li.net'),
  (173, 'Matthew', 'Henderson', 'm.henderson15@lewis.com'),
  (174, 'Lucas', 'Potter', 'lucasp44@martinez.com'),
  (175, 'Mark', 'Waters', 'mwaters15@sanders.com'),
  (176, 'Jeffrey', 'Solis', 'jeffrey.s@fletcher-hansen.com'),
  (177, 'Grace', 'Hunter', 'grace.hunter57@simpson.net'),
  (178, 'Scott', 'Henry', 'scott.henry@ingram.info'),
  (179, 'Kim', 'Richmond', 'k.richmond8@williams-schmitt.com'),
  (180, 'Lisa', 'Park', 'lisapark90@nguyen.com'),
  (181, 'Ronald', 'Sparks', 'ronald.s43@cuevas.com'),
  (182, 'Melissa', 'Holder', 'melissaholder@mcintyre.biz'),
  (183, 'Alexander', 'Paul', 'a.paul64@horn.org'),
  (184, 'Natalie', 'Jensen', 'nataliej@craig.com'),
  (185, 'Valerie', 'Medina', 'vmedina57@bailey-thomas.info'),
  (186, 'Cathy', 'Taylor', 'cathytaylor@roman.info'),
  (187, 'Tiffany', 'Allen', 'tallen@phillips-buchanan.com'),
  (188, 'Patricia', 'Anderson', 'p.anderson57@french.com'),
  (189, 'Gabriella', 'Kim', 'gabriellak@dillon-cooper.org'),
  (190, 'Emma', 'Torres', 'emma.torres@guerra.info'),
  (191, 'Shawn', 'Esparza', 'sesparza11@rose-gonzalez.com'),
  (192, 'Jesus', 'Dixon', 'j.dixon@nguyen.com'),
  (193, 'Michael', 'Nelson', 'michael.n76@hobbs.info'),
  (194, 'William', 'Miranda', 'williammiranda45@smith-edwards.biz'),
  (195, 'Eric', 'Coleman', 'ecoleman@russell.org'),
  (196, 'Aaron', 'Carroll', 'acarroll17@morgan.com'),
  (197, 'Kristina', 'Reyes', 'kristinareyes@wright.org'),
  (198, 'Joanna', 'Moss', 'joanna.m@flores.com'),
  (199, 'Ronald', 'Hoover', 'ronald.h@morrison.biz'),
  (200, 'Amy', 'Rose', 'a.rose@andrews.org'),
  (201, 'Tammy', 'Adams', 't.adams59@clark.biz'),
  (202, 'Andrea', 'Martinez', 'andreamartinez@franco.com'),
  (203, 'Peggy', 'Kelly', 'p.kelly@thornton.org'),
  (204, 'David', 'Huff', 'dhuff@white-gonzales.com'),
  (205, 'Whitney', 'Peters', 'whitneyp96@murphy.com'),
  (206, 'Sean', 'Ortiz', 's.ortiz@washington.com'),
  (207, 'Michael', 'Martin', 'michaelm@johnson-mosley.biz'),
  (208, 'Maria', 'Davis', 'm.davis39@parker.net'),
  (209, 'Gregory', 'Reese', 'g.reese@ferrell.com'),
  (210, 'Todd', 'Phillips', 'toddphillips47@mora-fields.info'),
  (211, 'Travis', 'Paul', 'travis.p97@smith.com'),
  (212, 'John', 'Dodson', 'jdodson12@mcgee-huerta.com'),
  (213, 'Edwin', 'Mccullough', 'emccullough@floyd.com'),
  (214, 'Megan', 'Pierce', 'megan.p@marks-wilson.com'),
  (215, 'Timothy', 'Lewis', 'tlewis@schroeder.org'),
  (216, 'Kimberly', 'Peterson', 'k.peterson10@kelly.com'),
  (217, 'John', 'Garcia', 'john.g85@howell.com'),
  (218, 'Justin', 'Watkins', 'justinw@miller.com'),
  (219, 'Elizabeth', 'Marshall', 'e.marshall@ayers.com'),
  (220, 'Elizabeth', 'Johnson', 'elizabeth.johnson@woodward-ellis.com'),
  (221, 'Holly', 'Valenzuela', 'hollyvalenzuela39@johnson.com'),
  (222, 'Spencer', 'Nguyen', 'spencern36@atkins.com'),
  (223, 'Ruth', 'Edwards', 'ruth.e@hamilton.com'),
  (224, 'Jon', 'Lawson', 'jlawson@martinez-blair.com'),
  (225, 'Dorothy', 'Cox', 'dorothy.cox88@cervantes.com'),
  (226, 'John', 'Guerrero', 'jguerrero@jackson.biz'),
  (227, 'Justin', 'Cook', 'jcook27@medina-maynard.com'),
  (228, 'Karen', 'Gross', 'kgross12@hines.info'),
  (229, 'Lisa', 'Gonzalez', 'l.gonzalez@avery.org'),
  (230, 'Timothy', 'Ashley', 'tashley41@swanson-wood.com'),
  (231, 'Peggy', 'Davis', 'pdavis@becker.com'),
  (232, 'Tina', 'Moon', 'tinamoon@gordon.com'),
  (233, 'Michael', 'Marshall', 'mmarshall37@patel-velasquez.org'),
  (234, 'Amy', 'Brown', 'abrown@barnes-terry.net'),
  (235, 'Erica', 'Silva', 'e.silva@pacheco-roach.com'),
  (236, 'Jennifer', 'Davis', 'jdavis33@carroll.com'),
  (237, 'Tony', 'Stone', 't.stone98@romero.com'),
  (238, 'Madison', 'Deleon', 'mdeleon89@miller.com'),
  (239, 'Kim', 'Williams', 'kimwilliams@clark.com'),
  (240, 'Angel', 'Moses', 'amoses@lewis-carpenter.com'),
  (241, 'Anthony', 'Conrad', 'aconrad84@butler-smith.org'),
  (242, 'David', 'Alvarado', 'dalvarado38@wallace-heath.net'),
  (243, 'Jacob', 'Burton', 'jacobb56@thomas-melton.com'),
  (244, 'Christopher', 'Johnson', 'cjohnson30@harper-black.com'),
  (245, 'Shelley', 'Lloyd', 'slloyd@harrell.biz'),
  (246, 'Shelby', 'Shepherd', 'shelby.shepherd@randall.com'),
  (247, 'Robert', 'Mccormick', 'r.mccormick28@brown-hughes.biz'),
  (248, 'Dawn', 'Hale', 'dhale@williams.com'),
  (249, 'Crystal', 'Mejia', 'c.mejia57@cannon.com'),
  (250, 'Rick', 'Ortiz', 'rick.o85@martin-adams.com')
;

--
-- Data for Name: Publishers; Type: TABLE DATA; Schema: Library Management; Owner: -
--

INSERT INTO "Publishers" (id, "Name") VALUES 
  (239, 'The Mysterious Press'),
  (268, 'WH Allen'),
  (71, 'Dover Publications'),
  (84, 'Fine Communications'),
  (76, 'Dynamite Entertainment'),
  (40, 'Broadway Books'),
  (34, 'Berkley'),
  (67, 'Dodd, Mead'),
  (27, 'Bantam Classics'),
  (101, 'Hackett Pub Co'),
  (253, 'Unbound'),
  (214, 'Shroff Publishers & Distributors'),
  (19, 'Aspect'),
  (207, 'Sams Publishing'),
  (194, 'Putnam Juvenile'),
  (5, 'Adams Media'),
  (218, 'Simon & Brown'),
  (91, 'Geddes & Grosset'),
  (135, 'Knopf Books for Young Readers'),
  (45, 'Chronicle Books'),
  (221, 'Simon & Schuster UK'),
  (211, 'Scribner'),
  (271, 'William Morrow Cookbooks'),
  (116, 'Harvard University Press'),
  (96, 'Grand Central Publishing'),
  (178, 'Penguin Press'),
  (191, 'PuddleDancer Press'),
  (137, 'Kodansha USA'),
  (249, 'Turtleback'),
  (153, 'Miramax'),
  (113, 'Harper Voyager'),
  (243, 'Tor Books'),
  (264, 'VIZ Media LLC'),
  (179, 'Penguin Putnam'),
  (80, 'Faber and Faber'),
  (70, 'Doubleday Canada'),
  (226, 'Speak'),
  (79, 'Eos'),
  (105, 'Harper Business'),
  (56, 'Dark Horse'),
  (4, 'Ace Books'),
  (60, 'Dell'),
  (172, 'Pan Books'),
  (3, 'Academic Press Inc'),
  (219, 'Simon Pulse'),
  (186, 'Portfolio'),
  (25, 'Ballantine Books'),
  (237, 'Ten Speed Press'),
  (144, 'Macmillan Children''s Books'),
  (11, 'America''s Best Comics'),
  (35, 'Black Dog & Leventhal Publishers'),
  (133, 'Kingfisher'),
  (90, 'Gallery Books'),
  (198, 'Random House'),
  (206, 'Saga Press'),
  (210, 'Scholastic Press'),
  (108, 'Harper Perennial'),
  (55, 'Da Capo Press'),
  (124, 'Independently published'),
  (201, 'Riverhead Hardcover'),
  (161, 'New Riders'),
  (102, 'Harcourt'),
  (75, 'Dutton Juvenile'),
  (168, 'O''Reilly Media'),
  (57, 'DAW'),
  (97, 'Graphics Press'),
  (65, 'Diamond'),
  (208, 'Sceptre'),
  (111, 'HarperTorch'),
  (41, 'Cambridge University Press'),
  (88, 'Four Elephants Press'),
  (42, 'Chelsea Green Publishing'),
  (228, 'Stanford University Press'),
  (141, 'Little, Brown Spark'),
  (51, 'Cricket Books'),
  (261, 'Viking Juvenile'),
  (146, 'Mariner Books'),
  (21, 'Atria Books'),
  (250, 'Turtleback Books'),
  (74, 'Dreamhaven Books'),
  (270, 'William Morrow'),
  (244, 'Tor Classics'),
  (258, 'Vertigo'),
  (14, 'Anchor'),
  (6, 'Addison Wesley'),
  (120, 'Hill House Publishers'),
  (126, 'JABberwocky Literary Agency, Inc.'),
  (82, 'Fawcett'),
  (248, 'Trophy'),
  (252, 'T.Y. Crowell Junior Books'),
  (117, 'Harvill Secker'),
  (145, 'Macmillan USA'),
  (44, 'Children''s High Level Group'),
  (83, 'Feiwel & Friends'),
  (222, 'SitePoint'),
  (38, 'Bloomsbury USA'),
  (28, 'Bantam Dell'),
  (59, 'Delacorte Press'),
  (247, 'Tor Teen'),
  (129, 'Jonathan Cape'),
  (213, 'Shadow Mountain'),
  (100, 'Hachette Books'),
  (169, 'Orion Pub Co'),
  (229, 'Starscape'),
  (185, 'Popular Library'),
  (188, 'Pragmatic Bookshelf'),
  (216, 'Signet'),
  (86, 'Five Seven Five Books'),
  (107, 'HarperCollins Canada'),
  (236, 'Tachyon Publications'),
  (162, 'New World Library'),
  (50, 'CreateSpace'),
  (46, 'Citadel'),
  (259, 'Viking'),
  (251, 'Two Waves Books'),
  (118, 'Headline'),
  (180, 'Penguin Random House'),
  (2, 'A Book Apart'),
  (23, 'Back Bay Books'),
  (238, 'The Chicken House'),
  (8, 'Aladdin'),
  (150, 'Millennium Paperbacks'),
  (163, 'North River Press'),
  (1, '37signals'),
  (197, 'Quill Tree Books'),
  (147, 'Material World'),
  (24, 'Baen'),
  (262, 'Villard'),
  (276, NULL),
  (36, 'Black Swan'),
  (7, 'Aerie'),
  (183, 'Picador'),
  (132, 'Katherine Tegen Books'),
  (225, 'Sourcebooks Landmark'),
  (204, 'Rosenfeld Media'),
  (48, 'Collins'),
  (16, 'Apress'),
  (164, 'NTC/Contemporary Publishing Company'),
  (122, 'Hyperion Book CH'),
  (184, 'Pocket Books'),
  (182, 'Periplus Editions'),
  (69, 'Doubleday Business'),
  (256, 'Vault Books'),
  (106, 'HarperCollins'),
  (170, 'OUP Oxford'),
  (190, 'Prometheus Books'),
  (110, 'HarperTeen'),
  (266, 'Warner'),
  (61, 'Del Rey'),
  (22, 'Avon'),
  (257, 'Vault Comics'),
  (143, 'lulu.com'),
  (263, 'Vintage'),
  (140, 'Little Brown & Co'),
  (134, 'Knopf'),
  (200, 'Riverhead Books'),
  (138, 'K & S Ranch'),
  (177, 'Penguin India'),
  (154, 'MIT Press'),
  (10, 'Ambient Press'),
  (29, 'Bantam Doubleday Dell'),
  (235, 'Sybex'),
  (230, 'Stewart, Tabori & Chang'),
  (212, 'SeaWolf Press'),
  (136, 'Kodansha International'),
  (115, 'Harvard Business Review Press'),
  (187, 'Portfolio Penguin'),
  (215, 'Sierra Club Books'),
  (20, 'Atlantic Books'),
  (159, 'NESFA Press'),
  (93, 'Gotham'),
  (53, 'Currency'),
  (31, 'Basic Books'),
  (139, 'Laurel Leaf'),
  (175, 'Penguin Classics'),
  (103, 'Harcourt Brace Jovanovich'),
  (73, 'Drawn and Quarterly'),
  (220, 'Simon & Schuster'),
  (166, 'Orb Books'),
  (104, 'Harper'),
  (273, 'W. W. Norton & Company'),
  (199, 'Red Fox'),
  (49, 'Corgi'),
  (99, 'Grim Oak Press'),
  (157, 'Nancy Paulsen Books'),
  (202, 'Roc'),
  (165, 'Oneworld Publications'),
  (15, 'Andrews McMeel Publishing'),
  (63, 'Demco Media'),
  (167, 'Orbit'),
  (13, 'Analytics Press'),
  (265, 'Waldman Publishing Corp.'),
  (112, 'HarperTrophy'),
  (64, 'Dial'),
  (176, 'Penguin Global'),
  (89, 'Frank Chimero'),
  (54, 'Dabel Brothers'),
  (224, 'Sourcebooks Casablanca'),
  (173, 'Pantheon'),
  (33, 'Belknap Press'),
  (255, 'University of Chicago Press'),
  (121, 'Houghton Mifflin Harcourt'),
  (267, 'Washington Square Press'),
  (127, 'Jaico Publishing House'),
  (12, 'Amulet Books'),
  (241, 'Time Life'),
  (30, 'Barnes and Noble'),
  (254, 'Universe'),
  (156, 'Morgan Kaufmann'),
  (231, 'St. Martin''s Press'),
  (125, 'Indian Thought Publications'),
  (123, 'Impress Mystery'),
  (94, 'G.P. Putnam''s Sons Books for Young Readers'),
  (242, 'Titan Books Ltd'),
  (9, 'Alfred A. Knopf'),
  (130, 'Jones & Bartlett Learning'),
  (272, 'Wordsworth Editions Ltd'),
  (227, 'Spectra'),
  (66, 'Disney-Hyperion'),
  (78, 'Empire Books'),
  (37, 'Bloomsbury'),
  (233, 'Stripe Press'),
  (52, 'Crown'),
  (196, 'Quality Paperback Book Club'),
  (151, 'Minerva'),
  (43, 'Children''s Classics'),
  (246, 'Tor Science Fiction'),
  (152, 'Minotaur Books'),
  (274, 'Yale University Press'),
  (192, 'Puffin'),
  (205, 'Rupa & Co.'),
  (240, 'Three Rivers Press'),
  (193, 'Putnam Adult'),
  (234, 'Subterranean Press'),
  (148, 'Metropolitan Books'),
  (232, 'Strange Chemistry'),
  (217, 'Signet Classics'),
  (114, 'Harry N. Abrams'),
  (95, 'Grafton'),
  (109, 'HarperPress'),
  (72, 'Dragonsteel Entertainment'),
  (98, 'Greenwillow Books'),
  (119, 'Henry Holt and Co.'),
  (32, 'BCA'),
  (87, 'Fontana Books'),
  (269, 'Wiley'),
  (18, 'Artisan'),
  (174, 'Penguin Books'),
  (85, 'Firebird'),
  (181, 'Perfection Learning'),
  (160, 'New American Library'),
  (171, 'Oxford University Press'),
  (155, 'Modern Library'),
  (209, 'Scholastic Point'),
  (260, 'Viking Adult'),
  (62, 'Del Rey/Dabel Brothers'),
  (58, 'DC Comics'),
  (131, 'Jossey-Bass'),
  (26, 'Bantam Books'),
  (68, 'Doubleday'),
  (275, 'Yearling'),
  (245, 'Tor Fantasy'),
  (47, 'Clearway Logistics Phase 1a'),
  (142, 'Liveright Publishing Corporation'),
  (128, 'John Benjamins Publishing Company'),
  (92, 'Gollancz'),
  (17, 'Arthur A. Levine Books'),
  (149, 'Microsoft Press'),
  (77, 'Ecco'),
  (189, 'Prentice Hall Press'),
  (223, 'SMK Books'),
  (195, 'Pyr'),
  (39, 'Breadpig'),
  (158, 'National Geographic'),
  (81, 'Farrar, Straus and Giroux'),
  (203, 'ROC')
;


-- Reset sequences
SELECT pg_catalog.setval('"Authors_id_seq"', 377, true);
SELECT pg_catalog.setval('"Books_id_seq"', 1410, true);
SELECT pg_catalog.setval('"Checkouts_id_seq"', 1, false);
SELECT pg_catalog.setval('"Items_id_seq"', 1712, true);
SELECT pg_catalog.setval('"Media_id_seq"', 2, true);
SELECT pg_catalog.setval('"Patrons_id_seq"', 250, true);
SELECT pg_catalog.setval('"Publishers_id_seq"', 276, true);


-- Set up constraints

ALTER TABLE ONLY "Authors"
    ADD CONSTRAINT "Authors_pkey" PRIMARY KEY (id);

ALTER TABLE ONLY "Books"
    ADD CONSTRAINT "Books_ISBN_key" UNIQUE ("ISBN");
ALTER TABLE ONLY "Books"
    ADD CONSTRAINT "Books_pkey" PRIMARY KEY (id);

ALTER TABLE ONLY "Items"
    ADD CONSTRAINT "Items_Barcode_key" UNIQUE ("Barcode");
ALTER TABLE ONLY "Items"
    ADD CONSTRAINT "Items_pkey" PRIMARY KEY (id);

ALTER TABLE ONLY "Media"
    ADD CONSTRAINT "Media_pkey" PRIMARY KEY (id);

ALTER TABLE ONLY "Patrons"
    ADD CONSTRAINT "Patrons_pkey" PRIMARY KEY (id);

ALTER TABLE ONLY "Publishers"
    ADD CONSTRAINT "Publishers_pkey" PRIMARY KEY (id);


-- Add foreign keys

ALTER TABLE ONLY "Books"
    ADD CONSTRAINT "Books_Author_fkey" FOREIGN KEY ("Author") REFERENCES "Authors"(id);
ALTER TABLE ONLY "Books"
    ADD CONSTRAINT "Books_Publisher_fkey" FOREIGN KEY ("Publisher") REFERENCES "Publishers"(id);

ALTER TABLE ONLY "Items"
    ADD CONSTRAINT "Items_Book_fkey" FOREIGN KEY ("Book") REFERENCES "Books"(id);
